﻿class BioAiController extends AIController
    hidecategories(Navigation)
    config(AI)
    native
    placeable;

var transient int m_nFireLineObstructionFrequency;

var transient int m_nFireLineObstructionCount;

var transient float m_fMobTargetMoveTolerance;

var float AdjustToSlotTime;

var transient Actor SearchGoalActor;

var float MoveOffset;

var Vector MovePoint;

var Actor MoveGoal;

var CoverInfo LastCover;

var CoverInfo CoverGoal;

var CoverInfo Cover;

var BioPawnBehavior MyBPBehavior;

var BioPawn MyBP;

var float m_fActivateTime;

var transient BioAiController OldController;

var ECoverAction PendingCoverAction;

var config float EnemyDistance_Melee;

var config float EnemyDistance_Long;

var config float EnemyDistance_Medium;

var config float EnemyDistance_Short;

var() config float AI_AccMod_LongRange;

var() config float AI_AccMod_MediumRange;

var() config float AI_AccMod_ShortRange;

var() config float AI_AccMod_BriefVisibility;

var() config float AI_AccMod_BriefAcquire;

var() config float AI_AccMod_TargMove;

var() config float AI_AccMod_Move;

var() config float AI_Acc_Target;

var() config float AI_Acc_Base;

var config float AI_Acc_InstabilityDecayRate;

var transient float LastInstabilityTime;

var transient float AimInstability;

var transient bool m_bCheckLOS;

var transient bool bFailedToFire;

var transient bool bWeaponFiring;

var int RemainingBurstFireCount;

var float WeaponAimDelay;

var float BurstFireDelay;

var int BurstFireCount;

var float TargetAcquisitionTime;

var array<BioBaseSquad> IgnoredSquads;

var array<Actor> IgnoredTargets;

var Actor PreferredTarget;

var Actor ForcedTarget;

var Actor FireTarget;

var config float Response_MinEnemyHearTime;

var config float Response_MinEnemySeenTime;

var config float InterpEnemyLocSpeed;

var array<DelayUpdateInfo> DelayUpdateList;

var array<EnemyInfo> EnemyList;

var config array<Name> AILogFilter;

var FileLog AILogFile;

var bool bMovingToCover;

var bool bReachedCover;

var bool bReachedMoveGoal;

var bool bReevaluatePath;

var const bool bHighCoverOnly;

var bool bAcquireNewCover;

var(Debug) config bool bDebug_ThreatRadius;

var(Debug) config bool bDebug_AIEnemyList;

var(Debug) config bool bDebug_AIRange;

var(Debug) config bool bDebug_AI;

var(Debug) config bool bAILogToWindow;

var(Debug) config bool bAILogging;

var Pawn oKiller;

enum EPerceptionType
{
    PT_Sight,
    PT_Heard,
    PT_HurtBy,
    PT_NotifySight,
    PT_Force,
};

enum ELocationType
{
    LT_Known,
    LT_Interp,
    LT_Exact,
};

struct native DelayUpdateInfo
{
    var EPerceptionType Type;

    var Pawn Pawn;

    var float UpdateTime;

    var Name EventName;

}

struct native EnemyInfo
{
    var Pawn Pawn;

    var Vector KnownLocation;

    var Vector InterpLocation;

    var float InterpTime;

    var CoverInfo Cover;

    var float InitialSeenTime;

    var float LastSeenTime;

    var float LastFailedPathTime;

    var bool bVisible;

}

const AI_NearMissDistance = 200;
const AI_NearbyHurtDistance = 500;
const TIMER_ADJUSTTOSLOT_LIMIT = 5.f;
const MAXSEARCHPATHLENGTH = 2000.0f;
enum eWalkWaypointsTypes
{
    WWT_Linear,
    WWT_Looping,
    WWT_OutAndBack,
    WWT_OutAndBackLooping,
    WWT_Random,
};

native function BioOnPawnActionStateChanged();

native function MapName_Hench_FailToPath(float X, float Y);

native function MapName_Hench_Teleport(float X, float Y);

native final function bool IsHostile(Controller Other);

native final function bool IsFriendly(Controller Other);

native final function bool UpdateEnemyInfo(int idx, EPerceptionType Type, optional Name EventName);

native final function int GetEnemyIndex(Pawn TestPawn);

native final function Vector GetFireTargetLocation(optional ELocationType LT);

native final function Vector GetEnemyLocation(optional Pawn TestPawn, optional ELocationType LT);

native final function Vector GetEnemyLocationByIndex(int idx, optional ELocationType LT);

native final function SetKnownEnemyInfo(int EnemyIdx, Pawn EnemyPawn, Vector EnemyLoc);

native final function bool IsEnemyVisibleByIndex(int idx);

native final function InterpEnemyLocation(int idx);

final function float TimeSinceEnemyVisible(int idx)
{
    if (idx >= 0 && idx < EnemyList.Length)
    {
        if (EnemyList[idx].bVisible == false)
        {
            return TimeSince(EnemyList[idx].LastSeenTime);
        }
    }
    return 0;
}

final function bool IsShortRange(Vector TestLocation)
{
    return VSizeSq(TestLocation - Pawn.Location) <= EnemyDistance_Short * EnemyDistance_Short;
}

final function bool IsMediumRange(Vector TestLocation)
{
    return VSizeSq(TestLocation - Pawn.Location) <= EnemyDistance_Medium * EnemyDistance_Medium;
}

function FillEnemyList()
{
    local int idx;
    local int Num;
    local BioAiController AI;
    foreach WorldInfo.AllControllers(Class'BioAiController', AI)
    {
        if (AI != self && IsFriendly(AI))
        {
            Num = AI.EnemyList.Length;
            idx = 0;
            while (idx < Num)
            {
                AddEnemy(AI.EnemyList[idx].Pawn, GetSquadName());
                idx++;
            }
        }
    }
}

event AutoAcquireEnemy()
{
    local Controller PotentialEnemy;
    foreach WorldInfo.AllControllers(Class'Controller', PotentialEnemy)
    {
        if (PotentialEnemy != self && IsHostile(PotentialEnemy))
        {
            AddEnemy(PotentialEnemy.Pawn, true, true);
        }
    }
}

event AutoNotifyEnemy()
{
    local BioAiController AI;
    foreach WorldInfo.AllControllers(Class'BioAiController', AI)
    {
        if (AI != self && IsHostile(AI))
        {
            AI.AddEnemy(Pawn, true, true);
        }
    }
}

final function UpdateAllEnemyInfo()
{
    local int idx;
    local int EnemyIdx;
    if (EnemyList.Length != 0)
    {
        EnemyIdx = 0;
        while (EnemyIdx < EnemyList.Length)
        {
            if (EnemyList[EnemyIdx].Pawn == None || EnemyList[EnemyIdx].Pawn.IsValidTargetFor(self) == false)
            {
                EnemyList.Remove(EnemyIdx--, 1);
                jump 125;
            }
            EnemyIdx++;
        }
        if (EnemyList.Length == 0)
        {
            if (BioPawn(Pawn).Squad != None)
            {
                BioPawn(Pawn).Squad.NotifyNoEnemiesPerceived();
            }
        }
    }
    idx = 0;
    while (idx < DelayUpdateList.Length)
    {
        if (WorldInfo.TimeSeconds >= DelayUpdateList[idx].UpdateTime)
        {
            EnemyIdx = EnemyList.Find('Pawn', DelayUpdateList[idx].Pawn);
            if (EnemyIdx >= 0)
            {
                UpdateEnemyInfo(EnemyIdx, DelayUpdateList[idx].Type, DelayUpdateList[idx].EventName);
            }
            DelayUpdateList.Remove(idx--, 1);
        }
        idx++;
    }
    return;
}

final function AddDelayedUpdate(int EnemyIdx, EPerceptionType Type, Name EventName, float delay)
{
    local int idx;
    local DelayUpdateInfo Info;
    if (EnemyIdx < 0 || EnemyIdx >= EnemyList.Length)
    {
        return;
    }
    if (delay <= 0)
    {
        UpdateEnemyInfo(EnemyIdx, Type, EventName);
        return;
    }
    idx = DelayUpdateList.Find('Pawn', EnemyList[EnemyIdx].Pawn);
    if (idx < 0)
    {
        Info.Pawn = EnemyList[EnemyIdx].Pawn;
        Info.Type = Type;
        Info.EventName = EventName;
        Info.UpdateTime = WorldInfo.TimeSeconds + delay;
        DelayUpdateList[DelayUpdateList.Length] = Info;
    }
}

event SeePlayer(Pawn Seen)
{
    if (Seen != None && Seen != Pawn)
    {
        if (BioPawn(Pawn).IsHostile(Seen) && CanSeePawn(Seen))
        {
            AddEnemy(Seen);
            UpdateEnemyInfo(EnemyList.Find('Pawn', Seen), 0, 'SeePlayer');
        }
    }
}

event SeeMonster(Pawn Seen)
{
    if (Seen != None && Seen != Pawn)
    {
        if (BioPawn(Pawn).IsHostile(Seen) && CanSeePawn(Seen))
        {
            AddEnemy(Seen);
            UpdateEnemyInfo(EnemyList.Find('Pawn', Seen), 0, 'SeeMonster');
        }
    }
}

function bool CanSeePawn(Pawn Seen)
{
    return true;
}

event HearNoise(float Loudness, Actor NoiseMaker, optional Name NoiseType)
{
    local Pawn Heard;
    if (MyBP == None)
    {
        return;
    }
    if (Controller(NoiseMaker) != None)
    {
        Heard = Controller(NoiseMaker).Pawn;
    }
    else
    {
        Heard = Pawn(NoiseMaker);
    }
    if (Heard != None && Heard != Pawn && MyBP.IsHostile(Heard))
    {
        if (MyBP.IsHostile(Heard) && CanHearPawn(Heard, Loudness, NoiseType))
        {
            AddEnemy(Heard);
            AddDelayedUpdate(EnemyList.Find('Pawn', Heard), 1, NoiseType, Response_MinEnemyHearTime);
        }
    }
}

function bool CanHearPawn(Pawn Heard, float Loudness, Name NoiseType)
{
    local float Dist;
    switch (NoiseType)
    {
        case 'NoiseType_PowerRelease':
        case 'NoiseType_PowerImpact':
        case 'NoiseType_WeaponFire':
        case 'NoiseType_Death':
            return true;
        case 'NoiseType_Footstep':
            Dist = VSize(Pawn.Location - Heard.Location);
            if (Dist <= 300)
            {
                return true;
            }
            else
            {
                if (MyBP != None && MyBP.bSleeping && Dist <= MyBP.fSleepPerceptionDistance)
                {
                    return true;
                }
            }
        default:
            return false;
    }
}

function CheckNearMiss(Pawn Shooter, Weapon W, Vector WeapLoc, Vector LineDir, Vector HitLocation)
{
    local Vector ClosestPoint;
    if (BioPawn(Pawn).IsHostile(Shooter) || MyBP != None && MyBP.bAmbientCreature)
    {
        if (PointDistToLine(Pawn.Location, LineDir, WeapLoc, ClosestPoint) <= 200 || VSize(HitLocation - Pawn.Location) < 200)
        {
            AddEnemy(Shooter);
            NotifyNearMiss(HitLocation);
        }
    }
}

function NotifyNearMiss(Vector HitLocation);

event bool AddEnemy(Pawn NewEnemy, optional Name BroadcastSquad, optional bool bForceNotice, optional bool bSkipBroadcast)
{
    local int idx;
    local BioAiController AI;
    local bool bHadEnemies;
    if (NewEnemy == None || NewEnemy.IsValidTargetFor(self) == false)
    {
        return false;
    }
    if (BroadcastSquad != 'None' && BroadcastSquad != GetSquadName())
    {
        return false;
    }
    if (MyBP != None && MyBP.bSleeping)
    {
        MyBP.TriggerEventClass(Class'SFXSeqEvt_OnStoppedSleeping', NewEnemy);
        MyBP.bSleeping = false;
    }
    bHadEnemies = EnemyList.Length > 0;
    idx = EnemyList.Find('Pawn', NewEnemy);
    if (idx == -1)
    {
        idx = EnemyList.Length;
        EnemyList.Length = EnemyList.Length + 1;
        EnemyList[idx].Pawn = NewEnemy;
        UpdateEnemyInfo(idx, 4, 'AddEnemy');
        NotifyNewEnemy(NewEnemy);
        if (BroadcastSquad == 'None' && !bSkipBroadcast)
        {
            foreach WorldInfo.AllControllers(Class'BioAiController', AI)
            {
                if (AI != self && IsFriendly(AI))
                {
                    AI.AddEnemy(NewEnemy, GetSquadName(), true);
                }
            }
            SFXGame(WorldInfo.Game).TriggerVocalizationEvent(17, BioPawn(Pawn), BioPawn(NewEnemy));
        }
    }
    else
    {
        if (bForceNotice)
        {
            UpdateEnemyInfo(idx, 4, 'AddEnemy');
            NotifyNewEnemy(NewEnemy);
        }
    }
    if (!bHadEnemies)
    {
        if (BioPawn(Pawn).Squad != None)
        {
            BioPawn(Pawn).Squad.NotifyEnemyPerceived();
        }
    }
    return true;
}

final function Name GetSquadName()
{
    if (MyBP != None && MyBP.Squad != None)
    {
        return MyBP.Squad.Name;
    }
    return 'None';
}

final function bool HasAnyEnemies()
{
    return EnemyList.Length > 0;
}

final function bool HasValidEnemy(optional Pawn TestEnemy)
{
    if (TestEnemy == None)
    {
        TestEnemy = Enemy;
    }
    if (TestEnemy == None || TestEnemy.IsValidTargetFor(self) == false)
    {
        return false;
    }
    return true;
}

final function bool HasValidTarget(optional Actor TestTarget)
{
    local Pawn TestPawn;
    if (TestTarget == None)
    {
        TestTarget = FireTarget;
    }
    if (TestTarget == None)
    {
        return false;
    }
    TestPawn = Pawn(TestTarget);
    if (TestPawn != None && TestPawn.IsValidTargetFor(self) == false)
    {
        return false;
    }
    return true;
}

native final function bool CanFireAt(Actor ChkTarget, Vector ViewPt, optional bool bUseEyeLocation, optional bool bUseRotation);

function Vector GetAimLocation(int EnemyIdx, Vector StartLoc)
{
    local Vector AimLoc;
    if (EnemyIdx >= 0)
    {
        AimLoc = GetEnemyLocationByIndex(EnemyIdx);
    }
    else
    {
        AimLoc = FireTarget.Location;
    }
    return AimLoc;
}

function ResetAimInstability()
{
    AimInstability = 1;
    LastInstabilityTime = WorldInfo.TimeSeconds;
}

function UpdateAccuracy()
{
    local float DeltaTime;
    DeltaTime = TimeSince(LastInstabilityTime);
    LastInstabilityTime = WorldInfo.TimeSeconds;
    if (MyBP != None && FireTarget != None)
    {
        AimInstability *= AI_Acc_InstabilityDecayRate ** DeltaTime;
        if (FireTarget.Location - MyBP.Location Dot MyBP.Rotation < 0,9)
        {
            AimInstability = 1;
        }
        AimInstability = FClamp(AimInstability, 0, 1);
    }
}

function Rotator GetAdjustedAimFor(Weapon W, Vector StartFireLoc)
{
    local Vector AimLoc;
    local SFXWeapon Wpn;
    local float InaccuracyPct;
    local float Pct;
    local Rotator AccRot;
    local Rotator AimRot;
    local int EnemyIdx;
    local SFXModule_GameEffectManager GEModule;
    local float AccuracyPenalty;
    local Vector2D AccMinCone;
    local Vector2D AccMaxCone;
    local SFXHeavyWeapon oHeavyWeapon;
    Wpn = SFXWeapon(W);
    EnemyIdx = -1;
    if (Pawn(FireTarget) != None)
    {
        EnemyIdx = EnemyList.Find('Pawn', Pawn(FireTarget));
    }
    if (MyBP != None && FireTarget != None && Wpn != None)
    {
        AimLoc = GetAimLocation(EnemyIdx, StartFireLoc);
        UpdateAccuracy();
        InaccuracyPct = AI_Acc_Base;
        if (MyBP.CoverType != 0)
        {
            InaccuracyPct = AI_Acc_Target;
        }
        if (VSize(MyBP.Velocity) > 0)
        {
            InaccuracyPct += AI_AccMod_Move;
        }
        if (VSize(FireTarget.Velocity) > 0)
        {
            InaccuracyPct += AI_AccMod_TargMove;
        }
        Pct = FClamp(1 - TimeSince(TargetAcquisitionTime) / Response_MinEnemySeenTime, 0, 1);
        InaccuracyPct += AI_AccMod_BriefAcquire * Pct;
        if (EnemyIdx >= 0 && IsEnemyVisibleByIndex(EnemyIdx))
        {
            Pct = FClamp(1 - EnemyList[EnemyIdx].LastSeenTime - EnemyList[EnemyIdx].InitialSeenTime / Response_MinEnemySeenTime, 0, 1);
            InaccuracyPct += AI_AccMod_BriefVisibility * Pct;
        }
        else
        {
            InaccuracyPct = 1;
        }
        if (IsShortRange(AimLoc))
        {
            InaccuracyPct += AI_AccMod_ShortRange;
        }
        else
        {
            if (IsMediumRange(AimLoc))
            {
                InaccuracyPct += AI_AccMod_MediumRange;
            }
            else
            {
                InaccuracyPct += AI_AccMod_LongRange;
            }
        }
        InaccuracyPct = FClamp(InaccuracyPct, 0,01, 0,99);
        InaccuracyPct = InaccuracyPct ** 1 - AimInstability;
        InaccuracyPct = FClamp(InaccuracyPct, 0, 1);
        AccMinCone = Wpn.AI_AccCone_Min;
        AccMaxCone = Wpn.AI_AccCone_Max;
        GEModule = MyBP.GetModule(Class'SFXModule_GameEffectManager');
        if (GEModule != None)
        {
            oHeavyWeapon = SFXHeavyWeapon(Wpn);
            if (oHeavyWeapon == None)
            {
                AccuracyPenalty = GEModule.GetEffectValue(Class'SFXGameEffect_AIAccuracyPenalty');
                AccMinCone.X += AccuracyPenalty;
                AccMinCone.Y += AccuracyPenalty;
                AccMaxCone.X += AccuracyPenalty;
                AccMaxCone.Y += AccuracyPenalty;
            }
        }
        AccRot.Pitch = GetRangeValueByPct(vect2d(AccMinCone.X, AccMaxCone.X), InaccuracyPct) * 182,0444;
        AccRot.Yaw = GetRangeValueByPct(vect2d(AccMinCone.Y, AccMaxCone.Y), InaccuracyPct) * 182,0444;
        AccRot.Pitch *= 0,5 - FRand();
        AccRot.Yaw *= 0,5 - FRand();
        AimRot = AimLoc - StartFireLoc + AccRot;
        return AimRot;
    }
    return Pawn.Rotation;
}

simulated event GetPlayerViewPoint(out Vector out_Location, out Rotator out_Rotation)
{
    local bool bAimAtEnemy;
    if (HasValidTarget())
    {
        bAimAtEnemy = true;
    }
    out_Location = Pawn.GetPawnViewLocation();
    out_Rotation = Pawn.Rotation;
    if (bAimAtEnemy)
    {
        out_Rotation = GetFireTargetLocation(1) - Pawn.Location;
    }
}

final function bool IsFireLineObstructed()
{
    if (IsFriendlyBlockingFireLine())
    {
        return true;
    }
    if (m_bCheckLOS && CanAttack(FireTarget) == false)
    {
        return true;
    }
    return false;
}

final function bool IsFriendlyBlockingFireLine()
{
    local BioAiController C;
    local Vector FireLine;
    local Vector FireStart;
    local Vector VectToFriendly;
    local float FireLineWidth;
    local float DistFromLine;
    if (Pawn == None)
    {
        return false;
    }
    if (FireTarget == None)
    {
        FireTarget = Enemy;
    }
    if (FireTarget == None)
    {
        return false;
    }
    FireStart = Pawn.GetWeaponStartTraceLocation();
    FireLine = Normal(FireTarget.Location - FireStart);
    FireLineWidth = 48;
    foreach WorldInfo.AllControllers(Class'BioAiController', C)
    {
        if (C != self && C.Pawn != None && C.Pawn.IsDead() == false && C.IsFriendly(self))
        {
            VectToFriendly = Normal(C.Pawn.Location - FireStart);
            if (VectToFriendly Dot FireLine > 0)
            {
                DistFromLine = FMin(PointDistToLine(C.Pawn.GetPawnViewLocation(), FireLine, FireStart), PointDistToLine(C.Pawn.Location, FireLine, FireStart));
                if (DistFromLine < FireLineWidth && VSize(C.Pawn.Location - Pawn.Location) < VSize(FireTarget.Location - Pawn.Location))
                {
                    return true;
                }
            }
        }
    }
    return false;
}

final function bool HandleFireLineObstruction()
{
    if (m_nFireLineObstructionCount >= m_nFireLineObstructionFrequency)
    {
        m_nFireLineObstructionCount = 0;
        if (IsFireLineObstructed())
        {
            if (MyBP.IsInCover())
            {
                StopFiring();
            }
            else
            {
                SetTimer(0,5, true, 'PollRefireWeapon');
                StopFiring();
            }
            return true;
        }
    }
    else
    {
        m_nFireLineObstructionCount++;
    }
    return false;
}

function bool FireWeaponBurst()
{
    local SFXWeapon Wpn;
    local int BurstRounds;
    if (Pawn != None)
    {
        Wpn = SFXWeapon(Pawn.Weapon);
        if (Wpn != None)
        {
            BurstRounds = RandRange(Wpn.AI_BurstFireCount.X, Wpn.AI_BurstFireCount.Y) + 0,5;
            BurstRounds *= Wpn.BurstRounds;
            return FireWeapon(BurstRounds, RandRange(Wpn.AI_BurstFireDelay.X, Wpn.AI_BurstFireDelay.Y));
        }
    }
    return false;
}

function ModifyBurstDelay(SFXWeapon Weapon, out float BurstDelay);

function bool FireWeapon(optional int NewBurstCount, optional float NewBurstDelay)
{
    BurstFireCount = NewBurstCount;
    BurstFireDelay = NewBurstDelay;
    RemainingBurstFireCount = BurstFireCount;
    if (bWeaponFiring == false)
    {
        bWeaponFiring = true;
        if (IsReloading() || IsSwitchingWeapons() || IsFireLineObstructed())
        {
            SetTimer(0,5, true, 'PollRefireWeapon');
            return false;
        }
        StartFiring();
    }
    return true;
}

final function PollRefireWeapon()
{
    if (bWeaponFiring == false && IsTimerActive('RefireWeapon') == false)
    {
        ClearTimer('PollRefireWeapon');
        return;
    }
    if (IsFireLineObstructed() == false)
    {
        ClearTimer('PollRefireWeapon');
        RefireWeapon();
    }
    else
    {
        NotifyFireLineBlocked();
    }
}

function RefireWeapon()
{
    if (IsFireLineObstructed())
    {
        SetTimer(0,5, true, 'PollRefireWeapon');
        NotifyFireLineBlocked();
    }
    else
    {
        RemainingBurstFireCount = BurstFireCount;
        StartFiring();
    }
}

function NotifyFireLineBlocked();

event StartFiring()
{
    if (Pawn != None && Pawn.Weapon != None)
    {
        bFire = 1;
        Pawn.StartFire(0);
        m_nFireLineObstructionCount = 0;
    }
}

function StopFiring()
{
    StopFiring();
    if (bWeaponFiring)
    {
        bWeaponFiring = false;
        ClearTimer('RefireWeapon');
        ClearTimer('PollRefireWeapon');
    }
}

function bool ShouldRefire()
{
    return ShouldStopFiring() == false && RemainingBurstFireCount > 0;
}

function bool ShouldStopFiring()
{
    if (bFire == 0 || RemainingBurstFireCount <= 0 || ShouldAutoReload() || IsReloading() || HasValidTarget() == false)
    {
        return true;
    }
    return false;
}

function NotifyWeaponFired(Weapon W, byte FireMode)
{
    RemainingBurstFireCount--;
    HandleFireLineObstruction();
}

function NotifyWeaponFinishedFiring(Weapon W, byte FireMode)
{
    local bool bWantsRefire;
    local SFXWeapon Weapon;
    Weapon = SFXWeapon(W);
    bWantsRefire = !ShouldStopFiring();
    StopFiring();
    if (bWantsRefire)
    {
        bWeaponFiring = true;
        if (IsFireLineObstructed() == false)
        {
            ModifyBurstDelay(Weapon, BurstFireDelay);
            if (RemainingBurstFireCount > 0 && Weapon.BurstRefireTime > 0)
            {
                bFire = 0;
                BurstFireCount = RemainingBurstFireCount;
                SetTimer(Weapon.BurstRefireTime, false, 'RefireWeapon');
            }
            else
            {
                if (RemainingBurstFireCount <= 0 && BurstFireDelay > 0)
                {
                    bFire = 0;
                    SetTimer(BurstFireDelay, false, 'RefireWeapon');
                }
                else
                {
                    RefireWeapon();
                }
            }
        }
        else
        {
            SetTimer(0,5, true, 'PollRefireWeapon');
        }
    }
    else
    {
        WeaponAimDelay = Weapon.GetAIAimDelay();
    }
}

final function bool IsReloading()
{
    return MyBP != None && MyBP.IsReloading();
}

function bool ShouldAutoReload()
{
    local SFXWeapon oWeapon;
    if (MyBP != None)
    {
        oWeapon = SFXWeapon(MyBP.Weapon);
        return oWeapon != None && oWeapon.ShouldAutoReload();
    }
    return false;
}

final function bool IsSwitchingWeapons()
{
    return MyBP != None && MyBP.IsSwitchingWeapons();
}

native final latent function bool WaitForActionToComplete(BioSeqData oSeqData);

native final latent function PrecisionMoveTo(Vector vLoc, Rotator rRot);

native function bool DirectWalkCheck(const out Vector vTarget, Actor pTarget);

native final function bool IsCoverExposedToAnEnemy(CoverInfo TestCover, out float out_ExposedScale, optional Pawn TestEnemy, optional bool bActualLocation);

native final function bool GetCover(Pawn ChkPawn, out CoverInfo out_Cover);

native final function bool GetPawnCover(Pawn ChkPawn, out CoverInfo out_Cover);

native final function bool IsValidCover(const CoverInfo TestCover);

event InvalidateCover()
{
    if (MyBP == None || MyBP.IsInCover() == false)
    {
        return;
    }
    MyBP.LeaveCover();
    MyBP.ShouldCrouch(false);
}

native function bool ReadyMode(optional float fDelaySeconds);

native function bool ExploreMode(optional float fDelaySeconds);

native function Actor GetTarget();

native final latent function SteeringMovement();

native function bool GetSteeringVector(out Vector vSteering);

native function bool AdjustSteeringMoveSpeed(Vector vSteering);

native final latent function MobTarget(Actor oTarget, float fMoveTolerance);

native function BioSetAdjustLocation(Vector vAdjustLoc);

native function BioEpicPawnBehavior GetPawnBehavior(Pawn oPawn);

native function OnStreamingLevelEnteringStasis();

native function OnStreamingLevelLeavingStasis();

event OnEnteringStasis()
{
    local BioPawn oPawn;
    local SFXSeqEvt_EnteringStasis stasisEvent;
    local int idx;
    oPawn = BioPawn(Pawn);
    if (oPawn != None)
    {
        idx = 0;
        while (idx < oPawn.GeneratedEvents.Length)
        {
            stasisEvent = SFXSeqEvt_EnteringStasis(oPawn.GeneratedEvents[idx]);
            if (stasisEvent != None)
            {
                stasisEvent.CheckActivate(oPawn, oPawn);
            }
            idx++;
        }
    }
    bStasis = true;
}

event OnLeavingStasis()
{
    local SFXSeqEvt_LeavingStasis stasisEvent;
    local int idx;
    local BioPawn oPawn;
    oPawn = BioPawn(Pawn);
    if (oPawn != None)
    {
        idx = 0;
        while (idx < oPawn.GeneratedEvents.Length)
        {
            stasisEvent = SFXSeqEvt_LeavingStasis(oPawn.GeneratedEvents[idx]);
            if (stasisEvent != None)
            {
                stasisEvent.CheckActivate(oPawn, oPawn);
            }
            idx++;
        }
    }
    bStasis = false;
}

event LookAtPawn(Pawn oLookTarget, int nRank)
{
    local BioPawn oPawn;
    oPawn = BioPawn(Pawn);
    if (oPawn != None)
    {
        oPawn.SetLookAtTarget(oLookTarget, 0, nRank);
    }
}

event bool PlayerActivate(Actor Target_)
{
    local int I;
    local int J;
    local bool bActivated;
    bActivated = false;
    I = 0;
    while (I < Pawn.GeneratedEvents.Length)
    {
        if (ClassIsChildOf(Pawn.GeneratedEvents[I].Class, Class'BioSeqEvt_OnPlayerActivate') && Pawn.GeneratedEvents[I] != None)
        {
            J = 0;
            while (J < Pawn.GeneratedEvents[I].OutputLinks.Length)
            {
                if (Pawn.GeneratedEvents[I].OutputLinks[J].Links.Length > 0)
                {
                    if (Pawn.GeneratedEvents[I].CheckActivate(Pawn, Target_))
                    {
                        bActivated = true;
                        Pawn.GeneratedEvents[I].SetObjectVars("Pawn Activated", Pawn);
                    }
                }
                J++;
            }
        }
        I++;
    }
    if (bActivated)
    {
        m_fActivateTime = WorldInfo.TimeSeconds;
    }
    return bActivated;
}

event Tick(float DeltaTime);

final event AILog_Internal(coerce String LogText, optional Name LogCategory, optional bool bForce);

function DrawDebug(BioHUD HUD)
{
    local int idx;
    local SFXWeapon Weapon;
    if (bDebug_AI)
    {
        if (Enemy != None)
        {
            DrawDebugLine(Pawn.GetPawnViewLocation(), GetEnemyLocation(None, 1), 100, 0, 0);
            DrawDebugLine(Pawn.GetPawnViewLocation() + vect(0,0,10), GetEnemyLocation(), 255, 0, 0);
        }
    }
    if (bDebug_AIRange)
    {
        Weapon = SFXWeapon(Pawn.Weapon);
        if (Weapon != None)
        {
            DrawDebugCylinder(Pawn.Location, Pawn.Location, Weapon.Range_Melee, 8, 255, 255, 255);
            DrawDebugCylinder(Pawn.Location, Pawn.Location, Weapon.Range_Short, 8, 0, 255, 0);
            DrawDebugCylinder(Pawn.Location, Pawn.Location, Weapon.Range_Medium, 8, 0, 0, 255);
            DrawDebugCylinder(Pawn.Location, Pawn.Location, Weapon.Range_Long, 8, 255, 0, 0);
        }
    }
    if (bDebug_AIEnemyList)
    {
        idx = 0;
        while (idx < EnemyList.Length)
        {
            if (EnemyList[idx].Pawn != Enemy)
            {
                if (IsEnemyVisibleByIndex(idx))
                {
                    DrawDebugLine(Pawn.Location, GetEnemyLocationByIndex(idx), 0, 0, 255);
                }
                else
                {
                    DrawDebugLine(Pawn.Location, GetEnemyLocationByIndex(idx), 255, 0, 0);
                }
            }
            idx++;
        }
    }
    if (bDebug_ThreatRadius && MyBP != None && MyBP.ThreatRadiusSquared > 0)
    {
        DrawDebugCylinder(MyBP.Location, MyBP.Location, Sqrt(MyBP.ThreatRadiusSquared), 8, 255, 255, 255);
    }
}

function ChangeAI(class<BioAiController> NewAIClass, optional bool bCacheOldController)
{
    local BioAiController NewController;
    if (Pawn.Controller.Class == NewAIClass)
    {
        return;
    }
    NewController = Spawn(NewAIClass);
    if (NewController != None)
    {
        NewController.Possess(Pawn, false);
        NewController.Initialize();
        if (bCacheOldController)
        {
            NewController.OldController = self;
        }
        else
        {
            Destroy();
        }
    }
}

function RestoreAI()
{
    if (OldController != None)
    {
        OldController.Possess(Pawn, false);
        OldController.Initialize();
        OldController = None;
        Destroy();
    }
}

final function ClearRouteCache()
{
    RouteCache.Length = 0;
}

final function int RouteCacheSize()
{
    return RouteCache.Length;
}

final function Actor DecrementRouteCache()
{
    local int I;
    local int nShift;
    local Actor NewRoute;
    if (RouteCacheSize() == 0)
    {
        return None;
    }
    I = 0;
    nShift = 1;
    NewRoute = RouteCache[I];
    I = 0;
    while (I < RouteCache.Length && RouteCache[I] != None)
    {
        if (I + nShift < RouteCache.Length)
        {
            RouteCache[I] = RouteCache[I + nShift];
        }
        else
        {
            RouteCache[I] = None;
        }
        I++;
    }
    RouteCache.Length = Max(0, RouteCache.Length - nShift);
    return NewRoute;
}

final function Actor GetLastObjectInRouteCache(Actor oMoveTarget)
{
    if (RouteCache.Length == 0)
    {
        return oMoveTarget;
    }
    return RouteCache[RouteCache.Length - 1];
}

function PushOntoRouteCache(NavigationPoint Marker)
{
    local int I;
    local int CacheLen;
    CacheLen = RouteCacheSize();
    I = Min(CacheLen + 1, 15);
    while (I > 0)
    {
        RouteCache[I] = RouteCache[I - 1];
        RouteCache[I - 1] = None;
        I--;
    }
    if (RouteCache[0] != None)
    {
    }
    RouteCache[0] = Marker;
}

native function bool FindNearestOpenLocation(Vector vStartLocation, out Vector vFoundLocation, optional Pawn oTarget, optional int nMaxShellsToCheck);

event Possess(Pawn NewPawn, bool bVehicleTransition)
{
    Possess(NewPawn, bVehicleTransition);
    if (Pawn != None)
    {
        MyBP = BioPawn(NewPawn);
        MyBPBehavior = BioPawnBehavior(MyBP.oBioComponent);
        SetTimer(0,5, false, 'FillEnemyList');
        SetTimer(0,1 + 0,15 * FRand(), true, 'UpdateAllEnemyInfo');
    }
}

event UnPossess()
{
    UnPossess();
    MyBP = None;
    MyBPBehavior = None;
    ClearTimer('UpdateAllEnemyInfo');
}

native final function bool HasValidCover();

final event bool IsAtCover()
{
    return bReachedCover;
}

final function bool ClaimCover(CoverInfo NewCover)
{
    local bool bResult;
    if (IsValidCover(NewCover))
    {
        bReachedCover = false;
        if (Cover.Link != NewCover.Link || Cover.SlotIdx != NewCover.SlotIdx && Cover.Link != None)
        {
            if (MyBP != None)
            {
                MyBP.LeaveCover();
            }
        }
        bResult = NewCover.Link.Claim(self, NewCover.SlotIdx);
        if (bResult)
        {
            Cover = NewCover;
            jump 221;
        }
        jump 224;
    }
    return bResult;
}

final function UnClaimCover()
{
    if (Cover.Link != None)
    {
        LastCover = Cover;
        Cover.Link.UnClaim(self, Cover.SlotIdx, true);
    }
    Cover.Link = None;
    Cover.SlotIdx = -1;
    bReachedCover = false;
}

final function bool SetCoverType()
{
    local int LeftSlotIdx;
    local float SlotPct;
    local Vector LtToRt;
    local Vector ProjLocToCov;
    if (MyBP != None && Cover.Link != None && Cover.SlotIdx >= 0 && Cover.SlotIdx < Cover.Link.Slots.Length)
    {
        if (Cover.SlotIdx == Cover.Link.Slots.Length - 1)
        {
            LeftSlotIdx = Cover.SlotIdx - 1;
        }
        else
        {
            LeftSlotIdx = Cover.SlotIdx;
        }
        LtToRt = Cover.Link.GetSlotLocation(LeftSlotIdx + 1) - Cover.Link.GetSlotLocation(LeftSlotIdx);
        ProjLocToCov = MyBP.Location - Cover.Link.GetSlotLocation(LeftSlotIdx);
        SlotPct = ProjLocToCov Dot Normal(LtToRt) / VSize2D(LtToRt);
        SlotPct = FClamp(SlotPct, 0, 1);
        MyBP.SetCoverInfo(Cover.Link, Cover.SlotIdx, LeftSlotIdx, LeftSlotIdx + 1, SlotPct);
        return true;
    }
    return false;
}

final function bool IsLeaning()
{
    return MyBP != None && MyBP.IsLeaning(MyBP.CoverAction);
}

final function bool IsPeeking()
{
    return MyBP != None && MyBP.IsPeeking(MyBP.CoverAction);
}

final function bool GetCoverAction(CoverInfo ChkCover, Actor ChkTarget, out ECoverAction AttackerCoverAction, out ECoverAction TargetCoverAction, optional bool bSetPendingCoverAction = true)
{
    local BioPawn PawnTarget;
    local CoverInfo ChkTargetCover;
    local int nIndex;
    local ECoverType Type;
    local int nFireLinkIdx;
    local array<int> FireLinkIndices;
    local CoverSlot Slot;
    local array<FireLinkItem> FireLinkItems;
    if (ChkCover.Link == None || ChkCover.SlotIdx < 0)
    {
        return false;
    }
    if (ChkTarget == None)
    {
        ChkTarget = FireTarget != None ? FireTarget : Enemy;
    }
    if (ChkTarget == None)
    {
        return false;
    }
    PawnTarget = BioPawn(ChkTarget);
    if (ChkCover.Link == Cover.Link && ChkCover.SlotIdx == Cover.SlotIdx)
    {
        Type = MyBP.FindCoverType();
    }
    else
    {
        Type = ChkCover.Link.Slots[ChkCover.SlotIdx].CoverType;
    }
    if (PawnTarget != None && GetPawnCover(PawnTarget, ChkTargetCover) && PawnTarget.IsInCover())
    {
        if (ChkCover.Link.GetFireLinkTo(ChkCover.SlotIdx, ChkTargetCover, 0, 0, nFireLinkIdx, FireLinkIndices))
        {
            Slot = ChkCover.Link.Slots[ChkCover.SlotIdx];
            nIndex = 0;
            while (nIndex < FireLinkIndices.Length)
            {
                FireLinkItems[FireLinkItems.Length] = Slot.FireLinks[nFireLinkIdx].Items[FireLinkIndices[nIndex]];
                nIndex++;
            }
        }
    }
    if (FireLinkItems.Length == 0)
    {
        if (ChkCover.Link.Slots[ChkCover.SlotIdx].bLeanLeft && CanFireAt(ChkTarget, ChkCover.Link.GetSlotViewPoint(ChkCover.SlotIdx, Type, 3), true))
        {
            // unknown bytecode 54 ;
        }
    }
}

final function NotifyReachedCover()
{
    bReachedCover = true;
}

final function NotifyReachedCoverSlot(int SlotIdx)
{
    if (MyBP != None)
    {
        if (SlotIdx != MyBP.TargetSlotIdx && Cover.Link == MyBP.CurrentLink)
        {
            if (!Cover.Link.bClaimAllSlots)
            {
                Cover.Link.UnClaim(self, SlotIdx, false);
            }
            MyBP.CurrentSlotDirection = 0;
            MyBP.Velocity = vect(0,0,0);
            bReachedCover = true;
        }
    }
}

native final latent function AdjustToSlot(int TargetSlotIdx);

native final latent function MoveToSlot(Vector vSlotLocation);

function float GetMoveTimeOutDuration()
{
    return 15;
}

final private function float GetRouteCacheDistance()
{
    local float Distance;
    local int idx;
    local Actor LastAnchor;
    LastAnchor = Pawn;
    idx = 0;
    while (idx < RouteCache.Length)
    {
        if (RouteCache[idx] != None)
        {
            Distance += VSize(LastAnchor.Location - RouteCache[idx].Location);
            LastAnchor = RouteCache[idx];
        }
        idx++;
    }
    return Distance;
}

static function class<BioBaseSaveObject> GetSaveObjectClass()
{
    return Class'BioPlayerControllerSaveObject';
}

function PawnDied(Pawn inPawn)
{
    if (inPawn == Pawn)
    {
        UnClaimCover();
        RouteCache_Empty();
        EnemyList.Length = 0;
    }
    PawnDied(inPawn);
    Destroy();
}

event NotifyNewEnemy(Pawn NewEnemy);

event NotifyEnemyVisible(int EnemyIdx, float TimeSinceSeen);

function NotifyTakeHit(Controller InstigatedBy, Vector HitLocation, int Damage, class<DamageType> DamageType, Vector Momentum)
{
    local int EnemyIdx;
    if (InstigatedBy != None)
    {
        EnemyIdx = EnemyList.Find('Pawn', InstigatedBy.Pawn);
        if (EnemyIdx == -1)
        {
            if (!AddEnemy(InstigatedBy.Pawn))
            {
                return;
            }
            EnemyIdx = EnemyList.Find('Pawn', InstigatedBy.Pawn);
        }
        UpdateEnemyInfo(EnemyIdx, 2, 'NotifyTakeHit');
    }
}

function NotifyCastAt(Pawn Attacker, SFXPower Power)
{
    local int EnemyIdx;
    if (Power != None && Power.PowerType != 3)
    {
        if (Attacker != None && Attacker != MyBP)
        {
            EnemyIdx = EnemyList.Find('Pawn', Attacker);
            if (EnemyIdx == -1)
            {
                if (!AddEnemy(Attacker))
                {
                    return;
                }
                EnemyIdx = EnemyList.Find('Pawn', Attacker);
            }
            UpdateEnemyInfo(EnemyIdx, 2, 'NotifyCastAt');
        }
    }
}

function NotifyUnableToFireFromOpen();

event OnLeftPlaypen();

event OnEnteredPlaypen();

function Initialize();

function NotifyLimbDetached(EBioPartGroup ePartGroup);

state Debug
{
    event BeginState(Name PreviousStateName)
    {
    }

    event EndState(Name NextStateName)
    {
    }

    event PushedState()
    {
    }

    event PoppedState()
    {
    }

    event PausedState()
    {
    }

    event ContinuedState()
    {
    }

}

function bool CanAttack(Actor oTarget, optional Vector AttackOrigin)
{
    local CoverInfo TargetCover;
    local BioPawn TargetPawn;
    local SFXAI_Core TargetAI;
    local bool bCanAttack;
    local array<ECoverAction> CoverActions;
    local int ActionIndex;
    local Vector CoverViewPt;
    local ECoverAction TargetCoverAction;
    local ECoverAction AttackerCoverAction;
    TargetCoverAction = 0;
    AttackerCoverAction = 0;
    if (MyBP == None || oTarget == None)
    {
        return false;
    }
    TargetPawn = BioPawn(oTarget);
    bCanAttack = false;
    if (MyBP.IsInCover())
    {
        AttackOrigin = MyBP.GetPawnViewLocation();
    }
    else
    {
        if (IsZero(AttackOrigin))
        {
            if (MyBP.Weapon != None)
            {
                AttackOrigin = MyBP.Weapon.GetPhysicalFireStartLoc();
            }
            else
            {
                AttackOrigin = MyBP.GetPawnViewLocation();
            }
        }
    }
    if (MyBP.IsInCover())
    {
        if (GetCoverAction(Cover, oTarget, AttackerCoverAction, TargetCoverAction))
        {
            bCanAttack = true;
        }
    }
    else
    {
        if (TargetPawn != None && GetPawnCover(TargetPawn, TargetCover))
        {
            TargetAI = SFXAI_Core(TargetPawn.Controller);
            if (TargetAI != None)
            {
                if (!TargetPawn.IsInCover())
                {
                    bCanAttack = CanFireAt(oTarget, AttackOrigin, true);
                }
                else
                {
                    if (TargetAI.GetCoverAction(TargetCover, MyBP, TargetCoverAction, AttackerCoverAction, false))
                    {
                        bCanAttack = true;
                    }
                }
            }
            else
            {
                TargetCover.Link.GetSlotActions(TargetCover.SlotIdx, CoverActions);
                if (CoverActions.Length > 0)
                {
                    ActionIndex = 0;
                    while (ActionIndex < CoverActions.Length)
                    {
                        CoverViewPt = TargetCover.Link.GetSlotViewPoint(TargetCover.SlotIdx, 0, CoverActions[ActionIndex]);
                        bCanAttack = CanFireAt(MyBP, CoverViewPt, true);
                        if (bCanAttack)
                        {
                            break;
                        }
                        ActionIndex++;
                    }
                    if (bCanAttack)
                    {
                        TargetCoverAction = CoverActions[ActionIndex];
                    }
                }
                if (!bCanAttack)
                {
                    bCanAttack = CanFireAt(oTarget, AttackOrigin, true);
                }
            }
        }
        else
        {
            bCanAttack = CanFireAt(oTarget, AttackOrigin, true);
        }
    }
    SetBestTargetCoverAction(TargetCoverAction);
    return bCanAttack;
}

function SetBestTargetCoverAction(ECoverAction BestAction);

native function bool HasLOSToTarget(Actor pTarget, out float fTimeOfHit);

native function bool IsTargetInFiringArc(BioPawn pPawn, Actor pTarget, float fFiringArcAsDot);

static event GetBehaviourList(out array<Name> Behaviours);

function SetForcedTarget(Actor oTarget)
{
    ForcedTarget = oTarget;
}

function Actor GetForcedTarget()
{
    return ForcedTarget;
}

function SetPreferredTarget(Actor oTarget)
{
    PreferredTarget = oTarget;
}

function Actor GetPreferredTarget()
{
    return PreferredTarget;
}

function AddIgnoredTarget(Actor oTarget)
{
    if (IgnoredTargets.Find(oTarget) == -1)
    {
        IgnoredTargets.AddItem(oTarget);
    }
    if (FireTarget == oTarget)
    {
        FireTarget = None;
    }
    if (Enemy == oTarget)
    {
        Enemy = None;
    }
}

function RemoveIgnoredTarget(Actor oTarget)
{
    IgnoredTargets.RemoveItem(oTarget);
}

function AddIgnoredSquad(BioBaseSquad oSquad)
{
    local BioPawn oBioPawn;
    if (IgnoredSquads.Find(oSquad) == -1)
    {
        IgnoredSquads.AddItem(oSquad);
    }
    oBioPawn = BioPawn(FireTarget);
    if (oBioPawn != None && oBioPawn.Squad == oSquad)
    {
        FireTarget = None;
    }
    oBioPawn = BioPawn(Enemy);
    if (oBioPawn != None && oBioPawn.Squad == oSquad)
    {
        Enemy = None;
    }
}

function RemoveIgnoredSquad(BioBaseSquad oSquad)
{
    IgnoredSquads.RemoveItem(oSquad);
}

function OnLastManStanding();

defaultproperties
{
    bAILogToWindow=true
    InterpEnemyLocSpeed=0,15
    Response_MinEnemySeenTime=1
    Response_MinEnemyHearTime=0,5
    m_bCheckLOS=true
    AI_Acc_InstabilityDecayRate=0,5
    AI_Acc_Base=0,01
    AI_Acc_Target=0,01
    AI_AccMod_Move=0,1
    AI_AccMod_TargMove=0,15
    AI_AccMod_BriefAcquire=0,25
    AI_AccMod_BriefVisibility=0,15
    AI_AccMod_ShortRange=-0,15
    AI_AccMod_MediumRange=-0,05
    EnemyDistance_Short=600
    EnemyDistance_Medium=1800
    EnemyDistance_Long=3600
    EnemyDistance_Melee=300
    m_nFireLineObstructionFrequency=2
    Components=None
    ***Deleted***=System.Byte[]
}
