﻿class BioArtPlaceable extends Actor
    hidecategories(Navigation)
    config(Game)
    abstract
    native;

var const BioArtPlaceable NextPlaceable;

var transient bool bIsFrozen;

var() bool m_bCanCleanupCorpse;

var transient int m_nCorpseCleanupAttempts;

var const transient config float m_fMaxCorpseCleanupScreenSizeThreshold;

var const transient config float m_fMaxCorpseCleanupDistanceThreshold;

var const transient config int m_nMaxCorpseVisibilityCleanupAttempts;

var const transient config float m_fCorpseCleanupAttemptRetryTime;

var const transient config float m_fCorpseCleanupFirstAttemptTime;

var native transient MultiMap_Mirror m_maoBlockedReachSpecs;

var Name m_nmMatineeNodeName;

var native Map_Mirror m_mAttachedNavPoints;

var(Debug) /* 00000008 */ LightEnvironmentComponent m_pLightEnvComponent;

var transient array<BioVisualEffect> VisualEffects;

var BioPawn m_oLastInstigator;

var /* 00000008 */ BioFSMComponent FSM;

var int m_nNumParticipants;

var(Debug) /* 00000008 */ MeshComponent PlaceableMesh;

var(Debug) /* 00000008 */ MeshComponent DestroyedMesh;

var /* 00000008 */ AudioComponent PlacablesSounds;

var BioArtPlaceableBehavior m_oBehavior;

native function SetActionStationWP(NavigationPoint oWP, Name nmSocket);

native function UpdateWpPointPosition();

native function InitRagdoll();

native function TermRagdoll();

native function bool BioDoProximityCheck();

native function AddProximityTimer();

native function OverrideUseCase(eBioAPLUseCase UseCase);

native function MAT_BeginAnimControl(array<AnimSet> InAnimSets);

native function MAT_SetAnimPosition(Name SlotName, int ChannelIndex, Name InAnimSeqName, float InPosition, bool bFireNotifies, bool bLooping);

native function MAT_SetAnimWeights(array<AnimSlotInfo> SlotInfos);

native function MAT_FinishAnimControl();

native function FreezeAnimation();

native function UnfreezeAnimation();

function PostBeginPlay()
{
    PostBeginPlay();
    m_oBehavior = BioArtPlaceableBehavior(oBioComponent);
    m_oBehavior.PostBeginPlay();
    if (bPathColliding == false && m_oBehavior.IsPhysicsAffectable())
    {
        bPathColliding = true;
    }
    if (m_oBehavior.RequiresProximityTimer())
    {
        AddProximityTimer();
    }
    if (PlaceableMesh != None)
    {
        PlaceableMesh.SetLightEnvironment(m_pLightEnvComponent);
    }
    CreateSelection();
}

function CreateSelection()
{
    local bool bTargetable;
    local bool bCombatTargetable;
    local SFXSelectionModule SelMod;
    local BioActorType MyActorType;
    MyActorType = BioActorType(ActorType);
    if (m_oBehavior.m_bTargetableOverride == false)
    {
        if (MyActorType != None)
        {
            bTargetable = MyActorType.m_bTargetable;
        }
    }
    else
    {
        bTargetable = m_oBehavior.m_bTargetable;
    }
    if (m_oBehavior.m_bCombatTargetableOverride == false)
    {
        if (MyActorType != None)
        {
            bCombatTargetable = MyActorType.m_bCombatTargetable;
        }
    }
    else
    {
        bCombatTargetable = m_oBehavior.m_bCombatTargetable;
    }
    if (bTargetable || bCombatTargetable)
    {
        SelMod = GetModule(Class'SFXSelectionModule');
        if (SelMod == None)
        {
            SelMod = new(self,Class'SFXSelectionModule');
            Modules.AddItem(SelMod);
        }
        SelMod.m_bCombatTargetable = bCombatTargetable;
        SelMod.m_bTargetable = bTargetable;
    }
}

function OnAttacked(Pawn oAttacker, Name WeaponName)
{
    local BioSeqEvt_OnAttacked Event;
    local int I;
    if (oAttacker != None)
    {
        I = 0;
        while (I < GeneratedEvents.Length)
        {
            Event = BioSeqEvt_OnAttacked(GeneratedEvents[I]);
            if (Event != None)
            {
                if (Event.CheckActivate(self, oAttacker))
                {
                    Event.SetNameVars("Weapon", WeaponName);
                    Event.SetObjectVars("Who", oAttacker);
                }
            }
            I++;
        }
    }
}

function OnCastAt(Pawn oAttacker, SFXPower Power)
{
    local BioSeqEvt_OnCastAt Event;
    local int I;
    if (oAttacker != None)
    {
        I = 0;
        while (I < GeneratedEvents.Length)
        {
            Event = BioSeqEvt_OnCastAt(GeneratedEvents[I]);
            if (Event != None)
            {
                if (Event.CheckActivate(self, oAttacker))
                {
                    Event.SetNameVars("Power", Power.PowerName);
                    Event.SetObjectVars("Who", oAttacker);
                    Event.SetIntVars("Category", Power.Discipline);
                }
            }
            I++;
        }
    }
}

function bool IsDestroyed()
{
    local BioArtPlaceableType PlaceableType;
    if (m_oBehavior != None)
    {
        PlaceableType = BioArtPlaceableType(ActorType);
        if (m_oBehavior.m_nmLastState == PlaceableType.m_nmDestroyedState)
        {
            return true;
        }
    }
    return false;
}

event RanInto(Actor Other)
{
    if (Physics == 10 && Other.Physics != 10)
    {
        if (BioPawn(Other) != None && VSize(Velocity) > 2 && BioPlayerSquad(BioPawn(Other).m_oBehavior.Squad) == None)
        {
        }
    }
}

event AbortReactive(Name nmToState)
{
    local int nIndex;
    local BioReactive oReactive;
    local BioReactiveUseArtPlaceable oUseArtPlaceable;
    if (m_oBehavior.m_aActivatedReactive.Length > 0)
    {
        nIndex = 0;
        while (nIndex < m_oBehavior.m_aActivatedReactive.Length)
        {
            oReactive = m_oBehavior.m_aActivatedReactive[nIndex];
            if (oReactive != None && BioReactiveUseArtPlaceable(oReactive) != None)
            {
                oUseArtPlaceable = BioReactiveUseArtPlaceable(oReactive);
                oUseArtPlaceable.bDone = true;
                m_oBehavior.SetState(nmToState);
                break;
            }
            nIndex++;
        }
    }
}

event OnSkillGameFailed(BioPawn oUsingPawn)
{
    local int nIndex;
    if (m_oBehavior != None)
    {
        m_oBehavior.m_bSkillGameFailed = true;
    }
    AbortReactive(m_oBehavior.GetStateFailure());
    nIndex = 0;
    while (nIndex < GeneratedEvents.Length)
    {
        if (ClassIsChildOf(GeneratedEvents[nIndex].Class, Class'BioSeqEvt_ArtPlaceableSkillGameFailed'))
        {
            GeneratedEvents[nIndex].CheckActivate(oUsingPawn, self);
        }
        nIndex++;
    }
}

event OnSkillGameSucceeded(BioPawn oUsingPawn)
{
    local int nIndex;
    AbortReactive(m_oBehavior.GetStateSuccess());
    nIndex = 0;
    while (nIndex < GeneratedEvents.Length)
    {
        if (ClassIsChildOf(GeneratedEvents[nIndex].Class, Class'BioSeqEvt_ArtPlaceableSkillGameSuccess'))
        {
            GeneratedEvents[nIndex].CheckActivate(oUsingPawn, self);
        }
        nIndex++;
    }
}

event OnSkillGameCanceled(BioPawn oUsingPawn)
{
    local int nIndex;
    AbortReactive(m_oBehavior.GetStateFailure());
    nIndex = 0;
    while (nIndex < GeneratedEvents.Length)
    {
        if (ClassIsChildOf(GeneratedEvents[nIndex].Class, Class'BioSeqEvt_ArtPlaceableSkillGameFailed'))
        {
            GeneratedEvents[nIndex].CheckActivate(oUsingPawn, self);
        }
        nIndex++;
    }
}

static function class<BioBaseSaveObject> GetSaveObjectClass()
{
    return Class'BioArtPlaceableSaveObject';
}

simulated event BeginAnimControl(array<AnimSet> InAnimSets)
{
    MAT_BeginAnimControl(InAnimSets);
}

simulated event SetAnimPosition(Name SlotName, int ChannelIndex, Name InAnimSeqName, float InPosition, bool bFireNotifies, bool bLooping)
{
    MAT_SetAnimPosition(SlotName, ChannelIndex, InAnimSeqName, InPosition, bFireNotifies, bLooping);
}

simulated event SetAnimWeights(array<AnimSlotInfo> SlotInfos)
{
    MAT_SetAnimWeights(SlotInfos);
}

simulated event FinishAnimControl()
{
    MAT_FinishAnimControl();
}

event DestroyedCleanupTimer()
{
    local bool bCleanUpCorpse;
    local float DistFromCameraSquared;
    local BioWorldInfo pWorldInfo;
    local SkeletalMeshComponent SkelMesh;
    bCleanUpCorpse = false;
    if (m_bCanCleanupCorpse)
    {
        if (!PlayerCanSeeMe())
        {
            bCleanUpCorpse = true;
        }
        else
        {
            if (m_nCorpseCleanupAttempts > m_nMaxCorpseVisibilityCleanupAttempts)
            {
                SkelMesh = SkeletalMeshComponent(PlaceableMesh);
                if (SkelMesh != None)
                {
                    if (SkelMesh.MaxDistanceFactor < m_fMaxCorpseCleanupScreenSizeThreshold)
                    {
                        bCleanUpCorpse = true;
                    }
                }
                else
                {
                    pWorldInfo = BioWorldInfo(WorldInfo);
                    if (pWorldInfo != None)
                    {
                        if (pWorldInfo.GetLocalPlayerController() != None)
                        {
                            DistFromCameraSquared = VSizeSq(Location - pWorldInfo.GetLocalPlayerController().PlayerCamera.CameraCache.POV.Location);
                        }
                    }
                    if (DistFromCameraSquared > m_fMaxCorpseCleanupDistanceThreshold * m_fMaxCorpseCleanupDistanceThreshold)
                    {
                        bCleanUpCorpse = true;
                    }
                }
            }
        }
        if (bCleanUpCorpse)
        {
            Destroy();
        }
        else
        {
            m_nCorpseCleanupAttempts++;
            SetTimer(m_fCorpseCleanupAttemptRetryTime, false, 'DestroyedCleanupTimer');
        }
    }
    else
    {
        ClearTimer('DestroyedCleanupTimer');
    }
}

function ReactToDeath(float Damage, float fDeathDamage, Controller InstigatedBy, Vector HitLocation, Vector Momentum, class<DamageType> clsDamageType, optional TraceHitInfo HitInfo, optional SFXDamageType pDamage)
{
    if (m_bCanCleanupCorpse)
    {
        SetTimer(m_fCorpseCleanupFirstAttemptTime, false, 'DestroyedCleanupTimer');
    }
}

defaultproperties
{
    FSM=BioFSMComponent'oFSM'
    m_pLightEnvComponent=DynamicLightEnvironmentComponent'LightEnvComponent0'
    m_nmMatineeNodeName=MatineeAnim
    m_fCorpseCleanupFirstAttemptTime=5
    m_fCorpseCleanupAttemptRetryTime=2
    m_nMaxCorpseVisibilityCleanupAttempts=4
    m_fMaxCorpseCleanupDistanceThreshold=4000
    m_fMaxCorpseCleanupScreenSizeThreshold=0,2
    m_bCanCleanupCorpse=true
    Components=None
    // child object LightEnvComponent0 of type DynamicLightEnvironmentComponent
    // child object oFSM of type BioFSMComponent
    // child object GPModule of type SFXGamePropertiesModule
    // child object oBehavior of type BioArtPlaceableBehavior
}
