﻿class BioDebugMenu extends BioGUIBox
    native;

var array<String> m_aMenuEntries;

var int m_nLineCount;

var int m_nMaxScrollPosition;

var int m_nScrollPosition;

var int m_nSelectedEntry;

var() Color m_colorSelected;

function AddEntry(String sMenuEntry)
{
    m_aMenuEntries[m_aMenuEntries.Length] = sMenuEntry;
    m_nSelectedEntry = 0;
    m_bComputeDrawParameters = true;
}

function Clear()
{
    m_aMenuEntries.Length = 0;
}

protected function bool ShouldRender()
{
    return m_aMenuEntries.Length > 0;
}

protected function ComputeDrawParameters(Canvas oCanvas)
{
    local float fTextHeight;
    local float fMaxWidth;
    local float fMaxHeight;
    local float fWidth;
    local int I;
    if (!m_bComputeDrawParameters || !ShouldRender())
    {
        return;
    }
    m_bComputeDrawParameters = false;
    m_nCanvasWidth = oCanvas.SizeX;
    m_nCanvasHeight = oCanvas.SizeY;
    oCanvas.TextSize(m_aMenuEntries[0], m_fWidth, fTextHeight);
    I = 1;
    while (I < m_aMenuEntries.Length)
    {
        oCanvas.TextSize(m_aMenuEntries[I], fWidth, fTextHeight);
        if (fWidth > m_fWidth)
        {
            m_fWidth = fWidth;
        }
        I++;
    }
    fMaxWidth = 0,75 * oCanvas.SizeX;
    if (m_fWidth > fMaxWidth)
    {
        m_fWidth = fMaxWidth;
    }
    m_fHeight = fTextHeight * m_aMenuEntries.Length;
    fMaxHeight = 0,75 * oCanvas.SizeY;
    m_nScrollPosition = 0;
    m_nLineCount = m_aMenuEntries.Length;
    if (m_fHeight > fMaxHeight)
    {
        m_nLineCount = m_aMenuEntries.Length - m_fHeight - fMaxHeight / fTextHeight;
    }
    m_nMaxScrollPosition = m_aMenuEntries.Length - m_nLineCount;
    if (m_fHeight > fMaxHeight)
    {
        m_fHeight = fMaxHeight;
    }
    m_fX = oCanvas.SizeX - m_fWidth / 2;
    m_fY = oCanvas.SizeY - m_fHeight / 2;
}

protected function RenderContent(Canvas oCanvas)
{
    local int I;
    oCanvas.CurY = m_fY;
    oCanvas.CurYL = 0;
    I = 0;
    while (I < m_nLineCount)
    {
        oCanvas.CurX = m_fX;
        if (I == m_nSelectedEntry - m_nScrollPosition)
        {
            oCanvas.DrawColor = m_colorSelected;
        }
        else
        {
            oCanvas.DrawColor = m_colorForeground;
        }
        oCanvas.DrawText(m_aMenuEntries[m_nScrollPosition + I]);
        I++;
    }
}

function ScrollDown()
{
    if (m_nSelectedEntry < m_aMenuEntries.Length - 1)
    {
        m_nSelectedEntry++;
    }
    if (m_nSelectedEntry > m_nScrollPosition + m_nLineCount - 1)
    {
        m_nScrollPosition = m_nSelectedEntry - m_nLineCount + 1;
    }
}

function ScrollUp()
{
    if (m_nSelectedEntry > 0)
    {
        m_nSelectedEntry--;
    }
    if (m_nSelectedEntry < m_nScrollPosition)
    {
        m_nScrollPosition = m_nSelectedEntry;
    }
}

defaultproperties
{
    m_colorSelected=(B=0,G=255,R=0,A=255)
}
