﻿class BioEpicPawnBehavior extends BioActorBehavior
    config(Game)
    native
    editinlinenew;

var(VisualEffects) array<EpicPawnVisualEffect> m_aVisualEffects;

var transient int m_nDoorProximityRefCount;

var int m_nCurrentPowerIndex;

var(Debug) array<BioPower> m_powers;

var array<BioCapability> m_aoCapabilities;

var /* 00000008 */ BioAttributesEpicPawn m_PawnAttributes;

var class<BioAttributesEpicPawn> m_attributesClass;

var int m_nCombatTransitionEnabled;

var int m_nPowersEnabled;

var int m_nGrenadesEnabled;

var int m_nWeaponsEnabled;

var int m_nStackEnabled;

var int m_nQueueEnabled;

var int m_nALLEnabled;

var(Squad) BioBaseSquad Squad;

var bool m_bDoCameraBehaviorTransition;

var EBioEpicPawnBehaviorWeaponFireMode m_eWeaponFireMode;

var EBioAnimNodeCombatModeFadeOut m_combatModeFadeoutState;

var EActionStateVariable m_ePreviousActionState;

var EActionStateVariable m_eCurrentActionState;

enum EBioAnimNodeCombatModeFadeOut
{
    BIO_ANIM_NODE_COMBAT_MODE_FADEOUT_NONE,
    BIO_ANIM_NODE_COMBAT_MODE_FADEOUT_ANIMATING_ENTER,
    BIO_ANIM_NODE_COMBAT_MODE_FADEOUT_ENTER,
    BIO_ANIM_NODE_COMBAT_MODE_FADEOUT_ANIMATING_EXIT,
    BIO_ANIM_NODE_COMBAT_MODE_FADEOUT_EXIT,
};

enum EBioEpicPawnBehaviorWeaponFireMode
{
    BIO_EPIC_PAWN_BEHAVIOR_WEAPON_FIRE_MODE_NORMAL,
    BIO_EPIC_PAWN_BEHAVIOR_WEAPON_FIRE_MODE_ALWAYS_HIT,
    BIO_EPIC_PAWN_BEHAVIOR_WEAPON_FIRE_MODE_ALWAYS_MISS,
};

struct native EpicPawnVisualEffect
{
    var() Name m_nmLabel;

    var() BioVFXTemplate m_oVFXTemplate;

    var() Name m_nmTagOfActorAtVFXLocation;

}

native function TickBehavior(float DeltaTime);

native final private function InitializeActionFramework();

native final private function int AbilityEnable_AI(bool bEnable, EDisableType eEnableType, Name nmSubType);

native final private function bool IsAbilityEnabled_AI(EDisableType eEnableType, Name nmSubType);

native final private function int AbilityEnable_Offense(bool bEnable, EDisableType eEnableType, Name nmSubType);

native final private function bool IsAbilityEnabled_Offense(EDisableType eEnableType, Name nmSubType);

native final private function int AbilityEnable_Power(bool bEnable, EDisableType eEnableType, Name nmSubType);

native final private function bool IsAbilityEnabled_Power(EDisableType eEnableType, Name nmSubType);

native function DoEnabledEvent(EDisableType eEnableType, Name nmSubType);

native function DoDisabledEvent(EDisableType eEnableType, Name nmSubType);

native function bool IsAbilityEnabledAll();

native function bool IsAbilityEnabledQueue();

native function bool IsAbilityEnabledStack();

native function bool IsAbilityEnabledWeapons();

native function bool IsAbilityEnabledGrenades();

native function bool IsAbilityEnabledPowers();

native function bool IsAbilityEnabledCombatTransition();

native function bool SetCurrentPower(Name nmPower, out float fCooldownRemaining);

native function ClearCurrentPower();

native function TickPowers(float DeltaTime);

native function EActionStateVariable SetActionState(EActionStateVariable eState, optional float fDelay);

native function EActionStateVariable SetPreviousActionState();

native function ModifyDoorProximityRefCount(bool bIncrement);

native function int GetDoorProximityRefCount();

event BioBaseSaveObject WriteSaveObject(Object InOuter, optional BioBaseSaveObject Save)
{
    if (!BioPawn(Owner).IsDead() || BioPawn(Owner).GetClassification() == 6)
    {
        return WriteSaveObject(InOuter, Save);
    }
    else
    {
        return None;
    }
}

static function class<BioBaseSaveObject> GetSaveObjectClass()
{
    return Class'BioEpicPawnBehaviorSaveObject';
}

native function BioPower FindPower(Name nmPowerName);

native function int FindPowerIndex(Name nmPowerName);

native function bool IsHealing();

event int GetInventoryResource(EInventoryResourceTypes eResourceType)
{
    local BioPawn oPlayerPawn;
    local BioWorldInfo oBWI;
    oBWI = BioWorldInfo(Owner.WorldInfo);
    if (oBWI == None)
    {
        return 0;
    }
    oPlayerPawn = BioPawn(oBWI.GetLocalPlayerController().Pawn);
    if (oPlayerPawn == None || SFXInventoryManager(oPlayerPawn.InvManager) == None)
    {
        return 0;
    }
    return SFXInventoryManager(oPlayerPawn.InvManager).GetResource(eResourceType);
}

event AdjustInventoryResource(EInventoryResourceTypes eResourceType, int nAmt, optional bool bNotifyTicker = false)
{
    local BioPawn oPlayerPawn;
    local BioWorldInfo oBWI;
    oBWI = BioWorldInfo(Owner.WorldInfo);
    if (oBWI == None)
    {
        return;
    }
    oPlayerPawn = BioPawn(oBWI.GetLocalPlayerController().Pawn);
    if (oPlayerPawn == None || SFXInventoryManager(oPlayerPawn.InvManager) == None)
    {
        return;
    }
    SFXInventoryManager(oPlayerPawn.InvManager).AdjustResource(eResourceType, nAmt, bNotifyTicker);
}

native function bool DisplayEpicPawnVisualEffect(Name nmLabel);

defaultproperties
{
    m_eCurrentActionState=BIO_ACTION_STATE_COMBAT
    m_nALLEnabled=1
    m_nQueueEnabled=1
    m_nStackEnabled=1
    m_nWeaponsEnabled=1
    m_nGrenadesEnabled=1
    m_nPowersEnabled=1
    m_nCombatTransitionEnabled=1
}
