﻿class BioHintSystemBase extends Object
    config(Game)
    native;

const HistoryLength = 10;
var config array<SFXNotificationData> m_NotificationData;

var transient SFXNotification m_nCurrentlyDisplayedNotification;

var transient bool m_bNotificationIsVisible;

var transient bool m_bNotificationIsPaused;

var transient array<SFXNotification> m_aNotifications;

var transient int m_nCurrentlyDisplayedHint;

var transient float m_fCurrentTime;

// m_mTrackingData: MapProperty[52]
var config array<HintDefinition> m_aHints;

var config float m_fMinimumTimeBetweenAnyHints;

var bool m_bDisabledForTutorial;

var config bool m_bEnabled;

var config int nNotificationBodyLength;

struct native HintTrackingData
{
    var int Num;

    var float LastTime;

    var float FirstTime;

    var float Times[10];

    var int QueueHead;

}

struct native HintDefinition
{
    var Name HintName;

    var bool Enabled;

// DefaultText: StringRefProperty[44]
// PS3Text: StringRefProperty[44]
// PCText: StringRefProperty[44]
    var float DisplayDuration;

    var float CooldownTime;

    var float UpdateTime;

    var float TimeRemaining;

    var int MaxDifficulty;

    var Name ClearEvent;

    var Name ClearContext;

    var SFXHintPosition HintPosition;

}

enum SFXNotificationPriotity
{
    NOTIFICATIONPRIORITY_UNDEFINED,
    NOTIFICATIONPRIORITY_NORMAL,
    NOTIFICATIONPRIORITY_HIGH,
};

struct native SFXNotification
{
    var Name nmType;

    var int nPriority;

    var float CreationTime;

    var float fDisplayTime;

    var String sTitle;

    var String sSubtitle;

    var String sBody;

    var Texture oImage;

    var Name nRemoteEvent;

    var Name nmSound;

    var Name nmStopSound;

    var int nFlourishID;

    var int Data1;

}

struct native SFXNotificationData
{
    var Name nmType;

// srTitle: StringRefProperty[44]
// srSubtitle: StringRefProperty[44]
// srBody: StringRefProperty[44]
// srAltTitle: StringRefProperty[44]
// srAltBody: StringRefProperty[44]
// srAltSubtitle: StringRefProperty[44]
    var float DisplayTime;

    var int Priority;

    var String sImageResource;

    var Name nRemoteEvent;

    var Name nmSound;

    var int nFlourishID;

    var bool bCanBeMerged;

}

native function AddNotification(SFXNotification oNotification);

native function AddNotification_JournalChange();

native function AddNotification_CodexChange();

native function AddNotification_CreditRecovery(int nAmount);

native function AddNotification_MedigelRecovery(int nAmount);

native function AddNotification_ParagonChange(int nAmount);

native function AddNotification_RenegadeChange(int nAmount);

native function AddNotification_IridiumRecovery(int nAmount);

native function AddNotification_PalladiumRecovery(int nAmount);

native function AddNotification_PlatinumRecovery(int nAmount);

native function AddNotification_ElementZeroRecovery(int nAmount);

native function AddNotification_HeavyWeaponAmmoRecovery(int nAmount);

native function AddNotification_AmmoRecovery(int nAmount);

native function AddNotification_AchievementChange(String sTitle, String sSubtitle, String sBody, int currentCount, int totalCount);

native function AddNotification_AchievementUnlocked(AchievementData Data, GrinderAchievement Grinder);

native function AddNotification_Custom(float nDisplayTime, String sTitle, String sSubtitle, String sBody, Texture Image, Name nRemoteEvent, optional Name nmType, optional Name nmSound, optional int Priority, optional int FlourishID);

native function ShowNotification(SFXNotification oNotification);

native function SFXNotificationData GetNotificationData(Name nmNotificationType);

native protected function PopulateNotificationOneParameter(SFXNotificationData oNoteData, out SFXNotification oNotification, int nParam1);

event CodexUpdate()
{
    AddNotification_CodexChange();
}

event JournalUpdate()
{
    AddNotification_JournalChange();
}

function SetNotificationPaused(bool bVal)
{
    m_bNotificationIsPaused = bVal;
}

event bool IsHintsOptionDisabled()
{
    local BioPlayerController oPC;
    oPC = BioWorldInfo(Class'Engine'.GetCurrentWorldInfo()).GetLocalPlayerController();
    if (oPC != None && oPC.ProfileSettings != None)
    {
        return !oPC.ProfileSettings.AreHintsEnabled();
    }
    return true;
}

function OnNotificationCompleted()
{
    local SFXNotification CurrentNotification;
    local BioWorldInfo oWorldInfo;
    local BioPlayerController oController;
    if (m_nCurrentlyDisplayedNotification.nRemoteEvent != 'None')
    {
        oWorldInfo = BioWorldInfo(Class'Engine'.GetCurrentWorldInfo());
        if (oWorldInfo != None)
        {
            oController = oWorldInfo.GetLocalPlayerController();
            if (oController != None)
            {
                oController.CauseEvent(CurrentNotification.nRemoteEvent);
            }
        }
    }
    m_bNotificationIsVisible = false;
    m_nCurrentlyDisplayedNotification.oImage = None;
}

function ClearNotifications()
{
    m_bNotificationIsVisible = false;
    m_aNotifications.Length = 0;
    m_nCurrentlyDisplayedNotification.oImage = None;
}

native function HintEvent(Name nmEvent, optional Name nmContext);

native function Clear();

native function HintTrackingData GetTrackingData(Name nmEvent, optional Name nmContext);

native function float GetPreviousTime(out HintTrackingData oData, int nAge);

native function float TimeSince(float fTime);

native function Tick(float TimeDelta);

event CacheCurrentState();

event String TruncateNotificationBody(String sBody)
{
    sBody = Repl(sBody, Chr(13), " ", false);
    if (Len(sBody) > nNotificationBodyLength)
    {
        sBody = Left(sBody, nNotificationBodyLength - 3) $ "...";
    }
    return sBody;
}

defaultproperties
{
    nNotificationBodyLength=100
    m_bEnabled=true
    m_fMinimumTimeBetweenAnyHints=30
    m_aHints=System.Byte[]
    m_nCurrentlyDisplayedHint=-1
    m_NotificationData=System.Byte[]
}
