﻿class BioPlanet extends SFXGalaxyMapObject
    hidecategories(Object)
    config(Game)
    native;

enum EFeatureType
{
    FEATURE_INVALID,
    FEATURE_MINERAL,
    FEATURE_LABEL,
    FEATURE_PROBES,
    FEATURE_ARTIFACT,
    FEATURE_LANDINGSITE,
};

enum EMineralType
{
    MINERAL_RED,
    MINERAL_BLUE,
    MINERAL_GREEN,
    MINERAL_ALPHA,
};

enum ESystemLevelType
{
    SL_PLANET,
    SL_ANOMALY,
    SL_RINGPLANET,
    SL_MASSRELAY,
    SL_DEPOT,
    SL_SUN,
};

enum EOrbitRingType
{
    OR_NONE,
    OR_ORBIT,
    OR_ASTEROID,
};

enum EPlanetType
{
    NOSCAN_PLANET,
    ROCK_PLANET,
    DESERT_PLANET,
    OCEAN_PLANET,
    GARDEN_PLANET,
    GIANT_ICE_PLANET,
    GIANT_JOVIAN_PLANET,
    GIANT_PEGASID_PLANET,
    POST_GARDEN,
    BROWN_DWARF,
    TIDAL_LOCK,
};

struct native PlanetSun
{
    var() LinearColor SunColor;

    var() float Brightness;

}

var transient Bio2DA m_pMapTable;

var transient Bio2DA m_pPlanetTable;

var transient int ActiveWorld;

var int PlanetPlotLabelCondition;

// PlanetPlotLabel: StringRefProperty[44]
// PlanetEventMessage: StringRefProperty[44]
var(Conditions) int PlanetEventTransitionParameter;

var int PlanetEventTransition;

var(Conditions) int PlanetEventParameter;

var int PlanetEventFunction;

var int PlanetEventCondition;

var(Conditions) String PlanetEvent;

var int PlanetLandCondition;

var(Appearance) String ImageBackground;

var String ExitStartPoint;

var String ExitMapName;

var config float ScanBarMaxMineralSize;

var config float EezoMineralsBase;

var config float RandomMineralsBase;

var config float PlacedMineralsPool;

var transient bool bInitialized;

var transient float UnscaledMineralsSum;

var String StartPoint;

var(Map) String MapName;

var array<SFXPlanetFeature> Features;

var float TerrainDepthFactor;

var float DefaultDisplaySize;

var float RelativeSize;

var(Appearance) Texture2D TextureParam;

var(Appearance) Prefab ScenePrefab;

var StaticMesh Mesh;

var(Appearance) MaterialInstanceConstant CloudMaterial;

var(Appearance) MaterialInstanceConstant PlanetMaterial;

var(Appearance) float FringeBloom;

var(Appearance) float Opacity;

var(Appearance) LinearColor CoronaColor;

var(Appearance) array<PlanetSun> Suns;

var(Appearance) LinearColor CloudColor;

var(Appearance) LinearColor RingColor;

var(Appearance) float PlanetRotation;

var(Planet) float IridiumPct;

var(Planet) float PlatinumPct;

var(Planet) float PalladiumPct;

var(Planet) int NumResources;

var(Planet) float ResourceRichness;

var(PlotLabel) EBioPlanetPlotLabelConditionAutoSet PlanetPlotLabelConditionName;

var(PlotLabel) EBioPlanetPlotLabelConditionPlotAutoSet PlanetPlotLabelConditionPlot;

var(PlotLabel) EBioRegionAutoSet PlanetPlotLabelConditionRegion;

var(Conditions) EBioPlanetEventTransitionAutoSet PlanetEventTransitionName;

var(Conditions) EBioPlanetEventTransitionPlotAutoSet PlanetEventTransitionPlot;

var(Conditions) EBioRegionAutoSet PlanetEventTransitionRegion;

var(Conditions) EBioPlanetEventConditionAutoSet PlanetEventConditionName;

var(Conditions) EBioPlanetEventConditionPlotAutoSet PlanetEventConditionPlot;

var(Conditions) EBioRegionAutoSet PlanetEventConditionRegion;

var(Conditions) EBioPlanetLandConditionAutoSet PlanetLandConditionName;

var(Conditions) EBioPlanetLandConditionPlotAutoSet PlanetLandConditionPlot;

var(Conditions) EBioRegionAutoSet PlanetLandConditionRegion;

var(Appearance) EBioGalaxyMap_PlanetType PlanetLevelType;

var(Appearance) ESystemLevelType SystemLevelType;

var(Appearance) EOrbitRingType OrbitRing;

var(Planet) EPlanetType PlanetType;

var(Label) bool AlreadyExplored;

var(Label) int MissionBriefingStrRefOverride;

// MissionBriefing: StringRefProperty[44]
var(Label) GFxMovieInfo PreviewImageResource;

// ButtonLabel: StringRefProperty[44]
// Description: StringRefProperty[44]
enum EBioPlanetEventConditionPlotAutoSet
{
    BioPlanetEventConditionPlot_Unset,
};

enum EBioPlanetEventConditionAutoSet
{
    BioPlanetEventCondition_Unset,
};

enum EBioPlanetEventTransitionPlotAutoSet
{
    BioPlanetEventTransitionPlot_Unset,
};

enum EBioPlanetEventTransitionAutoSet
{
    BioPlanetEventTransition_Unset,
};

enum EBioPlanetLandConditionPlotAutoSet
{
    BioPlanetLandConditionPlot_Unset,
};

enum EBioPlanetLandConditionAutoSet
{
    BioPlanetLandCondition_Unset,
};

enum EBioPlanetPlotLabelConditionPlotAutoSet
{
    BioPlanetPlotLabelConditionPlot_Unset,
};

enum EBioPlanetPlotLabelConditionAutoSet
{
    BioPlanetPlotLabelCondition_Unset,
};

native function bool LoadFrom2DA();

native function SetupMinerals();

native function bool IsVisited();

native function SetVisited(bool bVisited);

native function SaveProbeImpact(Vector vImpact);

native function LoadProbeImpacts(out array<Vector> vImpacts);

function LoadPlanetData(InterpActor Planet, out array<ParticleSystemComponent> TemporaryComponents)
{
    local SFXPlanetFeature F;
    local int I;
    local Vector V;
    local Vector vPos;
    local array<Vector> vProbeImpacts;
    local ParticleSystemComponent Particle;
    local float fDist;
    local int nRetrievedAmount;
    local SFXGameModeOrbital GameMode;
    local BioPlayerController PC;
    if (PlanetMaterial != None)
    {
        Planet.StaticMeshComponent.SetMaterial(0, PlanetMaterial);
    }
    PC = BioWorldInfo(Planet.WorldInfo).GetLocalPlayerController();
    if (PC != None)
    {
        GameMode = SFXGameModeOrbital(PC.GameModeManager2.HACK_GetOrbitalMode());
    }
    LoadProbeImpacts(vProbeImpacts);
    if (GameMode.GetGameData().ProbeLocationMarker != None)
    {
        foreach vProbeImpacts(V)
        {
            vPos = PlanePosToSphere(V, 100);
            Particle = new(Class'ParticleSystemComponent');
            Particle.SetTemplate(GameMode.GetGameData().ProbeLocationMarker);
            Particle.SetAbsolute(false, false, false);
            Particle.SetTranslation(vPos);
            Particle.SetRotation(vPos);
            Particle.bAutoActivate = true;
            Planet.AttachComponent(Particle);
            TemporaryComponents.AddItem(Particle);
        }
    }
    SetupMinerals();
    foreach Features(F)
    {
        // unresolved reference 117708544 ;
    }
}

function float GetPlanetSize()
{
    return DefaultDisplaySize * RelativeSize;
}

function Vector PlanePosToSphere(Vector vPosition2D, float SphereRadius)
{
    local Vector vSphere;
    local float XP;
    local float yp;
    XP = -vPosition2D.X - 0,5 * 2 * 3,141593;
    yp = 0,5 - vPosition2D.Y * 3,141593;
    vSphere.X = Cos(XP) * Cos(yp);
    vSphere.Y = Sin(XP) * Cos(yp);
    vSphere.Z = Sin(yp);
    vSphere *= SphereRadius;
    return vSphere;
}

function Vector SphereToPlanePos(Vector vSphere, float SphereRadius)
{
    local Vector vPosition2D;
    local float angularPos;
    vSphere /= SphereRadius;
    angularPos = Atan(vSphere.Y, vSphere.X) - 3,141593 * 2;
    vPosition2D.X = -angularPos / 3,141593 * 2 + 0,5;
    vPosition2D.Y = 0,5 - Asin(vSphere.Z) / 3,141593;
    return vPosition2D;
}

function SFXGalaxyMapObject AddChild()
{
    local SFXPlanetFeature newFeature;
    newFeature = new(self,Class'SFXPlanetFeature');
    Features.AddItem(newFeature);
    return newFeature;
}

function GetChildren(out array<SFXGalaxyMapObject> Children)
{
    Children = Features;
}

function RemoveChild(SFXGalaxyMapObject Child)
{
    Features.RemoveItem(SFXPlanetFeature(Child));
}

event int GetScaledResourceAmount(EMineralType MineralType, float UnscaledAmount)
{
    if (MineralType == 0)
    {
        return UnscaledAmount * EezoMineralsBase;
    }
    return UnscaledAmount / UnscaledMineralsSum * PlacedMineralsPool * ResourceRichness;
}

event EnumerateChildren()
{
    local SFXCluster Cluster;
    local SFXSystem System;
    System = SFXSystem(Outer);
    Cluster = SFXCluster(System.Outer);
    ActiveWorld = TableID + 100 * System.TableID + 10000 * Cluster.TableID;
}

defaultproperties
{
    OrbitRing=OR_ORBIT
    PlanetLevelType=eBioGM_PlanetType_Planet
    ResourceRichness=1
    PlanetRotation=1
    RingColor=(R=0,G=0,B=0,A=1)
    CloudColor=(R=0,G=0,015,B=0,0125,A=1)
    CoronaColor=(R=0,G=0,B=0,A=1)
    Opacity=1
    RelativeSize=1
    DefaultDisplaySize=550
    TerrainDepthFactor=1
    PlacedMineralsPool=25000
    RandomMineralsBase=350
    EezoMineralsBase=1500
    ScanBarMaxMineralSize=1500
    PlanetLandCondition=51
    PlanetEvent="Land"
    PlanetPlotLabelCondition=52
    m_pPlanetTable=Bio2DA'Default2DA'
    m_pMapTable=Bio2DA'Default2DA'
    Tag="Planet"
    Scale=3
}
