﻿class BioPowerScript extends Object
    within BioPower
    abstract
    native;

// NotRecommended_TargetImmune: StringRefProperty[44]
// NotRecommended_WeaponPowerAlreadyOn: StringRefProperty[44]
// NotRecommended_NoSquadMemberInjured: StringRefProperty[44]
// NotRecommended_NoSquadMemberDead: StringRefProperty[44]
// NotRecommended_NoMedigel: StringRefProperty[44]
// NotRecommended_TargetHasArmor: StringRefProperty[44]
// NotRecommended_TargetHasBiotics: StringRefProperty[44]
// NotRecommended_TargetHasShields: StringRefProperty[44]
// NotRecommended_TargetMachine: StringRefProperty[44]
// NotRecommended_TargetOrganic: StringRefProperty[44]
// Recommended_TargetVulnerable: StringRefProperty[44]
var transient bool m_bPlayerOrderedPowerUse;

var transient Vector m_vLocationToAimAt;

var transient Actor m_oTargetToAimAt;

var ForceFeedbackWaveform m_ImpactWaveForm;

var array<Actor> m_ImpactedActors;

var() Vector m_vProjectileLocation;

var() Vector m_vProjectileVelocity;

var() Vector m_vWorldImpactNormal;

var() Vector m_vWorldImpactLocation;

var() bool bShouldTick;

var() bool m_bWeaponModePower;

event InitializePowerScript();

function OnWeaponImpact(SFXWeapon Weapon, ImpactInfo Impact);

function OnWeaponEquip(SFXWeapon Weapon);

function OnWeaponUnequip(SFXWeapon Weapon);

function OnWeaponReload(SFXWeapon Weapon, bool QuickReload);

function OnProjectileExploded(Actor oCaster, Vector HitLocation, Vector HitNormal, Vector Velocity);

event Tick(float DeltaTime);

event bool CanStartPower(Actor oCaster)
{
    return true;
}

event bool StartPhase(EBioPowerState ePhase, Actor oCaster, float fDuration)
{
    return true;
}

event bool EndPhase(EBioPowerState ePhase, Actor oCaster)
{
    local SFXGame oGame;
    local float Cooldown;
    if (ePhase == 2 && oCaster != None)
    {
        oGame = SFXGame(oCaster.WorldInfo.Game);
        if (oGame != None && oGame.gameconfig.UseSharedPowerCooldown)
        {
            if (GetPowerCooldown(Cooldown))
            {
                SetPowerCooldowns(BioPawn(oCaster), Cooldown);
            }
        }
    }
    return true;
}

function bool GetPowerCooldown(out float Cooldown)
{
    if (self.Outer == None)
    {
        return false;
    }
    switch (self.Outer.m_nmPower)
    {
        case 'LW_Marksman':
            Cooldown = 6;
            break;
        case 'BW_Carnage':
            Cooldown = 6;
            break;
        case 'SW_Critical':
            Cooldown = 6;
            break;
        case 'TECH_Overload':
            Cooldown = 6;
            break;
        case 'AT_Immunity':
            Cooldown = 12;
            break;
        case 'AT_ConcussiveShot':
            Cooldown = 3;
            break;
        case 'AR_ShieldBoost':
            Cooldown = 12;
            break;
        case 'DC_Sabotage':
            Cooldown = 3;
            break;
        case 'DC_Hacking':
            Cooldown = 6;
            break;
        case 'MD_Shock':
            Cooldown = 6;
            break;
        case 'TECH_Fissure':
            Cooldown = 3;
            break;
        case 'TECH_CombatDrone':
            Cooldown = 12;
            break;
        case 'TK_Throw':
            Cooldown = 3;
            break;
        case 'TK_Lift':
            Cooldown = 6;
            break;
        case 'TK_Pull':
            Cooldown = 3;
            break;
        case 'SD_Warp':
            Cooldown = 3;
            break;
        case 'SD_Singularity':
            Cooldown = 12;
            break;
        case 'KF_Barrier':
            Cooldown = 12;
            break;
        case 'KF_Stasis':
            Cooldown = 12;
            break;
        case 'HENCH_Crush':
            Cooldown = 6;
            break;
        case 'HENCH_PhaseShift':
            Cooldown = 12;
            break;
        case 'HENCH_ShieldJack':
            Cooldown = 6;
            break;
        case 'HENCH_ImplantOverdrive':
            Cooldown = 3;
            break;
        case 'HENCH_Concussive':
            Cooldown = 6;
            break;
        default:
            return false;
    }
    Cooldown *= self.Outer.GetDifficultyCooldownMultiplier();
    return true;
}

function SetPowerCooldowns(BioPawn Pawn, float Cooldown)
{
    local BioPawnBehavior Behavior;
    local int nIndex;
    local BioPower Power;
    if (Pawn == None)
    {
        return;
    }
    Behavior = Pawn.m_oBehavior;
    if (Behavior == None)
    {
        return;
    }
    nIndex = 0;
    while (nIndex < Behavior.m_powers.Length)
    {
        Power = Behavior.m_powers[nIndex];
        if (Power != None)
        {
            if (PowerSharesCooldown(Power))
            {
                Power.SetTotalCooldownTime(Cooldown);
                Power.SetCooldownRemaining(Cooldown);
            }
        }
        nIndex++;
    }
}

function bool PowerSharesCooldown(BioPower Power)
{
    switch (Power.m_nmPower)
    {
        case 'Rifle_Butt':
        case 'Pistol_Whip':
        case 'MD_SquadHeal':
            return false;
        default:
            return true;
    }
}

event bool OnImpact(Actor oCaster, float fCasterStability, Actor oImpacted, int nPreviouslyImpacted);

event bool EffectUnapply(Actor oCaster, float fCasterStability, Actor oImpacted, int nPreviouslyImpacted);

event bool AdjustCooldown(out float fCoolDownTime)
{
    local float fStaticBonus;
    if (Pawn(Outer.Owner) != None && Pawn(Outer.Owner).InPlayerParty() && Outer.NewPower != None)
    {
        fStaticBonus = 1;
        GetCooldownResearchBonus(fStaticBonus);
        fCoolDownTime *= fStaticBonus + Outer.NewPower.DynamicCooldownBonus - 1;
    }
    return true;
}

function GetCooldownResearchBonus(out float fStaticResearchBonus)
{
    if (Pawn(Outer.Owner) != None && Pawn(Outer.Owner).InPlayerParty() && Outer.NewPower != None)
    {
        if (Outer.NewPower.Discipline == 1)
        {
            fStaticResearchBonus *= SFXGame(Pawn(Outer.Owner).WorldInfo.Game).BonusList.RchBioticCooldownCache;
        }
        else
        {
            if (Outer.NewPower.Discipline == 2)
            {
                fStaticResearchBonus *= SFXGame(Pawn(Outer.Owner).WorldInfo.Game).BonusList.RchTechCooldownCache;
            }
        }
    }
}

native function bool GetFloorLocation(Vector vStartLocation, out Vector vFloorLocation);

native function bool PlayGuiSound(Name nmSound);

function bool PlayForceFeedback(Actor oCaster, Actor oImpacted)
{
    local BioPawn pPawn;
    local BioPlayerController pPC;
    local BioPlayerController pController;
    local BioVehicleBase pVehicle;
    local BioWorldInfo pWorldInfo;
    local bool bPlayForceFeedback;
    bPlayForceFeedback = false;
    pPawn = BioPawn(oCaster);
    if (pPawn != None)
    {
        pWorldInfo = BioWorldInfo(oCaster.WorldInfo);
        if (pWorldInfo != None)
        {
            pController = pWorldInfo.GetLocalPlayerController();
            pPC = BioPlayerController(pPawn.Controller);
            if (pController != None && pPC == pController)
            {
                bPlayForceFeedback = true;
            }
        }
    }
    if (!bPlayForceFeedback)
    {
        pVehicle = BioVehicleBase(oCaster);
        if (pVehicle != None)
        {
            pWorldInfo = BioWorldInfo(oCaster.WorldInfo);
            if (pWorldInfo != None)
            {
                pController = pWorldInfo.GetLocalPlayerController();
                pPC = BioPlayerController(pVehicle.Controller);
                if (pController != None && pPC == pController)
                {
                    bPlayForceFeedback = true;
                }
            }
        }
    }
    if (!bPlayForceFeedback)
    {
        pPawn = BioPawn(oImpacted);
        if (pPawn != None)
        {
            pWorldInfo = BioWorldInfo(oCaster.WorldInfo);
            if (pWorldInfo != None)
            {
                pController = pWorldInfo.GetLocalPlayerController();
                pPC = BioPlayerController(pPawn.Controller);
                if (pController != None && pPC == pController)
                {
                    bPlayForceFeedback = true;
                }
            }
        }
    }
    if (!bPlayForceFeedback)
    {
        pVehicle = BioVehicleBase(oImpacted);
        if (pVehicle != None)
        {
            pWorldInfo = BioWorldInfo(oCaster.WorldInfo);
            if (pWorldInfo != None)
            {
                pController = pWorldInfo.GetLocalPlayerController();
                pPC = BioPlayerController(pVehicle.Controller);
                if (pController != None && pPC == pController)
                {
                    bPlayForceFeedback = true;
                }
            }
        }
    }
    if (bPlayForceFeedback)
    {
        pController.ClientPlayForceFeedbackWaveform(m_ImpactWaveForm);
    }
    return true;
}

simulated function float GetElapsedTime()
{
    return 0;
}

simulated function float GetDuration()
{
    return 0;
}

event bool ShouldUsePower(Actor Caster, Actor Target, out String sOptionalInfo)
{
    sOptionalInfo = "";
    return true;
}

function bool ShouldUsePowerOnShields(BioPawn Target, class<SFXDamageType> DamageType, out String sOptionalInfo)
{
    local SFXShield_Base CurrentShields;
    if (Target != None && Target.GetCurrentShields() > 0)
    {
        CurrentShields = GetCurrentShields(Target);
        if (CurrentShields == None)
        {
            return true;
        }
        if (SFXShield_Energy(CurrentShields) != None)
        {
            if (DamageType.Default.Resistance.Shield < 2)
            {
                sOptionalInfo = NotRecommended_TargetHasShields;
                return false;
            }
        }
        else
        {
            if (SFXShield_Biotic(CurrentShields) != None)
            {
                if (DamageType.Default.Resistance.Biotic < 2)
                {
                    sOptionalInfo = NotRecommended_TargetHasBiotics;
                    return false;
                }
            }
            else
            {
                if (SFXShield_Armour(CurrentShields) != None)
                {
                    if (DamageType.Default.Resistance.Armour < 2)
                    {
                        sOptionalInfo = NotRecommended_TargetHasArmor;
                        return false;
                    }
                }
                else
                {
                    return false;
                }
            }
        }
    }
    return true;
}

function SFXShield_Base GetCurrentShields(BioPawn Target)
{
    local SFXShield_Base Shields;
    if (Target != None && Target.InvManager != None)
    {
        foreach Target.InvManager.InventoryActors(Class'SFXShield_Base', Shields)
        {
            if (Shields.CurrentShields > 0)
            {
                return Shields;
            }
        }
    }
}

function OnPowerAdded(SFXPower Power);

function OnPowerRankIncreased();

function OnPowersLoaded();

function OnSquadMemberAdded(Pawn Pawn);

function OnOwnerDied();

function ReloadAmmoPower(BioPawn Target, SFXWeapon Weapon);

event String GetHUDWheelIconInfo(BioPawn OwnerPawn)
{
    return "";
}

function DebugDraw_Power(BioHUD H);

defaultproperties
{
    m_ImpactWaveForm=ForceFeedbackWaveform'ForceFeedbackWaveformBase'
    Recommended_TargetVulnerable=System.Byte[]
    NotRecommended_TargetOrganic=System.Byte[]
    NotRecommended_TargetMachine=System.Byte[]
    NotRecommended_TargetHasShields=System.Byte[]
    NotRecommended_TargetHasBiotics=System.Byte[]
    NotRecommended_TargetHasArmor=System.Byte[]
    NotRecommended_NoMedigel=System.Byte[]
    NotRecommended_NoSquadMemberDead=System.Byte[]
    NotRecommended_NoSquadMemberInjured=System.Byte[]
    NotRecommended_WeaponPowerAlreadyOn=System.Byte[]
    NotRecommended_TargetImmune=System.Byte[]
    // child object ForceFeedbackWaveformBase of type ForceFeedbackWaveform
}
