﻿class BioSFHandler_CharacterRecord2 extends BioSFHandler
    config(UI)
    transient;

enum PowerRankState
{
    POWER_RANK_STATE_LOCKED,
    POWER_RANK_STATE_BUYABLE,
    POWER_RANK_STATE_BOUGHT,
};

const CHARACTER_HANDLER_EVENT_INITIALIZE = 1;
const CHARACTER_HANDLER_EVENT_NEXT_CHARACTER = 2;
const CHARACTER_HANDLER_EVENT_AUTO_LEVEL_UP = 3;
const CHARACTER_HANDLER_EVENT_PURCHASE_TALENT_RANK = 4;
const CHARACTER_HANDLER_EVENT_CANCEL = 6;
const CHARACTER_HANDLER_EVENT_COMMIT = 7;
const CHARACTER_HANDLER_EVENT_PREV_CHARACTER = 8;
const CHARACTER_HANDLER_EVENT_END_SUSPEND = 10;
// m_srFaceCodeFormat: StringRefProperty[44]
// m_srRankDescriptionFormat: StringRefProperty[44]
// m_srTalentDescriptionFormat: StringRefProperty[44]
// m_srShepardLevelClassFormat: StringRefProperty[44]
// m_srOk: StringRefProperty[44]
// m_srParagonRenegadeMessagePS3: StringRefProperty[44]
// m_srSpendTalentPointsMessagePS3: StringRefProperty[44]
// m_srParagonRenegadeMessagePC: StringRefProperty[44]
// m_srSpendTalentPointsMessagePC: StringRefProperty[44]
// m_srParagonRenegadeMessageXBox: StringRefProperty[44]
// m_srSpendTalentPointsMessageXBox: StringRefProperty[44]
// m_srXPFormat: StringRefProperty[44]
var transient float m_InitialHiddenTimer;

var transient array<BioPawn> m_SpawnedSquadMembersSource;

var transient array<int> m_bPowerLockedStatus;

var transient array<Texture> m_SquadIcons;

var transient SFXPowerLevelUpHelper m_Helper;

var transient array<???> m_SquadNames;

var transient bool m_bCloseGUIButtonPressed;

var transient float m_fScrollValue;

var transient int m_CurrentPawnIndex;

var transient BioWorldInfo m_WorldInfo;

var config array<float> m_ParagonRenegadeBarValues;

var config int m_nMaxParagon;

var config int m_nMaxRenegade;

const DEFAULT_INITIAL_HIDDEN_TIMER = 0.1;
delegate OnCloseCallback();

function SetOnCloseCallback(delegate<OnCloseCallback> fn_OnCloseDelegate)
{
    __OnCloseCallback__Delegate = fn_OnCloseDelegate;
}

function HandleEvent(byte nCommand, const out array<String> Parameters)
{
    switch (nCommand)
    {
        case 1:
            InitializeCharacterRecord();
            break;
        case 2:
            NextCharacter(true);
            break;
        case 8:
            NextCharacter(false);
            break;
        case 3:
            AutoLevelUp();
            break;
        case 4:
            PurchasePowerRank(Parameters[0]);
            break;
        case 6:
            CancelChanges();
            break;
        case 7:
            m_Helper.ApplyChanges();
            CleanupCharacter();
            break;
        case 10:
            UpdateAll();
            break;
    }
}

function HandleInputEvent(BioGuiEvents Event, optional float fValue = 1)
{
    if (Event == 13 || Event == 21)
    {
        oWorldInfo.GetInputLock(-1);
        m_bCloseGUIButtonPressed = true;
    }
    else
    {
        if (m_bCloseGUIButtonPressed || BioWorldInfo(oWorldInfo).GetGuiInputPermission(Event) == false)
        {
            return;
        }
    }
    switch (Event)
    {
        case 5:
            ProcessRStickAxisInput(fValue);
            break;
        default:
            HandleInputEvent(Event, fValue);
    }
}

function ProcessRStickAxisInput(float fValue)
{
    if (Abs(fValue) > 0,25)
    {
        m_fScrollValue = -fValue;
    }
    else
    {
        m_fScrollValue = 0;
    }
}

function Update(float fDeltaT)
{
    local ASParams stParam;
    local array<ASParams> lstParams;
    stParam.Type = 1;
    stParam.fVar = m_fScrollValue;
    lstParams.AddItem(stParam);
    oPanel.InvokeMethodArgs("scrollDetailsAnalog", lstParams);
    if (m_Helper != None && m_Helper.m_Pawn != None && m_WorldInfo != None && m_WorldInfo.EventNotifier != None)
    {
        if (m_WorldInfo.EventNotifier.PendingTalentNotify(m_Helper.m_Pawn))
        {
            lstParams[0].Type = 0;
            lstParams[0].nVar = 2;
            stParam.Type = 3;
            stParam.bVar = false;
            lstParams.AddItem(stParam);
            oPanel.InvokeMethodArgs("SuspendWithPeriod", lstParams);
            m_WorldInfo.EventNotifier.ShowTalentNotify(m_Helper.m_Pawn);
        }
    }
    if (m_Helper != None && m_Helper.m_Pawn != None && m_InitialHiddenTimer > 0)
    {
        m_InitialHiddenTimer -= fDeltaT;
        if (m_InitialHiddenTimer <= 0)
        {
            m_WorldInfo.m_UIWorld.HidePawn(m_Helper.m_Pawn, false);
        }
    }
}

function OnPanelRemoved()
{
    local BioPawn SourcePawn;
    OnPanelRemoved();
    if (m_WorldInfo != None)
    {
        if (m_Helper != None && m_Helper.m_Pawn != None)
        {
            if (m_WorldInfo.EventNotifier != None)
            {
                m_WorldInfo.EventNotifier.RemoveTalentNotify(m_Helper.m_Pawn);
            }
        }
        if (m_WorldInfo.m_UIWorld != None)
        {
            foreach m_SpawnedSquadMembersSource(SourcePawn)
            {
                m_WorldInfo.m_UIWorld.DestroyPawn(SourcePawn);
            }
            m_WorldInfo.m_UIWorld.ResetActors();
            m_WorldInfo.m_UIWorld.TriggerEvent('LightsOff', m_WorldInfo);
        }
    }
    m_WorldInfo = None;
    m_CurrentPawnIndex = 0;
    m_fScrollValue = 0;
    m_bCloseGUIButtonPressed = false;
    m_SquadNames.Length = 0;
    __OnCloseCallback__Delegate = None;
    m_Helper = None;
}

function OnPanelAdded()
{
    local SFXGame oGame;
    local BioSFHandler_MessageBox MessageBox;
    local BioMessageBoxOptionalParams stParams;
    oPanel.SetExternalInterface(self);
    OnPanelAdded();
    if (oPanel != None)
    {
        oPanel.bApplyRightThumbstickDeadzone = true;
        oGame = SFXGame(Class'Engine'.GetCurrentWorldInfo().Game);
        if (oGame != None && oGame.bShowSquadScreenMessageBoxes)
        {
            MessageBox = MassEffectGuiManager(oPanel.oParentManager).CreateMessageBox();
            MessageBox.SetInputDelegate(Callback_ClosedTalentMsgBox);
            stParams.srAText = m_srOk;
            stParams.bModal = true;
            stParams.bNoFade = true;
            if (Class'WorldInfo'.IsConsoleBuild(1))
            {
                srMessage = m_srSpendTalentPointsMessageXBox;
            }
            else
            {
                if (Class'WorldInfo'.IsConsoleBuild(2))
                {
                    srMessage = m_srSpendTalentPointsMessagePS3;
                }
                else
                {
                    if (!Class'WorldInfo'.IsConsoleBuild(0))
                    {
                        srMessage = m_srSpendTalentPointsMessagePC;
                    }
                }
            }
            MessageBox.DisplayMessageBox(srMessage, stParams);
        }
    }
}

function Callback_ClosedTalentMsgBox(bool bAPressed, int Context)
{
    local BioSFHandler_MessageBox MessageBox;
    local BioMessageBoxOptionalParams stParams;
    if (oPanel != None)
    {
        MessageBox = MassEffectGuiManager(oPanel.oParentManager).CreateMessageBox();
        stParams.srAText = m_srOk;
        stParams.bModal = true;
        stParams.bNoFade = true;
        if (Class'WorldInfo'.IsConsoleBuild(1))
        {
            srMessage = m_srParagonRenegadeMessageXBox;
        }
        else
        {
            if (Class'WorldInfo'.IsConsoleBuild(2))
            {
                srMessage = m_srParagonRenegadeMessagePS3;
            }
            else
            {
                if (!Class'WorldInfo'.IsConsoleBuild(0))
                {
                    srMessage = m_srParagonRenegadeMessagePC;
                }
            }
        }
        MessageBox.DisplayMessageBox(srMessage, stParams);
    }
}

final private function bool InitializeCharacterRecord()
{
    local int nIndex;
    local int nSquadSize;
    local BioPawn oPawnToStartGUIWith;
    local BioPawn oPawn;
    local array<ASParams> lstParams;
    local ASParams stParam;
    local Texture henchIcon;
    m_WorldInfo = BioWorldInfo(oWorldInfo);
    if (m_WorldInfo == None || m_WorldInfo.m_playerSquad == None)
    {
        return false;
    }
    m_Helper = new(self,Class'SFXPowerLevelUpHelper');
    if (m_Helper == None)
    {
        return false;
    }
    stParam.Type = 0;
    stParam.nVar = ScreenLayout;
    lstParams.AddItem(stParam);
    oPanel.InvokeMethodArgs("SetPlatformLayout", lstParams);
    lstParams.Length = 0;
    m_Helper.Initialize(m_WorldInfo);
    nSquadSize = m_WorldInfo.m_playerSquad.GetSquadSize();
    m_SquadNames.Length = 0;
    nIndex = 0;
    if (nIndex < nSquadSize)
    {
        oPawn = BioPawn(m_WorldInfo.m_playerSquad.GetMember(nIndex));
        if (oPawn != None)
        {
            if (SFXPawn(oPawn) != None)
            {
                srName = SFXPawn(oPawn).GetPrettyName();
                henchIcon = SFXPawn(oPawn).GetGUIIcon();
            }
        }
        else
        {
            srName = // unknown bytecode 4F ;
        }
    }
}

final private function bool NextCharacter(bool bNext)
{
    local BioPawn NextPawn;
    local int nNextPawnIndex;
    if (m_WorldInfo == None || m_WorldInfo.m_playerSquad == None)
    {
        return false;
    }
    if (m_WorldInfo != None && m_WorldInfo.EventNotifier != None)
    {
        m_WorldInfo.EventNotifier.RemoveTalentNotify(m_Helper.m_Pawn);
    }
    nNextPawnIndex = m_CurrentPawnIndex;
    while (NextPawn == None)
    {
        if (bNext)
        {
            nNextPawnIndex++;
            if (nNextPawnIndex >= m_WorldInfo.m_playerSquad.GetSquadSize())
            {
                nNextPawnIndex = 0;
            }
        }
        else
        {
            nNextPawnIndex--;
            if (nNextPawnIndex < 0)
            {
                nNextPawnIndex = m_WorldInfo.m_playerSquad.GetSquadSize() - 1;
            }
        }
        NextPawn = BioPawn(m_WorldInfo.m_playerSquad.GetMember(nNextPawnIndex));
        if (NextPawn == None && nNextPawnIndex == m_CurrentPawnIndex)
        {
            PlayGuiSound('GuiError');
            return false;
        }
    }
    m_Helper.ApplyChanges();
    CleanupCharacter();
    return ChangeToCharacter(NextPawn);
}

final private function bool ChangeToCharacter(BioPawn NextCharacter)
{
    local ASParams CurrentParameter;
    local array<ASParams> Parameters;
    if (m_WorldInfo == None || m_WorldInfo.m_UIWorld == None || m_WorldInfo.m_playerSquad == None)
    {
        return false;
    }
    if (NextCharacter == None)
    {
        return false;
    }
    m_Helper.SetPawn(NextCharacter);
    m_CurrentPawnIndex = m_WorldInfo.m_playerSquad.GetMemberIndex(m_Helper.m_Pawn);
    CurrentParameter.Type = 0;
    CurrentParameter.nVar = 2;
    Parameters.AddItem(CurrentParameter);
    CurrentParameter.Type = 3;
    CurrentParameter.bVar = true;
    Parameters.AddItem(CurrentParameter);
    oPanel.InvokeMethodArgs("SuspendWithPeriod", Parameters);
    m_WorldInfo.m_UIWorld.TriggerEvent('SetupCharRec', m_WorldInfo);
    if (m_InitialHiddenTimer <= 0)
    {
        m_WorldInfo.m_UIWorld.HidePawn(m_Helper.m_Pawn, false);
    }
    PlayGuiSound('ChangeCharacter');
    return true;
}

final private function AutoLevelUp()
{
    m_Helper.AutoLevelUp();
    CreatePowerList();
    UpdatePowerList();
    UpdatePoints();
    UpdateParagonRenegadeValues();
}

final private function PurchasePowerRank(int nPowerIndex)
{
    local ASParams CurrentParameter;
    local array<ASParams> Parameters;
    local MassEffectGuiManager GUIManager;
    if (m_Helper.IncreaseRank(nPowerIndex) == false)
    {
        return;
    }
    if (m_Helper.CanEvolvePower(nPowerIndex))
    {
        CurrentParameter.Type = 0;
        CurrentParameter.nVar = nPowerIndex;
        Parameters.AddItem(CurrentParameter);
        oPanel.InvokeMethodArgs("ShowTalentEvolve", Parameters);
        GUIManager = MassEffectGuiManager(oPanel.oParentManager);
        if (GUIManager != None)
        {
            if (GUIManager.ShowSpecializationGUI(m_Helper.m_Pawn, m_Helper.m_powers[nPowerIndex].PowerName))
            {
                if (GUIManager.oSpecializationHandler != None)
                {
                    GUIManager.oSpecializationHandler.__EvolvePowerDelegate__Delegate = EvolvePower;
                    GUIManager.oSpecializationHandler.__CancelEvolutionDelegate__Delegate = CancelEvolution;
                }
            }
        }
    }
    UpdatePowerList();
    UpdatePoints();
    UpdateParagonRenegadeValues(true);
    Parameters.Length = 0;
    CurrentParameter.Type = 0;
    CurrentParameter.nVar = nPowerIndex;
    Parameters.AddItem(CurrentParameter);
    CurrentParameter.Type = 1;
    CurrentParameter.fVar = m_Helper.GetCurrentRank(nPowerIndex) - 1;
    Parameters.AddItem(CurrentParameter);
    CurrentParameter.Type = 3;
    CurrentParameter.bVar = true;
    Parameters.AddItem(CurrentParameter);
    oPanel.InvokeMethodArgs("SetTallentRankSelected", Parameters);
}

final private function CleanupCharacter()
{
    if (m_WorldInfo == None || m_WorldInfo.m_UIWorld == None)
    {
        return;
    }
    m_WorldInfo.m_UIWorld.HidePawn(m_Helper.m_Pawn, true);
    if (SFXPawn_Player(m_Helper.m_Pawn) != None)
    {
        SFXPawn_Player(m_Helper.m_Pawn).AutoMap();
    }
}

final private function UpdateAll()
{
    local ASParams CurrentParameter;
    local array<ASParams> Parameters;
    if (m_WorldInfo == None || m_WorldInfo.m_playerSquad == None)
    {
        return;
    }
    CurrentParameter.Type = 3;
    CurrentParameter.bVar = m_WorldInfo.m_playerSquad.GetSquadSize() > 1;
    Parameters.AddItem(CurrentParameter);
    oPanel.InvokeMethodArgs("setCanChangeCharacter", Parameters);
    UpdateCharacterStats();
    CreatePowerList();
    UpdatePowerList();
    UpdatePoints();
    UpdateParagonRenegadeValues();
    oPanel.IsVisible = true;
}

final private function UpdateCharacterStats()
{
    local BioAttributesPawn Attributes;
    local int PlayerLevel;
    local ASParams CurrentParameter;
    local array<ASParams> Parameters;
    local bool bShowPrevious;
    local bool bShowNext;
    local SFXPawn_Player PlayerPawn;
    local SFXGame Game;
    local Texture henchIcon1;
    local Texture henchIcon2;
    local SFXPawn_Henchman oPawn;
    local int nMaxHealth;
    local int nMaxShields;
    local int nXP_NextLevel;
    local int nXP_Current;
    local String sShieldsTitle;
    local String sXP;
    local String sFaceCode;
    local String sPlayerClass;
    local BioPlayerController oController;
    local SFXEngine oEngine;
    oEngine = None;
    if (oWorldInfo != None)
    {
        oController = BioWorldInfo(oWorldInfo).GetLocalPlayerController();
    }
    if (oController != None)
    {
        oEngine = SFXEngine(oController.Player.Outer);
    }
    if (m_Helper.m_Pawn == None || m_Helper.m_Pawn.m_oBehavior == None)
    {
        return;
    }
    Attributes = BioAttributesPawn(m_Helper.m_Pawn.m_oBehavior.m_PawnAttributes);
    if (Attributes == None)
    {
        return;
    }
    Game = oWorldInfo != None ? SFXGame(oWorldInfo.Game) : None;
    if (Game == None)
    {
        return;
    }
    Game.GetPlayerLevel(oEngine.CurrentUserID, PlayerLevel);
    CurrentParameter.Type = 2;
    PlayerPawn = SFXPawn_Player(m_Helper.m_Pawn);
    if (PlayerPawn != None)
    {
        CurrentParameter.Type = 2;
        CurrentParameter.sVar = PlayerPawn.GetFullName();
        Parameters.AddItem(CurrentParameter);
        sPlayerClass = Class'SFXGame'.GetSimpleString(PlayerPawn.PlayerClass.srClassName);
        ClearCustomTokens();
        SetCustomToken(0, PlayerLevel);
        SetCustomToken(1, sPlayerClass);
        CurrentParameter.Type = 2;
        CurrentParameter.sVar = Class'SFXGame'.GetSimpleString(m_srShepardLevelClassFormat, true);
        ClearCustomTokens();
        Parameters.AddItem(CurrentParameter);
        if (Len(PlayerPawn.FaceCode) > 0)
        {
            ClearCustomTokens();
            SetCustomToken(0, PlayerPawn.FaceCode);
            sFaceCode = Class'SFXGame'.GetSimpleString(m_srFaceCodeFormat, true);
            ClearCustomTokens();
        }
        else
        {
            sFaceCode = "";
        }
    }
    else
    {
        oPawn = SFXPawn_Henchman(m_Helper.m_Pawn);
        if (oPawn != None)
        {
            CurrentParameter.Type = 2;
            CurrentParameter.sVar = Game.GetSimpleString(oPawn.GetPrettyName());
            Parameters.AddItem(CurrentParameter);
            CurrentParameter.sVar = "";
            Parameters.AddItem(CurrentParameter);
        }
        sFaceCode = "";
    }
    GetSquadMemberNamesAndIcons(srCurrent, srPrevious, srNext, henchIcon2, henchIcon1);
    if (henchIcon1 != None && henchIcon2 != None)
    {
        SetExternalTextureOnPanel("HenchPlaceholder1", henchIcon1);
        SetExternalTextureOnPanel("HenchPlaceholder2", henchIcon2);
    }
    if (srCurrent != srPrevious)
    {
        bShowPrevious = true;
    }
    else
    {
        bShowPrevious = false;
    }
    if (srCurrent != srNext && srNext != srPrevious)
    {
        bShowNext = true;
    }
    else
    {
        bShowNext = false;
    }
    CurrentParameter.Type = 2;
    CurrentParameter.sVar = srPrevious;
    Parameters.AddItem(CurrentParameter);
    CurrentParameter.Type = 3;
    CurrentParameter.bVar = bShowPrevious;
    Parameters.AddItem(CurrentParameter);
    CurrentParameter.Type = 2;
    CurrentParameter.sVar = srNext;
    Parameters.AddItem(CurrentParameter);
    CurrentParameter.Type = 3;
    CurrentParameter.bVar = bShowNext;
    Parameters.AddItem(CurrentParameter);
    CurrentParameter.Type = 2;
    CurrentParameter.sVar = sFaceCode;
    Parameters.AddItem(CurrentParameter);
    oPanel.InvokeMethodArgs("setCharInfo", Parameters);
    Parameters.Length = 0;
    nMaxHealth = m_Helper.m_Pawn.GetMaxHealth();
    sShieldsTitle = SFXPawn(m_Helper.m_Pawn).GetShieldTypeAsDisplayString();
    nMaxShields = m_Helper.m_Pawn.GetMaxShields();
    GetExperienceProgress(nXP_Current, nXP_NextLevel);
    sXP = nXP_NextLevel - nXP_Current;
    CurrentParameter.Type = 2;
    CurrentParameter.sVar = nMaxHealth;
    Parameters.AddItem(CurrentParameter);
    CurrentParameter.Type = 2;
    CurrentParameter.sVar = sShieldsTitle;
    Parameters.AddItem(CurrentParameter);
    CurrentParameter.Type = 2;
    CurrentParameter.sVar = nMaxShields;
    Parameters.AddItem(CurrentParameter);
    CurrentParameter.Type = 2;
    CurrentParameter.sVar = sXP;
    Parameters.AddItem(CurrentParameter);
    oPanel.InvokeMethodArgs("setCharStats", Parameters);
    Parameters.Length = 0;
    UpdateParagonRenegadeValues();
}

function GetParagonRenegadePercentage(SFXPawn_Player PlayerPawn, out float ParagonPercent, out float RenegadePercent)
{
    local int Index;
    local float TotalCharmBarValue;
    local float TotalIntimidateBarValue;
    local float CharmSkill;
    local float IntimidateSkill;
    local BioPawnBehavior Behavior;
    local float DisplayBarValue;
    if (PlayerPawn != None)
    {
        Behavior = BioPawnBehavior(PlayerPawn.oBioComponent);
        if (Behavior != None)
        {
            DisplayBarValue = 1 / m_ParagonRenegadeBarValues.Length;
            RenegadePercent = 0;
            IntimidateSkill = Behavior.GetIntimidateSkill(m_Helper) / m_nMaxRenegade;
            Index = 0;
            while (Index < m_ParagonRenegadeBarValues.Length)
            {
                if (IntimidateSkill >= TotalIntimidateBarValue + m_ParagonRenegadeBarValues[Index])
                {
                    RenegadePercent += DisplayBarValue;
                }
                else
                {
                    RenegadePercent += IntimidateSkill - TotalIntimidateBarValue / m_ParagonRenegadeBarValues[Index] * DisplayBarValue;
                    break;
                }
                TotalIntimidateBarValue += m_ParagonRenegadeBarValues[Index];
                Index++;
            }
            ParagonPercent = 0;
            CharmSkill = Behavior.GetCharmSkill(m_Helper) / m_nMaxParagon;
            Index = 0;
            while (Index < m_ParagonRenegadeBarValues.Length)
            {
                if (CharmSkill >= TotalCharmBarValue + m_ParagonRenegadeBarValues[Index])
                {
                    ParagonPercent += DisplayBarValue;
                }
                else
                {
                    ParagonPercent += CharmSkill - TotalCharmBarValue / m_ParagonRenegadeBarValues[Index] * DisplayBarValue;
                    break;
                }
                TotalCharmBarValue += m_ParagonRenegadeBarValues[Index];
                Index++;
            }
        }
    }
}

function UpdateParagonRenegadeValues(optional bool bAnimate = false)
{
    local ASParams CurrentParameter;
    local array<ASParams> Parameters;
    local SFXPawn_Player PlayerPawn;
    local SFXGame Game;
    local float CharmPercentage;
    local float IntimidatePercentage;
    if (m_Helper.m_Pawn == None || m_Helper.m_Pawn.m_oBehavior == None)
    {
        return;
    }
    PlayerPawn = SFXPawn_Player(m_Helper.m_Pawn);
    Game = oWorldInfo != None ? SFXGame(oWorldInfo.Game) : None;
    if (Game == None)
    {
        return;
    }
    CurrentParameter.Type = 0;
    if (PlayerPawn != None)
    {
        GetParagonRenegadePercentage(PlayerPawn, CharmPercentage, IntimidatePercentage);
        CharmPercentage *= 100;
        IntimidatePercentage *= 100;
        CurrentParameter.nVar = IntimidatePercentage;
        Parameters.AddItem(CurrentParameter);
        CurrentParameter.nVar = CharmPercentage;
        Parameters.AddItem(CurrentParameter);
        PlayerPawn.UpdateParameters();
        if (!bAnimate)
        {
            oPanel.InvokeMethodArgs("setRenegadeParagon", Parameters);
        }
        else
        {
            oPanel.InvokeMethodArgs("seekSetRenegadeParagon", Parameters);
        }
        Parameters.Length = 0;
        CurrentParameter.Type = 0;
        CurrentParameter.nVar = FloorEx(IntimidatePercentage / 20);
        Parameters.AddItem(CurrentParameter);
        CurrentParameter.Type = 0;
        CurrentParameter.nVar = FloorEx(CharmPercentage / 20);
        Parameters.AddItem(CurrentParameter);
        oPanel.InvokeMethodArgs("setCharmIntimidateRank", Parameters);
    }
    else
    {
        CurrentParameter.nVar = -1;
        Parameters.AddItem(CurrentParameter);
        Parameters.AddItem(CurrentParameter);
        oPanel.InvokeMethodArgs("setRenegadeParagon", Parameters);
    }
}

function float FloorEx(float Value)
{
    local float Result;
    Result = Round(Value);
    if (Result > Value)
    {
        Result = Result - 1;
    }
    return Result;
}

final private function CreatePowerList()
{
    local int nIndex;
    local ASParams CurrentParameter;
    local array<ASParams> Parameters;
    local SFXPower Power;
    if (m_Helper.m_Pawn == None || m_Helper.m_Pawn.PowerManager == None)
    {
        return;
    }
    CurrentParameter.Type = 0;
    CurrentParameter.nVar = m_Helper.m_powers.Length;
    Parameters.AddItem(CurrentParameter);
    oPanel.InvokeMethodArgs("CreateTalentList", Parameters);
    Parameters.Length = 0;
    CurrentParameter.Type = 0;
    Parameters.AddItem(CurrentParameter);
    CurrentParameter.Type = 2;
    Parameters.AddItem(CurrentParameter);
    m_bPowerLockedStatus.Length = 0;
    nIndex = 0;
    while (nIndex < m_Helper.m_powers.Length)
    {
        Power = m_Helper.m_powers[nIndex];
        if (Power != None)
        {
            m_bPowerLockedStatus.AddItem(m_Helper.IsUnlocked(nIndex) ? 1 : 0);
            Parameters[0].nVar = nIndex;
            Power.GetStringFromStringRef(Power.DisplayName, Parameters[1].sVar);
            oPanel.InvokeMethodArgs("CreateRankList", Parameters);
        }
        nIndex++;
    }
}

final private function UpdatePowerList()
{
    local int nIndex;
    local ASParams CurrentParameter;
    local array<ASParams> Parameters;
    local SFXPower Power;
    local bool bUnlocked;
    local int nNewUnlockedPower;
    local String sDescription;
    local String sUnlockDetails;
    local String sUnlockDetailsSeperator;
    CurrentParameter.Type = 0;
    Parameters.AddItem(CurrentParameter);
    CurrentParameter.Type = 3;
    Parameters.AddItem(CurrentParameter);
    CurrentParameter.Type = 0;
    Parameters.AddItem(CurrentParameter);
    CurrentParameter.Type = 2;
    Parameters.AddItem(CurrentParameter);
    nNewUnlockedPower = -1;
    nIndex = 0;
    while (nIndex < m_Helper.m_powers.Length)
    {
        bUnlocked = m_Helper.IsUnlocked(nIndex);
        if (bUnlocked == true && m_bPowerLockedStatus[nIndex] != bUnlocked ? 1 : 0)
        {
            nNewUnlockedPower = nIndex;
            m_bPowerLockedStatus[nIndex] = bUnlocked ? 1 : 0;
        }
        Parameters[0].nVar = nIndex;
        Parameters[1].bVar = bUnlocked;
        Power = m_Helper.m_powers[nIndex];
        if (Power != None)
        {
            if (Power.Ranks.Length > 0)
            {
                Parameters[2].nVar = Power.Ranks[0].Icon;
            }
            else
            {
                Parameters[2].nVar = 0;
            }
        }
        else
        {
            Parameters[2].nVar = 0;
        }
        Power.GetStringFromStringRef(Power.Description, sDescription, true, Power.Rank - 1);
        if (!m_Helper.IsUnlocked(nIndex))
        {
            Power.GetUnlockDetails(sUnlockDetails);
            sUnlockDetailsSeperator = "<BR><BR>";
        }
        else
        {
            sUnlockDetails = "";
            sUnlockDetailsSeperator = "";
        }
        ClearCustomTokens();
        SetCustomToken(0, sUnlockDetails);
        SetCustomToken(1, sUnlockDetailsSeperator);
        SetCustomToken(2, sDescription);
        Parameters[3].sVar = Class'SFXGame'.GetSimpleString(m_srTalentDescriptionFormat, true);
        ClearCustomTokens();
        oPanel.InvokeMethodArgs("PopulateTalent", Parameters);
        UpdatePowerRanks(nIndex);
        nIndex++;
    }
    if (nNewUnlockedPower >= 0)
    {
        Parameters.Length = 0;
        CurrentParameter.Type = 0;
        Parameters.AddItem(CurrentParameter);
        Parameters[0].nVar = nNewUnlockedPower;
        oPanel.InvokeMethodArgs("ShowTalentEvolve", Parameters);
        PlayGuiSound('SquadScreen_NewPower');
    }
}

final private function UpdatePowerRanks(int nPowerIndex)
{
    local int nRankIndex;
    local ASParams CurrentParameter;
    local array<ASParams> Parameters;
    local SFXPower Power;
    local int nAvailableRank;
    local String sDescription;
    local String sUnlockDetails;
    local String sUnlockDetailsSeperator;
    if (m_Helper.m_Pawn == None || m_Helper.m_Pawn.PowerManager == None)
    {
        return;
    }
    if (nPowerIndex < 0 || nPowerIndex >= m_Helper.m_powers.Length)
    {
        return;
    }
    Power = m_Helper.m_powers[nPowerIndex];
    if (Power == None)
    {
        return;
    }
    nAvailableRank = m_Helper.GetAvailableRank(nPowerIndex);
    CurrentParameter.Type = 0;
    Parameters.AddItem(CurrentParameter);
    CurrentParameter.Type = 0;
    Parameters.AddItem(CurrentParameter);
    CurrentParameter.Type = 0;
    Parameters.AddItem(CurrentParameter);
    CurrentParameter.Type = 2;
    Parameters.AddItem(CurrentParameter);
    CurrentParameter.Type = 0;
    Parameters.AddItem(CurrentParameter);
    CurrentParameter.Type = 3;
    Parameters.AddItem(CurrentParameter);
    CurrentParameter.Type = 0;
    Parameters.AddItem(CurrentParameter);
    nRankIndex = 0;
    while (nRankIndex < Power.Ranks.Length)
    {
        Parameters[0].nVar = nPowerIndex;
        Parameters[1].nVar = nRankIndex;
        Parameters[2].nVar = Power.Ranks[nRankIndex].Icon;
        Power.GetDescription(nRankIndex, sDescription);
        if (!m_Helper.IsUnlocked(nPowerIndex))
        {
            Power.GetUnlockDetails(sUnlockDetails);
            sUnlockDetailsSeperator = "<BR>";
        }
        else
        {
            sUnlockDetails = "";
            sUnlockDetailsSeperator = "";
        }
        ClearCustomTokens();
        SetCustomToken(0, Power.RankCosts[nRankIndex]);
        SetCustomToken(1, sUnlockDetails);
        SetCustomToken(2, sUnlockDetailsSeperator);
        SetCustomToken(3, sDescription);
        Parameters[3].sVar = Class'SFXGame'.GetSimpleString(m_srRankDescriptionFormat, true);
        ClearCustomTokens();
        if (nRankIndex < m_Helper.m_UpdatedPowers[nPowerIndex].CurrentRank)
        {
            Parameters[4].nVar = 2;
        }
        else
        {
            if (nRankIndex < nAvailableRank)
            {
                Parameters[4].nVar = 1;
            }
            else
            {
                Parameters[4].nVar = 0;
            }
        }
        if (Power.Ranks[nRankIndex].Icon != m_Helper.m_Pawn.PowerManager.PassiveRankIcon)
        {
            Parameters[5].bVar = true;
        }
        else
        {
            Parameters[5].bVar = false;
        }
        Parameters[6].nVar = Power.Ranks[nRankIndex].Name;
        oPanel.InvokeMethodArgs("PopulateRank", Parameters);
        nRankIndex++;
    }
    nRankIndex = Power.Ranks.Length;
    while (nRankIndex < Power.MaxRank)
    {
        Parameters[0].nVar = nPowerIndex;
        Parameters[1].nVar = nRankIndex;
        Parameters[2].nVar = 0;
        Parameters[3].sVar = "";
        Parameters[4].nVar = 0;
        Parameters[5].bVar = false;
        Parameters[6].nVar = 0;
        Parameters[7].nVar = 0;
        oPanel.InvokeMethodArgs("PopulateRank", Parameters);
        nRankIndex++;
    }
}

final private function UpdatePoints()
{
    local ASParams CurrentParameter;
    local array<ASParams> Parameters;
    CurrentParameter.Type = 0;
    CurrentParameter.nVar = m_Helper.GetRemainingPoints();
    Parameters.AddItem(CurrentParameter);
    CurrentParameter.Type = 3;
    CurrentParameter.bVar = m_Helper.m_nPointsSpent > 0;
    Parameters.AddItem(CurrentParameter);
    CurrentParameter.Type = 3;
    CurrentParameter.bVar = m_Helper.CanMakePurchase();
    Parameters.AddItem(CurrentParameter);
    oPanel.InvokeMethodArgs("setAvailablePoints", Parameters);
}

final private function bool GetExperienceProgress(out int CurrentXP, out int TargetXP)
{
    local PlayerController PC;
    local SFXPawn_Player Player;
    local SFXGame GameInfo;
    PC = oPanel.oParentManager.GetPlayerController();
    Player = PC != None ? SFXPawn_Player(PC.Pawn) : None;
    if (Player != None)
    {
        GameInfo = SFXGame(Player.WorldInfo.Game);
        if (Player.CharacterLevel >= GameInfo.gameconfig.MaxPlayerLevel)
        {
            return false;
        }
        CurrentXP = Player.TotalXP;
        if (Class'BioLevelUpSystem'.ExperienceNeededForLevel(Player.CharacterLevel + 1, TargetXP) == false)
        {
            return false;
        }
    }
    return true;
}

final private function GetSquadMemberNamesAndIcons(out Texture NextMemberIcon, out Texture PrevMemberIcon)
{
    srCurrentMember = // unknown bytecode 4F ;
}

final private function CancelChanges()
{
    m_Helper.CancelChanges();
    UpdatePowerList();
    UpdatePoints();
    UpdateParagonRenegadeValues();
}

function EvolvePower(SFXPower OriginalPower, class<Object> EvolvedPowerClass)
{
    local int nIndex;
    local SFXPower Power;
    local ASParams CurrentParameter;
    local array<ASParams> Parameters;
    local int nEvolvedPowerIndex;
    nEvolvedPowerIndex = -1;
    nIndex = 0;
    while (nIndex < m_Helper.m_powers.Length)
    {
        Power = m_Helper.m_powers[nIndex];
        if (Power != None && Power == OriginalPower)
        {
            nEvolvedPowerIndex = nIndex;
        }
        nIndex++;
    }
    m_Helper.EvolvePower(OriginalPower, EvolvedPowerClass);
    CreatePowerList();
    UpdatePowerList();
    UpdatePoints();
    UpdateParagonRenegadeValues(true);
    if (nEvolvedPowerIndex >= 0)
    {
        CurrentParameter.Type = 0;
        Parameters.AddItem(CurrentParameter);
        Parameters[0].nVar = nEvolvedPowerIndex;
        oPanel.InvokeMethodArgs("ShowTalentEvolve", Parameters);
        PlayGuiSound('SquadScreen_TallentEvolved');
    }
}

function CancelEvolution(SFXPower OriginalPower)
{
    m_Helper.CancelEvolution(OriginalPower);
    UpdatePowerList();
    UpdatePoints();
    UpdateParagonRenegadeValues();
}

function AdditionalShutdownHandling()
{
    if (__OnCloseCallback__Delegate != None)
    {
        OnCloseCallback();
    }
}

defaultproperties
{
    m_nMaxRenegade=1900
    m_nMaxParagon=1957
    m_ParagonRenegadeBarValues(0)=0,2
    m_ParagonRenegadeBarValues(1)=0,2
    m_ParagonRenegadeBarValues(2)=0,2
    m_ParagonRenegadeBarValues(3)=0,2
    m_ParagonRenegadeBarValues(4)=0,2
    m_srXPFormat=System.Byte[]
    m_srSpendTalentPointsMessageXBox=System.Byte[]
    m_srParagonRenegadeMessageXBox=System.Byte[]
    m_srSpendTalentPointsMessagePC=System.Byte[]
    m_srParagonRenegadeMessagePC=System.Byte[]
    m_srSpendTalentPointsMessagePS3=System.Byte[]
    m_srParagonRenegadeMessagePS3=System.Byte[]
    m_srOk=System.Byte[]
    m_srShepardLevelClassFormat=System.Byte[]
    m_srTalentDescriptionFormat=System.Byte[]
    m_srRankDescriptionFormat=System.Byte[]
    m_srFaceCodeFormat=System.Byte[]
    nHandlerID=3
}
