﻿class BioSFHandler_Conversation extends BioSFHandler
    config(UI)
    transient
    native;

const MODE_None = 0;
const MODE_Investigate = 1;
const MODE_Charm = 2;
const MODE_Intimidate = 3;
const MODE_Illegal = 4;
enum BioConvWheelPositions
{
    REPLY_WHEEL_MIDDLE_RIGHT,
    REPLY_WHEEL_BOTTOM_RIGHT,
    REPLY_WHEEL_BOTTOM_LEFT,
    REPLY_WHEEL_MIDDLE_LEFT,
    REPLY_WHEEL_TOP_LEFT,
    REPLY_WHEEL_TOP_RIGHT,
};

const SkipNode = 1;
const SelectEntry = 2;
const ShowReplyWheel = 3;
const QueueEntry = 4;
const SelectRenegadeInterrupt = 5;
const SelectParagonInterrupt = 6;
const InitializationComplete = 7;
var transient bool m_bWasTriggered;

var transient bool m_bSFScreenInitialized;

var int m_nSlotsUsed[6];

var int m_aInvestigateLocations[6];

var int m_aReplyLocations[6];

var bool m_bDIsplayingRenegadeInterrupt;

var bool m_bDisplayingParagonInterrupt;

var bool m_bDisplayingWheel;

var bool m_bDisplayInvestigateSubMenu;

var float fLastRadius;

var Vector vInput;

var config int m_nInvestigateLocationMiddleRight;

var config int m_nInvestigateLocationBottomRight;

var config int m_nInvestigateLocationTopRight;

var config int m_nInvestigateLocationBottomLeft;

var config int m_nInvestigateLocationTopLeft;

var config int m_nInvestigateLocationMiddleLeft;

var config int m_nReplyLocationMiddleRight;

var config int m_nReplyLocationBottomRight;

var config int m_nReplyLocationTopRight;

var config int m_nReplyLocationBottomLeft;

var config int m_nReplyLocationTopLeft;

var config int m_nReplyLocationMiddleLeft;

// m_srTextReturn: StringRefProperty[44]
// m_srTextInvestigate: StringRefProperty[44]
const UNUSED_SLOT = -1;
const RETURN_OPTION = -2;
const INVESTIGATE_OPTION = -3;
function HighlightDefaultConvSegment();

final function SelectConversationSegment()
{
    oPanel.InvokeMethod("SelectConversationSegment");
}

final function SkipConversation()
{
    oPanel.InvokeMethod("SelectOrSkipConversation");
}

final function InterruptParagon()
{
    oPanel.InvokeMethod("InteruptParagon");
}

final function InterruptRenegade()
{
    oPanel.InvokeMethod("InteruptRenegade");
}

final function int GetNumReplies(BioConversation oConversation)
{
    return oConversation.m_lstCurrentReplyIndices.Length;
}

function OnPanelAdded()
{
    local ASParams stParam;
    local array<ASParams> lstParams;
    m_aReplyLocations[0] = m_nReplyLocationMiddleRight;
    m_aReplyLocations[1] = m_nReplyLocationBottomRight;
    m_aReplyLocations[2] = m_nReplyLocationBottomLeft;
    m_aReplyLocations[3] = m_nReplyLocationMiddleLeft;
    m_aReplyLocations[4] = m_nReplyLocationTopLeft;
    m_aReplyLocations[5] = m_nReplyLocationTopRight;
    m_aInvestigateLocations[0] = m_nInvestigateLocationMiddleRight;
    m_aInvestigateLocations[1] = m_nInvestigateLocationBottomRight;
    m_aInvestigateLocations[2] = m_nInvestigateLocationBottomLeft;
    m_aInvestigateLocations[3] = m_nInvestigateLocationMiddleLeft;
    m_aInvestigateLocations[4] = m_nInvestigateLocationTopLeft;
    m_aInvestigateLocations[5] = m_nInvestigateLocationTopRight;
    oPanel.bUseThumbstickAsDPad = false;
    m_bDIsplayingRenegadeInterrupt = false;
    m_bDisplayingParagonInterrupt = false;
    oPanel.bIgnoreGCOnPanelCleanup = true;
    oPanel.SetInputDisabled(true);
    stParam.Type = 0;
    stParam.nVar = ScreenLayout;
    lstParams.AddItem(stParam);
    oPanel.InvokeMethodArgs("SetPlatformLayout", lstParams);
}

function OnPanelRemoved()
{
    OnPanelRemoved();
    fLastRadius = 0;
    m_bDisplayInvestigateSubMenu = false;
    m_bDisplayingWheel = false;
}

function HandleEvent(byte nCommand, const out array<String> lstArguments)
{
    local BioWorldInfo oBioWorldInfo;
    oBioWorldInfo = BioWorldInfo(oWorldInfo);
    if (oBioWorldInfo.m_oCurrentConversation == None)
    {
        return;
    }
    switch (nCommand)
    {
        case 1:
            if (oBioWorldInfo.m_oCurrentConversation.SkipNode() == false)
            {
                PlayGuiSound('ConvError');
            }
            break;
        case 2:
            if (SelectConversationEntry(lstArguments[0]))
            {
                if (oBioWorldInfo.m_oCurrentConversation.SkipNode() == false)
                {
                    PlayGuiSound('ConvError');
                }
            }
            break;
        case 3:
            oBioWorldInfo.m_oCurrentConversation.m_bForceShowReplies = true;
            break;
        case 4:
            SelectConversationEntry(lstArguments[0]);
            break;
        case 6:
            PlayGuiSound('InterruptTriggeredParagon');
            oBioWorldInfo.m_oCurrentConversation.SelectInterruption();
            m_bWasTriggered = true;
            break;
        case 5:
            PlayGuiSound('InterruptTriggeredRenegade');
            oBioWorldInfo.m_oCurrentConversation.SelectInterruption();
            m_bWasTriggered = true;
            break;
        case 7:
            m_bSFScreenInitialized = true;
            break;
    }
}

function HandleInputEvent(BioGuiEvents Event, optional float fValue = 1)
{
    switch (Event)
    {
        case 2:
            vInput.X = fValue;
            break;
        case 3:
            vInput.Y = fValue;
            break;
        default:
            HandleInputEvent(Event, fValue);
    }
}

function UpdateInterruptVisuals(BioConversation oConversation)
{
    local bool bDisplayRenegade;
    local bool bDisplayParagon;
    bDisplayRenegade = false;
    bDisplayParagon = false;
    if (oConversation != None)
    {
        if (oConversation.NeedToDisplayInterrupt())
        {
            switch (oConversation.m_tInterruptInfo.eInterruptType);
            case 0;
            bDisplayRenegade = true;
            jump 111;
            case 1;
            bDisplayParagon = true;
            jump 111;
            default;
        }
    }
    if (bDisplayRenegade != m_bDIsplayingRenegadeInterrupt)
    {
        oPanel.InvokeMethod(bDisplayRenegade ? "showRenegadeInterrupt" : "hideRenegadeInterrupt");
        m_bDIsplayingRenegadeInterrupt = bDisplayRenegade;
        if (!m_bWasTriggered)
        {
            PlayGuiSound(bDisplayRenegade ? 'InterruptAppearRenegade' : 'InterruptDisappear');
        }
        m_bWasTriggered = false;
    }
    if (bDisplayParagon != m_bDisplayingParagonInterrupt)
    {
        oPanel.InvokeMethod(bDisplayParagon ? "showParagonInterrupt" : "hideParagonInterrupt");
        m_bDisplayingParagonInterrupt = bDisplayParagon;
        if (!m_bWasTriggered)
        {
            PlayGuiSound(bDisplayParagon ? 'InterruptAppearParagon' : 'InterruptDisappear');
        }
        m_bWasTriggered = false;
    }
    return;
}

function CalculateArrowPosition(out ASParams stParam, out array<ASParams> lstParams)
{
    local float fRadius;
    local Rotator R;
    if (Abs(vInput.X) > 0,25 || Abs(vInput.Y) > 0,25)
    {
        fRadius = Square(vInput.X) + Square(vInput.Y);
        if (fRadius >= fLastRadius || fRadius > 0,9)
        {
            R = vInput;
            stParam.nVar = R.Yaw * 0,005493164;
            if (stParam.nVar < 0)
            {
                stParam.nVar += 360;
            }
            lstParams.AddItem(stParam);
            oPanel.InvokeMethodArgs("setArrowPosition", lstParams);
        }
        fLastRadius = fRadius;
    }
    else
    {
        stParam.nVar = 360;
        lstParams.AddItem(stParam);
        oPanel.InvokeMethodArgs("setArrowPosition", lstParams);
        fLastRadius = 0;
    }
}

function Update(float fDeltaT)
{
    local BioWorldInfo oBioWorldInfo;
    local BioSubtitles oSubtitles;
    local BioConversation oConversation;
    local ASParams stParam;
    local array<ASParams> lstParams;
    local Color subtitleColor;
    local bool bAlert;
    local BioPlayerController PC;
    oBioWorldInfo = BioWorldInfo(oWorldInfo);
    oConversation = oBioWorldInfo.m_oCurrentConversation;
    if (oConversation != None && oConversation.NeedToDisplayReplies())
    {
        if (m_bDisplayingWheel == false)
        {
            UpdateConversationOptions(oConversation);
            m_bDisplayingWheel = true;
        }
        stParam.Type = 0;
        CalculateArrowPosition(stParam, lstParams);
        lstParams.Length = 0;
    }
    else
    {
        if (m_bDisplayingWheel)
        {
            m_bDisplayingWheel = false;
            oPanel.InvokeMethod("HideWheel");
        }
    }
    UpdateInterruptVisuals(oConversation);
    PC = BioPlayerController(GetPlayerController());
    oSubtitles = oBioWorldInfo.GetSubtitles();
    if (oSubtitles.HasSubtitle() && PC.GameModeManager2.ShouldShowSubtitles())
    {
        bAlert = oSubtitles.GetAlert();
        stParam.Type = 2;
        stParam.sVar = oSubtitles.GetSubtitle();
        lstParams.AddItem(stParam);
        stParam.Type = 0;
        if (oSubtitles.GetRenderMode() == 3)
        {
            stParam.nVar = 0;
        }
        else
        {
            if (oSubtitles.GetRenderMode() == 5)
            {
                stParam.nVar = 5;
            }
            else
            {
                if (oSubtitles.GetRenderMode() == 4)
                {
                    stParam.nVar = 2;
                }
                else
                {
                    if (PC != None && PC.GameModeManager2.IsActive(5) || bAlert)
                    {
                        stParam.nVar = 2;
                    }
                    else
                    {
                        stParam.nVar = 1;
                    }
                }
            }
        }
        lstParams.AddItem(stParam);
        subtitleColor = oSubtitles.GetFontColor();
        stParam.nVar = subtitleColor.R << 16 | subtitleColor.G << 8 | subtitleColor.B;
        lstParams.AddItem(stParam);
        stParam.nVar = subtitleColor.A / 2,55;
        lstParams.AddItem(stParam);
        stParam.Type = 3;
        stParam.bVar = bAlert;
        lstParams.AddItem(stParam);
        oPanel.InvokeMethodArgs("setSubtitle", lstParams);
    }
    else
    {
        oPanel.InvokeMethod("HideSubtitle");
    }
}

final function UpdateConversationOptions(BioConversation oConversation)
{
    local String sConversationOptions;
    local int nConversationOptionModes;
    local BioConvWheelPositions ePosition;
    local int I;
    local int nInvestigateReplyCount;
    local int nInvestigateReplyIteration;
    local int nReplyCategory;
    local bool bDisplayReply;
    local bool bFirstInvestigateFound;
    local ASParams stParam;
    local array<ASParams> lstParams;
    nInvestigateReplyCount = GetInvestigateReplyCount(oConversation);
    I = 0;
    while (I < 6)
    {
        m_nSlotsUsed[I] = -1;
        I++;
    }
    I = 0;
    if (I < GetNumReplies(oConversation))
    {
        srOption = // unknown bytecode 4F ;
    }
}

final function int MapGuiStyleToOptionMode(EConvGUIStyles eGUIStyle)
{
    local int nMode;
    nMode = 0;
    switch (eGUIStyle)
    {
        case 1:
            nMode = 2;
            break;
        case 2:
            nMode = 3;
            break;
        case 4:
            nMode = 4;
            break;
    }
    return nMode;
}

final function bool SelectConversationEntry(BioConvWheelPositions nWheelLocation)
{
    local BioWorldInfo oBioWorldInfo;
    local BioConversation oConversation;
    local bool bChangingMenu;
    local int nReplyIndex;
    oBioWorldInfo = BioWorldInfo(oWorldInfo);
    oConversation = oBioWorldInfo.GetConversation();
    bChangingMenu = false;
    nReplyIndex = m_nSlotsUsed[nWheelLocation];
    if (nReplyIndex >= 0)
    {
        if (oConversation.GetReplyGUIStyle(nReplyIndex) == 4)
        {
            PlayGuiSound('ConvError');
            return false;
        }
        m_bDisplayInvestigateSubMenu = false;
        PlayGuiSound('ConvSelect');
        if (oConversation.QueueReply(nReplyIndex) == false)
        {
            oBioWorldInfo.EndCurrentConversation();
        }
    }
    else
    {
        if (nReplyIndex == -2 && m_bDisplayInvestigateSubMenu)
        {
            m_bDisplayInvestigateSubMenu = false;
            bChangingMenu = true;
        }
        else
        {
            if (nReplyIndex == -3 && !m_bDisplayInvestigateSubMenu)
            {
                m_bDisplayInvestigateSubMenu = true;
                bChangingMenu = true;
            }
            else
            {
                PlayGuiSound('ConvError');
            }
        }
    }
    if (bChangingMenu)
    {
        UpdateConversationOptions(oConversation);
    }
    return !bChangingMenu;
}

final function BioConvWheelPositions GetInvestigateReplyLocation(int nInvestigateSlot)
{
    local int I;
    local int J;
    I = nInvestigateSlot;
    while (I < 6)
    {
        J = 0;
        while (J < 6)
        {
            if (m_aInvestigateLocations[J] == I)
            {
                if (m_nSlotsUsed[J] == -1)
                {
                    return J;
                }
                break;
            }
            J++;
        }
        I++;
    }
    return 6;
}

final function BioConvWheelPositions GetReplyLocation(int nReplyCategory)
{
    local int I;
    I = 0;
    while (I < 6)
    {
        if (nReplyCategory == m_aReplyLocations[I] || I == 6 - 1)
        {
            if (m_nSlotsUsed[I] == -1)
            {
                return I;
            }
            else
            {
                return 6;
            }
        }
        I++;
    }
    return 6;
}

final function int GetInvestigateReplyCount(BioConversation oConversation)
{
    local int nCount;
    local int I;
    if (oConversation == None)
    {
        return 0;
    }
    nCount = 0;
    I = 0;
    while (I < GetNumReplies(oConversation))
    {
        if (oConversation.GetReplyCategory(I) == 5)
        {
            nCount++;
        }
        I++;
    }
    return nCount;
}

final event SetVisible(bool bVal)
{
    if (!bVal)
    {
        m_bDisplayInvestigateSubMenu = false;
        m_bDisplayingWheel = false;
    }
}

final event bool IsVisible()
{
    return m_bDisplayingWheel || m_bDisplayInvestigateSubMenu;
}

function Enforce16x9(bool bVal);

defaultproperties
{
    m_srTextInvestigate=System.Byte[]
    m_srTextReturn=System.Byte[]
    m_nReplyLocationMiddleLeft=5
    m_nReplyLocationTopLeft=3
    m_nReplyLocationBottomLeft=4
    m_nReplyLocationTopRight=1
    m_nReplyLocationBottomRight=2
    m_nInvestigateLocationMiddleLeft=3
    m_nInvestigateLocationTopLeft=2
    m_nInvestigateLocationBottomLeft=1
    m_nInvestigateLocationTopRight=5
    m_nInvestigateLocationBottomRight=4
    nHandlerID=5
    bSetGameMode=false
}
