﻿class BioSFHandler_PCOptions extends BioSFHandler_Options
    transient
    native;

const Option_ResetPCBindings = 7;
const Option_ApplyPCBindings = 8;
const Option_ApplySettings = 9;
const PCOptions_MaxNumKeysPerAlias = 2;
var() int CaptureBindNumber;

var() int CaptureBindIndex;

var() transient array<NonBindableKeyDefinition> UnBindableKeys;

var() transient array<GuiBind> AliasMap;

// CannotBindConstantKey: StringRefProperty[44]
// CancelResetPCBindingsText: StringRefProperty[44]
// ConfirmResetPCBindingsText: StringRefProperty[44]
// ResetPCBindingsText: StringRefProperty[44]
// KeyHasBeenUnbound: StringRefProperty[44]
// KeyHasBeenBound: StringRefProperty[44]
var() array<KeyBind> KeyBinds;

struct native ModeAliasPair
{
    var() String Alias;

    var() EGameModes GameMode;

}

struct native SubordinateDesc extends ModeAliasPair
{
    var() bool bAllGameModes;

}

struct native GuiBind extends ModeAliasPair
{
// Name: StringRefProperty[44]
    var() bool bCategory;

    var() KeyBind Keys[2];

    var() String KeysLocName[2];

    var() array<SubordinateDesc> Subordinates;

}

struct native NonBindableKeyDefinition
{
    var() Name Name;

    var() String Command;

    var() bool Control;

    var() bool Shift;

    var() bool Alt;

    var() bool ModifierIndependent;

}

function OnPanelAdded()
{
    SetMouseShown(true);
    OnPanelAdded();
}

function HandleEvent(byte Command, const out array<String> lstArguments)
{
    local MassEffectGuiManager GuiMan;
    local BioPlayerController PC;
    local BioSFHandler_MessageBox MessageBox;
    local BioMessageBoxOptionalParams Params;
    if (bWaitingOnMsgBox)
    {
        return;
    }
    GuiMan = MassEffectGuiManager(oPanel.oParentManager);
    if (GuiMan == None)
    {
        return;
    }
    PC = BioPlayerController(GuiMan.Player);
    if (PC == None)
    {
        return;
    }
    switch (Command)
    {
        case 7:
            MessageBox = GuiMan.CreateMessageBox();
            MessageBox.SetInputDelegate(Callback_ResetPCBindings);
            Params.srAText = ConfirmResetPCBindingsText;
            Params.srBText = CancelResetPCBindingsText;
            Params.bNoFade = true;
            MessageBox.DisplayMessageBox(ResetPCBindingsText, Params);
            bWaitingOnMsgBox = true;
            break;
        case 2:
            Helper_CreateResetConfirmPopup(GuiMan, Callback_ResetPCToDefaults);
            m_lstOptsToReset.Length = 0;
            break;
        case 5:
        case 6:
            ApplySettings();
            ApplyPCBindings();
            PC.SavePCInputConfiguration(NewSettings);
            HandleEvent(Command, lstArguments);
            break;
        case 9:
            break;
        case 3:
            InitSystemSettingsCache();
            InitPCBindings(false);
            HandleEvent(Command, lstArguments);
            break;
        default:
            HandleEvent(Command, lstArguments);
            break;
    }
}

function onExIntResetToDefaults(String sPackedIDs)
{
    local MassEffectGuiManager GuiMan;
    GuiMan = MassEffectGuiManager(oPanel.oParentManager);
    if (GuiMan != None)
    {
        Helper_CreateResetConfirmPopup(GuiMan, Callback_ResetPCToDefaults);
    }
    Helper_ExtractPackedOptIDs(m_lstOptsToReset, sPackedIDs);
}

function Callback_ResetPCBindings(bool bAPressed, int Context)
{
    if (bAPressed)
    {
        Helper_DoPCBindingReset();
    }
    bWaitingOnMsgBox = false;
}

function Helper_DoPCBindingReset()
{
    local BioPlayerController PC;
    PC = BioPlayerController(GetPlayerController());
    if (PC != None)
    {
        InitPCBindings(true);
        RefreshGUIBindings();
    }
}

function Callback_ResetPCToDefaults(bool bAPressed, int Context)
{
    if (bAPressed)
    {
        ResetPCToDefaults(m_lstOptsToReset);
        if (m_lstOptsToReset.Length < 1)
        {
            Helper_DoPCBindingReset();
        }
    }
    Callback_ResetToDefaults(bAPressed, Context);
}

native function BuildControlBindingList();

function CaptureNewBinding(int BindIndex, int BindNumber)
{
    local PlayerController PC;
    PC = GetPlayerController();
    if (PC != None && PC.PlayerInput != None)
    {
        PC.PlayerInput.__OnReceivedNativeInputKey__Delegate = ProcessInput;
    }
    CaptureBindIndex = BindIndex;
    CaptureBindNumber = BindNumber;
    SetMouseShown(false);
}

native function bool ProcessInput(int ControllerId, Name Key, EInputEvent EventType, optional float AmountDepressed, optional bool bGamepad);

native final function RefreshGUIBindings();

native final function ApplyPCBindings();

native final function InitPCBindings(bool Defaults);

native final function InitSystemSettingsCache();

native final function ResetPCToDefaults(const out array<int> lstOptsToReset);

native final function ApplySettings();

native function SetOptionsOnGUI(SFXProfileSettings Settings);

native function SetOption(int Type, int Value);

defaultproperties
{
    KeyHasBeenBound=System.Byte[]
    KeyHasBeenUnbound=System.Byte[]
    ResetPCBindingsText=System.Byte[]
    ConfirmResetPCBindingsText=System.Byte[]
    CancelResetPCBindingsText=System.Byte[]
    CannotBindConstantKey=System.Byte[]
    AliasMap=System.Byte[]
    NewSettings=SFXProfileSettings'Settings0'
    ScreenLayout=System.Byte[]
    // child object Settings0 of type SFXProfileSettings
}
