﻿class BioSFHandler_Specialization extends BioSFHandler
    config(UI)
    transient;

const SPECIALIZATION_HANDLER_EVENT_CHOOSE_SPECIALIZATION = 1;
const SPECIALIZATION_HANDLER_EVENT_INITIALIZE = 2;
const SPECIALIZATION_HANDLER_EVENT_CANCEL_SPECIALIZATION = 3;
var transient SFXPower m_EvolvedPower2;

var transient SFXPower m_EvolvedPower1;

var transient SFXPower m_OriginalPower;

var transient float m_fRScrollValue;

// m_Description: StringRefProperty[44]
delegate EvolvePowerDelegate(SFXPower OriginalPower, class<Object> EvolvedPowerClass);

delegate CancelEvolutionDelegate(SFXPower OriginalPower);

function bool Initialize(BioPawn oPawn, Name nmPowerName)
{
    if (oPawn == None || oPawn.PowerManager == None)
    {
        return false;
    }
    m_OriginalPower = oPawn.PowerManager.GetPower(nmPowerName);
    if (m_OriginalPower == None)
    {
        return false;
    }
    if (m_OriginalPower.EvolvedPowerClass1 == None || m_OriginalPower.EvolvedPowerClass2 == None)
    {
        return false;
    }
    m_EvolvedPower1 = SFXPower(new(oPawn,m_OriginalPower.EvolvedPowerClass1));
    if (m_EvolvedPower1 == None)
    {
        return false;
    }
    m_EvolvedPower2 = SFXPower(new(oPawn,m_OriginalPower.EvolvedPowerClass2));
    if (m_EvolvedPower2 == None)
    {
        return false;
    }
    return true;
}

function OnPanelAdded()
{
    SetMouseShown(true);
}

function OnPanelRemoved()
{
    OnPanelRemoved();
    m_OriginalPower = None;
    m_EvolvedPower1 = None;
    m_EvolvedPower2 = None;
}

function HandleEvent(byte nCommand, const out array<String> Parameters)
{
    switch (nCommand)
    {
        case 2:
            UpdatePowerList();
            break;
        case 1:
            EvolvePower(Parameters[0]);
            break;
        case 3:
            CancelEvolution();
            break;
    }
}

function HandleInputEvent(BioGuiEvents Event, optional float fValue = 1)
{
    switch (Event)
    {
        case 5:
            if (Abs(fValue) > 0,25)
            {
                m_fRScrollValue = -fValue;
            }
            else
            {
                m_fRScrollValue = 0;
            }
            break;
        default:
            HandleInputEvent(Event, fValue);
    }
}

function Update(float fDeltaT)
{
    local ASParams stParam;
    local array<ASParams> lstParams;
    stParam.Type = 1;
    stParam.fVar = m_fRScrollValue;
    lstParams.AddItem(stParam);
    oPanel.InvokeMethodArgs("scrollDetailsAnalog", lstParams);
}

final private function UpdatePowerList()
{
    local ASParams CurrentParameter;
    local array<ASParams> Parameters;
    if (m_EvolvedPower1 == None || m_EvolvedPower2 == None)
    {
        return;
    }
    CurrentParameter.Type = 2;
    m_EvolvedPower1.GetStringFromStringRef(m_EvolvedPower1.DisplayName, CurrentParameter.sVar);
    Parameters.AddItem(CurrentParameter);
    CurrentParameter.Type = 2;
    m_EvolvedPower2.GetStringFromStringRef(m_EvolvedPower2.DisplayName, CurrentParameter.sVar);
    Parameters.AddItem(CurrentParameter);
    CurrentParameter.Type = 2;
    m_EvolvedPower1.GetStringFromStringRef(m_EvolvedPower1.TalentDescription, CurrentParameter.sVar, true, m_EvolvedPower1.Ranks.Length - 1);
    Parameters.AddItem(CurrentParameter);
    CurrentParameter.Type = 2;
    m_EvolvedPower2.GetStringFromStringRef(m_EvolvedPower2.TalentDescription, CurrentParameter.sVar, true, m_EvolvedPower2.Ranks.Length - 1);
    Parameters.AddItem(CurrentParameter);
    CurrentParameter.Type = 0;
    CurrentParameter.nVar = 0;
    Parameters.AddItem(CurrentParameter);
    CurrentParameter.Type = 0;
    CurrentParameter.nVar = 1;
    Parameters.AddItem(CurrentParameter);
    CurrentParameter.Type = 2;
    CurrentParameter.sVar = Class'SFXGame'.GetSimpleString(m_Description);
    Parameters.AddItem(CurrentParameter);
    CurrentParameter.Type = 0;
    CurrentParameter.nVar = ScreenLayout;
    Parameters.AddItem(CurrentParameter);
    oPanel.InvokeMethodArgs("populateSpecializations", Parameters);
}

final private function EvolvePower(int nIndex)
{
    if (m_OriginalPower == None)
    {
        return;
    }
    if (nIndex == 0)
    {
        EvolvePowerDelegate(m_OriginalPower, m_OriginalPower.EvolvedPowerClass1);
    }
    else
    {
        EvolvePowerDelegate(m_OriginalPower, m_OriginalPower.EvolvedPowerClass2);
    }
    MassEffectGuiManager(oPanel.oParentManager).HideSpecializationGui();
}

final private function CancelEvolution()
{
    CancelEvolutionDelegate(m_OriginalPower);
    MassEffectGuiManager(oPanel.oParentManager).HideSpecializationGui();
}

defaultproperties
{
    m_Description=System.Byte[]
    nHandlerID=23
}
