﻿class BioSkillGame_Handler extends BioSFHandler
    config(UI)
    transient
    native;

// m_preGameButtonXText: StringRefProperty[44]
// m_preGameButtonBText: StringRefProperty[44]
// m_preGameButtonAText: StringRefProperty[44]
// m_GameMessageDuringGame: StringRefProperty[44]
// m_GameMessageFailure: StringRefProperty[44]
// m_GameMessageSuccess: StringRefProperty[44]
var bool m_bSkillGameFailed;

var bool m_bDoNotUseSalvage;

var bool m_bSuccessfulEndGame;

var bool m_bButtonIconVisible;

var BioPawn m_oUsingPawn;

var Object m_oDependent;

var float m_fButtonDelayTime;

var float m_fButtonTimeout;

var int m_nButtonTimoutSequence;

var int m_nTotalButtonsToDisplay;

var int m_nButtonsSuccessfullyPressed;

var int m_nDisplayedButton;

var int m_nButtonInSequence;

var byte m_nSkillGameType;

enum ESkillGameButtons
{
    SKILL_GAME_BUTTON_A,
    SKILL_GAME_BUTTON_B,
    SKILL_GAME_BUTTON_X,
    SKILL_GAME_BUTTON_Y,
};

delegate OnSuccessCallback(int n_SalvageUsed);

delegate OnFailureCallback(int n_SalvageUsed);

delegate OnCancelCallback(int n_SalvageUsed);

event InitializeSkillGameData(BioPawn oUsingPawn, Object oDependent, byte nSkillGameType, int nTotalButtonsToDisplay, float fButtonTimeout, float fButtonDelayTime)
{
    m_nSkillGameType = nSkillGameType;
    m_nButtonsSuccessfullyPressed = 0;
    m_nButtonInSequence = 0;
    m_nButtonTimoutSequence = 0;
    m_bButtonIconVisible = false;
    m_nTotalButtonsToDisplay = nTotalButtonsToDisplay;
    m_fButtonTimeout = fButtonTimeout;
    m_fButtonDelayTime = fButtonDelayTime;
    m_oDependent = oDependent;
    m_oUsingPawn = oUsingPawn;
    if (m_nTotalButtonsToDisplay <= 0)
    {
        m_nTotalButtonsToDisplay = 1;
    }
    switch (m_nSkillGameType)
    {
        case 0:
            InitializeStringReferences(// unknown bytecode 4F ;
    }
}

event InitializeStringReferences()
{
    m_GameMessageSuccess = i_GameMessageSuccess;
    m_GameMessageFailure = i_GameMessageFailure;
    m_GameMessageDuringGame = i_GameMessageDuringGame;
    m_preGameButtonAText = i_preGameButtonAText;
    m_preGameButtonBText = i_preGameButtonBText;
    m_preGameButtonXText = i_preGameButtonXText;
}

native function StartSkillGameEx(BioPawn oUsingPawn, Object oPlaceable, byte nSkillGameType, int nTotalButtonsToDisplay, float fButtonTimeout, float fButtonDelayTime);

native function EndSkillGameEx(bool bCompleted);

event EndSkillGame(bool bCompleted, optional bool bNoSalvage = false, optional bool bGameCanceled = false)
{
    local BioWorldInfo oBWorldInfo;
    local BioSFHandler_MessageBox oMessageBox;
    local BioPlayerController PlayerController;
    local BioMessageBoxOptionalParams stParams;
    oBWorldInfo = BioWorldInfo(Class'Engine'.GetCurrentWorldInfo());
    PlayerController = oBWorldInfo.LocalPlayerController;
    m_nButtonTimoutSequence = 0;
    if (bCompleted == true)
    {
        if (bNoSalvage)
        {
            if (m_oDependent != None && BioArtPlaceable(m_oDependent) != None && BioArtPlaceable(m_oDependent).m_oBehavior != None)
            {
                if (oBWorldInfo != None && BioArtPlaceable(m_oDependent).m_oBehavior.m_bSkillGameXpAwarded == false)
                {
                    Class'BioConsequenceFunctions'.GrantScaledXP(oBWorldInfo, 5);
                    BioArtPlaceable(m_oDependent).m_oBehavior.m_bSkillGameXpAwarded = true;
                }
                BioArtPlaceable(m_oDependent).OnSkillGameSucceeded(m_oUsingPawn);
            }
        }
        else
        {
            if (m_oDependent != None && BioArtPlaceable(m_oDependent) != None && BioArtPlaceable(m_oDependent).m_oBehavior != None)
            {
                if (oBWorldInfo != None && BioArtPlaceable(m_oDependent).m_oBehavior.m_bSkillGameXpAwarded == false)
                {
                    Class'BioConsequenceFunctions'.GrantScaledXP(oBWorldInfo, 5);
                    BioArtPlaceable(m_oDependent).m_oBehavior.m_bSkillGameXpAwarded = true;
                }
                if (BioArtPlaceable(m_oDependent) != None)
                {
                    BioArtPlaceable(m_oDependent).OnSkillGameSucceeded(m_oUsingPawn);
                }
            }
            else
            {
                if (m_oDependent != None && BioArtPlaceable(m_oDependent) != None)
                {
                    BioArtPlaceable(m_oDependent).OnSkillGameFailed(m_oUsingPawn);
                }
                if (oBWorldInfo != None)
                {
                    oMessageBox = MassEffectGuiManager(PlayerController.GetScaleFormManager()).CreateMessageBox();
                    if (oMessageBox != None)
                    {
                        stParams.srAText = // unknown bytecode 4F ;
                    }
                }
            }
        }
    }
}

event HandleSkillGameButtonPress(int nButtonPressed)
{
    if (m_bButtonIconVisible == false)
    {
        return;
    }
    m_bButtonIconVisible = false;
    m_nButtonInSequence++;
    if (nButtonPressed != m_nDisplayedButton)
    {
        EndSkillGameEx(false);
        return;
    }
    else
    {
        m_nButtonsSuccessfullyPressed++;
        if (m_nButtonInSequence < m_nTotalButtonsToDisplay)
        {
            SkillGameTriggerNextButton(m_fButtonDelayTime);
        }
        else
        {
            EndSkillGameEx(true);
        }
    }
}

function HandleSkillGameButtonTimeout(Object Params)
{
    if (m_nButtonTimoutSequence < m_nButtonInSequence)
    {
        m_nButtonTimoutSequence++;
        return;
    }
    EndSkillGameEx(false);
}

event SkillGameTriggerNextButton(float fButtonDelayTime)
{
    local Object Params;
    local BioWorldInfo oBWorldInfo;
    Params = m_oDependent;
    oBWorldInfo = BioWorldInfo(Class'Engine'.GetCurrentWorldInfo());
    ClearAllButtons();
    oBWorldInfo.TimerList.AddTimer(HandleSkillGameDisplayButton, Params, fButtonDelayTime, "HandleSkillGameDisplayButton");
}

function HandleSkillGameDisplayButton(Object Params)
{
    local int nButtonType;
    local BioWorldInfo oBWorldInfo;
    m_bButtonIconVisible = true;
    oBWorldInfo = BioWorldInfo(Class'Engine'.GetCurrentWorldInfo());
    oBWorldInfo.TimerList.AddTimer(HandleSkillGameButtonTimeout, Params, m_fButtonTimeout, "HandleSkillGameButtonTimeout");
    nButtonType = RandRange(0, 4);
    if (nButtonType == 0)
    {
        m_nDisplayedButton = 0;
        SetButtonVisible(0);
    }
    else
    {
        if (nButtonType == 1)
        {
            m_nDisplayedButton = 1;
            SetButtonVisible(1);
        }
        else
        {
            if (nButtonType == 2)
            {
                m_nDisplayedButton = 2;
                SetButtonVisible(2);
            }
            else
            {
                m_nDisplayedButton = 3;
                SetButtonVisible(3);
            }
        }
    }
}

native function ClearAllButtons();

native function SetButtonVisible(int i_ButtonType);

native function UpdateSalvage(int i_salvage);

native function SetupGame(int i_initialSalvage);

function OnPanelAdded()
{
    oPanel.bUseThumbstickAsDPad = false;
}

defaultproperties
{
    m_nTotalButtonsToDisplay=3
    m_fButtonTimeout=1
    m_fButtonDelayTime=1
    nHandlerID=30
}
