﻿class MassEffectGuiManager extends BioSFManager
    config(UI)
    transient
    native;

struct native BioMessageBoxData
{
    var Name m_nmName;

    var int m_nPriority;

    var Object m_pCallbackObject;

    var Name m_nmCallbackFunction;

// m_srMessage: StringRefProperty[44]
    var BioMessageBoxOptionalParams m_stParams;

    var int m_nContext;

}

var array<BioMessageBoxData> m_aMessageBoxQueue;

var transient SFXAreaMapData m_CurrentAreaMap;

var() transient PlayerController Player;

var transient bool bGUIRequestedMouseVisiblityState;

var transient bool bMantleWidgetDisplayed;

var transient bool bClimbWidgetDisplayed;

var transient bool bCoverWidgetDisplayed;

var MEBrowserWheelSubPages m_eLastBrowserWheelSubPage;

var GFxMovie m_oCachedHintMovie;

var BioSFHandler_MessageBox m_oMessageBox;

var BioSFHandler_MessageBox m_oCachedHint;

var BioSFHandler_MessageBox m_oHintMsgBox;

var float m_fHintDisplayTimeRemaining;

var /* 00000008 */ WwiseAudioComponent m_wwiseComponent;

var BioSFResources m_oSFResources;

var BioSFHandler_BlackScreen m_oBlackScreenHandler;

var BioSFHandler_Specialization oSpecializationHandler;

var Name m_nmCurrentTutorial;

var BioSFHandler_Conversation m_oConversationHandler;

var BioWorldInfo oBioWorldInfo;

var bool m_bGameWasPaused;

var bool m_bSFHudCached;

var SFXSFHandler_SavingDisplay m_oSavingDisplayHandler;

var BioSFHandler_BrowserWheel m_oBWHandler;

var BioSFHandler_DesignerUI m_oDUIHandler;

var SFXSFHandler_PowerWheel m_oSFPowerWheelHandler;

var SFXSFHandler_Reticle m_oSFReticleHandler;

var SFXSFHandler_HUD m_oSFHudHandler;

enum EGuiHandlers
{
    GUI_HANDLER_NONE,
    GUI_HANDLER_INVENTORY,
    GUI_HANDLER_INGAMEGUI,
    GUI_HANDLER_CHARACTER_RECORD,
    GUI_HANDLER_LOOT,
    GUI_HANDLER_CONVERSATION,
    GUI_HANDLER_SHOP,
    GUI_HANDLER_GALAXYMAP,
    GUI_HANDLER_MAINMENU,
    GUI_HANDLER_NEW_CHARACTER,
    GUI_HANDLER_SELECT_CHARACTER,
    GUI_HANDLER_JOURNAL,
    GUI_HANDLER_HUD,
    GUI_HANDLER_PARTYSELECT,
    GUI_HANDLER_XMODS,
    GUI_HANDLER_SQUADCOMMAND,
    GUI_HANDLER_DATACODEX,
    GUI_HANDLER_SAVELOAD,
    GUI_HANDLER_ACHIEVEMENT,
    GUI_HANDLER_AREAMAP,
    GUI_HANDLER_SHAREDINGAMEGUI,
    GUI_HANDLER_MENUBROWSER,
    GUI_HANDLER_GAMEOVER,
    GUI_HANDLER_SPECIALIZATION,
    GUI_HANDLER_MESSAGEBOX,
    GUI_HANDLER_INTROTEXT,
    GUI_HANDLER_BLACKSCREEN,
    GUI_HANDLER_CREDITS,
    GUI_HANDLER_OPTIONS,
    GUI_HANDLER_ADDITONALCONTENT,
    GUI_HANDLER_SKILLGAME,
    GUI_HANDLER_SPLASH_SCREEN,
    GUI_HANDLER_REPLAYCHARACTERSELECT,
    GUI_HANDLER_CHOICEGUI,
    GUI_HANDLER_SNIPEROVERLAY,
};

enum BioTutorialPosition
{
    BTP_Top,
    BTP_Bottom,
    BTP_MessageBox,
};

enum SFXDisplayableSquadCommands
{
    SFX_DSC_READY,
    SFX_DSC_ATTACK,
    SFX_DSC_MOVETO,
    SFX_DSC_FOLLOW,
};

enum MainMenuEntries
{
    MM_ENTRY_DEFAULT,
    MM_ENTRY_NEWGAME,
    MM_ENTRY_NEWGAME_MALE,
    MM_ENTRY_NEWGAME_FEMALE,
    MM_ENTRY_LOADGAME,
    MM_ENTRY_EXTRAS,
    MM_ENTRY_ACHIEVEMENTS,
    MM_ENTRY_ALERT_DLC,
    MM_ENTRY_LIVECONTENT,
    MM_ENTRY_OPTIONS,
    MM_ENTRY_CREDITS,
    MM_ENTRY_IMPORT,
    MM_ENTRY_IMPORT_ME2,
    MM_ENTRY_MARKETPLACE,
    MM_ENTRY_EXITGAME,
    MM_ENTRY_EMPTY,
};

enum Cerberus3DState
{
    C3D_Default,
    C3D_RightPanel_Open,
    C3D_RightPanel_Closed,
    C3D_RightPanel_Opening,
    C3D_RightPanel_Closing,
};

native final function QueueNamedMessageBox(Name nmName, int nPriority, out BioMessageBoxOptionalParams stParams, delegate<InputCallback> pCallback, optional int nContext);

native final function RemoveNamedMessageBox(Name nmName);

native final function DisplayQueuedMessageBoxes();

native final function HideQueuedMessageBoxes();

native final function QueuedMessageBoxCallback(bool bAPressed, int nContext);

function Update(float fDeltaT)
{
    if (!m_oConversationHandler.IsVisible() && oBioWorldInfo != None && oBioWorldInfo.GetSubtitles().HasSubtitle())
    {
        if (oBioWorldInfo.m_oCurrentConversation != None)
        {
            ShowConversationGui(oBioWorldInfo.m_oCurrentConversation.IsAmbient());
        }
        else
        {
            ShowConversationGui(true);
        }
    }
}

function SetupBackground()
{
    oBioWorldInfo.m_UIWorld.TriggerEvent('SetBGShot', oBioWorldInfo);
}

function ClearAllNonModalPanels()
{
    local int I;
    local BioSFHandler_MessageBox oMessageBox;
    if (bSafeToRemovePanels)
    {
        I = m_lstPanels.Length - 1;
        while (I >= 0)
        {
            oMessageBox = BioSFHandler_MessageBox(m_lstPanels[I].GetDefaultHandler());
            if (oMessageBox == None || oMessageBox.m_bIsModal == false)
            {
                RemovePanel(m_lstPanels[I]);
            }
            I--;
        }
    }
    else
    {
        I = m_lstPanels.Length - 1;
        while (I >= 0)
        {
            oMessageBox = BioSFHandler_MessageBox(m_lstPanels[I].GetDefaultHandler());
            if (oMessageBox == None || oMessageBox.m_bIsModal == false)
            {
                m_lstPanels[I].SetToBeRemoved();
            }
            I--;
        }
    }
}

final event bool IsInSplashScreen()
{
    local BioSFPanel Panel;
    local BioSFHandler Handler;
    local BioSFHandler_Splash SplashHandler;
    Panel = GetPanelByTag('Splash');
    if (Panel != None)
    {
        foreach Panel.ScriptHandlers(Handler)
        {
            SplashHandler = BioSFHandler_Splash(Handler);
            if (SplashHandler != None && !SplashHandler.bPressedStart)
            {
                return true;
            }
        }
    }
    return false;
}

final event BioSFHandler_MainMenu ShowMainMenu(optional MainMenuEntries eEntry = 0)
{
    local BioSFPanel oPanel;
    local BioSFHandler_MainMenu oHandler;
    local BioPlayerController PC;
    local bool bWasCreated;
    bWasCreated = false;
    oPanel = GetPanelByTag('MainMenu');
    if (oPanel == None)
    {
        oPanel = CreatePanel('MainMenu', true);
        bWasCreated = true;
    }
    oHandler = BioSFHandler_MainMenu(oPanel.GetDefaultHandler());
    oHandler.nSelectedEntry = eEntry;
    if (bWasCreated)
    {
        PC = GetBioPlayerController();
        if (PC != None)
        {
            PC.GameModeManager2.EnableMode(7);
            if (eEntry != 0)
            {
                PC.CauseEvent('ShowDesk');
            }
        }
    }
    else
    {
        oHandler.SetEnabledAndVisible(true);
    }
    return oHandler;
}

native function HackReloadMainMenu();

final event ShowConversationGui(optional bool bIsSubtitle = false)
{
    m_oConversationHandler.oPanel.bInputDisabled = bIsSubtitle;
    m_oConversationHandler.oPanel.bHandleInputThisFrame = bIsSubtitle;
    m_oConversationHandler.SetVisible(true);
}

final event HideConversationGui()
{
    m_oConversationHandler.oPanel.bInputDisabled = true;
    m_oConversationHandler.oPanel.bHandleInputThisFrame = true;
    m_oConversationHandler.SetVisible(false);
}

final event RecreateHudPanel()
{
    local BioSFPanel oPanel;
    local BioSFPanel oReticlePanel;
    local BioSFPanel oWheelPanel;
    oPanel = GetPanelByTag('HUD');
    oReticlePanel = GetPanelByTag('Reticle');
    oWheelPanel = GetPanelByTag('PowerWheel');
    if (oPanel != None && oReticlePanel != None && oWheelPanel != None)
    {
        RemovePanel(oPanel);
        RemovePanel(oReticlePanel);
        RemovePanel(oWheelPanel);
        m_oSFHudHandler = None;
        m_oSFReticleHandler = None;
        m_oSFPowerWheelHandler = None;
        m_bSFHudCached = false;
    }
    oReticlePanel = CreatePanel('Reticle', true);
    oWheelPanel = CreatePanel('PowerWheel', true);
    oPanel = CreatePanel('HUD', true);
    m_oSFHudHandler = SFXSFHandler_HUD(oPanel.GetDefaultHandler());
    m_oSFReticleHandler = SFXSFHandler_Reticle(oReticlePanel.GetDefaultHandler());
    m_oSFPowerWheelHandler = SFXSFHandler_PowerWheel(oWheelPanel.GetDefaultHandler());
    GetDUIHandler();
}

final function ReleaseHudCache()
{
    m_oSFHudHandler = None;
    m_oSFReticleHandler = None;
    m_oSFPowerWheelHandler = None;
    m_bSFHudCached = false;
}

function OnAllowCameraRotation(bool i_bValue)
{
    if (m_oSFPowerWheelHandler != None)
    {
        m_oSFPowerWheelHandler.OnAllowCameraRotation(i_bValue);
    }
}

native final function CancelLeaderActive();

final event ShowBrowserWheel()
{
    local BioPlayerController PC;
    if (!oBioWorldInfo.m_bAllowBrowserWheel)
    {
        return;
    }
    if (oBioWorldInfo.m_playerSquad != None)
    {
        CancelLeaderActive();
    }
    PC = GetBioPlayerController();
    if (PC != None)
    {
        m_bGameWasPaused = PC.WorldInfo.bPlayersOnly;
        PC.WorldInfo.bPlayersOnly = true;
        PC.GameModeManager2.EnableMode(7);
    }
    ReturnToBrowserWheel();
    SetupBackground();
    PlaySound('InGameGuiEnter');
}

final event ReturnToBrowserWheel(optional BioSFPanel oCurrentPanel, optional bool bExitImmediately)
{
    local BioSFPanel oBrowserPanel;
    if (oCurrentPanel != None)
    {
        RemovePanel(oCurrentPanel, true);
    }
    oBrowserPanel = CreatePanel('MenuBrowser', true);
    oBrowserPanel.bFullScreen = true;
    if (bExitImmediately)
    {
        HideBrowserWheel();
    }
}

final event HideBrowserWheel(optional BioSFPanel oCurrentPanel)
{
    local BioPlayerController PC;
    local BioSFPanel oDUIPanel;
    if (oCurrentPanel != None)
    {
        RemovePanel(oCurrentPanel, true);
    }
    else
    {
        RemovePanel(GetPanelByTag('MenuBrowser'));
    }
    PC = GetBioPlayerController();
    if (PC != None)
    {
        PC.GameModeManager2.DisableMode(7);
        PC.WorldInfo.bPlayersOnly = m_bGameWasPaused;
    }
    if (m_oDUIHandler != None && m_oDUIHandler.nElementVisibleCount == 0)
    {
        oDUIPanel = GetPanelByTag('DesignerUI');
        if (oDUIPanel != None)
        {
            oDUIPanel.IsVisible = false;
        }
    }
    PlayGuiSound('InGameGuiExit');
}

function ShowAchievementGui()
{
    local BioSFPanel oPanel;
    local BioPlayerController PC;
    oPanel = CreatePanel('Achievement', true);
    oPanel.bFullScreen = true;
    PC = GetBioPlayerController();
    if (PC != None)
    {
        PC.GameModeManager2.EnableMode(7);
        m_bGameWasPaused = PC.WorldInfo.bPlayersOnly;
        PC.WorldInfo.bPlayersOnly = true;
    }
    SetupBackground();
}

function HideAchievementGui()
{
    local BioPlayerController PC;
    RemovePanel(GetPanelByTag('Achievement'));
    PC = GetBioPlayerController();
    if (PC != None)
    {
        PC.GameModeManager2.DisableMode(7);
        PC.WorldInfo.bPlayersOnly = m_bGameWasPaused;
    }
}

final event HideGuiByTag(Name sTag)
{
    local BioPlayerController PC;
    RemovePanel(GetPanelByTag(sTag));
    PC = GetBioPlayerController();
    if (PC != None)
    {
        PC.GameModeManager2.DisableMode(7);
        PC.WorldInfo.bPlayersOnly = m_bGameWasPaused;
    }
}

function bool ShowSpecializationGUI(BioPawn oPawn, Name nmPowerName)
{
    local BioSFPanel oPanel;
    local bool didSucceed;
    if (oPawn == None)
    {
        return false;
    }
    oPanel = CreatePanel('Specialization', true);
    if (oPanel != None)
    {
        if (oSpecializationHandler != None)
        {
            didSucceed = oSpecializationHandler.Initialize(oPawn, nmPowerName);
            oPanel.AddScriptHandler(oSpecializationHandler);
        }
        else
        {
            oSpecializationHandler = BioSFHandler_Specialization(oPanel.GetDefaultHandler());
            didSucceed = oSpecializationHandler.Initialize(oPawn, nmPowerName);
        }
        if (!didSucceed)
        {
            HideSpecializationGui();
        }
        jump 206;
    }
    return didSucceed;
    return;
}

final event HideSpecializationGui()
{
    RemovePanel(GetPanelByTag('Specialization'));
}

final function BioSFHandler_PartySelection ShowPartySelect(optional Name nmForcedHenchman)
{
    local BioSFPanel oPanel;
    local BioSFHandler_PartySelection oPSHandler;
    local BioPlayerController PC;
    PC = GetBioPlayerController();
    if (PC != None)
    {
        PC.GameModeManager2.EnableMode(7);
    }
    oPanel = CreatePanel('PartySelect', true);
    oPanel.bFullScreen = true;
    oPSHandler = BioSFHandler_PartySelection(oPanel.GetDefaultHandler());
    oPSHandler.nmForcedHenchman = nmForcedHenchman;
    SetupBackground();
    return oPSHandler;
}

final function BioSFHandler_Credits PlayCredits()
{
    local BioSFPanel oPanel;
    local BioSFHandler_Credits oCreditsHandler;
    local BioPlayerController PC;
    PC = GetBioPlayerController();
    if (PC != None)
    {
        PC.WorldInfo.bPlayersOnly = true;
        PC.GameModeManager2.EnableMode(7);
    }
    oPanel = CreatePanel('Credits', true);
    oPanel.bFullScreen = true;
    oCreditsHandler = BioSFHandler_Credits(oPanel.GetDefaultHandler());
    return oCreditsHandler;
}

native final function BioSFHandler_NewCharacter PlayCharacterCreation();

final event BioSFHandler_NewCharacter LaunchCharacterCreation(array<AnimSet> a_AnimSets, array<class<SFXCharacterClass> > a_CharacterClasses, SFXMorphFaceFrontEndDataSource a_MaleDataSource, SFXMorphFaceFrontEndDataSource a_FemaleDataSource)
{
    local BioSFPanel oPanel;
    local BioPlayerController PC;
    PC = GetBioPlayerController();
    if (PC != None)
    {
        oBioWorldInfo.bPlayersOnly = true;
        PC.GameModeManager2.EnableMode(7);
    }
    oPanel = CreatePanel('NewCharacter', true);
    oPanel.bFullScreen = true;
    oBioWorldInfo.m_UIWorld.m_oNCHandler = BioSFHandler_NewCharacter(oPanel.GetDefaultHandler());
    oBioWorldInfo.m_UIWorld.m_oNCHandler.lstClassAnimSetRefs = a_AnimSets;
    oBioWorldInfo.m_UIWorld.m_oNCHandler.lstCharacterClasses = a_CharacterClasses;
    oBioWorldInfo.m_UIWorld.m_oNCHandler.MaleDataSource = a_MaleDataSource;
    oBioWorldInfo.m_UIWorld.m_oNCHandler.FemaleDataSource = a_FemaleDataSource;
    oBioWorldInfo.m_UIWorld.m_oNCHandler.Setup3DModel();
    oBioWorldInfo.m_UIWorld.FlushPendingCommands();
    oBioWorldInfo.m_UIWorld.m_oNCHandler.bOpenedFromMainMenu = true;
    return oBioWorldInfo.m_UIWorld.m_oNCHandler;
}

final function HidePartySelect()
{
    local BioPlayerController PC;
    RemovePanel(GetPanelByTag('PartySelect'));
    PC = GetBioPlayerController();
    if (PC != None)
    {
        PC.GameModeManager2.DisableMode(7);
        PC.WorldInfo.bPlayersOnly = m_bGameWasPaused;
    }
}

final function ShowGameOverGui()
{
    local BioSFPanel oPanel;
    local BioSFHandler_GameOver oGameOver;
    local BioPlayerController PC;
    PC = GetBioPlayerController();
    if (PC != None)
    {
        m_bGameWasPaused = PC.WorldInfo.bPlayersOnly;
        PC.WorldInfo.bPlayersOnly = true;
        PC.GameModeManager2.EnableMode(7);
    }
    oPanel = CreatePanel('GameOver', true);
    oGameOver = BioSFHandler_GameOver(oPanel.GetDefaultHandler());
    if (srGameOverString != 0)
    {
        oGameOver.GameOverStringOverride = srGameOverString;
    }
}

final function HideGameOverGui()
{
    local BioPlayerController PC;
    RemovePanel(GetPanelByTag('GameOver'));
    PC = GetBioPlayerController();
    if (PC != None)
    {
        PC.GameModeManager2.DisableMode(7);
        PC.WorldInfo.bPlayersOnly = m_bGameWasPaused;
    }
}

final function ShowBlackScreen(bool bWithFade)
{
    HideHint();
    if (m_oBlackScreenHandler != None)
    {
        m_oBlackScreenHandler.Show(bWithFade);
    }
}

final function HideBlackScreen(bool bWithFade)
{
    if (m_oBlackScreenHandler != None)
    {
        m_oBlackScreenHandler.Hide(bWithFade);
        RestoreHint();
    }
}

final function bool BlackScreenFadeFinished()
{
    local bool bVal;
    bVal = true;
    if (m_oBlackScreenHandler != None)
    {
        bVal = !m_oBlackScreenHandler.IsFading();
    }
    return bVal;
}

final event BioSkillGame_Handler ShowSkillGame()
{
    local BioSFPanel oPanel;
    local BioPlayerController PC;
    oPanel = CreatePanel('SkillGame', true);
    PC = GetBioPlayerController();
    if (PC != None)
    {
        m_bGameWasPaused = PC.WorldInfo.bPlayersOnly;
        PC.WorldInfo.bPlayersOnly = true;
        PC.GameModeManager2.EnableMode(7);
    }
    return BioSkillGame_Handler(oPanel.GetDefaultHandler());
}

final event HideSkillGame()
{
    local BioPlayerController PC;
    local BioWorldInfo oBWorldInfo;
    local BioSFPanel oSkillGamePanel;
    oBWorldInfo = BioWorldInfo(Class'Engine'.GetCurrentWorldInfo());
    if (oBWorldInfo != None)
    {
        oBWorldInfo.TimerList.KillTimer("HandleSkillGameButtonTimeout");
        oBWorldInfo.TimerList.KillTimer("HandleSkillGameDisplayButton");
    }
    oSkillGamePanel = GetPanelByTag('SkillGame');
    if (oSkillGamePanel != None)
    {
        RemovePanel(oSkillGamePanel);
        PC = GetBioPlayerController();
        if (PC != None)
        {
            PC.GameModeManager2.DisableMode(7);
            PC.WorldInfo.bPlayersOnly = m_bGameWasPaused;
        }
    }
}

final event InterruptActiveSkillGame()
{
    local BioSkillGame_Base_Handler Handler;
    local BioSFPanel oSkillGamePanel;
    local array<Name> skillGameTags;
    local int I;
    skillGameTags[0] = 'SkillGameDecryption';
    skillGameTags[1] = 'SkillGameBypass';
    I = 0;
    while (I < skillGameTags.Length)
    {
        if (oSkillGamePanel == None)
        {
            oSkillGamePanel = GetPanelByTag(skillGameTags[I]);
        }
        I++;
    }
    if (oSkillGamePanel != None && oSkillGamePanel.ScriptHandlers.Length > 0)
    {
        Handler = BioSkillGame_Base_Handler(oSkillGamePanel.ScriptHandlers[0]);
        Handler.Cancel();
    }
}

final event BioSkillGame_Decryption_Handler ShowSkillGameDecryption()
{
    local BioSFPanel oPanel;
    local BioPlayerController PC;
    HideHint();
    oPanel = CreatePanel('SkillGameDecryption', true);
    PC = GetBioPlayerController();
    if (PC != None)
    {
        PC.GameModeManager2.EnableMode(7);
    }
    return BioSkillGame_Decryption_Handler(oPanel.GetDefaultHandler());
}

final event HideSkillGameDecryption()
{
    local BioPlayerController PC;
    local BioSFPanel oSkillGamePanel;
    oSkillGamePanel = GetPanelByTag('SkillGameDecryption');
    if (oSkillGamePanel != None)
    {
        RemovePanel(oSkillGamePanel);
        PC = GetBioPlayerController();
        if (PC != None)
        {
            PC.GameModeManager2.DisableMode(7);
        }
        RestoreHint();
    }
}

final event BioSkillGame_Bypass_Handler ShowSkillGameBypass()
{
    local BioSFPanel oPanel;
    local BioPlayerController PC;
    oPanel = CreatePanel('SkillGameBypass', true);
    PC = GetBioPlayerController();
    if (PC != None)
    {
        PC.GameModeManager2.EnableMode(7);
    }
    return BioSkillGame_Bypass_Handler(oPanel.GetDefaultHandler());
}

final event HideSkillGameBypass()
{
    local BioPlayerController PC;
    local BioSFPanel oSkillGamePanel;
    oSkillGamePanel = GetPanelByTag('SkillGameBypass');
    if (oSkillGamePanel != None)
    {
        RemovePanel(oSkillGamePanel);
        PC = GetBioPlayerController();
        if (PC != None)
        {
            PC.GameModeManager2.DisableMode(7);
        }
    }
}

function OnPlayerDeath()
{
    local BioSFPanel oDUIPanel;
    HideSkillGame();
    if (m_oDUIHandler != None)
    {
        m_oDUIHandler.ClearAll(true);
        m_oDUIHandler.ClearAll(false);
        oDUIPanel = GetPanelByTag('DesignerUI');
        if (oDUIPanel != None)
        {
            oDUIPanel.IsVisible = false;
            RemovePanel(oDUIPanel);
        }
    }
}

final event ShowCoverWidget();

final event HideCoverWidget();

final event ShowClimbMantleWidget(bool bMantle);

final event HideClimbMantleWidget();

native final function SetWheelState(MEBrowserWheelSubPages nPage, BioBrowserStates nState);

native final function SFXSFHandler_HUD GetCachedHudHandler();

native final function BioSFHandler_DesignerUI GetDUIHandler();

native final function BioSFHandler_BrowserWheel GetBrowserHandler();

event ShowPlatformSpecificHint(float fDisplayTime, optional SFXHintPosition ePosition = 2, optional bool bCached = false)
{
    local bool bIsPS3;
    local bool bIsPC;
    srText = srDefaultText;
    bIsPS3 = Class'WorldInfo'.IsConsoleBuild(2);
    bIsPC = !Class'WorldInfo'.IsConsoleBuild(0);
    if (bIsPC && srPCText != 0)
    {
        srText = srPCText;
    }
    else
    {
        if (bIsPS3 && srPS3Text != 0)
        {
            srText = srPS3Text;
        }
    }
    if (srText != 0)
    {
        ShowHint(srText, fDisplayTime, 0, ePosition, bCached);
    }
}

event ShowHint(float fDisplayTime, optional int nIcon = 0, optional SFXHintPosition ePosition = 2, optional bool bCached = false)
{
    CancelHint();
    if (m_oHintMsgBox == None)
    {
        m_oHintMsgBox = CreateHintBox();
    }
    if (m_oHintMsgBox == None)
    {
        return;
    }
    if (fDisplayTime > 0)
    {
        m_fHintDisplayTimeRemaining = fDisplayTime;
        m_oHintMsgBox.SetUpdateDelegate(UpdateHint);
    }
    m_oHintMsgBox.m_bCached = bCached;
    m_oHintMsgBox.DisplayHintMessage(srText, nIcon, ePosition);
}

event CancelHint()
{
    local BioPlayerController PC;
    if (m_oHintMsgBox == None)
    {
        return;
    }
    m_oHintMsgBox.HideMessageBox(false);
    m_oHintMsgBox = None;
    PC = GetBioPlayerController();
    if (PC != None)
    {
        PC.HintSystem.m_nCurrentlyDisplayedHint = -1;
    }
}

event HideHint()
{
    if (m_oHintMsgBox != None && m_oHintMsgBox.oPanel != None && m_oHintMsgBox.m_bCached)
    {
        if (m_oCachedHint != None)
        {
            m_oCachedHint.HideMessageBox(true, true);
        }
        m_oCachedHint = m_oHintMsgBox;
        m_oCachedHint.m_nHiddenDepth = 1;
        m_oCachedHint.oPanel.IsVisible = false;
        m_oHintMsgBox = None;
    }
    else
    {
        CancelHint();
        if (m_oCachedHint != None)
        {
            m_oCachedHint.m_nHiddenDepth++;
        }
    }
}

event RestoreHint()
{
    CancelHint();
    if (m_oCachedHint != None && m_oCachedHint.oPanel != None)
    {
        m_oCachedHint.m_nHiddenDepth--;
        if (m_oCachedHint.m_nHiddenDepth <= 0)
        {
            m_oHintMsgBox = m_oCachedHint;
            m_oCachedHint = None;
            m_oHintMsgBox.oPanel.IsVisible = true;
        }
    }
}

event SFXSFHandler_EANetworking CreateNetworkGUI(Name HandlerId)
{
    local SFXSFHandler_EANetworking oHandler;
    local BioSFPanel oPanel;
    oPanel = CreatePanel(HandlerId, true);
    if (oPanel != None)
    {
        oHandler = SFXSFHandler_EANetworking(oPanel.GetDefaultHandler());
    }
    return oHandler;
}

event DestroyNetworkGUI(Name HandlerId)
{
    RemovePanel(GetPanelByTag(HandlerId));
}

final function UpdateHint(float fDeltaT, BioSFHandler_MessageBox oMsgBox)
{
    m_fHintDisplayTimeRemaining -= fDeltaT;
    if (m_fHintDisplayTimeRemaining <= 0)
    {
        m_fHintDisplayTimeRemaining = 0;
        if (oMsgBox == m_oHintMsgBox)
        {
            CancelHint();
        }
    }
}

native function BioSFHandler_MessageBox CreateHintBox();

native function BioSFHandler_MessageBox CreateMessageBox();

native function BioSFHandler_ChoiceGUI CreateChoiceGUI(optional Name nmTag);

native function bool PlaySound(Name nmSound);

native function bool StopSound(Name nmSound);

native function CleanUp();

function GameSessionEnded()
{
    GameSessionEnded();
    m_oSFHudHandler = None;
    m_oSFPowerWheelHandler = None;
    m_oSFReticleHandler = None;
    m_oDUIHandler = None;
    m_oBWHandler = None;
    oBioWorldInfo = None;
    oSpecializationHandler = None;
    m_oConversationHandler = None;
    m_oMessageBox = None;
    m_oHintMsgBox = None;
    m_oCachedHint = None;
    m_CurrentAreaMap = None;
    m_oSavingDisplayHandler = None;
    m_oBlackScreenHandler = None;
    CleanUp();
}

final function BioSFHandler_Conversation GetConversationHandler()
{
    return m_oConversationHandler;
}

final function ShowSniperOverlay()
{
    if (GetPanelByTag('SniperOverlay') == None)
    {
        CreatePanel('SniperOverlay', true);
    }
}

final function HideSniperOverlay()
{
    RemovePanel(GetPanelByTag('SniperOverlay'));
}

function SetDistanceOnSniperOverlay(int nDistance)
{
    local BioSFHandler_SniperOverlay o_SniperOverlay;
    local int X;
    local BioSFPanel oPanel;
    oPanel = GetPanelByTag('SniperOverlay');
    if (oPanel != None)
    {
        X = 0;
        while (X < oPanel.ScriptHandlers.Length)
        {
            o_SniperOverlay = BioSFHandler_SniperOverlay(oPanel.ScriptHandlers[X]);
            if (o_SniperOverlay != None)
            {
                o_SniperOverlay.SetDistance(nDistance);
                break;
            }
            X++;
        }
    }
}

function ShowPersonalizationGUI()
{
    local BioSFPanel oPanel;
    oPanel = GetPanelByTag('Personalization');
    if (oPanel == None)
    {
        oPanel = CreatePanel('Personalization', true);
        oPanel.bFullScreen = true;
    }
}

function HidePersonalizationGUI()
{
    RemovePanel(GetPanelByTag('Personalization'));
}

final function BioSFHandler_MainMenu GetMainMenuHandler()
{
    local BioSFHandler_MainMenu oMainMenu;
    local int X;
    local BioSFPanel oPanel;
    oPanel = GetPanelByTag('MainMenu');
    if (oPanel != None)
    {
        X = 0;
        while (X < oPanel.ScriptHandlers.Length)
        {
            oMainMenu = BioSFHandler_MainMenu(oPanel.ScriptHandlers[X]);
            if (oMainMenu != None)
            {
                return oMainMenu;
            }
            X++;
        }
    }
    return None;
}

function BioSkillGame_Decryption_Handler GetDecryptionHandler()
{
    local BioSFPanel oPanel;
    local BioSkillGame_Decryption_Handler Handler;
    local int X;
    oPanel = GetPanelByTag('SkillGameDecryption');
    if (oPanel != None)
    {
        X = 0;
        while (X < oPanel.ScriptHandlers.Length)
        {
            Handler = BioSkillGame_Decryption_Handler(oPanel.ScriptHandlers[X]);
            if (Handler != None)
            {
                return Handler;
            }
            X++;
        }
    }
    return None;
}

final function SFXSFHandler_AreaMap GetAreaMapHandler()
{
    local BioSFPanel oPanel;
    oPanel = GetPanelByTag('AreaMap');
    if (oPanel != None)
    {
        return SFXSFHandler_AreaMap(oPanel.GetDefaultHandler());
    }
    return None;
}

native final function ForceSquadCommandFeedback(SFXDisplayableSquadCommands Command, int MemberIdx, bool bCommandSuccess);

final function GUICMDStringToCategories(out array<ESFPanelInvokeCategories> categories, String sArg1, String sArg2)
{
    if (sArg1 == "radar")
    {
        categories.AddItem(2);
    }
    else
    {
        if (sArg1 == "squad")
        {
            categories.AddItem(3);
        }
        else
        {
            if (sArg1 == "target")
            {
                categories.AddItem(4);
            }
            else
            {
                if (sArg1 == "weapon")
                {
                    categories.AddItem(8);
                }
                else
                {
                    if (sArg1 == "env")
                    {
                        categories.AddItem(6);
                    }
                    else
                    {
                        if (sArg1 == "reticle" || sArg1 == "reticule")
                        {
                            if (sArg2 == "combat")
                            {
                                categories.AddItem(11);
                            }
                            else
                            {
                                if (sArg2 == "selection")
                                {
                                    categories.AddItem(12);
                                }
                                else
                                {
                                    if (sArg2 == "los")
                                    {
                                        categories.AddItem(13);
                                    }
                                    else
                                    {
                                        categories.AddItem(11);
                                        categories.AddItem(12);
                                        categories.AddItem(13);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }
}

function GUICMD(out String sResultMessage, Name nmPanelTag, String sCmd, optional String sArg1, optional String sArg2)
{
    local BioSFPanel oPanel;
    local array<ESFPanelInvokeCategories> aCategories;
    local int I;
    sResultMessage = "GUICMD: Received command ( " $ nmPanelTag $ " , " $ sCmd $ " , " $ sArg1 $ " , " $ sArg2 $ " )";
    oPanel = GetPanelByTag(nmPanelTag);
    if (oPanel == None)
    {
        sResultMessage = "MEGUICMD: Cannot find active panel serving gui " $ nmPanelTag;
        return;
    }
    if (sCmd == "block")
    {
        if (nmPanelTag != 'HUD')
        {
            sResultMessage = "MEGUICMD: block commands are currently not supported for non-HUD guis";
            return;
        }
        GUICMDStringToCategories(aCategories, sArg1, sArg2);
        I = 0;
        while (I != aCategories.Length)
        {
            oPanel.SetInvokePrereq(aCategories[I], 0, false);
            ++I;
        }
    }
    else
    {
        if (sCmd == "unblock")
        {
            if (nmPanelTag != 'HUD')
            {
                sResultMessage = "MEGUICMD: block commands are currently not supported for non-HUD guis";
                return;
            }
            if (sArg1 == "")
            {
                oPanel.SetInvokePrereq(0, 0, true);
            }
            else
            {
                GUICMDStringToCategories(aCategories, sArg1, sArg2);
                I = 0;
                while (I != aCategories.Length)
                {
                    oPanel.SetInvokePrereq(aCategories[I], 0, true);
                    ++I;
                }
            }
        }
    }
}

function PlayerController GetPlayerController()
{
    if (Player == None)
    {
        Player = oBioWorldInfo.GetLocalPlayerController();
    }
    return Player;
}

function BioPlayerController GetBioPlayerController()
{
    return BioPlayerController(GetPlayerController());
}

function SetAreaMapData(SFXAreaMapData oMapData)
{
    m_CurrentAreaMap = oMapData;
    if (m_CurrentAreaMap != None)
    {
        m_CurrentAreaMap.ReCalculate();
    }
}

function SFXAreaMapData GetAreaMapData()
{
    return m_CurrentAreaMap;
}

function bool ShowAreaMap()
{
    local BioSFPanel oPanel;
    local BioPlayerController oController;
    local SFXAreaMapData oMapData;
    oMapData = GetAreaMapData();
    if (oMapData == None)
    {
        return false;
    }
    oPanel = CreatePanel('AreaMap', true);
    if (oPanel == None)
    {
        return false;
    }
    oController = GetBioPlayerController();
    m_bGameWasPaused = oController.WorldInfo.bPlayersOnly;
    oController.WorldInfo.bPlayersOnly = true;
    PlaySound('InGameAreaMapEnter');
    return true;
}

function bool HideAreaMap()
{
    local BioPlayerController oController;
    local BioSFPanel oPanel;
    oController = GetBioPlayerController();
    oPanel = GetPanelByTag('AreaMap');
    if (oPanel != None)
    {
        RemovePanel(oPanel);
        oController.GenerateTutorialEvent(8);
    }
    else
    {
        return false;
    }
    oController.WorldInfo.bPlayersOnly = m_bGameWasPaused;
    PlayGuiSound('InGameAreaMapExit');
    return true;
}

function SFXSFHandler_PRCShop CreatePCRStore()
{
    local SFXSFHandler_PRCShop oHandler;
    local BioSFPanel oPanel;
    oPanel = CreatePanel('PRCStore', true);
    if (oPanel != None)
    {
        oPanel.bFullScreen = true;
        oHandler = SFXSFHandler_PRCShop(oPanel.GetDefaultHandler());
    }
    return oHandler;
}

function DestroyPCRStore()
{
    RemovePanel(GetPanelByTag('PRCStore'));
}

native function bool SetMouseVisibility(Object pObject, bool bShow);

native function UpdateMouseVisibilityState();

defaultproperties
{
    m_oSFResources=BioSFResources'GUI_Sound_Mappings'
    m_eLastBrowserWheelSubPage=MBW_SP_Save
}
