﻿class SFXAI_Ambient extends SFXAI_Core
    hidecategories(Navigation)
    config(AI)
    placeable;

var int m_nWaypointBodyStance;

var Name m_WaypointAnimName;

var eWalkWaypointsTypes m_eWalkOrder;

var bool m_bCountingUp;

var bool m_bPaused;

var SFXNav_WayPoint m_oCurrentWaypoint;

var int m_nWaypointIndex;

function SetMovementSpeed();

function int FindNearestRoutePoint(Vector vLoc, Route oRoute)
{
    local int I;
    local int nNearest;
    local float vShortDist;
    local float vDist;
    local NavigationPoint oNavPoint;
    nNearest = 0;
    vShortDist = 1E+09;
    I = 0;
    while (I < oRoute.NavList.Length)
    {
        oNavPoint = oRoute.NavList[I].Nav;
        if (oNavPoint != None)
        {
            vDist = VSizeSq(oNavPoint.Location - vLoc);
            if (vDist < vShortDist)
            {
                vShortDist = vDist;
                nNearest = I;
            }
        }
        I++;
    }
    return nNearest;
}

function MoveOnScriptedRoute(Route oRoute, int nNearestWaypoint, bool bWalking, eWalkWaypointsTypes eWalkOrder, bool bSkipNodes)
{
    if (IsInState('WalkingWaypoints', true))
    {
        return;
    }
    ScriptedRoute = oRoute;
    m_nWaypointIndex = -1;
    if (ScriptedRoute == None || ScriptedRoute.NavList.Length <= 0)
    {
        return;
    }
    if (nNearestWaypoint < 0 && MyBP != None)
    {
        m_nWaypointIndex = FindNearestRoutePoint(MyBP.Location, ScriptedRoute);
    }
    else
    {
        m_nWaypointIndex = nNearestWaypoint;
    }
    MyBP.SetWalking(bWalking);
    m_eWalkOrder = eWalkOrder;
    m_bCountingUp = true;
    m_bCanSkipNodes = bSkipNodes;
    PushState('WalkingWaypoints');
}

state StandingIdle
{
Begin:
    if (m_nWaypointIndex != -1)
    {
        PushState('WalkingWaypoints');
    }
}

state WalkingWaypoints
{
    function SetNextWayPoint()
    {
        if (m_nWaypointIndex != -1)
        {
            if (m_nWaypointIndex < ScriptedRoute.NavList.Length)
            {
                m_oCurrentWaypoint = SFXNav_WayPoint(ScriptedRoute.NavList[m_nWaypointIndex].Nav);
            }
            else
            {
                m_nWaypointIndex = -1;
                m_oCurrentWaypoint = None;
            }
        }
        else
        {
            m_nWaypointIndex = -1;
            m_oCurrentWaypoint = None;
        }
    }

    function bool UpdateNextWaypoint()
    {
        local int PathLength;
        local int nCurrentPathNext;
        nCurrentPathNext = m_nWaypointIndex;
        PathLength = -1;
        PathLength = ScriptedRoute.NavList.Length;
        if (PathLength <= 0)
        {
            m_nWaypointIndex = -1;
            return false;
        }
        switch (m_eWalkOrder)
        {
            case 0:
                m_nWaypointIndex++;
                if (m_nWaypointIndex >= PathLength)
                {
                    return false;
                }
                break;
            case 1:
                m_nWaypointIndex++;
                if (m_nWaypointIndex == PathLength)
                {
                    m_nWaypointIndex = 0;
                }
                break;
            case 2:
                if (m_bCountingUp)
                {
                    m_nWaypointIndex++;
                }
                else
                {
                    m_nWaypointIndex--;
                }
                if (m_bCountingUp && m_nWaypointIndex == PathLength)
                {
                    m_bCountingUp = false;
                    m_nWaypointIndex -= 2;
                }
                if (!m_bCountingUp && m_nWaypointIndex == -1)
                {
                    return false;
                }
                break;
            case 3:
                if (m_bCountingUp)
                {
                    m_nWaypointIndex++;
                }
                else
                {
                    m_nWaypointIndex--;
                }
                if (m_bCountingUp && m_nWaypointIndex == PathLength)
                {
                    m_bCountingUp = false;
                    m_nWaypointIndex -= 2;
                }
                if (!m_bCountingUp && m_nWaypointIndex == -1)
                {
                    m_bCountingUp = true;
                    m_nWaypointIndex += 2;
                }
                break;
            case 4:
                while (nCurrentPathNext == m_nWaypointIndex && PathLength > 1)
                {
                    m_nWaypointIndex = Rand(PathLength);
                }
                break;
        }
        return true;
    }

    event PoppedState()
    {
        PoppedState();
        m_oCurrentWaypoint = None;
        bPreciseDestination = false;
        MyBP.ClearLatentAction(Class'BioSeqAct_WalkWaypointSet');
    }

    function bool MaintainVelocity()
    {
        local bool bMaintainVelocity;
        local int PathLength;
        PathLength = ScriptedRoute.NavList.Length;
        bMaintainVelocity = m_oCurrentWaypoint.m_bMaintainVelocity;
        switch (m_eWalkOrder)
        {
            case 0:
                if (m_nWaypointIndex >= PathLength - 1)
                {
                    bMaintainVelocity = false;
                }
                break;
            case 2:
                if (!m_bCountingUp && m_nWaypointIndex == 0)
                {
                    bMaintainVelocity = false;
                }
                break;
            default:
                break;
        }
        return bMaintainVelocity;
    }

    SetNextWayPoint();
    if (m_oCurrentWaypoint != None)
    {
        bPreciseDestination = m_oCurrentWaypoint.m_bPrecisionMovement;
        DesiredRotation = m_oCurrentWaypoint.Rotation;
        m_bClearVelocityAfterMove = !MaintainVelocity();
        MoveToGoal(m_oCurrentWaypoint, m_oCurrentWaypoint.m_fMoveOffset);
        if (bReachedMoveGoal)
        {
            if (m_oCurrentWaypoint.m_bPrecisionMovement)
            {
                PrecisionMoveTo(m_oCurrentWaypoint.Location, m_oCurrentWaypoint.Rotation);
            }
            m_bPaused = m_oCurrentWaypoint.OnPawnReachedWayPoint(MyBP);
            if (m_bPaused)
            {
                PushState('PausedOnWaypoint');
            }
            Sleep(m_oCurrentWaypoint.m_fDelay);
            m_bPaused = false;
            jump 258;
        }
    }
    if (!UpdateNextWaypoint() == false) jump 0;
    PopState();
    // invalid native function 205 ;
}

function ResumeWalking();

state PausedOnWaypoint
{
    function ResumeWalking()
    {
        PopState();
    }

}

function PlayWaypointAnim(Name AnimName, int eStance)
{
    m_WaypointAnimName = AnimName;
    m_nWaypointBodyStance = eStance;
    PushState('CustomAction_WaypointAnimation');
}

state CustomAction_WaypointAnimation
{
    function bool ExecuteCustomAction()
    {
        local BioCustomAction_WaypointAnimation oWPAnim;
        local bool bSetAnim;
        bSetAnim = false;
        if (VerifyCAHasBeenInstanced(11))
        {
            oWPAnim = BioCustomAction_WaypointAnimation(CustomActions[11]);
            if (oWPAnim != None)
            {
                bSetAnim = oWPAnim.SetWaypointAnim(m_WaypointAnimName, m_nWaypointBodyStance);
            }
        }
        if (bSetAnim)
        {
            DoCustomAction(11);
        }
        return bSetAnim;
    }

}

state Idle
{
Begin:
    if (m_nmCurrentBehavior != 'None')
    {
        PushState(m_nmCurrentBehavior);
    }
}

defaultproperties
{
    m_nWaypointIndex=-1
    m_nmCurrentBehavior=StandingIdle
    BehaviourList(0)=StandingIdle
    AtCover_WeaponRange=Goal_AtCover'AtCov_EvalWeaponRange'
    AtCover_Defensive=Goal_AtCover'AtCov_EvalDefensive'
    AtCover_Aggressive=Goal_AtCover'AtCov_EvalAggressive'
    AtCover_NearMoveGoal=Goal_AtCover'AtCov_EvalNearGoal'
    AtCover_AIGoHere=Goal_AtGoHereCover'AtCov_EvalGoHere'
    // child object AtCov_EvalAggressive of type Goal_AtCover
    // child object AtCov_EvalDefensive of type Goal_AtCover
    // child object AtCov_EvalNearGoal of type Goal_AtCover
    // child object AtCov_EvalWeaponRange of type Goal_AtCover
    // child object AtCov_EvalGoHere of type Goal_AtGoHereCover
}
