﻿class SFXAI_Core extends SFXAI_NativeBase
    hidecategories(Navigation)
    config(AI)
    placeable;

var transient float m_fLastApproachCheckTime;

var transient float m_fLastApproachDistance;

var transient Actor m_oLastApproachTarget;

var transient float m_fLastPushTime;

var transient Vector m_vPushDirection;

var transient BioPawn m_oPushingPawn;

var transient float m_fLastBumpTime;

var transient Pawn m_oLastBumped;

var transient Actor m_oGoalActor;

var const int m_nTargetTicketCost;

var transient int m_nCurrentAttackID;

var transient SFXWeapon m_oNextWeapon;

var transient float m_fRunThreshold;

var /* 00000008 */ Goal_AtGoHereCover AtCover_AIGoHere;

var /* 00000008 */ Goal_AtCover AtCover_NearMoveGoal;

var /* 00000008 */ Goal_AtCover AtCover_Aggressive;

var /* 00000008 */ Goal_AtCover AtCover_Defensive;

var /* 00000008 */ Goal_AtCover AtCover_WeaponRange;

var transient int m_nMoveCompletionReason;

var transient int m_nWeaponCompletionReason;

var transient int m_nPowerCompletionReason;

var transient Actor m_ActorToFollow;

var transient Actor m_RequestedActorToFollow;

var transient float m_fReloadThreshold;

var transient float m_fLastCoverEnterTime;

var transient int m_nMoveAttemptCounter;

var transient int m_nReservationID;

var array<Name> BehaviourList;

var transient float m_fSpecialMoveTimeDelay;

var transient Vector m_PowerTargetLocation;

var transient Actor m_PowerTargetActor;

var transient int m_nCancelReasons;

var transient float m_fFiringArcAngle;

var transient Name m_nmPowerToUse;

var transient ECoverAction m_eBestTargetCoverAction;

var transient AttackResult m_AttackResult;

var transient EAICustomAction DamageReaction;

var transient bool bDying;

var transient bool m_bFailedTicket;

var transient bool m_bFirstIdle;

var transient bool bActorReachable;

var transient bool m_bUsePeriodicCoverCheck;

var transient bool m_bCustomActionFailed;

var transient bool m_bPendingWeaponSwitch;

var transient bool m_bPlayedCustomActivationAnim;

var transient bool m_bAllowWeaponSwitch;

var transient bool m_bDelayWeaponUse;

var transient bool m_bAttacking;

var transient bool m_bAvoidFireFromPlayerOnly;

var transient bool m_bAvoidDangerLinks;

var transient bool m_bUsePartialPaths;

var transient bool m_bStayLeanedOut;

var transient bool m_bIgnorePowerSuppression;

var transient bool m_bAllowedToLeavePlaypen;

var transient bool bDoingCustomAction;

var transient bool bAbortedCustomAction;

var transient bool bFinishedCustomAction;

var transient bool m_bUsePowerReservations;

var transient bool m_bClearVelocityAfterMove;

var transient bool m_bInvalidatedCover;

var transient bool m_bWaitBeforeNextAttack;

var transient bool m_bCancelAction;

var transient bool m_bPowerProjectileReleased;

var transient bool m_bSpecialMoveAllowed;

var transient Name m_nmCurrentBehavior;

var const bool m_bAIGoHereCanUseCover;

var config bool bUseTicketing;

const CANCEL_REASON_SHOT = 1;
const CANCEL_REASON_NEW_ORDER = 2;
const CANCEL_REASON_ORDER_CANCELLED = 4;
const CANCEL_REASON_TARGET_CHANGED = 8;
const CANCEL_REASON_IN_PLAYER_LINE_OF_FIRE = 16;
enum AttackResult
{
    ATTACK_SUCCESS,
    ATTACK_FAIL,
    ATTACK_FAIL_RELOADING,
    ATTACK_FAIL_NO_LOS,
};

enum EAICompletionReasons
{
    AI_Cancelled,
    AI_Success,
    AI_Failed,
    AI_LOS,
    AI_Cooldown,
    AI_Disabled,
};

enum EAITicketType
{
    AI_NoTicket,
    AI_TargetTicket,
    AI_AttackTicket,
};

enum EAICustomActionTypes
{
    AI_DefaultCustomAction,
    AI_DamageCustomAction,
    AI_DeathCustomAction,
};

delegate UsePowerDelegate(int nReason);

delegate FireWeaponDelegate(int nReason);

delegate MoveToDelegate(int nReason);

delegate bool TryInvalidateTargetRange(float fRangeToTarget);

delegate bool TryInvalidateTargetFlank();

function NotifyPlaypenChanged();

function UsePowerOnTarget(Name nmPowerToUse, Actor oTarget, optional delegate<UsePowerDelegate> PowerDelegate, optional bool bIgnoreSuppression)
{
    if (PowerDelegate != None)
    {
        PowerDelegate.UsePowerDelegate(2);
    }
}

function FireWeaponAtTarget(Actor oTarget, bool bCheckLOS, optional delegate<FireWeaponDelegate> FireDelegate)
{
    if (FireDelegate != None)
    {
        FireDelegate.FireWeaponDelegate(2);
    }
}

function MoveToGoalExternal(Actor NewMoveGoal, optional float NewMoveOffset, optional delegate<MoveToDelegate> MoveDelegate)
{
    if (MoveDelegate != None)
    {
        MoveDelegate.MoveToDelegate(2);
    }
}

function bool StartFollowingActor(Actor ActorToFollow)
{
    return false;
}

function StopFollowingActor()
{
    if (IsInState('FollowingActor', true))
    {
        CancelAction(4);
    }
    m_RequestedActorToFollow = None;
}

function AssignGoHereDelegates(delegate<TryInvalidateTargetRange> RangeDel, delegate<TryInvalidateTargetFlank> FlankDel)
{
    __TryInvalidateTargetRange__Delegate = RangeDel;
    __TryInvalidateTargetFlank__Delegate = FlankDel;
}

function ClearGoHereDelegates()
{
    __TryInvalidateTargetRange__Delegate = None;
    __TryInvalidateTargetFlank__Delegate = None;
}

static event GetBehaviourList(out array<Name> Behaviours)
{
    local Name nmBehaviour;
    Behaviours.Length = 0;
    foreach Default.BehaviourList(nmBehaviour)
    {
        Behaviours.AddItem(nmBehaviour);
    }
}

function bool SetDefaultBehaviour(Name NewBehaviour)
{
    local int nBehaviourIndex;
    nBehaviourIndex = BehaviourList.Find(NewBehaviour);
    if (nBehaviourIndex != -1 && NewBehaviour != m_nmCurrentBehavior)
    {
        m_nmCurrentBehavior = NewBehaviour;
        return true;
    }
    return false;
}

function bool ChangeBehaviour(Name NewBehaviour)
{
    if (SetDefaultBehaviour(NewBehaviour))
    {
        if (CanSwitchBehaviours())
        {
            GotoState('Idle');
        }
        return true;
    }
    return false;
}

function PostBeginPlay()
{
    PostBeginPlay();
    if (!bDeleteMe && WorldInfo.NetMode != 3 && PlayerReplicationInfo == None)
    {
        InitPlayerReplicationInfo();
    }
}

function Initialize()
{
    if (MyBP != None)
    {
        if (MyBP.m_nmAIBehaviour != 'None')
        {
            SetDefaultBehaviour(MyBP.m_nmAIBehaviour);
        }
        else
        {
            SetDefaultBehaviour(MyBP.PawnType.m_nmDefaultAIBehaviour);
        }
        if (MyBP.m_oBehavior != None && MyBP.m_oBehavior.Squad != None)
        {
            if (!MyBP.m_oBehavior.Squad.bSquadEnabled)
            {
                InitialState = 'Disabled';
            }
        }
        if (MyBP.m_bPlayCustomActivationAnim && !m_bPlayedCustomActivationAnim)
        {
            MyBP.SetHidden(true);
        }
    }
}

event SpawnedByKismet()
{
    if (MyBP.m_bPlayCustomActivationAnim && !m_bPlayedCustomActivationAnim)
    {
        MyBP.SetHidden(true);
    }
}

function bool SelectTarget()
{
    local Actor OldTarget;
    local int idx;
    local int BestIdx;
    local float Rating;
    local float BestRating;
    local float Distance;
    local Pawn EnemyPawn;
    local CoverInfo EnemyCover;
    local bool bResult;
    local Pawn ForcedEnemy;
    if (HasValidTarget() && TimeSince(TargetAcquisitionTime) < 0,1)
    {
        return true;
    }
    OldTarget = FireTarget;
    ForcedEnemy = Pawn(ForcedTarget);
    if (ForcedEnemy != None && ForcedEnemy.IsValidTargetFor(self) && ForcedEnemy.IsInvisible() == false)
    {
        Enemy = ForcedEnemy;
    }
    else
    {
        BestIdx = -1;
        BestRating = -1;
        idx = 0;
        while (idx < EnemyList.Length)
        {
            EnemyPawn = EnemyList[idx].Pawn;
            if (EnemyPawn == None || EnemyPawn.IsValidTargetFor(self) == false)
            {
            }
            else
            {
                if (EnemyPawn.IsInState('Downed'))
                {
                }
                else
                {
                    if (EnemyPawn.IsInvisible())
                    {
                    }
                    else
                    {
                        if (IgnoredTargets.Find(EnemyPawn) >= 0)
                        {
                        }
                        else
                        {
                            if (BioPawn(EnemyPawn) != None && BioPawn(EnemyPawn).Squad != None && IgnoredSquads.Find(BioPawn(EnemyPawn).Squad) >= 0)
                            {
                            }
                            else
                            {
                                Distance = VSize(GetEnemyLocationByIndex(idx) - Pawn.Location);
                                if (Distance < EnemyDistance_Melee)
                                {
                                    Rating = GetRangeValueByPct(vect2d(8, 4), Distance / EnemyDistance_Melee);
                                }
                                else
                                {
                                    if (Distance < EnemyDistance_Short)
                                    {
                                        Rating = GetRangeValueByPct(vect2d(4, 2), Distance - EnemyDistance_Melee / EnemyDistance_Short - EnemyDistance_Melee);
                                    }
                                    else
                                    {
                                        if (Distance < EnemyDistance_Medium)
                                        {
                                            Rating = GetRangeValueByPct(vect2d(2, 1), Distance - EnemyDistance_Short / EnemyDistance_Medium - EnemyDistance_Short);
                                        }
                                        else
                                        {
                                            if (Distance < EnemyDistance_Long)
                                            {
                                                Rating = GetRangeValueByPct(vect2d(1, 0,5), Distance - EnemyDistance_Long / EnemyDistance_Long - EnemyDistance_Medium);
                                            }
                                            else
                                            {
                                                Rating = 0,35;
                                            }
                                        }
                                    }
                                }
                                if (EnemyPawn == OldTarget)
                                {
                                    if (m_bFailedTicket)
                                    {
                                        Rating *= 0,6;
                                        m_bFailedTicket = false;
                                    }
                                    else
                                    {
                                        Rating *= 1,5;
                                    }
                                }
                                if (EnemyPawn == PreferredTarget)
                                {
                                    Rating *= 1,9;
                                }
                                if (EnemyPawn.IsHumanControlled())
                                {
                                    Rating *= 1,5;
                                }
                                if (Pawn.LastHitBy == EnemyPawn.Controller && Enemy == None || Enemy.Controller == None || Enemy.Controller.Enemy != self)
                                {
                                    Rating *= 1,25;
                                }
                                if (Pawn.LastHitBy != None)
                                {
                                    jump 981;
                                }
                                if (Distance > EnemyDistance_Short)
                                {
                                    if (IsEnemyVisibleByIndex(idx) == false)
                                    {
                                        if (TimeSinceEnemyVisible(idx) < 5)
                                        {
                                            Rating *= 0,5;
                                        }
                                        else
                                        {
                                            Rating *= 0,1;
                                        }
                                    }
                                }
                                if (GetPawnCover(EnemyPawn, EnemyCover) == false)
                                {
                                    Rating *= 1,5;
                                }
                                if (Distance > EnemyDistance_Short)
                                {
                                    AdjustRatingByTickets(Rating, idx);
                                }
                                AdjustEnemyRating(EnemyPawn, Rating);
                                if (Rating >= 0 && BestRating < 0 || Rating > BestRating)
                                {
                                    BestIdx = idx;
                                    BestRating = Rating;
                                }
                            }
                        }
                    }
                }
            }
            idx++;
        }
        if (BestIdx >= 0 && BestIdx < EnemyList.Length)
        {
            Enemy = EnemyList[BestIdx].Pawn;
        }
        else
        {
            Enemy = None;
        }
    }
    bResult = HasValidEnemy();
    if (bResult)
    {
        FireTarget = Enemy;
    }
    else
    {
        FireTarget = None;
    }
    if (OldTarget != FireTarget)
    {
        TargetAcquisitionTime = WorldInfo.TimeSeconds;
        ReleaseTicket(OldTarget, 2, true);
        ReleaseTicket(OldTarget, 1);
        AcquireTicket(FireTarget, 1);
        TriggerAttackVocalization();
        OnTargetChanged();
    }
    ShotTarget = Pawn(FireTarget);
    return bResult;
}

function AdjustRatingByTickets(out float out_Rating, int EnemyIdx)
{
    local float fTicketAdjustmentFactor;
    local BioPawn oEnemyPawn;
    if (EnemyIdx < 0 || EnemyIdx >= EnemyList.Length)
    {
        return;
    }
    oEnemyPawn = BioPawn(EnemyList[EnemyIdx].Pawn);
    if (oEnemyPawn != None)
    {
        fTicketAdjustmentFactor = oEnemyPawn.GetMaxTargetTickets() - oEnemyPawn.m_nTargetTickets;
        if (oEnemyPawn == FireTarget)
        {
            fTicketAdjustmentFactor += m_nTargetTicketCost;
        }
        fTicketAdjustmentFactor = oEnemyPawn.GetMaxTargetTickets() > 0 ? fTicketAdjustmentFactor / oEnemyPawn.GetMaxTargetTickets() : 0;
        if (fTicketAdjustmentFactor <= 0)
        {
            fTicketAdjustmentFactor = 0,01;
        }
        out_Rating *= fTicketAdjustmentFactor;
    }
}

function AdjustEnemyRating(Pawn EnemyPawn, out float fRating);

function OnTargetChanged()
{
    if (!IsReturningToPlaypen())
    {
        CancelAction(8);
    }
}

function TriggerAttackVocalization()
{
    if (BioPawn(FireTarget) != None && MyBP != None)
    {
        SFXGame(WorldInfo.Game).TriggerVocalizationEvent(13, MyBP, BioPawn(FireTarget));
    }
}

state Idle
{
    function BeginState(Name PrevState)
    {
        BeginState(PrevState);
        SetTimer(10, true, 'PlayAmbientVoc');
    }

    function PushedState()
    {
        PushedState();
        SetTimer(10, true, 'PlayAmbientVoc');
    }

    function EndState(Name NextState)
    {
        EndState(NextState);
        ClearTimer('PlayAmbientVoc');
    }

    function PoppedState()
    {
        PoppedState();
        ClearTimer('PlayAmbientVoc');
    }

    event PausedState()
    {
        PausedState();
        ClearTimer('PlayAmbientVoc');
    }

    event ContinuedState()
    {
        ContinuedState();
        SetTimer(10, true, 'PlayAmbientVoc');
    }

    function NotifyNewEnemy(Pawn NewEnemy)
    {
        NotifyNewEnemy(NewEnemy);
        if (MyBP != None && MyBP.IsDead() == false)
        {
            if (!MyBP.m_bPlayCustomActivationAnim || m_bPlayedCustomActivationAnim)
            {
                if (m_RequestedActorToFollow != None)
                {
                    m_ActorToFollow = m_RequestedActorToFollow;
                    PushState('FollowingActor');
                }
                if (IsActorInPlaypen(MyBP) == false)
                {
                    GotoState('ReturningToPlaypen');
                }
                else
                {
                    if (m_nmCurrentBehavior != 'None')
                    {
                        PushState(m_nmCurrentBehavior);
                    }
                }
            }
        }
    }

Begin:
    if (m_bFirstIdle)
    {
        Sleep(0,05);
        m_bFirstIdle = false;
    }
    if (MyBP != None)
    {
        if (MyBP.m_bPlayCustomActivationAnim && !m_bPlayedCustomActivationAnim)
        {
            StartCustomAction(8);
        }
    }
    if (m_RequestedActorToFollow != None)
    {
        m_ActorToFollow = m_RequestedActorToFollow;
        PushState('FollowingActor');
    }
    if (IsActorInPlaypen(MyBP) == false)
    {
        GotoState('ReturningToPlaypen');
    }
    else
    {
        while (true)
        {
            if (MyBP != None && MyBP.IsDead() == false && MyBP.Squad != None)
            {
                if (HasAnyEnemies())
                {
                    if (m_nmCurrentBehavior != 'None')
                    {
                        PushState(m_nmCurrentBehavior);
                    }
                }
            }
            Sleep(1);
        }
    }
}

state IdleDebug
{
Begin:
    if (CoverGoal.Link != None)
    {
        PushState('MovingToCover');
    }
    else
    {
        if (MoveGoal != None)
        {
            PushState('MovingToGoal');
        }
    }
}

function FindNewCover()
{
    bAcquireNewCover = true;
}

function NotifyTakeHit(Controller InstigatedBy, Vector HitLocation, int Damage, class<DamageType> DamageType, Vector Momentum)
{
    NotifyTakeHit(InstigatedBy, HitLocation, Damage, DamageType, Momentum);
    LastShotAtTime = WorldInfo.TimeSeconds;
    NotifyUnderAttack(true);
}

function NotifyNearMiss(Vector HitLocation)
{
    NotifyNearMiss(HitLocation);
    LastShotAtTime = WorldInfo.TimeSeconds;
    NotifyUnderAttack(false);
}

function NotifyUnderAttack(bool bHit);

function NotifyPowerFinished(Name nmPower);

simulated function NotifyCoverDisabled(CoverLink Link, int SlotIdx)
{
    local BioPawn MyPawn;
    MyPawn = BioPawn(Pawn);
    if (MyPawn != None && MyPawn.IsInCover())
    {
        MyPawn.LeaveCover();
        MyPawn.PlayAnimatedReaction(58, self);
        MyBP.ShouldCrouch(false);
    }
}

function bool NotifyBump(Actor Other, Vector HitNormal)
{
    local BioCustomAction Action;
    local SFXAI_Core oOtherAI;
    m_oLastBumped = Pawn(Other);
    m_fLastBumpTime = WorldInfo.TimeSeconds;
    if (m_oLastBumped != None)
    {
        oOtherAI = SFXAI_Core(m_oLastBumped.Controller);
        if (oOtherAI != None)
        {
            oOtherAI.PushPawn(MyBP, HitNormal);
        }
    }
    if (GetCurrentCustomAction(Action) && Action.NotifyBump(Other, HitNormal))
    {
        return true;
    }
    return NotifyBump(Other, HitNormal);
}

function bool NotifyHitWall(Vector HitNormal, Actor Wall)
{
    local BioCustomAction Action;
    if (GetCurrentCustomAction(Action) && Action.NotifyHitWall(HitNormal, Wall))
    {
        return true;
    }
    return NotifyHitWall(HitNormal, Wall);
}

function PushPawn(BioPawn oPawn, Vector HitNormal)
{
    m_oPushingPawn = oPawn;
    m_vPushDirection = HitNormal;
    m_fLastPushTime = WorldInfo.TimeSeconds;
}

function bool RespondToPush()
{
    if (TimeSince(m_fLastPushTime) < 0,3 && !MyBP.IsInCover())
    {
        MovePoint = MyBP.Location + m_vPushDirection * -300;
        MovePoint.Z = MyBP.Location.Z;
        MoveOffset = 0;
        MoveTimer = 3;
        PushState('MovingToLocation');
        return true;
    }
    return false;
}

function SetBestTargetCoverAction(ECoverAction BestAction)
{
    m_eBestTargetCoverAction = BestAction;
}

function ECoverAction GetBestTargetCoverAction(BioPawn TargetPawn, CoverInfo TargetCover)
{
    if (TargetPawn == None || TargetCover.Link == None)
    {
        return 0;
    }
    if (TargetPawn.CoverAction != 0 && TargetPawn.CoverAction < 9)
    {
        return TargetPawn.CoverAction;
    }
    if (IsTargetInFiringArc(TargetPawn, MyBP, 0,7))
    {
        if (m_eBestTargetCoverAction != 0)
        {
            return m_eBestTargetCoverAction;
        }
        if (TargetCover.Link.Slots[TargetCover.SlotIdx].bLeanLeft)
        {
            return 3;
        }
        if (TargetCover.Link.Slots[TargetCover.SlotIdx].bLeanRight)
        {
            return 4;
        }
        if (TargetCover.Link.Slots[TargetCover.SlotIdx].CoverType != 1 && TargetCover.Link.Slots[TargetCover.SlotIdx].bCanPopUp)
        {
            return 7;
        }
    }
    return 0;
}

function Vector GetAimLocation(int EnemyIdx, Vector StartLoc)
{
    local Vector AimLoc;
    local CoverInfo EnemyCover;
    local BioPawn oEnemyPawn;
    local Vector LeftPos;
    local Vector RightPos;
    local ECoverAction TargetCoverAction;
    local ECoverType CoverSlotType;
    if (EnemyIdx >= 0)
    {
        oEnemyPawn = BioPawn(EnemyList[EnemyIdx].Pawn);
    }
    else
    {
        oEnemyPawn = BioPawn(FireTarget);
    }
    if (oEnemyPawn != None && GetPawnCover(oEnemyPawn, EnemyCover) && oEnemyPawn.IsInCover())
    {
        TargetCoverAction = GetBestTargetCoverAction(oEnemyPawn, EnemyCover);
        if (TargetCoverAction != 0)
        {
            CoverSlotType = EnemyCover.Link.Slots[EnemyCover.SlotIdx].CoverType;
            LeftPos = EnemyCover.Link.GetSlotViewPoint(oEnemyPawn.LeftSlotIdx, CoverSlotType, TargetCoverAction);
            RightPos = EnemyCover.Link.GetSlotViewPoint(oEnemyPawn.RightSlotIdx, CoverSlotType, TargetCoverAction);
            AimLoc = LeftPos * 1 - oEnemyPawn.CurrentSlotPct + RightPos * oEnemyPawn.CurrentSlotPct;
            if (oEnemyPawn.CoverAction != 7)
            {
                AimLoc.Z -= 35;
            }
        }
        else
        {
            AimLoc = GetAimLocation(EnemyIdx, StartLoc);
        }
    }
    else
    {
        AimLoc = GetAimLocation(EnemyIdx, StartLoc);
    }
    return AimLoc;
}

function ReEvaluateCover();

function NotifyWeaponDelayFinished()
{
    m_bDelayWeaponUse = false;
}

state FiringWeapon
{
    function bool CanStartFiring()
    {
        if (m_bDelayWeaponUse)
        {
            return false;
        }
        if (MyBP.Weapon == None)
        {
            return false;
        }
        if (m_bAttacking)
        {
            return false;
        }
        if (MyBP.IsReloading(true))
        {
            return false;
        }
        if (HasValidTarget() == false)
        {
            return false;
        }
        if (IsTargetInFiringArc(MyBP, FireTarget, m_fFiringArcAngle) == false)
        {
            m_nWeaponCompletionReason = 3;
            return false;
        }
        if (Vehicle(FireTarget) != None)
        {
            if (VSize(FireTarget.Location - MyBP.Location) > MyBP.SightRadius)
            {
                return false;
            }
        }
        if (CanShootWeapon(FireTarget) == false)
        {
            return false;
        }
        if (m_bCheckLOS && CanAttack(FireTarget) == false)
        {
            m_nWeaponCompletionReason = 3;
            return false;
        }
        if (!AcquireTicket(FireTarget, 2))
        {
            m_bFailedTicket = true;
            return false;
        }
        return true;
    }

    function ReEvaluateCover()
    {
        bAcquireNewCover = true;
    }

    function bool StartFiringWeapon()
    {
        if (m_bAttacking)
        {
            return false;
        }
        bFailedToFire = false;
        AimWeapon();
        m_bAttacking = true;
        return true;
    }

    function StopFiringWeapon(optional bool bReleaseAttackTicket = true)
    {
        if (bReleaseAttackTicket)
        {
            ReleaseTicket(FireTarget, 2);
        }
        ClearTimer('UpdateFiring');
        ClearTimer('ShootWeapon');
        if (m_bAttacking)
        {
            if (MyBP.IsInCover())
            {
                if (m_bStayLeanedOut == false && MyBP.CoverAction != 0)
                {
                    MyBP.SetCoverAction(0);
                    MyBP.ShouldCrouch(MyBP.CoverType == 2);
                }
                ResetAimInstability();
            }
            StopFiring();
            m_bAttacking = false;
            if (m_bStayLeanedOut == false)
            {
                if (MyBP.IsInCover())
                {
                    PushState('WaitingForAnimatedTransition');
                }
            }
            else
            {
                m_bStayLeanedOut = false;
            }
        }
    }

    function UpdateFiring()
    {
        if (MyBP == None || MyBPBehavior == None)
        {
            StopFiringWeapon();
            return;
        }
        if (m_bCancelAction)
        {
            StopFiringWeapon();
            m_nWeaponCompletionReason = 0;
            return;
        }
        if (IsTargetInFiringArc(MyBP, FireTarget, m_fFiringArcAngle) == false)
        {
            StopFiringWeapon();
            return;
        }
        if (bWeaponFiring == false || HasValidTarget() == false || bFailedToFire)
        {
            StopFiringWeapon();
            return;
        }
    }

    function NotifyFireLineBlocked()
    {
        local BioPawn TargetAsPawn;
        TargetAsPawn = BioPawn(FireTarget);
        if (TargetAsPawn != None && TargetAsPawn.GetCurrentHealth() > 0)
        {
            SFXGame(WorldInfo.Game).TriggerVocalizationEvent(16, MyBP, TargetAsPawn);
        }
        bFailedToFire = true;
        m_nWeaponCompletionReason = 3;
    }

    function AimWeapon()
    {
        MyBP.PlayFireCharging();
        if (WeaponAimDelay > 0)
        {
            SetTimer(WeaponAimDelay, false, 'ShootWeapon');
        }
        else
        {
            ShootWeapon();
        }
    }

    function ShootWeapon()
    {
        local BioPawn oTarget;
        FireWeaponBurst();
        oTarget = BioPawn(FireTarget);
        if (oTarget != None)
        {
            oTarget.OnAttacked(MyBP, MyBP.Weapon.Name);
        }
        SetTimer(0,5, true, 'UpdateFiring');
    }

    function float GetBurstDelay(SFXWeapon Wpn)
    {
        return BurstFireDelay;
    }

    function NotifyWeaponFinishedFiring(Weapon W, byte FireMode)
    {
        local SFXWeapon Wpn;
        local float fDelay;
        NotifyWeaponFinishedFiring(W, FireMode);
        if (FireMode == 0 && !bWeaponFiring)
        {
            Wpn = SFXWeapon(Pawn.Weapon);
            if (Wpn != None)
            {
                fDelay = GetBurstDelay(Wpn);
                ModifyBurstDelay(Wpn, fDelay);
                if (fDelay > 0)
                {
                    m_bDelayWeaponUse = true;
                    SetTimer(fDelay, false, 'NotifyWeaponDelayFinished');
                }
            }
        }
    }

    event PausedState()
    {
        StopFiringWeapon(false);
        PausedState();
    }

    event PoppedState()
    {
        StopFiringWeapon();
        if (__FireWeaponDelegate__Delegate != None)
        {
            FireWeaponDelegate(m_nWeaponCompletionReason);
            __FireWeaponDelegate__Delegate = None;
        }
        m_bCheckLOS = true;
        PoppedState();
    }

    event PushedState()
    {
        local SFXWeapon oWeapon;
        PushedState();
        WeaponAimDelay = 0;
        if (MyBP != None)
        {
            oWeapon = SFXWeapon(MyBP.Weapon);
            if (oWeapon != None)
            {
                WeaponAimDelay = oWeapon.GetAIAimDelay();
            }
        }
        ClearCancelAction();
    }

    function bool LeanBeforeShooting()
    {
        local ECoverAction AttackerCoverAction;
        local ECoverAction TargetCoverAction;
        if (MyBP.IsInCover() && MyBP.IsInCoverLeaning() == false)
        {
            if (PendingCoverAction != 0)
            {
                MyBP.SetCoverAction(PendingCoverAction);
            }
            else
            {
                if (GetCoverAction(Cover, FireTarget, AttackerCoverAction, TargetCoverAction) == false)
                {
                    return false;
                }
                MyBP.SetCoverAction(AttackerCoverAction);
            }
            MyBP.ShouldCrouch(MyBP.CoverType == 2 && MyBP.CoverAction != 7);
            MyBP.SetAnimatedTransitionPending();
            PushState('WaitingForAnimatedTransition');
        }
        return true;
    }

    m_nWeaponCompletionReason = 2;
    if (MyBP == None || MyBP.IsDead())
    {
        PopState();
    }
    if (CanStartFiring() == false)
    {
        PopState();
    }
    if (LeanBeforeShooting() == false)
    {
        PopState();
    }
    if (StartFiringWeapon() == false)
    {
        PopState();
    }
    m_nWeaponCompletionReason = 1;
    while (m_bAttacking)
    {
        Sleep(0,25);
    }
    PopState();
    // invalid native function 205 ;
}

function bool DebugMoveToCover(CoverInfo NewCoverGoal)
{
    if (IsInState('MovingToCover') || IsInState('MovingToGoal'))
    {
        return false;
    }
    if (MyBP == None)
    {
        return false;
    }
    if (IsValidCover(NewCoverGoal) == false || NewCoverGoal.Link.Slots[NewCoverGoal.SlotIdx].SlotMarker == None)
    {
        return false;
    }
    if (MyBP.CurrentLink == NewCoverGoal.Link && MyBP.CurrentSlotIdx == NewCoverGoal.SlotIdx)
    {
        return false;
    }
    if (ClaimCover(NewCoverGoal) == false)
    {
        return false;
    }
    CoverGoal = NewCoverGoal;
    GotoState('IdleDebug');
    return true;
}

function bool DebugMoveToGoal(Actor NewMoveGoal, optional float NewMoveOffset)
{
    if (IsInState('MovingToCover') || IsInState('MovingToGoal'))
    {
        return false;
    }
    if (SetMoveGoal(NewMoveGoal, NewMoveOffset) == false)
    {
        return false;
    }
    GotoState('IdleDebug');
    return true;
}

function ApplyBasePathConstraints()
{
    if (!m_bAllowedToLeavePlaypen && m_nEnabledFlags == 0)
    {
        Class'SFXPath_WithinPlaypen'.WithinPlaypen(MyBP, IsReturningToPlaypen());
    }
}

function bool AcquireCoverGoal(Goal_AtCover GoalEvaluator, Actor GoalActor, bool bIgnoreAIGoHere, out CoverInfo NewCoverGoal)
{
    local CoverSlotMarker Marker;
    if (MyBP == None)
    {
        return false;
    }
    if (!bIgnoreAIGoHere && m_AIGoHereTarget != None)
    {
        GoalEvaluator = AtCover_AIGoHere;
        GoalActor = m_AIGoHereTarget;
    }
    if (GoalEvaluator == None)
    {
        return false;
    }
    if (GoalActor == None)
    {
        return false;
    }
    GoalEvaluator.Init(self, GoalActor);
    ApplyBasePathConstraints();
    if (GoalEvaluator.MoveTowardsGoalActor)
    {
        Class'Path_TowardGoal'.TowardGoal(MyBP, GoalActor);
    }
    DebugResetNavWeights();
    if (FindPathToward(GoalActor) != None)
    {
        Marker = CoverSlotMarker(RouteCache[RouteCache.Length - 1]);
        NewCoverGoal = Marker.OwningSlot;
    }
    if (IsValidCover(NewCoverGoal) == false || NewCoverGoal.Link.Slots[NewCoverGoal.SlotIdx].SlotMarker == None)
    {
        return false;
    }
    if (MyBP.CurrentLink == NewCoverGoal.Link && MyBP.CurrentSlotIdx == NewCoverGoal.SlotIdx)
    {
        return true;
    }
    if (ClaimCover(NewCoverGoal) == false)
    {
        return false;
    }
    return true;
}

function bool MoveToCover(Goal_AtCover GoalEvaluator, Actor GoalActor, optional bool bUsePeriodicCheck = true, optional bool bIgnoreAIGoHere)
{
    local CoverInfo NewCoverGoal;
    m_bInvalidatedCover = false;
    if (AcquireCoverGoal(GoalEvaluator, GoalActor, bIgnoreAIGoHere, NewCoverGoal) == false)
    {
        return false;
    }
    if (MyBP != None && MyBP.CurrentLink == NewCoverGoal.Link && MyBP.CurrentSlotIdx == NewCoverGoal.SlotIdx)
    {
        return true;
    }
    CoverGoal = NewCoverGoal;
    m_bUsePeriodicCoverCheck = bUsePeriodicCheck;
    if (!bIgnoreAIGoHere && m_AIGoHereTarget != None)
    {
        PushState('AIGoHere');
    }
    else
    {
        PushState('MovingToCover');
    }
    return true;
}

function bool IsMovingToCover()
{
    return CoverGoal.Link != None;
}

function PeriodicCoverValidation()
{
    local CoverSlotMarker oCoverSlot;
    local int idx;
    local Pawn oEnemyPawn;
    local float fDistFromEnemySq;
    if (CoverGoal.Link != None && CoverGoal.SlotIdx != -1)
    {
        oCoverSlot = CoverGoal.Link.Slots[CoverGoal.SlotIdx].SlotMarker;
        if (oCoverSlot != None)
        {
            idx = 0;
            while (idx < EnemyList.Length && !m_bInvalidatedCover)
            {
                oEnemyPawn = EnemyList[idx].Pawn;
                if (oEnemyPawn != None)
                {
                    fDistFromEnemySq = VSizeSq(oCoverSlot.Location - oEnemyPawn.Location);
                    m_bInvalidatedCover = fDistFromEnemySq <= EnemyDistance_Short * EnemyDistance_Short;
                }
                idx++;
            }
        }
    }
    if (m_bInvalidatedCover)
    {
        MoveTarget = None;
        bReachedMoveGoal = false;
    }
}

state MovingToCover
{
    event PoppedState()
    {
        CoverGoal.Link = None;
        CoverGoal.SlotIdx = -1;
        if (m_bUsePeriodicCoverCheck)
        {
            ClearTimer('PeriodicCoverValidation');
        }
        m_bUsePartialPaths = Default.m_bUsePartialPaths;
        PoppedState();
    }

Begin:
    m_bUsePartialPaths = false;
    if (MyBP == None || MyBP.IsDead() || CoverGoal.Link == None)
    {
        PopState();
    }
    if (IsValidCover(CoverGoal) == false || CoverGoal.Link.Slots[CoverGoal.SlotIdx].SlotMarker == None)
    {
        PopState();
    }
    m_bInvalidatedCover = false;
    if (m_bUsePeriodicCoverCheck)
    {
        SetTimer(0,5, true, 'PeriodicCoverValidation');
    }
    Sleep(0,01);
    SFXGame(WorldInfo.Game).TriggerVocalizationEvent(18, MyBP);
    MoveToGoal(CoverGoal.Link.Slots[CoverGoal.SlotIdx].SlotMarker, 0, None, true);
    if (m_bUsePeriodicCoverCheck)
    {
        ClearTimer('PeriodicCoverValidation');
    }
    if (bReachedMoveGoal == false)
    {
        if (CoverGoal.Link.Slots[CoverGoal.SlotIdx].SlotOwner == self)
        {
            CoverGoal.Link.SetInvalidUntil(CoverGoal.SlotIdx, WorldInfo.TimeSeconds + 0,5);
        }
        UnClaimCover();
        PopState();
    }
    PushState('EnteringCover');
    m_fLastCoverEnterTime = WorldInfo.TimeSeconds;
    PopState();
}

function bool SetMoveGoal(Actor NewMoveGoal, optional float NewMoveOffset)
{
    if (NewMoveGoal == None)
    {
        return false;
    }
    MoveGoal = NewMoveGoal;
    MoveOffset = NewMoveOffset;
    return true;
}

function PeriodicMoveCheck()
{
    SetMovementSpeed();
    if (IsInState('FollowingActor', true))
    {
        if (m_ActorToFollow != None)
        {
            if (DirectWalkCheck(m_ActorToFollow.Location, m_ActorToFollow))
            {
                MoveGoal = None;
            }
        }
    }
}

function SetMovementSpeed()
{
    local float fDistance;
    local Vector vMove;
    if (MyBP == None)
    {
        return;
    }
    if (MoveGoal != None)
    {
        fDistance = VSize(MoveGoal.Location - MyBP.Location);
    }
    else
    {
        fDistance = VSize(MovePoint - MyBP.Location);
    }
    if (WantsToRun(fDistance))
    {
        if (MyBP.bIsWalking)
        {
            MyBP.SetWalking(false);
        }
    }
    else
    {
        if (MoveGoal == None || MoveTarget == MoveGoal)
        {
            if (MyBP.bIsWalking == false)
            {
                MyBP.SetWalking(true);
                if (MoveTarget != None)
                {
                    vMove = MoveTarget.Location - MyBP.Location;
                    SetMoveTimer(vMove);
                }
            }
        }
    }
    MyBP.SetDesiredSpeed(1);
}

function bool WantsToRun(float fDistance)
{
    return fDistance > m_fRunThreshold;
}

state EnteringCover
{
Begin:
    if (MyBP == None || MyBP.IsDead() || IsValidCover(CoverGoal) == false || CoverGoal.Link.Slots[CoverGoal.SlotIdx].SlotMarker == None)
    {
        PopState();
    }
    SetCoverType();
    if (MyBP.CoverType == 2)
    {
        MyBP.ShouldCrouch(true);
    }
    else
    {
        MyBP.ShouldCrouch(false);
    }
    if (MyBP.CanDoCoverAction(3))
    {
        MyBP.SetCoverDirection(1);
    }
    else
    {
        MyBP.SetCoverDirection(2);
    }
    AdjustToSlot(CoverGoal.SlotIdx);
    Pawn.SetAnchor(CoverGoal.Link.Slots[CoverGoal.SlotIdx].SlotMarker);
    MyBP.CurrentLink = CoverGoal.Link;
    MyBP.CurrentSlotIdx = CoverGoal.SlotIdx;
    NotifyReachedCover();
    PopState();
}

function bool IsAttacking()
{
    return m_bAttacking;
}

function CancelAction(optional int nReason)
{
    m_bCancelAction = true;
    if (nReason > 0)
    {
        m_nCancelReasons = m_nCancelReasons | nReason;
    }
}

function ClearCancelAction()
{
    m_bCancelAction = false;
    m_nCancelReasons = 0;
}

function AttackResult ATTACK()
{
    local Name nmPowerName;
    m_AttackResult = 1;
    if (MyBP == None || MyBPBehavior == None)
    {
        return m_AttackResult;
    }
    if (HasValidTarget() == false)
    {
        return m_AttackResult;
    }
    if (m_bAttacking)
    {
        return m_AttackResult;
    }
    if (MyBPBehavior.m_oCurrentModalActive != None)
    {
        return m_AttackResult;
    }
    if (MyBP.Weapon != None)
    {
        if (MyBP.IsReloading(true))
        {
            m_AttackResult = 2;
            return m_AttackResult;
        }
        else
        {
            if (MyBP.IsSwitchingWeapons())
            {
                return m_AttackResult;
            }
        }
    }
    Focus = FireTarget;
    if (ChooseAttack(FireTarget, nmPowerName) == false)
    {
        return m_AttackResult;
    }
    if (nmPowerName != 'None')
    {
        m_nmPowerToUse = nmPowerName;
        PushState('UsingPower');
    }
    else
    {
        PushState('FiringWeapon');
    }
    m_AttackResult = 0;
    return m_AttackResult;
}

function bool ChooseAttack(Actor oTarget, out Name nmPowerName)
{
    local SFXWeapon oWeapon;
    local int nRequiresAttackTicket;
    local Vector AttackOrigin;
    nRequiresAttackTicket = 0;
    if (ChooseDefensivePower(nmPowerName))
    {
        return true;
    }
    if (MyBP == None || oTarget == None)
    {
        return false;
    }
    if (IsTargetInFiringArc(MyBP, oTarget, m_fFiringArcAngle) == false)
    {
        return false;
    }
    if (Vehicle(oTarget) != None)
    {
        if (VSize(oTarget.Location - MyBP.Location) > MyBP.SightRadius)
        {
            return false;
        }
    }
    if (FRand() <= GetPowerUsePercent())
    {
        ChooseAttackPower(oTarget, nmPowerName, nRequiresAttackTicket, AttackOrigin);
    }
    if (nmPowerName == 'None')
    {
        oWeapon = SFXWeapon(MyBP.Weapon);
        if (oWeapon == None)
        {
            return false;
        }
        if (CanShootWeapon(oTarget) == false)
        {
            return false;
        }
        if (ShouldReload() && Reload())
        {
            m_AttackResult = 2;
            return false;
        }
        nRequiresAttackTicket = 1;
    }
    if (nRequiresAttackTicket > 0)
    {
        if (AcquireTicket(oTarget, 2) == false)
        {
            m_bFailedTicket = true;
            if (nmPowerName != 'None')
            {
                ClearPowerReservation(nmPowerName);
            }
            return false;
        }
    }
    if (m_bCheckLOS && CanAttack(oTarget, AttackOrigin) == false)
    {
        ReleaseTicket(oTarget, 2);
        m_AttackResult = 3;
        if (nmPowerName != 'None')
        {
            ClearPowerReservation(nmPowerName);
        }
        return false;
    }
    return true;
}

function bool ShouldReload()
{
    local SFXWeapon Weapon;
    local float fMagazineSize;
    local float fRemaining;
    if (MyBP == None)
    {
        return false;
    }
    Weapon = SFXWeapon(MyBP.Weapon);
    if (Weapon == None)
    {
        return false;
    }
    if (Weapon.CanReload() == false)
    {
        return false;
    }
    fMagazineSize = Weapon.GetMagazineSize();
    if (fMagazineSize > 0)
    {
        fRemaining = fMagazineSize - Weapon.AmmoUsedCount;
        if (fRemaining < fMagazineSize * m_fReloadThreshold)
        {
            return true;
        }
    }
    return false;
}

function bool Reload()
{
    local SFXWeapon Weapon;
    if (MyBP == None)
    {
        return false;
    }
    Weapon = SFXWeapon(MyBP.Weapon);
    if (Weapon == None)
    {
        return false;
    }
    Weapon.TryReload(false);
    return true;
}

function bool RequestPowerReservation(Name nmPower, optional bool bForceSuccess = false)
{
    local BioPower oPower;
    local BioWorldInfo oWorldInfo;
    if (m_bUsePowerReservations == false)
    {
        return true;
    }
    if (MyBP == None || MyBPBehavior == None)
    {
        return false;
    }
    oPower = MyBPBehavior.FindPower(nmPower);
    if (oPower == None)
    {
        return false;
    }
    oWorldInfo = BioWorldInfo(WorldInfo);
    if (oWorldInfo == None || oWorldInfo.m_oPowerManager == None)
    {
        return false;
    }
    m_nReservationID = oWorldInfo.m_oPowerManager.MakeReservation(oPower, MyBP, bForceSuccess);
    if (m_nReservationID == 0)
    {
        return false;
    }
    return true;
}

function ClearPowerReservation(Name nmPower, optional bool bSkipProjectilePowers = false)
{
    local BioPower oPower;
    local BioWorldInfo oWorldInfo;
    if (m_bUsePowerReservations == false)
    {
        return;
    }
    if (m_nReservationID == 0)
    {
        return;
    }
    if (MyBP == None || MyBPBehavior == None)
    {
        return;
    }
    oPower = MyBPBehavior.FindPower(nmPower);
    if (oPower == None)
    {
        return;
    }
    oWorldInfo = BioWorldInfo(WorldInfo);
    if (oWorldInfo == None || oWorldInfo.m_oPowerManager == None)
    {
        return;
    }
    if (oPower.GetType() == 1 && bSkipProjectilePowers)
    {
        return;
    }
    oWorldInfo.m_oPowerManager.ClearReservation(m_nReservationID);
}

function bool AcquireTicket(Actor oTarget, EAITicketType eTicket, optional int nAttackTicketCost = 1)
{
    local BioPawn oTargetPawn;
    local bool bAcquiredTicket;
    if (bUseTicketing)
    {
        if (oTarget == None)
        {
            return false;
        }
        bAcquiredTicket = false;
        oTargetPawn = BioPawn(oTarget);
        if (oTargetPawn != None)
        {
            switch (eTicket)
            {
                case 1:
                    bAcquiredTicket = oTargetPawn.AcquireTargetTicket(m_nTargetTicketCost);
                    break;
                case 2:
                    if (m_nCurrentAttackID != 0)
                    {
                        bAcquiredTicket = oTargetPawn.HasValidAttackTicket(m_nCurrentAttackID);
                    }
                    if (!bAcquiredTicket)
                    {
                        m_nCurrentAttackID = oTargetPawn.AcquireAttackTicket(nAttackTicketCost);
                        bAcquiredTicket = m_nCurrentAttackID > 0;
                    }
                    break;
                default:
                    break;
            }
            return bAcquiredTicket;
        }
        else
        {
            return true;
        }
    }
    else
    {
        return true;
    }
}

function ReleaseTicket(Actor oTarget, EAITicketType eTicket, optional bool bKillAttackTicket = false)
{
    local BioPawn oTargetPawn;
    if (bUseTicketing)
    {
        oTargetPawn = BioPawn(oTarget);
        if (oTargetPawn != None)
        {
            switch (eTicket);
            case 1;
            oTargetPawn.ReleaseTargetTicket(m_nTargetTicketCost);
            jump 159;
            case 2;
            if (!m_nCurrentAttackID > 0) jump 145;
            oTargetPawn.ReleaseAttackTicket(m_nCurrentAttackID, bKillAttackTicket);
            if (!bKillAttackTicket) jump 142;
            m_nCurrentAttackID = 0;
            jump 145;
            jump 159;
            case 0;
            jump 159;
            default;
        }
    }
    return;
}

function float GetPowerUsePercent()
{
    if (MyBPBehavior != None)
    {
        return MyBPBehavior.GetPowerUsePercent();
    }
    else
    {
        return 0;
    }
}

function SetSquadIntoCombat()
{
    local BioAiController oMember;
    if (MyBP == None || MyBP.Squad == None)
    {
        return;
    }
    SFXGame(WorldInfo.Game).TriggerVocalizationEvent(1, MyBP);
    foreach MyBP.Squad.SquadMembers(oMember)
    {
        oMember.ReadyMode();
    }
}

function SwitchWeapon(SFXWeapon oWeapon)
{
    if (MyBP != None)
    {
        CancelAction();
        m_bPendingWeaponSwitch = true;
        m_oNextWeapon = oWeapon;
    }
}

state SwitchingWeapons
{
    event PoppedState()
    {
        m_bPendingWeaponSwitch = false;
        PoppedState();
    }

    function ChangeWeapons()
    {
        local SFXInventoryManager oInventory;
        if (MyBP != None)
        {
            oInventory = SFXInventoryManager(MyBP.InvManager);
            if (oInventory != None)
            {
                if (m_oNextWeapon != None)
                {
                    oInventory.SetWeaponIfAvailable(m_oNextWeapon);
                }
            }
        }
    }

    function bool IsWeaponSwitching()
    {
        if (MyBP != None)
        {
            return MyBP.IsSwitchingWeapons();
        }
        return false;
    }

    ChangeWeapons();
    while (IsWeaponSwitching())
    {
        Sleep(0,1);
    }
    PopState();
    // invalid native function 205 ;
}

function bool IsInFightingState();

function Taunt()
{
    if (FireTarget != None)
    {
        SFXGame(WorldInfo.Game).TriggerVocalizationEvent(39, MyBP, BioPawn(FireTarget), 0);
    }
}

function PlayAmbientVoc()
{
    SFXGame(WorldInfo.Game).TriggerVocalizationEvent(40, MyBP, None, 0);
}

state Combat
{
    function bool IsInFightingState()
    {
        return true;
    }

    function CheckPendingWeaponSwitch()
    {
        if (m_bPendingWeaponSwitch)
        {
            PushState('SwitchingWeapons');
        }
    }

    function BeginState(Name PreviousStateName)
    {
        BeginState(PreviousStateName);
        SetSquadIntoCombat();
        SelectTarget();
        SetTimer(0,5 + FRand() * 0,2, true, 'SelectTarget');
        SetTimer(10 + FRand() * 30, true, 'ReEvaluateCover');
        SetTimer(10, true, 'Taunt');
        CheckPendingWeaponSwitch();
    }

    function EndState(Name NextStateName)
    {
        ClearTimer('SelectTarget');
        ClearTimer('ReEvaluateCover');
        ClearTimer('Taunt');
        EndState(NextStateName);
    }

    event PushedState()
    {
        PushedState();
        SetSquadIntoCombat();
        SetTimer(0,5 + FRand() * 0,2, true, 'SelectTarget');
        SetTimer(10 + FRand() * 10, true, 'ReEvaluateCover');
        SetTimer(10, true, 'Taunt');
        CheckPendingWeaponSwitch();
    }

    event PoppedState()
    {
        ClearTimer('SelectTarget');
        ClearTimer('ReEvaluateCover');
        ClearTimer('Taunt');
        PoppedState();
    }

    event ContinuedState()
    {
        ContinuedState();
        SetTimer(0,1, false, 'CheckPendingWeaponSwitch');
    }

    function SwitchWeapon(SFXWeapon oWpn)
    {
        Global.SwitchWeapon(oWpn);
        PushState('SwitchingWeapons');
    }

}

function CheckPendingWeaponSwitch();

state UsingPower
{
    function bool CanStartPower()
    {
        local SFXPower oPower;
        if (MyBP == None || MyBP.PowerManager == None || m_nmPowerToUse == 'None')
        {
            return false;
        }
        if (m_bAttacking)
        {
            return false;
        }
        oPower = MyBP.PowerManager.GetPower(m_nmPowerToUse);
        if (oPower == None)
        {
            return false;
        }
        if (oPower.IsEnabled() == false)
        {
            m_nPowerCompletionReason = 5;
            return false;
        }
        if (!m_bIgnorePowerSuppression && oPower.IsSuppressed())
        {
            m_nPowerCompletionReason = 5;
            return false;
        }
        if (oPower.GetCooldownRemaining() > 0)
        {
            m_nPowerCompletionReason = 4;
            return false;
        }
        return true;
    }

    function bool ChoosePowerTarget(out Actor oTargetActor, out Vector vTargetLocation)
    {
        local BioPower oPower;
        local EBioPowerType ePowerType;
        local BioPawn oPawn;
        if (MyBP == None || MyBPBehavior == None || m_nmPowerToUse == 'None')
        {
            return false;
        }
        oPower = MyBPBehavior.FindPower(m_nmPowerToUse);
        if (oPower == None)
        {
            return false;
        }
        ePowerType = oPower.GetType();
        if (ePowerType == 3)
        {
            oTargetActor = None;
            vTargetLocation = MyBP.Location + 100 * MyBP.Rotation;
        }
        else
        {
            if (FireTarget == None)
            {
                return false;
            }
            oTargetActor = FireTarget;
            oPawn = BioPawn(FireTarget);
            if (oPawn == None || oPawn.GetAimNodeLocation(4, vTargetLocation) == false)
            {
                vTargetLocation = FireTarget.Location;
            }
        }
        return true;
    }

    function bool WasPowerUseOrderedByPlayer()
    {
        return false;
    }

    function bool StartPower()
    {
        local float fCooldownRemaining;
        local BioActivePower oActive;
        if (MyBP == None || MyBPBehavior == None || m_nmPowerToUse == 'None')
        {
            return false;
        }
        if (m_bAttacking)
        {
            return false;
        }
        if (MyBPBehavior.SetCurrentPower(m_nmPowerToUse, fCooldownRemaining) == false)
        {
            return false;
        }
        oActive = BioActivePower(MyBPBehavior.AcquireActive(Class'BioActivePower', false));
        if (oActive == None)
        {
            return false;
        }
        oActive.SetTargetActor(m_PowerTargetActor);
        oActive.SetTargetLocation(m_PowerTargetLocation);
        oActive.UpdatePowerOrder(WasPowerUseOrderedByPlayer(), m_bIgnorePowerSuppression);
        oActive.m_bUsePowerReservation = m_bUsePowerReservations;
        oActive.m_nReservationID = m_nReservationID;
        if (oActive.StartActiveFromScript() == false)
        {
            MyBPBehavior.m_oCurrentModalActive = None;
            return false;
        }
        if (oActive.m_oPower != None)
        {
            if (oActive.m_oPower.CanCasterMoveDuringPower() == false)
            {
                MyBP.StopMovement();
            }
            MyBP.SetTicketDuration(m_nCurrentAttackID, oActive.m_oPower.GetEffectDuration());
        }
        m_bAttacking = true;
        return true;
    }

    function UpdatePower()
    {
        if (MyBPBehavior == None)
        {
            CleanUp(false);
            return;
        }
        if (BioActivePower(MyBPBehavior.m_oCurrentModalActive) == None)
        {
            CleanUp(false);
            return;
        }
        if (ShouldCancelPower())
        {
            CleanUp(true);
            m_nPowerCompletionReason = 0;
        }
    }

    function bool ShouldCancelPower()
    {
        if (m_bCancelAction && m_nCancelReasons != 8)
        {
            return true;
        }
        return false;
    }

    function CleanUp(bool bOnlyCleanUpIfCasting)
    {
        local BioActivePower oActive;
        if (MyBPBehavior != None)
        {
            oActive = BioActivePower(MyBPBehavior.m_oCurrentModalActive);
            if (oActive != None)
            {
                if (oActive.CancelPower(bOnlyCleanUpIfCasting) == false)
                {
                    return;
                }
                MyBPBehavior.ReleaseActiveByClass(Class'BioActivePower');
            }
        }
        if (IsTimerActive('UpdatePower'))
        {
            ClearTimer('UpdatePower');
        }
        m_bAttacking = false;
    }

    event PushedState()
    {
        PushedState();
        ClearCancelAction();
    }

    event PoppedState()
    {
        CleanUp(false);
        ReleaseTicket(FireTarget, 2);
        ClearPowerReservation(m_nmPowerToUse, m_bPowerProjectileReleased);
        m_nmPowerToUse = 'None';
        m_bPowerProjectileReleased = false;
        m_PowerTargetActor = None;
        m_PowerTargetLocation = vect(0,0,0);
        if (__UsePowerDelegate__Delegate != None)
        {
            UsePowerDelegate(m_nPowerCompletionReason);
            __UsePowerDelegate__Delegate = None;
        }
        m_bIgnorePowerSuppression = false;
        PoppedState();
    }

    event PausedState()
    {
        if (m_bAttacking)
        {
            ClearTimer('UpdatePower');
        }
        PausedState();
    }

    event ContinuedState()
    {
        ContinuedState();
        if (m_bAttacking)
        {
            SetTimer(0,05, true, 'UpdatePower');
        }
    }

    m_nPowerCompletionReason = 2;
    if (MyBP == None || MyBPBehavior == None || MyBP.IsDead())
    {
        PopState();
    }
    if (CanStartPower() == false)
    {
        PopState();
    }
    if (ChoosePowerTarget(m_PowerTargetActor, m_PowerTargetLocation) == false)
    {
        PopState();
    }
    if (m_PowerTargetActor != None)
    {
        Focus = m_PowerTargetActor;
    }
    else
    {
        FocalPoint = m_PowerTargetLocation;
    }
    if (StartPower() == false)
    {
        PopState();
    }
    if (MyBP.IsInAnimatedTransition())
    {
        PushState('WaitingForAnimatedTransition');
    }
    SFXGame(WorldInfo.Game).TriggerVocalizationEvent(19, MyBP, BioPawn(m_PowerTargetActor));
    m_nPowerCompletionReason = 1;
    SetTimer(0,05, true, 'UpdatePower');
    while (m_bAttacking)
    {
        Sleep(0,05);
    }
    if (m_nPowerCompletionReason == 1)
    {
        jump 324;
    }
    if (MyBP.IsInAnimatedTransition())
    {
        PushState('WaitingForAnimatedTransition');
    }
    PopState();
    // invalid native function 205 ;
}

state Dying
{
    function OnLeftPlaypen();

    function OnEnteredPlaypen();

    function TakeDamage(float DamageAmount, Controller EventInstigator, Vector HitLocation, Vector Momentum, class<DamageType> DamageType, optional TraceHitInfo HitInfo, optional Actor DamageCauser);

    function NotifyUnderAttack(bool bHit);

    function SwitchWeapon(SFXWeapon oWeapon);

    function bool CanInterruptCurrentState()
    {
        return false;
    }

}

state AIGoHereSitAndShoot
{
    while (true)
    {
        if (FireTarget == None)
        {
            while (SelectTarget() == false && m_AIGoHereTarget == None || m_bAIGoHereTargetArrived == false)
            {
                Sleep(1);
            }
        }
        if (m_AIGoHereTarget == None || m_bAIGoHereTargetArrived == false)
        {
            break;
        }
        if (MyBP.IsInCover() && IsFlankedByTarget(FireTarget) && __TryInvalidateTargetFlank__Delegate == None || TryInvalidateTargetFlank())
        {
            InvalidateCover();
            Focus = FireTarget;
            FinishRotation();
            break;
        }
        if (__TryInvalidateTargetRange__Delegate == None || TryInvalidateTargetRange(VSize(MyBP.Location - FireTarget.Location)))
        {
            InvalidateCover();
            Focus = FireTarget;
            FinishRotation();
            break;
        }
        ATTACK();
        Sleep(0,5 + FRand());
    }
    PopState();
    // invalid native function 205 ;
}

state AIGoHere
{
    event PushedState()
    {
        PushedState();
        m_bAIGoHereTargetArrived = false;
    }

    event PoppedState()
    {
        CoverGoal.Link = None;
        CoverGoal.SlotIdx = -1;
        MoveGoal = None;
        PoppedState();
    }

    function SetAIGoHereCoverGoal()
    {
        local CoverInfo NewCoverGoal;
        if (AcquireCoverGoal(AtCover_AIGoHere, m_AIGoHereTarget, false, NewCoverGoal) == false)
        {
        }
        CoverGoal = NewCoverGoal;
    }

    function SetAIGoHereMoveGoal()
    {
        SetMoveGoal(m_AIGoHereTarget);
    }

    function AIGoHereMoveToCover()
    {
        PushState('MovingToCover');
    }

    function AIGoHereMoveToGoal()
    {
        PushState('MovingToGoal');
    }

Begin:
    while (true)
    {
        if (m_AIGoHereTarget == None || m_bAIGoHereTargetArrived == true)
        {
            break;
        }
        if (CoverGoal.Link != None)
        {
            if (CoverLink(m_AIGoHereTarget) == None)
            {
                CoverGoal.Link = None;
                CoverGoal.SlotIdx = -1;
                SetAIGoHereMoveGoal();
            }
        }
        if (m_bAIGoHereCanUseCover && IsValidCover(CoverGoal))
        {
            AIGoHereMoveToCover();
            if (m_AIGoHereTarget != None && m_AIGoHereTarget == MyBP.CurrentLink)
            {
                m_bAIGoHereTargetArrived = true;
            }
            else
            {
                SetAIGoHereCoverGoal();
            }
        }
        else
        {
            if (MoveGoal != None)
            {
                AIGoHereMoveToGoal();
                if (bReachedMoveGoal)
                {
                    m_bAIGoHereTargetArrived = true;
                }
                else
                {
                    SetAIGoHereMoveGoal();
                }
            }
            else
            {
                break;
            }
        }
        if (m_bAIGoHereTargetArrived)
        {
            PushState('AIGoHereSitAndShoot');
        }
        Sleep(0,5);
    }
    PopState();
}

function bool UpdateDeathState()
{
    return true;
}

function NotifyDeathBlow()
{
    ReleaseTicket(FireTarget, 1);
    ReleaseTicket(FireTarget, 2, true);
    FireTarget = None;
    bDying = true;
    EndCustomAction();
    PopState(true);
    GotoState('Dying');
}

event OnEnteringStasis()
{
    ReleaseTicket(FireTarget, 1);
    ReleaseTicket(FireTarget, 2, true);
    OnEnteringStasis();
}

event bool GetCurrentCustomAction(out BioCustomAction pAction)
{
    if (CurrentCustomAction > 0 && CurrentCustomAction < CustomActions.Length)
    {
        pAction = CustomActions[CurrentCustomAction];
        return true;
    }
    return false;
}

function Name GetCustomActionState(EAICustomAction eAction)
{
    switch (eAction)
    {
        case 8:
            return 'CustomAction_CombatActivate';
        case 9:
            return 'CustomAction_CoverClimbUp';
        case 10:
            return 'CustomAction_CoverMantle';
        default:
            return 'None';
    }
}

event bool StartCustomAction(EAICustomAction NewAction, optional EAICustomActionTypes CustomActionType)
{
    local Name CustomActionState;
    if (CanDoCustomAction(NewAction))
    {
        switch (CustomActionType)
        {
            case 1:
                CustomActionState = 'CustomAction_DamageReaction';
                DamageReaction = NewAction;
                break;
            case 2:
                CustomActionState = 'CustomAction_DeathReaction';
                DamageReaction = NewAction;
                break;
            case 0:
                CustomActionState = GetCustomActionState(NewAction);
        }
        if (CustomActionState != 'None')
        {
            if (IsInState('UsingPower'))
            {
                PopState();
            }
            if (CurrentCustomAction != 0 && CurrentCustomAction != NewAction)
            {
                CustomActionEnded(CurrentCustomAction);
            }
            CurrentCustomAction = NewAction;
            PushState(CustomActionState);
        }
        return true;
    }
    return false;
}

final event DoCustomAction(EAICustomAction NewAction)
{
    if (NewAction != CurrentCustomAction)
    {
        if (!CanDoCustomAction(NewAction))
        {
            return;
        }
        if (CurrentCustomAction != 0)
        {
            CustomActionEnded(CurrentCustomAction);
        }
        CurrentCustomAction = NewAction;
    }
    if (CurrentCustomAction != 0)
    {
        CustomActions[CurrentCustomAction].StartCustomAction();
    }
}

final function EndCustomAction()
{
    if (bDying != true && MyBP.Physics == 10)
    {
    }
    if (CurrentCustomAction != 0)
    {
        CustomActionEnded(CurrentCustomAction);
    }
    CurrentCustomAction = 0;
    bFinishedCustomAction = true;
    DoCustomAction(0);
}

event bool CanDoCustomAction(EAICustomAction CAction)
{
    if (CAction != 0 && CAction < CustomActionClasses.Length && CustomActionClasses[CAction] != None)
    {
        if (VerifyCAHasBeenInstanced(CAction))
        {
            return CustomActions[CAction].CanDoCustomAction();
        }
    }
    return false;
}

final function CustomActionEnded(EAICustomAction PrevAction)
{
    if (PrevAction != 0)
    {
        if (CustomActions[PrevAction] != None)
        {
            CustomActions[PrevAction].StopCustomAction();
            jump 65;
        }
    }
    return;
}

final function bool IsDoingACustomAction()
{
    return bDoingCustomAction;
}

final function bool VerifyCAHasBeenInstanced(EAICustomAction CAIndex)
{
    if (CAIndex > 0)
    {
        if (CAIndex >= CustomActions.Length || CustomActions[CAIndex] == None)
        {
            if (CAIndex < CustomActionClasses.Length && CustomActionClasses[CAIndex] != None)
            {
                CustomActions[CAIndex] = new(Outer,CustomActionClasses[CAIndex]);
                CustomActions[CAIndex].oPawn = BioPawn(Pawn);
            }
            else
            {
                CustomActions[CAIndex] = None;
                return false;
            }
        }
        return true;
    }
    return false;
}

state CustomAction
{
    function OnLeftPlaypen();

    function bool CanInterruptCurrentState()
    {
        return false;
    }

    event bool EnableAI(bool bEnable, int nRequestedBy)
    {
        if (bEnable)
        {
            m_nEnabledFlags = m_nEnabledFlags & ~nRequestedBy;
        }
        else
        {
            m_nEnabledFlags = m_nEnabledFlags | nRequestedBy;
        }
        return true;
    }

    event PushedState()
    {
        PushedState();
        ClearTimer('CustomActionTimeout');
        bReachedCover = false;
        bPreparingMove = true;
        bForceDesiredRotation = true;
        bAbortedCustomAction = false;
        bFinishedCustomAction = false;
        bDoingCustomAction = true;
        if (MyBP != None)
        {
            DesiredRotation = MyBP.DesiredRotation;
            MyBP.StopMovement(false);
        }
    }

    event PoppedState()
    {
        PoppedState();
        bDoingCustomAction = false;
        if (bDying != true && MyBP.Physics == 10)
        {
        }
        if (!bFinishedCustomAction)
        {
            EndCustomAction();
        }
        bPreparingMove = false;
        bPreciseDestination = false;
        bForceDesiredRotation = false;
        bReachedCover = false;
    }

    function bool ShouldFinishRotation();

    function bool ShouldFinishPostRotation();

    function FinishedCustomAction();

    function float GetPostCustomActionSleepTime();

    function bool ExecuteCustomAction();

    if (ShouldFinishRotation())
    {
        FinishRotation();
    }
    if (ExecuteCustomAction())
    {
        Sleep(0,1);
        if (!Pawn == None || bFinishedCustomAction || bAbortedCustomAction) jump 29;
        if (!!bAbortedCustomAction) jump 143;
        if (!ShouldFinishPostRotation()) jump 97;
        FinishRotation();
        FinishedCustomAction();
        if (!GetPostCustomActionSleepTime() > 0) jump 140;
        Sleep(GetPostCustomActionSleepTime());
        jump 151;
        Sleep(0,5);
    }
    Sleep(0,5);
    if (!MyBP.IsDead()) jump 201;
    GotoState('Dying');
    jump 351;
    if (!!CanCustomActionTerminateEarly()) jump 351;
    if (!m_nEnabledFlags != 0) jump 244;
    GotoState('Disabled');
    jump 351;
    if (!!m_bAllowedToLeavePlaypen && IsActorInPlaypen(MyBP) == false && MyBP.Physics != 10) jump 322;
    GotoState('ReturningToPlaypen');
    jump 351;
    if (!IsInState(m_nmCurrentBehavior, true) == false) jump 351;
    GotoState('Idle');
    PopState();
    jump 0;
    // unresolved reference 0 ;
}

function NotifyLimbDetached(EBioPartGroup ePartGroup)
{
    if (IsInState('Dying'))
    {
        return;
    }
    if (MyBP != None)
    {
        MyBP.Died(self, Class'SFXDamageType_Suicide', MyBP.Location);
    }
    NotifyLimbDetached(ePartGroup);
}

function bool IsPlayingReaction();

function bool IsPlayingDeathReaction();

state CustomAction_DamageReaction
{
    function bool IsPlayingReaction()
    {
        return true;
    }

    function bool ExecuteCustomAction()
    {
        DoCustomAction(DamageReaction);
        return true;
    }

    function bool ShouldFinishRotation()
    {
        return SFXCustomAction_DamageReaction(CustomActions[DamageReaction]).bRotateOnHit;
    }

    function float GetPostCustomActionSleepTime()
    {
        return 0,1;
    }

}

state CustomAction_DeathReaction
{
    function bool IsPlayingDeathReaction()
    {
        return true;
    }

}

state CustomAction_CoverMantleClimbBase
{
    function bool ShouldFinishRotation()
    {
        return true;
    }

    event PushedState()
    {
        Focus = None;
        MyBP.DesiredRotation = MoveTarget.Location - Pawn.Location;
        PushedState();
    }

    function FinishedCustomAction()
    {
        if (GetCustomActionMovementState() == true)
        {
            m_bCustomActionFailed = false;
            if (Pawn != None)
            {
                Pawn.SetAnchor(NavigationPoint(MoveTarget));
            }
        }
        else
        {
            m_bCustomActionFailed = true;
        }
    }

    function bool GetCustomActionMovementState()
    {
        if (VSize(Pawn.Location - MoveTarget.Location) < VSize(Pawn.Location - Pawn.Anchor.Location))
        {
            return true;
        }
        return false;
    }

}

state CustomAction_CoverMantle
{
    function bool ExecuteCustomAction()
    {
        DoCustomAction(10);
        return true;
    }

    event PushedState()
    {
        PushedState();
    }

}

state CustomAction_CoverClimbUp
{
    function bool ExecuteCustomAction()
    {
        DoCustomAction(9);
        return true;
    }

    event PushedState()
    {
        PushedState();
    }

}

state CustomAction_CombatActivate
{
    function WakeOnNotify()
    {
        if (CustomActions[8] != None)
        {
            CustomActions[8].ContinueCustomAction();
        }
    }

    function NotifyNewEnemy(Pawn NewEnemy)
    {
        NotifyNewEnemy(NewEnemy);
        if (IsTimerActive('WakeOnNotify') == false)
        {
            SetTimer(FRand() * 2, false, 'WakeOnNotify');
        }
    }

    function bool ExecuteCustomAction()
    {
        DoCustomAction(8);
        return true;
    }

Begin:
    m_bPlayedCustomActivationAnim = true;
    if (ShouldFinishRotation())
    {
        FinishRotation();
    }
    if (ExecuteCustomAction())
    {
        if (MyBP != None && MyBP.IsDead() == false && MyBP.Squad != None)
        {
            if (!HasAnyEnemies())
            {
                if (CustomActions[8] != None)
                {
                    CustomActions[8].PauseCustomAction();
                }
            }
        }
        MyBP.SetHidden(false);
        Sleep(0,1);
        if (!Pawn == None || bFinishedCustomAction || bAbortedCustomAction) jump 160;
        if (!!bAbortedCustomAction) jump 274;
        if (!ShouldFinishPostRotation()) jump 228;
        FinishRotation();
        FinishedCustomAction();
        if (!GetPostCustomActionSleepTime() > 0) jump 271;
        Sleep(GetPostCustomActionSleepTime());
        jump 282;
        Sleep(0,5);
    }
    Sleep(0,5);
    PopState();
}

function CustomActionTimeout();

function bool IsFlankedByTarget(Actor oTarget)
{
    local Vector vPawnRotation;
    local Vector vTargetLocation;
    local float fDot;
    if (MyBP == None || oTarget == None)
    {
        return false;
    }
    vPawnRotation = MyBP.Rotation;
    vTargetLocation = Normal(oTarget.Location - MyBP.Location);
    fDot = vTargetLocation Dot vPawnRotation;
    if (fDot <= 0,5)
    {
        return true;
    }
    return false;
}

function bool IsReturningToPlaypen()
{
    return IsInState('ReturningToPlaypen', true);
}

state ReturningToPlaypen
{
    event BeginState(Name PreviousStateName)
    {
        BeginState(PreviousStateName);
        SetTimer(0,5 + FRand() * 0,2, true, 'SelectTarget');
    }

    event EndState(Name NextStateName)
    {
        ClearTimer('SelectTarget');
        EndState(NextStateName);
    }

    function bool MoveToPlaypen()
    {
        local Actor oNodeInPlaypen;
        if (MyBP == None || MyBP.Squad == None)
        {
            return false;
        }
        oNodeInPlaypen = MyBP.Squad.GetPlaypenReturnPoint(MyBP);
        if (oNodeInPlaypen == None)
        {
            return false;
        }
        MoveToGoal(oNodeInPlaypen, 0, None, true);
        return true;
    }

    MoveToPlaypen();
    if (IsActorInPlaypen(MyBP) == false)
    {
        Sleep(1);
    }
    GotoState('Idle');
    // invalid native function 205 ;
}

event OnLeftPlaypen()
{
    if (MyBP == None || MyBP.Squad == None)
    {
        return;
    }
    if (m_bAllowedToLeavePlaypen)
    {
        return;
    }
    if (IsReturningToPlaypen())
    {
        if (MoveGoal != None && MyBP.Squad.IsPositionInPlaypen(MoveGoal.Location))
        {
            return;
        }
    }
    GotoState('ReturningToPlaypen');
}

function bool IsActorInPlaypen(Actor oActor)
{
    if (oActor != None)
    {
        if (MyBP != None && MyBP.Squad != None)
        {
            if (!MyBP.Squad.HasPlaypen())
            {
                return true;
            }
            else
            {
                if (MyBP.Squad.IsPositionInPlaypen(oActor.Location))
                {
                    return true;
                }
            }
            return false;
        }
    }
    return true;
}

function bool CanSwitchBehaviours()
{
    return !IsReturningToPlaypen() && CanInterruptCurrentState();
}

state Disabled
{
    function OnLeftPlaypen();

    function OnEnteredPlaypen();

    function bool CanSwitchBehaviours()
    {
        return false;
    }

    function UsePowerOnTarget(Name nmPowerToUse, Actor oTarget, optional delegate<UsePowerDelegate> PowerDelegate, optional bool bIgnoreSuppression)
    {
        ReadyMode();
        m_bIgnorePowerSuppression = bIgnoreSuppression;
        RequestPowerReservation(nmPowerToUse, true);
        FireTarget = oTarget;
        m_nmPowerToUse = nmPowerToUse;
        __UsePowerDelegate__Delegate = PowerDelegate;
        PushState('UsingPower');
    }

    function FireWeaponAtTarget(Actor oTarget, bool bCheckLOS, optional delegate<FireWeaponDelegate> FireDelegate)
    {
        if (MyBP != None && MyBP.Weapon == None)
        {
            ReadyMode();
        }
        m_bCheckLOS = bCheckLOS;
        FireTarget = oTarget;
        __FireWeaponDelegate__Delegate = FireDelegate;
        PushState('FiringWeapon');
    }

    function MoveToGoalExternal(Actor NewMoveGoal, optional float NewMoveOffset, optional delegate<MoveToDelegate> MoveDelegate)
    {
        MoveToGoal(NewMoveGoal, NewMoveOffset, MoveDelegate);
    }

    function bool StartFollowingActor(Actor ActorToFollow)
    {
        if (MyBP == None || MyBP.IsDead() || ActorToFollow == None)
        {
            return false;
        }
        m_RequestedActorToFollow = ActorToFollow;
        m_ActorToFollow = m_RequestedActorToFollow;
        PushState('FollowingActor');
        return true;
    }

Begin:
    while (true)
    {
        Sleep(1);
    }
}

event BioClearCrossLevelReferences(Level oLevel)
{
    if (m_PowerTargetActor != None && IsActorInLevel(m_PowerTargetActor, oLevel))
    {
        m_PowerTargetActor = None;
    }
    if (m_RequestedActorToFollow != None && IsActorInLevel(m_RequestedActorToFollow, oLevel))
    {
        m_RequestedActorToFollow = None;
    }
    if (m_ActorToFollow != None && IsActorInLevel(m_ActorToFollow, oLevel))
    {
        m_ActorToFollow = None;
    }
}

function DrawLocationMarker(Vector vLocation, optional float fSize = 10, optional int nRed = 255, optional int nGreen = 0, optional int nBlue = 0)
{
    local Vector vStart;
    local Vector vEnd;
    if (MyBP == None)
    {
        return;
    }
    vStart = vLocation;
    vStart.X -= fSize;
    vEnd = vLocation;
    vEnd.X += fSize;
    MyBP.DrawDebugLine(vStart, vEnd, nRed, nGreen, nBlue, true);
    vStart = vLocation;
    vStart.Z -= fSize;
    vEnd = vLocation;
    vEnd.Z += fSize;
    MyBP.DrawDebugLine(vStart, vEnd, nRed, nGreen, nBlue, true);
}

function StartSpecialMoveDelayTimer()
{
    if (m_fSpecialMoveTimeDelay > 0)
    {
        m_bSpecialMoveAllowed = false;
        SetTimer(m_fSpecialMoveTimeDelay, true, 'EnableSpecialMove');
    }
}

function EnableSpecialMove()
{
    ClearTimer('EnableSpecialMove');
    m_bSpecialMoveAllowed = true;
}

state WaitingForAnimatedTransition
{
    while (MyBP != None && MyBP.IsInAnimatedTransition())
    {
        Sleep(0,05);
    }
    PopState();
    // invalid native function 205 ;
}

function bool RespondToBump(Actor Other, Vector HitNormal)
{
    SFXGame(WorldInfo.Game).TriggerVocalizationEvent(80, MyBP, BioPawn(Other), 0);
    return false;
}

state MovingToLocation
{
    event PushedState()
    {
        PushedState();
        bReachedMoveGoal = false;
        m_nMoveCompletionReason = 1;
    }

    event PoppedState()
    {
        MovePoint = vect(0,0,0);
        MoveOffset = 0;
        MoveTimer = 0;
        if (MyBP != None && m_bClearVelocityAfterMove)
        {
            MyBP.StopMovement(false);
        }
        m_bClearVelocityAfterMove = true;
        PoppedState();
    }

    function bool HasReachedLocation()
    {
        local Vector vMoveLocation;
        local Vector vPawnLocation;
        local float fCollisionRadius;
        if (bReachedMoveGoal)
        {
            return true;
        }
        if (MyBP == None)
        {
            return true;
        }
        vMoveLocation = MovePoint;
        vMoveLocation.Z = 0;
        vPawnLocation = MyBP.Location;
        vPawnLocation.Z = 0;
        if (MyBP.CylinderComponent != None)
        {
            fCollisionRadius = MyBP.CylinderComponent.CollisionRadius;
        }
        if (VSize(vMoveLocation - vPawnLocation) <= MoveOffset + fCollisionRadius)
        {
            return true;
        }
        return false;
    }

    function float GetBurstDelay(SFXWeapon Wpn)
    {
        BurstFireDelay = RandRange(Wpn.AI_BurstFireMovingDelay.X, Wpn.AI_BurstFireMovingDelay.Y);
        return BurstFireDelay;
    }

    function bool ShouldCancelMove()
    {
        return false;
    }

Begin:
    if (MyBP == None || MyBP.IsDead())
    {
        PopState();
    }
    if (IsZero(MovePoint))
    {
        PopState();
    }
    if (MyBP.IsInCover())
    {
        InvalidateCover();
    }
    if (MyBP.bIsCrouched)
    {
        MyBP.ShouldCrouch(false);
    }
    SetMovementSpeed();
    if (MyBP.bCanStrafe && HasValidTarget())
    {
        Focus = FireTarget;
        if (m_bAttacking == false)
        {
            if (CanStartFiring())
            {
                StartFiringWeapon();
            }
        }
    }
    else
    {
        Focus = None;
        FocalPoint = MovePoint + Normal(MovePoint - MyBP.Location) * 100;
    }
    if (MoveTimer <= 0)
    {
        MoveTimer = 20;
    }
    while (HasReachedLocation() == false && MoveTimer > 0 && ShouldCancelMove() == false)
    {
        MoveTowardLocation(MovePoint);
        Sleep(0,1);
    }
    if (MoveTimer > 0 && m_nMoveCompletionReason != 0)
    {
        bReachedMoveGoal = true;
        jump 408;
    }
    if (m_bAttacking)
    {
        StopFiringWeapon();
    }
    PopState();
}

function bool MoveToGoal(Actor NewMoveGoal, optional float NewMoveOffset, optional delegate<MoveToDelegate> MoveDelegate, optional bool bIgnoreAIGoHere)
{
    __MoveToDelegate__Delegate = MoveDelegate;
    if (!bIgnoreAIGoHere && m_AIGoHereTarget != None)
    {
        if (SetMoveGoal(m_AIGoHereTarget))
        {
            PushState('AIGoHere');
            return true;
        }
    }
    else
    {
        if (SetMoveGoal(NewMoveGoal, NewMoveOffset))
        {
            PushState('MovingToGoal');
            return true;
        }
    }
    return false;
}

state MovingToGoal
{
    event PushedState()
    {
        PushedState();
        SetTimer(GetPeriodicMoveInterval(), true, 'PeriodicMoveCheck');
        bReachedMoveGoal = false;
        MoveTarget = None;
        RouteCache_Empty();
        m_nMoveCompletionReason = 2;
        ClearCancelAction();
    }

    event PoppedState()
    {
        PoppedState();
        MoveGoal = None;
        MovePoint = vect(0,0,0);
        MoveOffset = 0;
        MoveTarget = None;
        RouteCache_Empty();
        if (MyBP != None && m_bClearVelocityAfterMove)
        {
            MyBP.StopMovement(false);
        }
        m_bClearVelocityAfterMove = true;
        ClearTimer('MoveTimedOut');
        ClearTimer('CheckPawnInPlaypen');
        ClearTimer('PeriodicMoveCheck');
        if (__MoveToDelegate__Delegate != None)
        {
            if (bReachedMoveGoal)
            {
                m_nMoveCompletionReason = 1;
            }
            MoveToDelegate(m_nMoveCompletionReason);
            __MoveToDelegate__Delegate = None;
        }
        if (MyBP.LastPhysicsSetter == self)
        {
            if (MyBP.Physics == 12)
            {
                MyBP.ForceGroundConform();
            }
            MyBP.SetPhysics(1);
        }
    }

    function float GetPeriodicMoveInterval()
    {
        return 0,25;
    }

    event OnEnteredPlaypen()
    {
        if (!m_bAllowedToLeavePlaypen && m_nEnabledFlags == 0)
        {
            SetTimer(0,5, true, 'CheckPawnInPlaypen');
        }
    }

    function CheckPawnInPlaypen()
    {
        if (MyBP != None && MyBP.Squad != None && MyBP.Squad.IsPositionInPlaypen(MyBP.Location))
        {
            GotoState('Idle');
        }
    }

    function bool CanSkipNodeAfterTimeout(Actor oMoveTarget)
    {
        local float fRetryDistSq;
        local Vector vDirToMoveTarget;
        local ReachSpec oReachSpec;
        if (oMoveTarget == None || RouteCache.Length == 0 || IsSpecialMoveAfterCurrentMoveTarget())
        {
            return false;
        }
        oReachSpec = NavigationPoint(oMoveTarget).GetReachSpecTo(RouteCache[0]);
        if (oReachSpec != None)
        {
            if (oReachSpec.IsBlockedFor(MyBP))
            {
                return false;
            }
        }
        vDirToMoveTarget = MyBP.Location - oMoveTarget.Location;
        fRetryDistSq = 10000;
        if (MyBP.CylinderComponent != None)
        {
            if (TimeSince(m_fLastBumpTime) < 1 && m_oLastBumped != None && m_oLastBumped.CylinderComponent != None)
            {
                fRetryDistSq = m_oLastBumped.CylinderComponent.CollisionRadius * 2;
            }
            else
            {
                fRetryDistSq = MyBP.CylinderComponent.CollisionRadius;
            }
            fRetryDistSq += MyBP.CylinderComponent.CollisionRadius + 5;
            fRetryDistSq *= fRetryDistSq;
        }
        vDirToMoveTarget.Z = 0;
        if (VSizeSq(vDirToMoveTarget) < fRetryDistSq)
        {
            return true;
        }
        return false;
    }

    function MoveTimedOut()
    {
        PopState();
    }

    function bool HasReachedGoal()
    {
        if (MoveGoal != None && Pawn != None)
        {
            if (Pawn.Anchor == MoveGoal)
            {
                return true;
            }
            else
            {
                if (Pawn.ReachedDestination(MoveGoal))
                {
                    return true;
                }
                if (IsInState('FollowingActor', true))
                {
                    if (m_ActorToFollow != None && m_ActorToFollow == MoveGoal)
                    {
                        if (RouteCache.Length == 0)
                        {
                            return true;
                        }
                    }
                }
                return false;
            }
        }
        return true;
    }

    function bool ApplyPathConstraints()
    {
        local bool bAppliedConstraints;
        local Vector LastMantleLocation;
        local Goal_AtActor oAtGoalEval;
        bAppliedConstraints = false;
        if (Class'Goal_AtActor'.AtActor(MyBP, MoveGoal, MoveOffset, m_bUsePartialPaths))
        {
            oAtGoalEval = Goal_AtActor(MyBP.PathGoalList);
            m_oGoalActor = oAtGoalEval.GoalActor;
            Class'Path_TowardGoal'.TowardGoal(MyBP, MoveGoal);
            ApplyBasePathConstraints();
            Class'SFXPath_AvoidClaimedCover'.AvoidClaimedCover(MyBP);
            if (TimeSince(MyBP.LastMantleTime) < 5)
            {
                LastMantleLocation = MyBP.LastMantleLocation;
            }
            if (CustomActionClasses.Length > 10)
            {
                if (CustomActionClasses[9] != None && CustomActionClasses[10] != None)
                {
                    Class'Path_MinDistBetweenSpecsOfType'.EnforceMinDist(MyBP, 500, Class'MantleReachSpec', LastMantleLocation);
                }
            }
            if (m_bAvoidDangerLinks)
            {
                Class'SFXPath_AvoidFireFromCover'.AvoidFireFromCover(MyBP, m_bAvoidFireFromPlayerOnly);
            }
            bAppliedConstraints = true;
        }
        return bAppliedConstraints;
    }

    function FinishPathGeneration()
    {
        if (RouteGoal != m_oGoalActor)
        {
            if (VSizeSq(RouteGoal.Location - m_oGoalActor.Location) > MoveOffset * MoveOffset)
            {
            }
        }
    }

    function bool GetFirstMoveTarget(out Actor FirstMoveTarget)
    {
        if (DirectWalkCheck(MoveGoal.Location, MoveGoal))
        {
            FirstMoveTarget = MoveGoal;
        }
        else
        {
            if (ApplyPathConstraints() == false)
            {
                return false;
            }
            FirstMoveTarget = FindPathToward(MoveGoal, false, 2000);
            if (FirstMoveTarget == None)
            {
                return false;
            }
            if (FirstMoveTarget == RouteCache[0])
            {
                RouteCache_RemoveIndex(0);
            }
            FinishPathGeneration();
        }
        m_nMoveAttemptCounter = 0;
        LogRoute();
        return true;
    }

    function bool IsSpecialMoveAfterCurrentMoveTarget()
    {
        local NavigationPoint NextTarget;
        if (MyBP != None && RouteCache.Length > 0 && MoveTarget != None)
        {
            NextTarget = RouteCache[0];
            if (MyBP.CanMantleBetween(NavigationPoint(MoveTarget), NextTarget) == true)
            {
                return true;
            }
        }
        return false;
    }

    function CancelAction(optional int nReason)
    {
        Global.CancelAction(nReason);
        MoveTarget = None;
        m_nMoveCompletionReason = 0;
        if (nReason == 8)
        {
            m_bClearVelocityAfterMove = true;
        }
    }

    function float GetBurstDelay(SFXWeapon Wpn)
    {
        BurstFireDelay = RandRange(Wpn.AI_BurstFireMovingDelay.X, Wpn.AI_BurstFireMovingDelay.Y);
        return BurstFireDelay;
    }

    event UpdateMovementActions()
    {
        local float fDist;
        local float fDot;
        local Vector vTargetVector;
        local Vector vMoveVector;
        if (MyBP.bCanStrafe && HasValidTarget())
        {
            fDist = VSizeSq(MyBP.Location - MoveTarget.Location);
            if (IsSpecialMoveAfterCurrentMoveTarget() && fDist < 250000)
            {
                Focus = MoveTarget;
                if (m_bAttacking && FireTarget != None)
                {
                    vTargetVector = FireTarget.Location - MyBP.Location;
                    vMoveVector = MoveTarget.Location - MyBP.Location;
                    fDot = vTargetVector Dot vMoveVector;
                    if (fDot < m_fFiringArcAngle)
                    {
                        StopFiringWeapon();
                    }
                }
            }
            else
            {
                Focus = FireTarget;
                if (m_bAttacking == false)
                {
                    if (CanStartFiring())
                    {
                        StartFiringWeapon();
                    }
                }
            }
        }
        else
        {
            Focus = MoveTarget;
        }
        MyBP.DestinationOffset = GetMoveOffset();
    }

    function float GetMoveOffset()
    {
        if (MoveTarget == None || MoveGoal == None)
        {
            return 0;
        }
        if (MoveTarget == MoveGoal)
        {
            return MoveOffset;
        }
        return 0;
    }

    function LogRoute()
    {
        local int nIndex;
        if (bAILogging == false)
        {
            return;
        }
        if (MoveTarget == None && RouteCache.Length == 0)
        {
            return;
        }
        if (MoveTarget != None)
        {
            DrawLocationMarker(MoveTarget.Location, 10, 0, 0, 255);
        }
        nIndex = 0;
        while (nIndex < RouteCache.Length)
        {
            DrawLocationMarker(RouteCache[nIndex].Location, 10, 0, 0, 255);
            nIndex++;
        }
    }

Begin:
    if (MyBP == None || MyBP.IsDead() || MoveGoal == None)
    {
        PopState();
    }
    if (MyBP.IsInCover())
    {
        MyBP.LeaveCover();
    }
    if (MyBP.bIsCrouched)
    {
        MyBP.ShouldCrouch(false);
    }
    if (GetFirstMoveTarget(MoveTarget) == false)
    {
        PopState();
    }
    if (MoveTarget != MoveGoal || BioPawn(MoveTarget) == None)
    {
        MyBP.SetPhysics(12);
    }
    else
    {
        MyBP.SetPhysics(1);
    }
    MyBP.LastPhysicsSetter = self;
    while (MoveTarget != None && MoveGoal != None)
    {
        SetMovementSpeed();
        SmoothPathMovement();
        if (m_bCustomActionFailed)
        {
            m_bCustomActionFailed = false;
            RouteCache_Empty();
            if (GetFirstMoveTarget(MoveTarget))
            {
            }
        }
        else
        {
            if (MoveTimer < 0)
            {
                if (CanSkipNodeAfterTimeout(MoveTarget))
                {
                    MoveTarget = RouteCache[0];
                    RouteCache_RemoveIndex(0);
                }
                else
                {
                    break;
                }
            }
        }
        Sleep(0,1);
    }
    bReachedMoveGoal = HasReachedGoal();
    if (m_bAttacking)
    {
        StopFiringWeapon();
    }
    if (bReachedMoveGoal == false)
    {
    }
    PopState();
}

event UpdateMovementActions();

function NotifySharedCooldownFinished();

function bool ChooseAttackPower(Actor oTarget, out Name nmPower, out int nRequiresAttackTicket, out Vector AttackOrigin)
{
    local int nIndex;
    local SFXPower oPower;
    local array<SFXPower> Powers;
    local Vector ProjectileOrigin;
    local int nNewIndex;
    local bool bAdded;
    if (MyBP == None || MyBP.PowerManager == None || oTarget == None)
    {
        return false;
    }
    nIndex = 0;
    while (nIndex < MyBP.PowerManager.Powers.Length)
    {
        oPower = MyBP.PowerManager.Powers[nIndex];
        if (oPower != None)
        {
            if (oPower.AISelectable)
            {
                if (oPower.CapabilityType == 1 || oPower.CapabilityType == 2 || oPower.CapabilityType == 3)
                {
                    if (CanUsePower(oPower, oTarget))
                    {
                        if (oPower.PowerType == 5)
                        {
                            Powers.Length = 0;
                            Powers.AddItem(oPower);
                            break;
                        }
                        else
                        {
                            nNewIndex = 0;
                            while (nNewIndex < Powers.Length)
                            {
                                if (oPower.Rank > Powers[nNewIndex].Rank)
                                {
                                    Powers.InsertItem(nNewIndex, oPower);
                                    bAdded = true;
                                    break;
                                }
                                nNewIndex++;
                            }
                            if (bAdded == false)
                            {
                                Powers.AddItem(oPower);
                            }
                            bAdded = false;
                        }
                    }
                }
            }
        }
        nIndex++;
    }
    if (Powers.Length == 0)
    {
        return false;
    }
    if (Powers.Length == 1)
    {
        oPower = Powers[0];
    }
    else
    {
        if (FRand() <= 0,5)
        {
            oPower = Powers[0];
        }
        else
        {
            oPower = Powers[Rand(Powers.Length)];
        }
    }
    if (oPower.GetScaledValue(oPower.MaximumRange) >= 1000)
    {
        nRequiresAttackTicket = 1;
    }
    if (RequestPowerReservation(oPower.PowerName) == false)
    {
        return false;
    }
    if (oPower.PowerType == 1)
    {
        if (oPower.OldPower != None && oPower.OldPower.GetProjectileAttachPoint(MyBP, ProjectileOrigin))
        {
            AttackOrigin = ProjectileOrigin;
        }
    }
    if (IsZero(AttackOrigin))
    {
        AttackOrigin = MyBP.GetPawnViewLocation();
    }
    nmPower = oPower.PowerName;
    return true;
}

function bool ChooseDefensivePower(out Name nmPower)
{
    local int nIndex;
    local SFXPower oPower;
    local array<SFXPower> Powers;
    if (MyBP == None || MyBP.PowerManager == None)
    {
        return false;
    }
    nIndex = 0;
    while (nIndex < MyBP.PowerManager.Powers.Length)
    {
        oPower = MyBP.PowerManager.Powers[nIndex];
        if (oPower != None)
        {
            if (oPower.AISelectable)
            {
                if (oPower.CapabilityType == 5 || oPower.CapabilityType == 7 || oPower.CapabilityType == 6)
                {
                    if (CanUsePower(oPower, None))
                    {
                        Powers.AddItem(oPower);
                    }
                }
            }
        }
        nIndex++;
    }
    if (Powers.Length == 0)
    {
        return false;
    }
    if (Powers.Length == 1)
    {
        oPower = Powers[0];
    }
    else
    {
        oPower = Powers[Rand(Powers.Length)];
    }
    if (RequestPowerReservation(oPower.PowerName) == false)
    {
        return false;
    }
    nmPower = oPower.PowerName;
    return true;
}

function bool ChooseDeathPower(out Name nmPower)
{
    local int nIndex;
    local SFXPower oPower;
    local array<SFXPower> Powers;
    if (MyBP == None || MyBP.PowerManager == None)
    {
        return false;
    }
    nIndex = 0;
    while (nIndex < MyBP.PowerManager.Powers.Length)
    {
        oPower = MyBP.PowerManager.Powers[nIndex];
        if (oPower != None && oPower.CapabilityType == 9)
        {
            Powers.AddItem(oPower);
        }
        nIndex++;
    }
    if (Powers.Length == 0)
    {
        return false;
    }
    if (Powers.Length == 1)
    {
        oPower = Powers[0];
    }
    else
    {
        oPower = Powers[Rand(Powers.Length)];
    }
    nmPower = oPower.PowerName;
    return true;
}

function bool CanUsePower(SFXPower oPower, optional Actor oTarget, optional bool bIgnoreDelay = false)
{
    local float fDistance;
    local BioPowerScript oPowerScript;
    local String sPowerInfo;
    if (oPower == None || MyBP == None)
    {
        return false;
    }
    if (oPower.IsEnabled() == false)
    {
        return false;
    }
    if (!m_bIgnorePowerSuppression && oPower.IsSuppressed())
    {
        return false;
    }
    if (oPower.GetCooldownRemaining() > 0)
    {
        return false;
    }
    if (bIgnoreDelay == false && oPower.TimeUntilNextUse > 0)
    {
        return false;
    }
    if (oTarget != None)
    {
        fDistance = VSize(oTarget.Location - MyBP.Location);
        if (fDistance < oPower.GetScaledValue(oPower.MinimumRange) || fDistance > oPower.GetScaledValue(oPower.MaximumRange))
        {
            return false;
        }
    }
    oPowerScript = oPower.GetPowerScript();
    if (oPowerScript == None || oPowerScript.ShouldUsePower(MyBP, oTarget, sPowerInfo) == false)
    {
        return false;
    }
    return true;
}

function bool CanShootWeapon(Actor oTarget)
{
    return true;
}

function bool UsePower(Name nmPower, optional Actor oTarget, optional bool bIgnoreSuppression, optional bool bIgnoreReservation, optional bool bCheckLOS)
{
    local SFXPower oPower;
    local Vector ProjectileOrigin;
    local Vector AttackOrigin;
    if (MyBP == None || MyBP.PowerManager == None)
    {
        return false;
    }
    oPower = MyBP.PowerManager.GetPower(nmPower);
    if (oPower == None)
    {
        return false;
    }
    m_bIgnorePowerSuppression = bIgnoreSuppression;
    if (CanUsePower(oPower, oTarget) == false)
    {
        return false;
    }
    if (oPower.CapabilityType == 1 || oPower.CapabilityType == 2 || oPower.CapabilityType == 3)
    {
        if (oTarget != None)
        {
            if (oPower.GetScaledValue(oPower.MaximumRange) >= 1000)
            {
                if (AcquireTicket(oTarget, 2) == false)
                {
                    return false;
                }
            }
        }
    }
    if (RequestPowerReservation(oPower.PowerName, bIgnoreReservation) == false)
    {
        ReleaseTicket(oTarget, 2);
        return false;
    }
    if (bCheckLOS)
    {
        if (oPower.PowerType == 1)
        {
            if (oPower.OldPower != None && oPower.OldPower.GetProjectileAttachPoint(MyBP, ProjectileOrigin))
            {
                AttackOrigin = ProjectileOrigin;
            }
        }
        if (IsZero(AttackOrigin))
        {
            AttackOrigin = MyBP.GetPawnViewLocation();
        }
        if (CanAttack(oTarget, AttackOrigin) == false)
        {
            ReleaseTicket(oTarget, 2);
            ClearPowerReservation(oPower.PowerName);
            return false;
        }
    }
    if (oTarget != None && oTarget != MyBP)
    {
        FireTarget = oTarget;
    }
    m_nmPowerToUse = oPower.PowerName;
    PushState('UsingPower');
    return true;
}

function float TargetRange()
{
    if (FireTarget != None && MyBP != None)
    {
        return VSizeSq(FireTarget.Location - MyBP.Location);
    }
    return 999999;
}

function bool IsInWeaponRange(Actor oTarget)
{
    local float fDistance;
    local float fWeaponRange;
    if (MyBP == None || oTarget == None)
    {
        return false;
    }
    fDistance = VSize(oTarget.Location - MyBP.Location);
    fWeaponRange = MyBP.GetWeaponRange(4);
    if (fDistance > fWeaponRange)
    {
        return false;
    }
    return true;
}

state FollowingActor
{
    event PushedState()
    {
        PushedState();
        SetTimer(0,5 + FRand() * 0,2, true, 'SelectTarget');
        SetTimer(0,5, true, 'UpdateFollowing');
        ClearCancelAction();
    }

    event PoppedState()
    {
        if (MyBP != None)
        {
            MyBP.StopMovement(false);
        }
        ClearTimer('SelectTarget');
        ClearTimer('UpdateFollowing');
        m_ActorToFollow = None;
        PoppedState();
    }

    event PausedState()
    {
        ClearTimer('SelectTarget');
        ClearTimer('UpdateFollowing');
        PausedState();
    }

    function bool ShouldCancelFollow()
    {
        if (m_bCancelAction && m_nCancelReasons & 4 != 0)
        {
            return true;
        }
        if (m_ActorToFollow == None)
        {
            return true;
        }
        return false;
    }

    event ContinuedState()
    {
        ContinuedState();
        if (ShouldCancelFollow())
        {
            PopState();
            return;
        }
        SetTimer(0,5 + FRand() * 0,2, true, 'SelectTarget');
        SetTimer(0,5, true, 'UpdateFollowing');
    }

    event GetActorToFollow(out Actor oActor, out Vector vLocation)
    {
        oActor = m_ActorToFollow;
        vLocation = m_ActorToFollow.Location;
    }

    function bool IsDirectlyReachable()
    {
        if (m_ActorToFollow == None)
        {
            return false;
        }
        return DirectWalkCheck(m_ActorToFollow.Location, m_ActorToFollow);
    }

    function bool PathfindToActor()
    {
        if (m_ActorToFollow == None)
        {
            return false;
        }
        if (MoveToGoal(m_ActorToFollow, 400) == false)
        {
            bReachedMoveGoal = false;
            return false;
        }
        return true;
    }

    function bool TeleportToActor()
    {
        if (m_ActorToFollow == None)
        {
            return false;
        }
        return Global.TeleportToActor(m_ActorToFollow);
    }

    function UpdateFollowing()
    {
        if (ShouldCancelFollow())
        {
            PopState();
            return;
        }
    }

    function float GetBurstDelay(SFXWeapon Wpn)
    {
        BurstFireDelay = RandRange(Wpn.AI_BurstFireMovingDelay.X, Wpn.AI_BurstFireMovingDelay.Y);
        return BurstFireDelay;
    }

    event bool NotifyBump(Actor Other, Vector HitNormal)
    {
        return RespondToBump(Other, HitNormal);
    }

Begin:
    if (MyBP == None || MyBP.IsDead())
    {
        PopState();
    }
    if (m_ActorToFollow == None)
    {
        PopState();
    }
    if (MyBP.IsInCover())
    {
        InvalidateCover();
    }
    if (MyBP.bIsCrouched)
    {
        MyBP.ShouldCrouch(false);
    }
    while (true)
    {
        if (MyBP.bCanStrafe && HasValidTarget())
        {
            Focus = FireTarget;
            if (IsAttacking() == false)
            {
                if (CanStartFiring())
                {
                    StartFiringWeapon();
                }
            }
        }
        SteeringMovement();
        Focus = FireTarget;
        FocalPoint = MyBP.Location + Normal(MyBP.Velocity) * 4000;
        if (IsAttacking())
        {
            StopFiringWeapon();
        }
        Sleep(0,1);
        if (ShouldCancelFollow())
        {
            PopState();
        }
        m_bUsePartialPaths = false;
        PathfindToActor();
        m_bUsePartialPaths = true;
        bActorReachable = IsDirectlyReachable();
        if (!bActorReachable || bReachedMoveGoal == false) jump 296;
        if (!bActorReachable == false) jump 477;
        m_nMoveAttemptCounter = 0;
        if (!TeleportToActor() == false && m_nMoveAttemptCounter < 3) jump 477;
        m_nMoveAttemptCounter++;
        Sleep(1);
        if (!ShouldCancelFollow()) jump 474;
        PopState();
        jump 409;
        jump 126;
    }
}

function bool TeleportToActor(Actor oActor, optional bool bForceTeleport = false)
{
    local Vector vDestination;
    local Vector vNearestOpenLocation;
    local PlayerController PlayerController;
    if (MyBP == None || oActor == None)
    {
        return false;
    }
    if (MyBP.GetTimeSinceLastRender() > 0,5 || bForceTeleport)
    {
        if (SFXPawn_Player(oActor) != None)
        {
            PlayerController = PlayerController(SFXPawn_Player(oActor).Controller);
            if (PlayerController != None && PlayerController.PlayerCamera != None)
            {
                vDestination = oActor.Location - PlayerController.PlayerCamera.CameraCache.POV.Rotation * 400;
            }
        }
        if (IsZero(vDestination))
        {
            vDestination = oActor.Location - oActor.Rotation * 400;
        }
        if (FindNearestOpenLocation(vDestination, vNearestOpenLocation, Pawn(oActor)))
        {
            MyBP.SafeSetLocation(vNearestOpenLocation);
            return true;
        }
    }
    return false;
}

function float FindClosestApproachDistance(Actor oActor)
{
    local Goal_AtActor oAtGoalEval;
    local Actor FirstMoveTarget;
    if (MyBP == None || oActor == None)
    {
        return 0;
    }
    if (m_oLastApproachTarget == oActor && TimeSince(m_fLastApproachCheckTime) < 5)
    {
        return m_fLastApproachDistance;
    }
    m_oLastApproachTarget = oActor;
    m_fLastApproachDistance = 0;
    if (!IsActorInPlaypen(oActor))
    {
        m_fLastApproachCheckTime = WorldInfo.TimeSeconds;
        if (Class'Goal_AtActor'.AtActor(MyBP, oActor, MoveOffset, true))
        {
            oAtGoalEval = Goal_AtActor(MyBP.PathGoalList);
            m_oGoalActor = oAtGoalEval.GoalActor;
            Class'Path_TowardGoal'.TowardGoal(MyBP, oActor);
            ApplyBasePathConstraints();
            FirstMoveTarget = FindPathToward(oActor, false);
            if (FirstMoveTarget != None)
            {
                if (RouteGoal != m_oGoalActor)
                {
                    m_fLastApproachDistance = VSize(RouteGoal.Location - oActor.Location);
                }
                jump 342;
            }
            RouteCache_Empty();
        }
    }
    return m_fLastApproachDistance;
}

function NotifyKnockedOutOfCover();

function bool CanCustomActionTerminateEarly()
{
    return false;
}

defaultproperties
{
    bUseTicketing=true
    m_bSpecialMoveAllowed=true
    m_bClearVelocityAfterMove=true
    m_bUsePowerReservations=true
    m_bUsePartialPaths=true
    m_bAvoidFireFromPlayerOnly=true
    m_bAllowWeaponSwitch=true
    m_bFirstIdle=true
    m_fFiringArcAngle=0,5
    m_fReloadThreshold=0,25
    AtCover_WeaponRange=Goal_AtCover'AtCov_EvalWeaponRange'
    AtCover_Defensive=Goal_AtCover'AtCov_EvalDefensive'
    AtCover_Aggressive=Goal_AtCover'AtCov_EvalAggressive'
    AtCover_NearMoveGoal=Goal_AtCover'AtCov_EvalNearGoal'
    AtCover_AIGoHere=Goal_AtGoHereCover'AtCov_EvalGoHere'
    m_fRunThreshold=200
    m_nTargetTicketCost=1
    CustomActionClasses(0)=None
    CustomActionClasses(1)=None
    CustomActionClasses(2)=None
    CustomActionClasses(3)=None
    CustomActionClasses(4)=class'SFXCustomAction_Frozen'
    CustomActionClasses(5)=class'SFXCustomAction_Ragdoll'
    CustomActionClasses(6)=None
    CustomActionClasses(7)=None
    CustomActionClasses(8)=class'BioCustomAction_CombatActivate'
    CustomActionClasses(9)=None
    CustomActionClasses(10)=None
    CustomActionClasses(11)=class'BioCustomAction_WaypointAnimation'
    InUseNodeCostMultiplier=10
    // child object AtCov_EvalWeaponRange of type Goal_AtCover
    // child object AtCov_EvalDefensive of type Goal_AtCover
    // child object AtCov_EvalAggressive of type Goal_AtCover
    // child object AtCov_EvalNearGoal of type Goal_AtCover
    // child object AtCov_EvalGoHere of type Goal_AtGoHereCover
    // child object CovGoal_Enemies0 of type CovGoal_Enemies
    // child object CovGoal_NearGoHereGoal of type CovGoal_GoalProximity
    // child object CovGoal_NearMoveGoal of type CovGoal_GoalProximity
    // child object CovGoal_MovDistAggressive of type CovGoal_MovementDistance
    // child object CovGoal_MovDistDefensive of type CovGoal_MovementDistance
    // child object CovGoal_MovDistGoHere of type CovGoal_MovementDistance
    // child object CovGoal_MovDistWeapon of type CovGoal_MovementDistance
    // child object CovGoal_TeamProx0 of type CovGoal_TeammateProximity
    // child object CovGoal_WeaponRange0 of type CovGoal_WeaponRange
}
