﻿class SFXAI_Henchman extends SFXAI_Core
    hidecategories(Navigation)
    config(AI)
    placeable;

enum HenchmanOrderType
{
    HENCHMAN_ORDER_NONE,
    HENCHMAN_ORDER_USE_POWER,
    HENCHMAN_ORDER_SWITCH_WEAPON,
    HENCHMAN_ORDER_ATTACK_TARGET,
    HENCHMAN_ORDER_FOLLOW,
    HENCHMAN_ORDER_HOLD_POSITION,
};

struct HenchmanOrder
{
    var HenchmanOrderType eOrderType;

    var Actor oTargetActor;

    var Vector vTargetLocation;

    var Name nmPower;

    var SFXWeapon oWeapon;

    var bool bInstantOrder;

    var bool bExecutingOrder;

    var bool bPowerUseIsInstant;

}

var transient float MaxBackAwayDistance;

var transient int m_nBackingAwayCount;

var transient int m_nCoverEvaluationCount;

var config bool m_bAllowInstantPowerWhileVisible;

var /* 00000008 */ Goal_AtCover AtCover_NearDefendGoal;

var transient float MinDistanceFromTargetForCrouch;

var transient float m_fDistanceFromActorToDefend;

var transient Actor m_ActorToDefend;

var transient float m_fHeadshotProbability;

var transient SFXAI_Henchman m_OtherHenchman;

var transient float DamagePercentToRemainInCover;

var transient float AttackDelayAfterBeingShot;

var transient float DistanceToStartCombatWhileFollowing;

var transient float TetherDistanceForFollowOrder;

var transient float TetherDistanceWhileExecutingOrder;

var transient float TetherDistance;

var transient Vector m_vHoldLocation;

var transient bool m_bBackingAwayFromTarget;

var transient bool m_bOriginalStopRootMotion;

var transient bool m_bOriginalStartRootMotion;

var transient bool m_bCheckPlayerReachability;

var transient bool m_bEnemiesPerceived;

var transient bool m_bMeleeAttacker;

var transient bool m_bTooFarToAttack;

var transient bool m_bDelayPowerUse;

var transient bool m_bResetHenchman;

var transient bool m_bHoldingPosition;

var transient bool m_bFollowPlayer;

var transient array<HenchmanOrder> m_Orders;

function bool AddOrder(HenchmanOrderType eOrder, Actor oTargetActor, Vector vTargetLocation, Name nmPower, SFXWeapon oWeapon, optional int nQueue)
{
    local int nIndex;
    local bool bInstantOrder;
    local int nCurrentIndex;
    local int nQueuePosition;
    if (MyBP == None || MyBP.IsDead())
    {
        return false;
    }
    if (m_ActorToDefend != None || m_RequestedActorToFollow != None)
    {
        if (eOrder == 4 || eOrder == 5)
        {
            return false;
        }
    }
    if (eOrder == 1)
    {
        bInstantOrder = true;
        nQueuePosition = 0;
        nIndex = -1;
        nCurrentIndex = 0;
        while (nCurrentIndex < m_Orders.Length)
        {
            if (m_Orders[nCurrentIndex].eOrderType == 1)
            {
                if (nQueuePosition == nQueue)
                {
                    nIndex = nCurrentIndex;
                    break;
                }
                else
                {
                    nQueuePosition++;
                }
            }
            nCurrentIndex++;
        }
        if (nIndex == -1)
        {
            // unknown bytecode 54 ;
        }
    }
}

function int GetInstantOrderCount()
{
    local int nCount;
    local int nIndex;
    nIndex = 0;
    while (nIndex < m_Orders.Length)
    {
        if (m_Orders[nIndex].bInstantOrder)
        {
            nCount++;
        }
        nIndex++;
    }
    return nCount;
}

function bool CancelOrder(HenchmanOrderType eOrder, Name nmPower)
{
    local int nIndex;
    if (MyBP == None || MyBP.IsDead())
    {
        return false;
    }
    if (m_Orders.Length > 0)
    {
        if (eOrder == 1)
        {
            nIndex = m_Orders.Length - 1;
            if (m_Orders[nIndex].nmPower == nmPower && m_Orders[nIndex].bExecutingOrder == false)
            {
                m_Orders.Remove(nIndex, 1);
                if (IsTimerActive('InstantUsePower'))
                {
                    ClearTimer('InstantUsePower');
                }
                return true;
            }
        }
    }
    return false;
}

function bool CanQueueOrder()
{
    local int nQueuedInstantOrders;
    nQueuedInstantOrders = GetInstantOrderCount();
    if (nQueuedInstantOrders > 1)
    {
        return false;
    }
    return true;
}

function bool CanStartImmediateOrder()
{
    local int nQueuedInstantOrders;
    nQueuedInstantOrders = GetInstantOrderCount();
    if (nQueuedInstantOrders > 0)
    {
        return false;
    }
    return true;
}

function RemoveOldSwitchWeaponOrders()
{
    local int nIndex;
    nIndex = m_Orders.Length - 1;
    while (nIndex >= 0)
    {
        if (m_Orders[nIndex].eOrderType == 2 && m_Orders[nIndex].bExecutingOrder == false)
        {
            m_Orders.Remove(nIndex, 1);
        }
        nIndex--;
    }
}

function ExecuteOrders()
{
    if (MyBP == None || MyBP.IsDead())
    {
        return;
    }
    if (m_Orders.Length > 0)
    {
        if (IsInState('ExecutingOrders', true))
        {
            return;
        }
        CancelAction(2);
    }
}

function HenchmanOrder GetNextOrder(HenchmanOrderType eOrder)
{
    local HenchmanOrder oNoneOrder;
    local int nOrder;
    nOrder = 0;
    while (nOrder < m_Orders.Length)
    {
        if (m_Orders[nOrder].eOrderType == eOrder)
        {
            return m_Orders[nOrder];
        }
        ++nOrder;
    }
    return oNoneOrder;
}

function OutOfWorld()
{
    m_bResetHenchman = true;
}

function TriggerAttackVocalization()
{
    if (BioPawn(FireTarget) != None && MyBP != None)
    {
        SFXGame(WorldInfo.Game).TriggerVocalizationEvent(14, MyBP, BioPawn(FireTarget));
    }
}

function NotifyChangedWeapon(Weapon PreviousWeapon, Weapon NewWeapon)
{
    NotifyChangedWeapon(PreviousWeapon, NewWeapon);
    if (PreviousWeapon != None)
    {
        SFXGame(WorldInfo.Game).TriggerVocalizationEvent(20, BioPawn(Pawn));
    }
}

state Idle
{
    function BeginState(Name PreviousStateName)
    {
        BeginState(PreviousStateName);
        SetTimer(0,25 + FRand() * 0,25, true, 'ExecuteOrders');
        SetTimer(0,5, true, 'CheckPlayerLineOfFire');
        SetTimer(1 + FRand() * 0,25, true, 'CheckInCombat');
    }

    function EndState(Name NextStateName)
    {
        EndState(NextStateName);
        ClearTimer('ExecuteOrders');
        ClearTimer('CheckPlayerLineOfFire');
        ClearTimer('CheckInCombat');
    }

    function ExecuteOrders()
    {
        if (m_Orders.Length > 0)
        {
            PushState('ExecutingOrders');
        }
    }

    event bool NotifyBump(Actor Other, Vector HitNormal)
    {
        return RespondToBump(Other, HitNormal);
    }

    function NotifyNewEnemy(Pawn NewEnemy)
    {
        m_bEnemiesPerceived = true;
    }

    function Name ChooseBehavior()
    {
        local BioPlayerSquad oPlayerSquad;
        local float fDistance;
        if (m_bResetHenchman)
        {
            return 'ResetHenchman';
        }
        if (m_RequestedActorToFollow != None)
        {
            m_vHoldLocation = vect(0,0,0);
            m_bHoldingPosition = false;
            m_ActorToFollow = m_RequestedActorToFollow;
            return 'FollowingActor';
        }
        if (m_ActorToDefend != None)
        {
            m_vHoldLocation = vect(0,0,0);
            m_bHoldingPosition = false;
            if (IsInCombat() == false)
            {
                return 'None';
            }
            return 'Combat_Henchman';
        }
        if (ShouldFollowPlayer())
        {
            if (m_bFollowPlayer == false && ForcedTarget != None)
            {
                SFXGame(WorldInfo.Game).TriggerVocalizationEvent(4, MyBP);
            }
            oPlayerSquad = MyBP != None ? BioPlayerSquad(MyBP.Squad) : None;
            if (oPlayerSquad != None && oPlayerSquad.m_playerPawn != None)
            {
                m_vHoldLocation = vect(0,0,0);
                m_bHoldingPosition = false;
                m_ActorToFollow = oPlayerSquad.m_playerPawn;
                return 'FollowingActor';
            }
        }
        if (MyBP != None && IsZero(m_vHoldLocation) == false && m_bHoldingPosition == false)
        {
            oPlayerSquad = BioPlayerSquad(MyBP.Squad);
            if (oPlayerSquad != None && oPlayerSquad.m_playerPawn != None)
            {
                fDistance = VSize(oPlayerSquad.m_playerPawn.Location - m_vHoldLocation);
                if (fDistance <= TetherDistanceWhileExecutingOrder)
                {
                    return 'MovingToHoldLocation';
                }
                SFXGame(WorldInfo.Game).TriggerVocalizationEvent(11, MyBP);
            }
            m_vHoldLocation = vect(0,0,0);
        }
        if (IsInCombat() == false && m_bHoldingPosition && ForcedTarget == None)
        {
            return 'None';
        }
        return 'Combat_Henchman';
    }

Begin:
    while (true)
    {
        ExecuteOrders();
        m_nmCurrentBehavior = ChooseBehavior();
        if (m_nmCurrentBehavior != 'None')
        {
            ClearCancelAction();
            PushState(m_nmCurrentBehavior);
        }
        ExecuteOrders();
        Sleep(0,5);
    }
}

state ResetHenchman
{
    function bool TeleportToPlayer()
    {
        local Pawn oPlayer;
        if (MyBP == None || MyBP.Squad == None)
        {
            return false;
        }
        oPlayer = MyBP.Squad.GetSquadLeader();
        if (oPlayer == None)
        {
            return false;
        }
        return TeleportToActor(oPlayer, m_bResetHenchman);
    }

Begin:
    m_bFollowPlayer = false;
    m_vHoldLocation = vect(0,0,0);
    m_bHoldingPosition = false;
    ForcedTarget = None;
    if (MyBP != None)
    {
        MyBP.StopMovement();
        if (MyBP.bHidden == true)
        {
            MyBP.SetHidden(false);
        }
        MyBP.SetCollision(true, true, false);
    }
    while (TeleportToPlayer() == false)
    {
        Sleep(0,5);
    }
    m_bResetHenchman = false;
    PopState();
}

state ExecutingOrders
{
    event PushedState()
    {
        PushedState();
        ClearTimer('ExecuteOrders');
    }

    event PoppedState()
    {
        m_Orders.Length = 0;
        SetTimer(0,5 + FRand() * 0,5, true, 'ExecuteOrders');
        PoppedState();
    }

    function ExecutePowerOrder()
    {
        if (MyBP != None)
        {
            MyBP.ReadyMode();
        }
        m_nmPowerToUse = m_Orders[0].nmPower;
        if (m_nmPowerToUse != 'None')
        {
            m_bIgnorePowerSuppression = true;
            PushState('UsingPower');
        }
    }

    function ExecuteAttackOrder()
    {
        if (MyBP != None)
        {
            MyBP.ReadyMode();
        }
        ForcedTarget = m_Orders[0].oTargetActor;
        m_bFollowPlayer = false;
        ApplyOrderBonus();
    }

    function ExecuteFollowOrder()
    {
        m_bFollowPlayer = true;
        ForcedTarget = None;
        m_vHoldLocation = vect(0,0,0);
        m_bHoldingPosition = false;
        SFXGame(WorldInfo.Game).TriggerVocalizationEvent(8, MyBP);
        ApplyOrderBonus();
    }

    function ExecuteSwitchWeaponOrder()
    {
        SFXGame(WorldInfo.Game).TriggerVocalizationEvent(6, MyBP, None);
        Global.SwitchWeapon(m_Orders[0].oWeapon);
        PushState('SwitchingWeapons');
    }

    function ExecuteHoldPositionOrder()
    {
        m_vHoldLocation = m_Orders[0].vTargetLocation;
        m_bHoldingPosition = false;
        ForcedTarget = None;
        m_bFollowPlayer = false;
        ApplyOrderBonus();
    }

    while (m_Orders.Length > 0)
    {
        m_Orders[0].bExecutingOrder = true;
        if (m_Orders[0].eOrderType == 1)
        {
            if (m_Orders[0].bPowerUseIsInstant == false || CanInstantlyUsePowers() == false || IsTimerActive('InstantUsePower') == false)
            {
                if (IsTimerActive('InstantUsePower'))
                {
                    ClearTimer('InstantUsePower');
                }
                m_Orders[0].bPowerUseIsInstant = false;
                while (MyBP != None && MyBP.Weapon != None && MyBP.IsReloading(true))
                {
                    Sleep(0,25);
                }
                ExecutePowerOrder();
                m_Orders.Remove(0, 1);
            }
            else
            {
                Sleep(0,25);
            }
        }
        else
        {
            if (m_Orders[0].eOrderType == 3)
            {
                ExecuteAttackOrder();
                m_Orders.Remove(0, 1);
            }
            else
            {
                if (m_Orders[0].eOrderType == 4)
                {
                    ExecuteFollowOrder();
                    m_Orders.Remove(0, 1);
                }
                else
                {
                    if (m_Orders[0].eOrderType == 2)
                    {
                        ExecuteSwitchWeaponOrder();
                        m_Orders.Remove(0, 1);
                    }
                    else
                    {
                        if (m_Orders[0].eOrderType == 5)
                        {
                            ExecuteHoldPositionOrder();
                            m_Orders.Remove(0, 1);
                        }
                    }
                }
            }
        }
    }
    PopState();
    jump 0;
    // unresolved reference 0 ;
}

state UsingPower
{
    function bool ChoosePowerTarget(out Actor oTargetActor, out Vector vTargetLocation)
    {
        local BioPower oPower;
        local EBioPowerType ePowerType;
        local BioPawn oPawn;
        if (MyBP == None || MyBPBehavior == None)
        {
            return false;
        }
        oPower = MyBPBehavior.FindPower(m_nmPowerToUse);
        if (oPower == None)
        {
            return false;
        }
        ePowerType = oPower.GetType();
        if (ePowerType == 3)
        {
            oTargetActor = None;
            vTargetLocation = MyBP.Location + 100 * MyBP.Rotation;
        }
        else
        {
            if (m_Orders.Length >= 1 && m_Orders[0].eOrderType == 1)
            {
                oTargetActor = m_Orders[0].oTargetActor;
                vTargetLocation = m_Orders[0].vTargetLocation;
            }
            else
            {
                oTargetActor = FireTarget;
                oPawn = BioPawn(FireTarget);
                if (oPawn == None || oPawn.GetAimNodeLocation(4, vTargetLocation) == false)
                {
                    vTargetLocation = FireTarget.Location;
                }
            }
        }
        return true;
    }

    function bool WasPowerUseOrderedByPlayer()
    {
        return m_Orders.Length >= 1 && m_Orders[0].eOrderType == 1;
    }

    function bool ShouldCancelPower()
    {
        return m_bCancelAction;
    }

}

event BioClearCrossLevelReferences(Level oLevel)
{
    local int nIndex;
    nIndex = 0;
    while (nIndex < m_Orders.Length)
    {
        if (m_Orders[nIndex].oTargetActor != None && IsActorInLevel(m_Orders[nIndex].oTargetActor, oLevel))
        {
            m_Orders[nIndex].oTargetActor = None;
        }
        nIndex++;
    }
    if (m_ActorToDefend != None && IsActorInLevel(m_ActorToDefend, oLevel))
    {
        m_ActorToDefend = None;
    }
}

state Combat_Henchman
{
    event PushedState()
    {
        PushedState();
        SetTimer(0,5 + FRand() * 0,5, true, 'SelectTarget');
        SetTimer(1 + FRand() * 0,25, true, 'CheckMeleeAttacker');
        SetTimer(1 + FRand() * 0,25, true, 'CheckThreatRadius');
        SetSquadIntoCombat();
    }

    event PoppedState()
    {
        ClearTimer('SelectTarget');
        ClearTimer('CheckMeleeAttacker');
        ClearTimer('CheckThreatRadius');
        ClearTimer('FindNewCover');
        PoppedState();
    }

    event ContinuedState()
    {
        ContinuedState();
        if (m_bCancelAction && m_nCancelReasons & 2 != 0)
        {
            PopState();
            return;
        }
    }

    event bool NotifyBump(Actor Other, Vector HitNormal)
    {
        return RespondToBump(Other, HitNormal);
    }

    if (ForcedTarget != None)
    {
        SFXGame(WorldInfo.Game).TriggerVocalizationEvent(2, MyBP, BioPawn(ForcedTarget));
        FireTarget = ForcedTarget;
        PreferredTarget = ForcedTarget;
        ForcedTarget = None;
    }
    while (true)
    {
        if (m_Orders.Length > 0)
        {
            PopState();
        }
        if (FireTarget == None && SelectTarget() == false)
        {
            PopState();
        }
        if (ShouldFollowPlayer())
        {
            PopState();
        }
        if (m_bHoldingPosition == false)
        {
            if (MyBP.IsInCover() == false || IsInWeaponRange(FireTarget) == false || CanAttack(FireTarget) == false || IsWithinThreatRadius())
            {
                HenchmanMoveToCover();
                if (m_bInvalidatedCover)
                {
                    HenchmanMoveToCover();
                }
                if (MyBP != None && MyBP.IsInCover() == false)
                {
                    if (IsWithinThreatRadius())
                    {
                        if (m_ActorToDefend == None)
                        {
                            PushState('BackingAwayFromTarget');
                        }
                    }
                    else
                    {
                        if (VSize(FireTarget.Location - MyBP.Location) > 2000 && m_ActorToDefend == None)
                        {
                            m_bTooFarToAttack = true;
                            SetTimer(2 + FRand() * 0,5, false, 'ResetFarAwayFlag');
                            PopState();
                        }
                        SetTimer(1,5 + FRand() * 0,5, false, 'FindNewCover');
                    }
                }
                jump 505;
            }
        }
        if (m_Orders.Length > 0)
        {
            PopState();
        }
        bAcquireNewCover = false;
        while (bAcquireNewCover == false)
        {
            if (m_Orders.Length > 0)
            {
                PopState();
            }
            if (FireTarget == None && SelectTarget() == false)
            {
                PopState();
            }
            if (MyBP != None && MyBP.IsInCover() == false)
            {
                Focus = FireTarget;
            }
            if (ShouldAttack())
            {
                ATTACK();
                if (m_AttackResult == 3)
                {
                    if (IsTimerActive('FindNewCover') == false)
                    {
                        SetTimer(3, false, 'FindNewCover');
                    }
                }
            }
            if (m_Orders.Length > 0 || ShouldFollowPlayer())
            {
                PopState();
            }
            if (IsInWeaponRange(FireTarget) == false && ShouldWaitForPlayer() == false)
            {
                if (IsTimerActive('FindNewCover') == false)
                {
                    SetTimer(3, false, 'FindNewCover');
                }
            }
            if (bAcquireNewCover == false)
            {
                if (m_bHoldingPosition == false && IsTargetInFiringArc(MyBP, FireTarget, m_fFiringArcAngle) == false)
                {
                    bAcquireNewCover = true;
                }
                if (MyBP.IsInCover() && IsFlankedByTarget(FireTarget))
                {
                    MyBP.LeaveCover();
                    Focus = FireTarget;
                }
                if (m_bWaitBeforeNextAttack)
                {
                    m_bWaitBeforeNextAttack = false;
                    Sleep(AttackDelayAfterBeingShot);
                }
                else
                {
                    Sleep(0,5 + FRand());
                }
            }
        }
        Sleep(0,5);
    }
    // invalid native function 205 ;
}

function bool HenchmanMoveToCover()
{
    local int nIndex;
    local CovGoal_GoalProximity Proximity;
    m_nCoverEvaluationCount++;
    if (IsTimerActive('FindNewCover'))
    {
        ClearTimer('FindNewCover');
    }
    if (m_ActorToDefend != None)
    {
        if (AtCover_NearDefendGoal != None)
        {
            nIndex = 0;
            while (nIndex < AtCover_NearDefendGoal.CoverGoalConstraints.Length)
            {
                Proximity = CovGoal_GoalProximity(AtCover_NearDefendGoal.CoverGoalConstraints[nIndex]);
                if (Proximity != None)
                {
                    Proximity.MaxGoalDist = m_fDistanceFromActorToDefend;
                }
                nIndex++;
            }
            if (MoveToCover(AtCover_NearDefendGoal, m_ActorToDefend, false))
            {
                return true;
            }
        }
    }
    else
    {
        if (FireTarget == None)
        {
            return false;
        }
        if (MoveToCover(AtCover_WeaponRange, FireTarget))
        {
            return true;
        }
    }
    if (MyBP != None && MyBP.IsInCover())
    {
        MyBP.LeaveCover();
    }
    return false;
}

simulated function bool NotifyCoverClaimViolation(Controller NewClaim, CoverLink Link, int SlotIdx)
{
    m_bInvalidatedCover = true;
    MoveTarget = None;
    bReachedMoveGoal = false;
    return true;
}

function ResetFarAwayFlag()
{
    m_bTooFarToAttack = false;
}

function NotifyUnderAttack(bool bHit)
{
    if (MyBP == None)
    {
        return;
    }
    if (IsInState('Combat_Henchman', true) == false)
    {
        return;
    }
    if (IsMovingToCover())
    {
        return;
    }
    if (MyBP.IsInCover())
    {
        if (bHit)
        {
            if (IsLeaning())
            {
                if (IsInState('UsingPower', true) == false)
                {
                    CancelAction(1);
                }
                m_bWaitBeforeNextAttack = true;
            }
            else
            {
                bAcquireNewCover = true;
            }
        }
    }
    else
    {
        bAcquireNewCover = true;
    }
}

state FollowingActor
{
    function bool ShouldCancelFollow()
    {
        local float fDistance;
        if (ShouldCancelFollow())
        {
            return true;
        }
        if (m_bCancelAction && m_nCancelReasons & 2 != 0)
        {
            return true;
        }
        if (SFXPawn_Player(m_ActorToFollow) != None && m_RequestedActorToFollow == None)
        {
            if (IsInCombat())
            {
                fDistance = VSize(m_ActorToFollow.Location - MyBP.Location);
                if (m_bFollowPlayer == false && IsWithinThreatRadius() && fDistance < TetherDistance)
                {
                    return true;
                }
                if (fDistance <= DistanceToStartCombatWhileFollowing)
                {
                    m_bFollowPlayer = false;
                    if (ShouldFollowPlayer() == false)
                    {
                        return true;
                    }
                    jump 209;
                }
            }
        }
        return false;
    }

}

function bool TeleportToActor(Actor oActor, optional bool bForceTeleport)
{
    local bool bResult;
    bResult = TeleportToActor(oActor, bForceTeleport);
    if (bResult)
    {
        MapName_Hench_Teleport(MyBP.Location.X, MyBP.Location.Y);
    }
    return bResult;
}

function bool ShouldFollowPlayer()
{
    local BioPlayerSquad oPlayerSquad;
    local float fDistance;
    if (m_ActorToDefend != None)
    {
        return false;
    }
    if (MyBP == None)
    {
        return false;
    }
    oPlayerSquad = BioPlayerSquad(MyBP.Squad);
    if (oPlayerSquad == None || oPlayerSquad.m_playerPawn == None)
    {
        return false;
    }
    fDistance = VSize(oPlayerSquad.m_playerPawn.Location - MyBP.Location);
    if (m_bHoldingPosition || IsZero(m_vHoldLocation) == false || ForcedTarget != None)
    {
        if (fDistance > TetherDistanceWhileExecutingOrder)
        {
            return true;
        }
        return false;
    }
    if (m_bFollowPlayer && fDistance >= TetherDistanceForFollowOrder)
    {
        return true;
    }
    if (IsInCombat() == false)
    {
        return true;
    }
    if (fDistance >= TetherDistance)
    {
        return true;
    }
    if (FireTarget == None && SelectTarget() == false)
    {
        return true;
    }
    if (m_bTooFarToAttack)
    {
        return true;
    }
    return false;
}

function bool IsInMoveTo()
{
    return !IsZero(m_vHoldLocation);
}

function bool IsInCombat()
{
    if (MyBP == None)
    {
        return false;
    }
    return MyBP.InCombat();
}

function SetSquadIntoCombat()
{
    local BioPlayerSquad oPlayerSquad;
    if (MyBP == None)
    {
        return;
    }
    oPlayerSquad = BioPlayerSquad(MyBP.Squad);
    if (oPlayerSquad == None)
    {
        return;
    }
    oPlayerSquad.SquadEnterCombatMode();
}

function bool WantsToRun(float fDistance)
{
    if (IsInState('MovingAwayFromPlayer', true) == true)
    {
        return true;
    }
    if (CoverSlotMarker(MoveGoal) != None)
    {
        return true;
    }
    return WantsToRun(fDistance);
}

state MovingToGoal
{
    event PoppedState()
    {
        PoppedState();
        if (m_bCheckPlayerReachability)
        {
            m_bCheckPlayerReachability = false;
            if (!IsPlayerReachable())
            {
                m_bResetHenchman = true;
                CancelAction(2);
            }
        }
    }

    function float GetPeriodicMoveInterval()
    {
        return 1;
    }

    function bool ApplyPathConstraints()
    {
        local Vector LastMantleLocation;
        if (m_bBackingAwayFromTarget == false)
        {
            return ApplyPathConstraints();
        }
        if (MyBP != None && MoveGoal != None)
        {
            if (Class'Goal_AwayFromPosition'.FleeFrom(MyBP, MoveGoal.Location, MaxBackAwayDistance))
            {
                Class'SFXPath_AvoidClaimedCover'.AvoidClaimedCover(MyBP);
                if (TimeSince(MyBP.LastMantleTime) < 5)
                {
                    LastMantleLocation = MyBP.LastMantleLocation;
                }
                if (CustomActionClasses.Length > 10)
                {
                    if (CustomActionClasses[9] != None && CustomActionClasses[10] != None)
                    {
                        Class'Path_MinDistBetweenSpecsOfType'.EnforceMinDist(MyBP, 500, Class'MantleReachSpec', LastMantleLocation);
                    }
                }
                Class'SFXPath_AvoidFireFromCover'.AvoidFireFromCover(MyBP, m_bAvoidFireFromPlayerOnly);
                return true;
            }
        }
        return false;
    }

    function FinishPathGeneration()
    {
        if (m_bBackingAwayFromTarget == false)
        {
            FinishPathGeneration();
            return;
        }
        MoveGoal = RouteGoal;
    }

    function bool GetFirstMoveTarget(out Actor FirstMoveTarget)
    {
        if (m_bBackingAwayFromTarget == false)
        {
            return GetFirstMoveTarget(FirstMoveTarget);
        }
        if (ApplyPathConstraints() == false)
        {
            return false;
        }
        FirstMoveTarget = FindPathToward(MoveGoal, false, 2000);
        if (FirstMoveTarget == None)
        {
            return false;
        }
        if (FirstMoveTarget == RouteCache[0])
        {
            RouteCache_RemoveIndex(0);
        }
        FinishPathGeneration();
        m_nMoveAttemptCounter = 0;
        LogRoute();
        return true;
    }

    function CancelAction(optional int nReason)
    {
        if (nReason == 2)
        {
            CancelAction();
        }
    }

}

state Dying
{
    MyBP.SetCollision(true, false);
    MyBP.Mesh.SetRBChannel(7);
    MyBP.Mesh.SetRBCollidesWithChannel(2, false);
    while (IsInCombat() && MyBP.IsInState('Downed'))
    {
        Sleep(0,5);
    }
    GotoState('Resurrecting');
    // invalid native function 205 ;
}

state Resurrecting
{
    function bool ResurrectHenchman()
    {
        local BioPlayerSquad oPlayerSquad;
        if (MyBP == None)
        {
            return false;
        }
        oPlayerSquad = BioPlayerSquad(MyBP.Squad);
        if (oPlayerSquad == None)
        {
            return false;
        }
        if (MyBP.Physics != 10 && MyBP.Mesh == MyBP.CollisionComponent)
        {
            MyBP.SetPhysics(10);
        }
        return MyBP.Resurrect(oPlayerSquad.GetPercentHealthOnResurrection(), false);
    }

Begin:
    if (MyBP.IsDead())
    {
        while (ResurrectHenchman() == false)
        {
            Sleep(1);
        }
    }
    MyBP.SetCollision(true, true);
    if (m_bResetHenchman)
    {
        MyBP.ForceEndRagdoll();
    }
    while (MyBP.IsInState('InRagdoll', true))
    {
        Sleep(0,1);
    }
    m_bFollowPlayer = false;
    m_vHoldLocation = vect(0,0,0);
    m_bHoldingPosition = false;
    ForcedTarget = None;
    if (m_nEnabledFlags != 0)
    {
        GotoState('Disabled');
    }
    else
    {
        GotoState('Auto');
    }
}

event bool NotifyBump(Actor Other, Vector HitNormal)
{
    if (MyBP == None)
    {
        return false;
    }
    if (SFXPawn_Player(Other) == None && SFXPawn_Henchman(Other) == None)
    {
        return false;
    }
    if (IsInState('MovingToHoldLocation', true))
    {
        if (IsInState('MovingToLocation', true))
        {
            if (VSize(MyBP.Location - MovePoint) <= 250)
            {
                bReachedMoveGoal = true;
                return true;
            }
        }
    }
    return false;
}

function bool RespondToBump(Actor Other, Vector HitNormal)
{
    if (SFXPawn_Player(Other) != None)
    {
        if (IsInState('MovingAwayFromPlayer', true))
        {
            return false;
        }
        SFXGame(WorldInfo.Game).TriggerVocalizationEvent(80, BioPawn(Other), MyBP);
        GotoState('MovingAwayFromPlayer');
        return true;
    }
    return false;
}

state MovingAwayFromPlayer
{
    function BeginState(Name PreviousStateName)
    {
        BeginState(PreviousStateName);
        if (MyBP != None)
        {
            m_bOriginalStartRootMotion = MyBP.m_bEnableStartRootMotion;
            m_bOriginalStopRootMotion = MyBP.m_bEnableStopRootMotion;
            MyBP.m_bEnableStartRootMotion = false;
            MyBP.m_bEnableStopRootMotion = false;
        }
    }

    event EndState(Name NextStateName)
    {
        if (MyBP != None)
        {
            MyBP.StopMovement(false);
            MyBP.m_bEnableStartRootMotion = m_bOriginalStartRootMotion;
            MyBP.m_bEnableStopRootMotion = m_bOriginalStopRootMotion;
        }
        EndState(NextStateName);
    }

    function bool GetMoveLocation(out Vector vLocation)
    {
        local BioPlayerSquad oPlayerSquad;
        local PlayerController oPlayerController;
        local Vector vCameraLocation;
        local Vector vPlayerDirection;
        local Vector vUp;
        local Vector vNewDirection;
        local Rotator rCameraRotation;
        local Rotator rToHenchman;
        if (bAILogging)
        {
            FlushPersistentDebugLines();
        }
        if (MyBP == None || WorldInfo == None)
        {
            return false;
        }
        oPlayerSquad = BioPlayerSquad(MyBP.Squad);
        if (oPlayerSquad == None || oPlayerSquad.m_playerPawn == None)
        {
            return false;
        }
        oPlayerController = PlayerController(oPlayerSquad.m_playerPawn.Controller);
        if (oPlayerController == None || oPlayerController.PlayerCamera == None)
        {
            return false;
        }
        vCameraLocation = oPlayerController.PlayerCamera.CameraCache.POV.Location;
        vPlayerDirection = Normal(MyBP.Location - vCameraLocation);
        vPlayerDirection.Z = 0;
        vPlayerDirection = Normal(vPlayerDirection);
        vUp.Z = 1;
        vNewDirection = vPlayerDirection Cross vUp;
        rCameraRotation = Normalize(oPlayerController.PlayerCamera.CameraCache.POV.Rotation);
        rToHenchman = MyBP.Location - oPlayerSquad.m_playerPawn.Location;
        if (rToHenchman.Yaw - rCameraRotation.Yaw >= 0)
        {
            vNewDirection *= -1;
        }
        vLocation = MyBP.Location + vNewDirection * 200;
        if (bAILogging)
        {
            DrawDebugLine(MyBP.Location, vLocation, 255, 0, 0, true);
        }
        if (PointReachable(vLocation) == false)
        {
            vNewDirection *= -1;
            vLocation = MyBP.Location + vNewDirection * 200;
            if (bAILogging)
            {
                DrawDebugLine(MyBP.Location, vLocation, 0, 255, 0, true);
            }
            if (PointReachable(vLocation) == false)
            {
                vLocation = MyBP.Location + vPlayerDirection * 200;
                if (bAILogging)
                {
                    DrawDebugLine(MyBP.Location, vLocation, 0, 0, 255, true);
                }
                if (PointReachable(vLocation) == false)
                {
                    return false;
                }
                jump 775;
            }
            jump 778;
        }
        return true;
    }

    if (MyBP == None)
    {
        GotoState('Idle');
    }
    if (GetMoveLocation(MovePoint) == false)
    {
        GotoState('Idle');
    }
    MoveOffset = 0;
    MoveTimer = 5;
    SetMovementSpeed();
    PushState('MovingToLocation');
    GotoState('Idle');
    jump 0;
    // unresolved reference 0 ;
}

function bool ChooseAttackPower(Actor oTarget, out Name nmPower, out int nRequiresAttackTicket, out Vector AttackOrigin)
{
    return ChooseAttackPowerHelper(oTarget, false, nmPower, nRequiresAttackTicket, AttackOrigin);
}

function bool ChooseAttackPowerHelper(Actor oTarget, bool bPlayerRequest, out Name nmPower, out int nRequiresAttackTicket, out Vector AttackOrigin)
{
    if (bPlayerRequest || CanUsePowers(true))
    {
        return ChooseAttackPower(oTarget, nmPower, nRequiresAttackTicket, AttackOrigin);
    }
    return false;
}

function bool ChooseDefensivePower(out Name nmPower)
{
    if (CanUsePowers(false) == false)
    {
        return false;
    }
    return ChooseDefensivePower(nmPower);
}

function bool CanUsePowers(bool bCheckProfileOption)
{
    local Pawn Player;
    local PlayerController Controller;
    local SFXPRI PRI;
    if (m_bDelayPowerUse)
    {
        return false;
    }
    if (m_OtherHenchman != None)
    {
        if (m_OtherHenchman.IsInState('UsingPower', true) || m_OtherHenchman.ArePowersCoolingDown())
        {
            return false;
        }
    }
    if (bCheckProfileOption && MyBP != None && MyBP.Squad != None)
    {
        Player = MyBP.Squad.GetSquadLeader();
        if (Player != None)
        {
            Controller = PlayerController(Player.Controller);
            if (Controller != None)
            {
                PRI = SFXPRI(Controller.PlayerReplicationInfo);
                if (PRI != None && PRI.bSquadUsesPowers == false)
                {
                    return false;
                }
            }
        }
    }
    return true;
}

state MovingToHoldLocation
{
    event PushedState()
    {
        PushedState();
        SetTimer(0,5 + FRand() * 0,5, true, 'SelectTarget');
    }

    event PoppedState()
    {
        ClearTimer('SelectTarget');
        PoppedState();
    }

    event ContinuedState()
    {
        ContinuedState();
        if (m_bCancelAction && m_nCancelReasons & 2 != 0)
        {
            PopState();
            return;
        }
    }

    function bool MoveToCoverNearHoldLocation()
    {
        local Actor oMoveIndicator;
        local CoverLink OriginalCoverLink;
        if (MyBP == None || MyBP.Squad == None)
        {
            return false;
        }
        oMoveIndicator = MyBP.Squad.GetMemberMoveIndicator(MyBP.Squad.GetMemberIndex(MyBP));
        if (oMoveIndicator == None)
        {
            return false;
        }
        OriginalCoverLink = MyBP.CurrentLink;
        if (MoveToCover(AtCover_NearMoveGoal, oMoveIndicator, false) == false)
        {
            return false;
        }
        MyBP.ReadyMode();
        if (OriginalCoverLink == None || CoverGoal.Link != None && CoverGoal.Link != OriginalCoverLink)
        {
            SFXGame(WorldInfo.Game).TriggerVocalizationEvent(9, MyBP);
        }
        else
        {
            SFXGame(WorldInfo.Game).TriggerVocalizationEvent(7, MyBP);
        }
        return true;
    }

    function bool PathfindToHoldLocation()
    {
        local Actor oMoveIndicator;
        if (MyBP == None || MyBP.Squad == None)
        {
            return false;
        }
        oMoveIndicator = MyBP.Squad.GetMemberMoveIndicator(MyBP.Squad.GetMemberIndex(MyBP));
        if (oMoveIndicator == None)
        {
            return false;
        }
        MoveToGoal(oMoveIndicator);
        return true;
    }

    m_bHoldingPosition = false;
    if (MyBP == None)
    {
        PopState();
    }
    if (IsZero(m_vHoldLocation))
    {
        PopState();
    }
    if (MoveToCoverNearHoldLocation())
    {
        if (bReachedMoveGoal && MyBP.IsInCover())
        {
            m_bHoldingPosition = true;
            PopState();
        }
    }
    else
    {
        SFXGame(WorldInfo.Game).TriggerVocalizationEvent(7, MyBP);
    }
    if (DirectWalkCheck(m_vHoldLocation, None))
    {
        MovePoint = m_vHoldLocation;
        MoveOffset = 0;
        MoveTimer = 20;
        PushState('MovingToLocation');
    }
    else
    {
        PathfindToHoldLocation();
    }
    if (IsInCombat())
    {
        if (FireTarget == None || VSize(FireTarget.Location - MyBP.Location) > MinDistanceFromTargetForCrouch)
        {
            MyBP.ShouldCrouch(true);
        }
    }
    m_bHoldingPosition = true;
    if (bReachedMoveGoal == false)
    {
        SFXGame(WorldInfo.Game).TriggerVocalizationEvent(12, MyBP);
    }
    PopState();
    jump 0;
    // unresolved reference 0 ;
}

function bool ShouldAttack()
{
    local int nPlayerIsFiring;
    if (MyBP == None)
    {
        return false;
    }
    if (MyBP.IsInCover())
    {
        if (MyBP.GetHealthPct() < DamagePercentToRemainInCover)
        {
            return false;
        }
        if (IsInPlayerLineOfFire(nPlayerIsFiring))
        {
            return false;
        }
    }
    return true;
}

event OnEnteringStasis()
{
    local BioWorldInfo oWorldInfo;
    local BioPlayerSquad oPlayerSquad;
    oWorldInfo = BioWorldInfo(WorldInfo);
    if (oWorldInfo != None && oWorldInfo.m_bWaitingForStreamingLoadVisibleComplete == false)
    {
        appScreenDebugMessage(Pawn $ " has been put into stasis.  The henchman will be teleported to the player.");
        if (MyBP != None)
        {
            MyBP.bStasis = false;
            m_bResetHenchman = true;
            if (MyBP.IsDead())
            {
                oPlayerSquad = BioPlayerSquad(MyBP.Squad);
                if (oPlayerSquad != None)
                {
                    if (MyBP.Physics != 10 && MyBP.Mesh == MyBP.CollisionComponent)
                    {
                        MyBP.SetPhysics(10);
                    }
                    MyBP.Resurrect(oPlayerSquad.GetPercentHealthOnResurrection(), true);
                }
            }
            if (MyBP.IsInState('InRagdoll', true))
            {
                MyBP.GotoState('Auto');
            }
            else
            {
                if (IsInState('Idle') == false)
                {
                    GotoState('Idle');
                }
            }
            return;
        }
    }
    OnEnteringStasis();
    bStasis = false;
}

function NotifyHenchmenLoaded()
{
    local Pawn oSquadMember;
    local SFXAI_Henchman oController;
    local int nCount;
    local int nIndex;
    if (MyBP == None || MyBP.Squad == None)
    {
        return;
    }
    nCount = MyBP.Squad.GetSquadSize();
    if (nCount >= 3)
    {
        nIndex = 0;
        while (nIndex < nCount)
        {
            oSquadMember = MyBP.Squad.GetMember(nIndex);
            if (oSquadMember != None && oSquadMember != Pawn)
            {
                oController = SFXAI_Henchman(oSquadMember.Controller);
                if (oController != None)
                {
                    m_OtherHenchman = oController;
                    oController.m_OtherHenchman = self;
                }
            }
            nIndex++;
        }
    }
}

function bool ArePowersCoolingDown()
{
    if (MyBP == None || MyBP.PowerManager == None)
    {
        return false;
    }
    if (MyBP.PowerManager.GetSharedCooldownRemaining() > 0)
    {
        return true;
    }
    return false;
}

function NotifySharedCooldownFinished()
{
    local SFXPawn_Henchman oPawn;
    oPawn = SFXPawn_Henchman(MyBP);
    if (oPawn != None && oPawn.PowerUseDelay > 0)
    {
        m_bDelayPowerUse = true;
        SetTimer(oPawn.PowerUseDelay, false, 'NotifyPowerDelayFinished');
    }
}

function NotifyPowerDelayFinished()
{
    m_bDelayPowerUse = false;
}

function Vector GetAimLocation(int EnemyIdx, Vector StartLoc)
{
    local BioPawn oEnemyPawn;
    local Vector vHeadLocation;
    if (SFXWeapon(Pawn.Weapon).bUseSniperCam)
    {
        if (FRand() <= m_fHeadshotProbability)
        {
            if (EnemyIdx >= 0)
            {
                oEnemyPawn = BioPawn(EnemyList[EnemyIdx].Pawn);
            }
            else
            {
                oEnemyPawn = BioPawn(FireTarget);
            }
            if (oEnemyPawn != None)
            {
                if (oEnemyPawn.GetAimNodeLocation(1, vHeadLocation, false))
                {
                    return vHeadLocation;
                }
            }
        }
    }
    return GetAimLocation(EnemyIdx, StartLoc);
}

state FiringWeapon
{
    function ExecuteOrders()
    {
        if (m_Orders.Length > 0)
        {
            if (m_Orders[0].eOrderType == 1)
            {
                if (MyBP.IsInCover() && MyBP.IsInCoverLeaning())
                {
                    m_bStayLeanedOut = true;
                }
            }
        }
        Global.ExecuteOrders();
    }

    event bool NotifyBump(Actor Other, Vector HitNormal)
    {
        return RespondToBump(Other, HitNormal);
    }

}

state WaitingForAnimatedTransition
{
    function ExecuteOrders()
    {
        if (m_Orders.Length > 0)
        {
            if (m_Orders[0].eOrderType == 1)
            {
                if (IsInState('FiringWeapon', true))
                {
                    if (MyBP.IsInCover() && MyBP.IsInCoverLeaning())
                    {
                        m_bStayLeanedOut = true;
                    }
                }
            }
        }
        Global.ExecuteOrders();
    }

}

function bool StartFollowingActor(Actor ActorToFollow)
{
    if (MyBP == None || MyBP.IsDead() || ActorToFollow == None)
    {
        return false;
    }
    m_RequestedActorToFollow = ActorToFollow;
    CancelAction(2);
    return true;
}

function bool StartDefendingActor(Actor ActorToDefend, float fMaxDistanceFromActor)
{
    if (MyBP == None || MyBP.IsDead() || ActorToDefend == None)
    {
        return false;
    }
    m_ActorToDefend = ActorToDefend;
    m_fDistanceFromActorToDefend = fMaxDistanceFromActor;
    CancelAction(2);
    return true;
}

function StopDefendingActor()
{
    m_ActorToDefend = None;
    m_fDistanceFromActorToDefend = 0;
}

function bool IsInPlayerLineOfFire(out int nPlayerIsFiring)
{
    local BioPlayerSquad PlayerSquad;
    local BioPawn Player;
    local BioPlayerController PlayerController;
    local SFXPlayerCamera PlayerCamera;
    local Vector vHenchmanLocationWithoutZ;
    local Vector vCameraRotationWithoutZ;
    local float fAngle;
    local float fDistance;
    local float fLineOfFireAngle;
    if (MyBP == None || MyBP.IsDead())
    {
        return false;
    }
    if (MyBP.GetTimeSinceLastRender() > 1)
    {
        return false;
    }
    PlayerSquad = BioPlayerSquad(MyBP.Squad);
    if (PlayerSquad == None)
    {
        return false;
    }
    Player = PlayerSquad.m_playerPawn;
    if (Player == None || Player.Weapon == None)
    {
        return false;
    }
    PlayerController = BioPlayerController(Player.Controller);
    if (PlayerController == None)
    {
        return false;
    }
    PlayerCamera = SFXPlayerCamera(PlayerController.PlayerCamera);
    if (PlayerCamera == None)
    {
        return false;
    }
    nPlayerIsFiring = Player.Weapon.IsFiring();
    if (PlayerController.IsZoomed() == false && nPlayerIsFiring == 0)
    {
        return false;
    }
    fDistance = VSize2D(MyBP.Location - Player.Location);
    if (fDistance > 1000)
    {
        return false;
    }
    vHenchmanLocationWithoutZ = MyBP.Location - PlayerCamera.CameraCache.POV.Location;
    vHenchmanLocationWithoutZ.Z = 0;
    vCameraRotationWithoutZ = PlayerCamera.CameraCache.POV.Rotation;
    vCameraRotationWithoutZ.Z = 0;
    fAngle = GetAngleBetween(vHenchmanLocationWithoutZ, vCameraRotationWithoutZ) * 57,29578;
    fLineOfFireAngle = 6;
    if (fDistance <= 500)
    {
        fLineOfFireAngle *= 2;
    }
    if (fAngle <= fLineOfFireAngle)
    {
        return true;
    }
    return false;
}

function CheckPlayerLineOfFire()
{
    local int nPlayerIsFiring;
    if (MyBP == None)
    {
        return;
    }
    if (IsInPlayerLineOfFire(nPlayerIsFiring))
    {
        if (MyBP.IsInCover())
        {
            if (IsLeaning())
            {
                if (IsInState('UsingPower', true) == false)
                {
                    CancelAction(16);
                }
            }
        }
        else
        {
            if (MyBP.bIsCrouched == false && IsZero(MyBP.Velocity))
            {
                if (nPlayerIsFiring != 0)
                {
                    SFXGame(WorldInfo.Game).TriggerVocalizationEvent(81, MyBP);
                }
                MyBP.ShouldCrouch(true);
                ClearTimer('CheckPlayerLineOfFire');
                SetTimer(3, true, 'CheckLineOfFireClear');
            }
        }
    }
}

function CheckLineOfFireClear()
{
    local int nPlayerIsFiring;
    if (MyBP == None)
    {
        return;
    }
    if (MyBP.bIsCrouched)
    {
        if (IsInPlayerLineOfFire(nPlayerIsFiring))
        {
            return;
        }
        if (MyBP.IsInCover() == false)
        {
            MyBP.ShouldCrouch(false);
        }
    }
    ClearTimer('CheckLineOfFireClear');
    if (IsInState('Idle', true))
    {
        SetTimer(0,5, true, 'CheckPlayerLineOfFire');
    }
}

function CheckMeleeAttacker()
{
    local Vector HitLocation;
    local Vector HitNormal;
    local Actor HitActor;
    m_bMeleeAttacker = false;
    if (MyBP == None || FireTarget == None)
    {
        return;
    }
    if (VSize(FireTarget.Location - MyBP.Location) > 700)
    {
        return;
    }
    HitActor = FireTarget.Trace(HitLocation, HitNormal, MyBP.Location, FireTarget.Location, true, vect(0,0,0));
    if (HitActor != None && HitActor != MyBP)
    {
        return;
    }
    m_bMeleeAttacker = true;
}

function OnTargetChanged()
{
    local BioPlayerSquad PlayerSquad;
    if (bAILogging)
    {
        PlayerSquad = BioPlayerSquad(MyBP.Squad);
        PlayerSquad.m_playerPawn.ClientMessage(MyBP.m_oBehavior.GetActorGameName() $ ": " $ FireTarget);
    }
    CheckMeleeAttacker();
    CheckThreatRadius();
    m_bTooFarToAttack = false;
    if (IsInState('Combat_Henchman', true))
    {
        CancelAction(8);
    }
}

function ModifyBurstDelay(SFXWeapon Weapon, out float BurstDelay)
{
    if (Weapon != None)
    {
        BurstDelay = BurstDelay * RandRange(Weapon.AI_HenchBurstFireMultiplier.X, Weapon.AI_HenchBurstFireMultiplier.Y);
    }
}

function ApplyOrderBonus()
{
    local SFXModule_GameEffectManager Manager;
    local SFXGameEffect Effect;
    if (MyBP != None)
    {
        Manager = MyBP.GetModule(Class'SFXModule_GameEffectManager');
        if (Manager != None)
        {
            Manager.RemoveEffectsByCategory('HenchmanAttackOrderBonus');
            Manager.RemoveEffectsByCategory('HenchmanMoveOrderBonus');
            Effect = Manager.CreateEffect(Class'SFXGameEffect_PassiveWeaponDamageBonus', 'HenchmanAttackOrderBonus', 3, 1, 0,5);
            if (Effect != None)
            {
                Effect.OnApplied();
            }
            if (Manager.HasEffectOfType(Class'SFXGameEffect_DamageMultiplier') == false)
            {
                Effect = Manager.CreateEffect(Class'SFXGameEffect_DamageMultiplier', 'HenchmanMoveOrderBonus', 5, 1, 0,6);
                if (Effect != None)
                {
                    Effect.OnApplied();
                }
            }
        }
    }
}

state MovingToLocation
{
    function bool ShouldCancelMove()
    {
        if (m_bCancelAction && m_nCancelReasons & 2 != 0)
        {
            m_nMoveCompletionReason = 0;
            return true;
        }
        return false;
    }

}

function NotifyNewEnemy(Pawn NewEnemy)
{
    m_bEnemiesPerceived = true;
}

function CheckInCombat()
{
    if (m_bEnemiesPerceived && MyBP == None || MyBP.InCombat() == false)
    {
        m_bEnemiesPerceived = false;
        m_vHoldLocation = vect(0,0,0);
        m_bHoldingPosition = false;
    }
}

function AdjustRatingByTickets(out float out_Rating, int EnemyIdx);

function AdjustEnemyRating(Pawn EnemyPawn, out float fRating)
{
    local BioPawn EnemyBioPawn;
    EnemyBioPawn = BioPawn(EnemyPawn);
    if (EnemyBioPawn != None)
    {
        if (EnemyBioPawn.GetHealthPct() <= 0,5)
        {
            fRating *= 1,25;
        }
    }
}

function bool ShouldWaitForPlayer()
{
    local BioPlayerSquad PlayerSquad;
    local float PlayerDistance;
    local float HenchmanDistance;
    if (MyBP != None && FireTarget != None)
    {
        PlayerSquad = MyBP != None ? BioPlayerSquad(MyBP.Squad) : None;
        if (PlayerSquad != None && PlayerSquad.m_playerPawn != None)
        {
            PlayerDistance = VSize(FireTarget.Location - PlayerSquad.m_playerPawn.Location);
            HenchmanDistance = VSize(FireTarget.Location - MyBP.Location);
            if (HenchmanDistance + 1000 < PlayerDistance)
            {
                return true;
            }
        }
    }
    return false;
}

function bool IsPlayerReachable()
{
    local Pawn oPlayer;
    local bool bReachable;
    local Actor oFirstMoveTarget;
    if (MyBP == None || MyBP.Squad == None)
    {
        return true;
    }
    oPlayer = MyBP.Squad.GetSquadLeader();
    if (oPlayer == None)
    {
        return true;
    }
    bReachable = false;
    if (DirectWalkCheck(oPlayer.Location, oPlayer))
    {
        bReachable = true;
    }
    else
    {
        if (Class'Goal_AtActor'.AtActor(MyBP, oPlayer, 0, false))
        {
            Class'Path_TowardGoal'.TowardGoal(MyBP, oPlayer);
            oFirstMoveTarget = FindPathToward(oPlayer, false, 2000);
            if (oFirstMoveTarget != None)
            {
                bReachable = true;
            }
            RouteCache_Empty();
        }
    }
    return bReachable;
}

function OnCombatStart()
{
    if (!IsInState('FollowingActor', true))
    {
        if (IsInState('MovingToGoal', true))
        {
            m_bCheckPlayerReachability = true;
        }
        else
        {
            if (!IsPlayerReachable())
            {
                m_bResetHenchman = true;
                CancelAction(2);
            }
        }
    }
}

function InstantUsePower()
{
    local Actor TargetActor;
    local Vector TargetLocation;
    local Name PowerName;
    local SFXPower Power;
    local EBioPowerType PowerType;
    local float CooldownRemaining;
    local BioActivePower PowerActive;
    if (MyBP == None || MyBPBehavior == None || MyBP.PowerManager == None)
    {
        return;
    }
    if (m_Orders.Length == 0 || m_Orders[0].eOrderType != 1)
    {
        return;
    }
    if (m_Orders[0].bPowerUseIsInstant == false)
    {
        return;
    }
    if (m_Orders[0].bExecutingOrder == false)
    {
        if (CanInstantlyUsePowers() == false)
        {
            m_Orders[0].bPowerUseIsInstant = false;
            return;
        }
    }
    TargetActor = m_Orders[0].oTargetActor;
    TargetLocation = m_Orders[0].vTargetLocation;
    PowerName = m_Orders[0].nmPower;
    m_Orders.Remove(0, 1);
    Power = MyBP.PowerManager.GetPower(PowerName);
    if (Power == None)
    {
        return;
    }
    if (Power.IsEnabled() == false)
    {
        return;
    }
    if (Power.GetCooldownRemaining() > 0)
    {
        return;
    }
    PowerType = Power.PowerType;
    if (PowerType == 3)
    {
        TargetActor = None;
        TargetLocation = MyBP.Location + 100 * MyBP.Rotation;
    }
    if (MyBPBehavior.SetCurrentPower(PowerName, CooldownRemaining) == false)
    {
        return;
    }
    PowerActive = new(MyBP,Class'BioActivePower');
    if (PowerActive == None)
    {
        return;
    }
    PowerActive.Initialize(MyBP);
    PowerActive.SetTargetActor(TargetActor);
    PowerActive.SetTargetLocation(TargetLocation);
    PowerActive.UpdatePowerOrder(true, true);
    PowerActive.m_bInstantPowerUse = true;
    if (PowerActive.StartActiveFromScript() == false)
    {
        return;
    }
    MyBPBehavior.m_InstantPowerActive = PowerActive;
    SFXGame(WorldInfo.Game).TriggerVocalizationEvent(19, MyBP, BioPawn(TargetActor));
}

function bool CanInstantlyUsePowers()
{
    if (MyBP != None && MyBP.GetTimeSinceLastRender() >= 0,25 || m_bAllowInstantPowerWhileVisible)
    {
        return true;
    }
    return false;
}

function NotifyKnockedOutOfCover()
{
    bAcquireNewCover = true;
}

function bool IsWithinThreatRadius()
{
    local BioPawn TargetPawn;
    TargetPawn = BioPawn(FireTarget);
    if (MyBP != None && TargetPawn != None)
    {
        if (m_bMeleeAttacker)
        {
            return true;
        }
        if (TargetPawn.ThreatRadiusSquared > 0 && VSizeSq(TargetPawn.Location - MyBP.Location) <= TargetPawn.ThreatRadiusSquared)
        {
            return true;
        }
    }
    return false;
}

function CheckThreatRadius()
{
    if (IsWithinThreatRadius())
    {
        if (bAcquireNewCover == false && IsInState('MovingToCover', true) == false)
        {
            bAcquireNewCover = true;
        }
    }
}

state BackingAwayFromTarget
{
    event PushedState()
    {
        PushedState();
        m_bBackingAwayFromTarget = true;
    }

    event PoppedState()
    {
        m_bBackingAwayFromTarget = false;
        PoppedState();
    }

Begin:
    m_nBackingAwayCount++;
    if (FireTarget != None)
    {
        MoveToGoal(FireTarget);
    }
    PopState();
}

defaultproperties
{
    TetherDistance=4000
    TetherDistanceWhileExecutingOrder=4000
    TetherDistanceForFollowOrder=500
    DistanceToStartCombatWhileFollowing=800
    AttackDelayAfterBeingShot=1
    DamagePercentToRemainInCover=1
    m_fHeadshotProbability=1
    MinDistanceFromTargetForCrouch=400
    AtCover_NearDefendGoal=Goal_AtCover'AtCov_EvalDefendGoal'
    MaxBackAwayDistance=2000
    m_nmCurrentBehavior=Combat_Henchman
    m_bUsePowerReservations=false
    m_bAvoidDangerLinks=true
    m_bAvoidFireFromPlayerOnly=false
    m_fSpecialMoveTimeDelay=1
    AtCover_WeaponRange=Goal_AtCover'AtCov_EvalWeaponRange'
    AtCover_Defensive=Goal_AtCover'AtCov_EvalDefensive'
    AtCover_Aggressive=Goal_AtCover'AtCov_EvalAggressive'
    AtCover_NearMoveGoal=Goal_AtCover'AtCov_EvalNearGoal'
    AtCover_AIGoHere=Goal_AtGoHereCover'AtCov_EvalGoHere'
    m_fRunThreshold=350
    CustomActionClasses(0)=None
    CustomActionClasses(1)=None
    CustomActionClasses(2)=None
    CustomActionClasses(3)=None
    CustomActionClasses(4)=class'SFXCustomAction_Frozen'
    CustomActionClasses(5)=class'SFXCustomAction_Ragdoll'
    CustomActionClasses(6)=None
    CustomActionClasses(7)=None
    CustomActionClasses(8)=class'BioCustomAction_CombatActivate'
    CustomActionClasses(9)=class'BioCustomAction_CoverClimb'
    CustomActionClasses(10)=class'BioCustomAction_CoverMantle'
    CustomActionClasses(11)=class'BioCustomAction_WaypointAnimation'
    CustomActionClasses(12)=None
    CustomActionClasses(13)=None
    CustomActionClasses(14)=class'SFXCustomAction_Knockback'
    CustomActionClasses(15)=None
    CustomActionClasses(16)=None
    CustomActionClasses(17)=None
    CustomActionClasses(18)=None
    CustomActionClasses(19)=None
    CustomActionClasses(20)=None
    CustomActionClasses(21)=None
    CustomActionClasses(22)=None
    CustomActionClasses(23)=None
    CustomActionClasses(24)=None
    CustomActionClasses(25)=None
    CustomActionClasses(26)=None
    CustomActionClasses(27)=class'SFXCustomAction_DeathKnockback'
    CustomActionClasses(28)=None
    CustomActionClasses(29)=None
    CustomActionClasses(30)=None
    CustomActionClasses(31)=None
    CustomActionClasses(32)=None
    CustomActionClasses(33)=None
    CustomActionClasses(34)=None
    CustomActionClasses(35)=None
    CustomActionClasses(36)=None
    CustomActionClasses(37)=None
    CustomActionClasses(38)=None
    CustomActionClasses(39)=None
    CustomActionClasses(40)=None
    CustomActionClasses(41)=None
    CustomActionClasses(42)=None
    CustomActionClasses(43)=None
    CustomActionClasses(44)=None
    CustomActionClasses(45)=None
    CustomActionClasses(46)=None
    CustomActionClasses(47)=class'SFXCustomAction_PlayerStagger'
    CustomActionClasses(48)=class'SFXCustomAction_PlayerMeleed'
    CustomActionClasses(49)=None
    CustomActionClasses(50)=None
    CustomActionClasses(51)=class'SFXCustomAction_PlayerStandardImpact'
    CustomActionClasses(52)=class'SFXCustomAction_PlayerKnockback'
    CustomActionClasses(53)=None
    CustomActionClasses(54)=None
    CustomActionClasses(55)=None
    CustomActionClasses(56)=None
    CustomActionClasses(57)=None
    CustomActionClasses(58)=class'SFXCustomAction_Meleed'
    CustomActionClasses(59)=None
    CustomActionClasses(60)=None
    CustomActionClasses(61)=None
    CustomActionClasses(62)=None
    CustomActionClasses(63)=class'SFXCustomAction_BloodyPlayerStandardImpact'
    CustomActionClasses(64)=class'SFXCustomAction_PlayerOnFire'
    CustomActionClasses(65)=None
    CustomActionClasses(66)=None
    CustomActionClasses(67)=None
    CustomActionClasses(68)=None
    CustomActionClasses(69)=None
    CustomActionClasses(70)=None
    CustomActionClasses(71)=class'SFXCustomAction_ElectricStandardImpact'
    CustomActionClasses(72)=None
    CustomActionClasses(73)=None
    CustomActionClasses(74)=None
    CustomActionClasses(75)=class'SFXCustomAction_MeleedII'
    CustomActionClasses(76)=None
    CustomActionClasses(77)=None
    CustomActionClasses(78)=None
    CustomActionClasses(79)=None
    CustomActionClasses(80)=None
    CustomActionClasses(81)=None
    CustomActionClasses(82)=None
    CustomActionClasses(83)=None
    CustomActionClasses(84)=None
    CustomActionClasses(85)=None
    CustomActionClasses(86)=class'SFXCustomAction_PlayerStandardImpact'
    CustomActionClasses(87)=class'SFXCustomAction_PlayerMeleed'
    CustomActionClasses(88)=class'SFXCustomAction_PlayerStagger'
    CustomActionClasses(89)=None
    CustomActionClasses(90)=class'SFXCustomAction_PlayerKnockback'
    // child object CovGoal_AvoidHenchEnemies of type CovGoal_AvoidEnemies
    // child object CovGoal_HenchEnemies of type CovGoal_Enemies
    // child object CovGoal_NearDefendGoal of type CovGoal_GoalProximity
    // child object CovGoal_HenchMovDistWeapon of type CovGoal_MovementDistance
    // child object CovGoal_HenchTeamProx of type CovGoal_TeammateProximity
    // child object CovGoal_HenchWeaponRange of type CovGoal_WeaponRange
    // child object AtCov_EvalAggressive of type Goal_AtCover
    // child object AtCov_EvalDefendGoal of type Goal_AtCover
    // child object AtCov_EvalDefensive of type Goal_AtCover
    // child object AtCov_EvalNearGoal of type Goal_AtCover
    // child object AtCov_EvalWeaponRange of type Goal_AtCover
    // child object AtCov_EvalGoHere of type Goal_AtGoHereCover
}
