﻿class SFXCameraMode_Interpolate extends SFXCameraMode;

var transient bool bDisableAimPointPreservation;

var Vector LocationOffset;

var Rotator RotationOffset;

var() InterpCurveFloat Curve;

var() EBioInterpolationMethod InterpMethod;

var() Vector AimPoint;

var() float AimPointDistance;

var() float DefaultTime;

var() bool bComplete;

var() SFXCameraMode To;

var() SFXCameraMode From;

var() float CurrentTime;

var() float TotalTime;

function float Interpolate(float T)
{
    local float O;
    Class'BioInterpolator'.InterpolateFloatCurve(O, Curve, 0, 1, T);
    return FClamp(O, 0, 1);
}

function bool CheckLoop(SFXCameraMode Mode, int RecurseLevel)
{
    if (self == Mode)
    {
        return true;
    }
    RecurseLevel -= 1;
    if (RecurseLevel <= 0)
    {
        From = None;
        bComplete = true;
        return false;
    }
    if (From.CheckLoop(self, RecurseLevel) || To.CheckLoop(self, RecurseLevel))
    {
        return true;
    }
    return From.CheckLoop(Mode, RecurseLevel) || To.CheckLoop(Mode, RecurseLevel);
}

function InitializeTransition(SFXCameraMode FromMode, SFXCameraMode ToMode, float Time, bool PreserveTarget)
{
    local Actor A;
    local Vector V;
    bComplete = false;
    CurrentTime = 0;
    TotalTime = Time;
    if (TotalTime == 0)
    {
        TotalTime = 1E-05;
    }
    From = FromMode;
    To = ToMode;
    To.Owner = From.Owner;
    A = None;
    if (PreserveTarget && !bDisableAimPointPreservation)
    {
        Trace(A, V);
    }
    if (A != None)
    {
        AimPointDistance = VSize(V - From.m_pov.Location);
        AimPoint = V;
        To.AimAtPoint(V);
        To.AimAtPoint(V);
        To.DoCameraCollision(To.m_pov.Location, To.m_pov.Rotation, Owner);
        if (To.CollisionDistance > 0,5 || AimPointDistance < 400)
        {
            To.m_pov.Rotation = From.m_pov.Rotation;
            To.m_pov.Location = To.GetCameraLocation();
        }
    }
    else
    {
        To.m_pov.Rotation = From.m_pov.Rotation;
        To.m_pov.Location = To.GetCameraLocation();
    }
    Owner.SetRotation(To.m_pov.Rotation);
    RotationOffset = To.m_pov.Rotation - From.m_pov.Rotation;
    LocationOffset = To.m_pov.Location - From.m_pov.Location;
}

function Trace(out Actor HitActor, out Vector HitLocation)
{
    local Vector TraceStart;
    local Vector TraceEnd;
    local Vector ToPC;
    local Vector cameraRot;
    local Vector Extent;
    local Vector HitNormal;
    local TraceHitInfo HitInfo;
    Extent = vect(0,0,0);
    TraceStart = Owner.PlayerCamera.CameraCache.POV.Location;
    ToPC = Owner.Pawn.Location - TraceStart;
    cameraRot = Owner.PlayerCamera.CameraCache.POV.Rotation;
    TraceStart += cameraRot * ToPC Dot cameraRot;
    TraceEnd = TraceStart + cameraRot * 10000;
    HitActor = Owner.Pawn.Trace(HitLocation, HitNormal, TraceEnd, TraceStart, true, Extent, HitInfo);
}

function Tick(float TimeDelta)
{
    local float T;
    CurrentTime += TimeDelta;
    if (CurrentTime >= TotalTime)
    {
        bComplete = true;
        CurrentTime = TotalTime;
    }
    To.Tick(TimeDelta);
    if (From == None)
    {
        return;
    }
    if (!bComplete)
    {
        From.Tick(TimeDelta);
    }
    T = Interpolate(CurrentTime / TotalTime);
    if (From != None && To != None)
    {
        Class'BioInterpolator'.InterpolateVector(m_pov.Location, InterpMethod, LocationOffset, vect(0,0,0), T);
        Class'BioInterpolator'.InterpolateRotator(m_pov.Rotation, InterpMethod, RotationOffset, rot(0,0,0), T);
        Class'BioInterpolator'.InterpolateFloat(m_pov.FOV, InterpMethod, From.m_pov.FOV, To.m_pov.FOV, T);
    }
    m_pov.Rotation = To.m_pov.Rotation - m_pov.Rotation;
    m_pov.Location = To.m_pov.Location - m_pov.Location;
}

function Rotator GetCurrentShake()
{
    local Rotator shake1;
    local Rotator shake2;
    local Rotator Shake;
    if (!bComplete)
    {
        shake1 = From.GetCurrentShake();
    }
    shake2 = To.GetCurrentShake();
    if (From != None && To != None)
    {
        Class'BioInterpolator'.InterpolateRotator(Shake, InterpMethod, shake1, shake2, CurrentTime / TotalTime);
    }
    return Shake;
}

defaultproperties
{
    DefaultTime=0,5
    InterpMethod=BIO_INTERPOLATION_METHOD_QUARTER_SIN
    Curve=(Points=?,InterpMethod=?)
}
