﻿class SFXCameraMode_Vehicle extends SFXCameraMode
    config(Game);

var transient float LastDeltaT;

var transient Vector HookLocation;

var transient bool bUpdatedThisFrame;

var transient float ElasticScale;

var float ElasticSpeed;

var transient Vector ElasticPosition;

var transient MotionBlurEffect Blur;

var config Vector AlternateCameraHookOffset;

var config float AlternateCameraArcRadius;

var config Vector RegularCameraHookOffset;

var config float RegularCameraArcRadius;

var transient float ArcRadius;

function Initialize(BioPlayerController PC)
{
    Initialize(PC);
    ArcRadius = RegularCameraArcRadius;
    HookOffset = RegularCameraHookOffset;
    if (PC.Pawn != None)
    {
        m_pov.Rotation = PC.Pawn.Rotation;
    }
    HookLocation = GetActorCameraHook();
    ElasticPosition = HookLocation - QuatRotateVector(QuatFromRotator(m_pov.Rotation), Offset);
}

function Vector GetCameraLocation()
{
    if (!bUpdatedThisFrame)
    {
        UpdatePosition();
        bUpdatedThisFrame = true;
    }
    return ElasticPosition - QuatRotateVector(QuatFromRotator(m_pov.Rotation), Offset);
}

function UpdatePosition()
{
    local float Length1;
    local float Length2;
    local Vector OldOffset;
    local Vector NewOffset;
    local Vector NiceOffset;
    OldOffset = ElasticPosition - HookLocation;
    Length1 = VSize(OldOffset);
    ElasticPosition = VLerp(ElasticPosition, GetActorCameraHook(), ElasticScale);
    HookLocation = GetActorCameraHook();
    NewOffset = ElasticPosition - HookLocation;
    Length2 = VSize(NewOffset);
    NiceOffset = Normal(NewOffset) * Lerp(Length1, Length2, 0,2);
    ElasticPosition = HookLocation + NiceOffset;
}

function Tick(float DeltaTime)
{
    Tick(DeltaTime);
    ElasticScale = 1 - ElasticSpeed ** DeltaTime;
    bUpdatedThisFrame = false;
    LastDeltaT = DeltaTime;
    if (Owner.bAlternateCamera != 0)
    {
        Offset.X = Lerp(450, 700, FClamp(Offset.X - 450 / 250 + DeltaTime, 0, 1));
    }
    else
    {
        Offset.X = Lerp(700, 450, FClamp(700 - Offset.X / 250 + DeltaTime, 0, 1));
    }
}

function MakeActive()
{
    LocalPlayer(Owner.Player).BioAddPostProcessEffect(Blur, Owner.PlayerCamera, 0);
}

function MakeInactive()
{
    LocalPlayer(Owner.Player).BioRemovePostProcessEffect(Blur);
}

defaultproperties
{
    RegularCameraArcRadius=335
    RegularCameraHookOffset=(X=0,Y=0,Z=200)
    AlternateCameraArcRadius=400
    AlternateCameraHookOffset=(X=-300,Y=0,Z=150)
    Blur=MotionBlurEffect's_DefaultBlur'
    ElasticSpeed=1E-06
    bIsCameraShakeEnabled=true
    Offset=(X=450,Y=0,Z=0)
    FOV=90
    Input=SFXCameraInput's_Input'
    // child object s_DefaultBlur of type MotionBlurEffect
    // child object s_Input of type SFXCameraInput
}
