﻿class SFXCameraModifier_ScreenShake extends CameraModifier
    config(Camera)
    native;

enum EShakeParam
{
    ESP_OffsetRandom,
    ESP_OffsetZero,
};

struct native ShakeParams
{
    var() EShakeParam X;

    var() EShakeParam Y;

    var() EShakeParam Z;

    var const transient byte Padding;

}

struct native ScreenShakeStruct
{
    var() float TimeToGo;

    var() float TimeDuration;

    var() Vector RotAmplitude;

    var() Vector RotFrequency;

    var Vector RotSinOffset;

    var() ShakeParams RotParam;

    var() Vector LocAmplitude;

    var() Vector LocFrequency;

    var Vector LocSinOffset;

    var() ShakeParams LocParam;

    var() float FOVAmplitude;

    var() float FOVFrequency;

    var float FOVSinOffset;

    var() EShakeParam FOVParam;

    var() Name ShakeName;

}

var() ScreenShakeStruct TestShake;

var array<ScreenShakeStruct> Shakes;

final function RemoveAllScreenShakes()
{
    Shakes.Length = 0;
}

final function RemoveScreenShake(Name ShakeName)
{
    local int idx;
    idx = Shakes.Find('ShakeName', ShakeName);
    if (idx != -1)
    {
        Shakes.Remove(idx, 1);
    }
}

final function AddScreenShake(ScreenShakeStruct NewShake)
{
    local int ShakeIdx;
    local int NumShakes;
    NumShakes = Shakes.Length;
    if (NewShake.ShakeName != 'None')
    {
        ShakeIdx = 0;
        while (ShakeIdx < NumShakes)
        {
            if (Shakes[ShakeIdx].ShakeName == NewShake.ShakeName)
            {
                Shakes[ShakeIdx] = InitializeShake(NewShake);
                return;
            }
            ++ShakeIdx;
        }
    }
    Shakes[NumShakes] = InitializeShake(NewShake);
}

final function ScreenShakeStruct InitializeShake(ScreenShakeStruct NewShake)
{
    NewShake.TimeToGo = NewShake.TimeDuration;
    if (!IsZero(NewShake.RotAmplitude))
    {
        NewShake.RotSinOffset.X = InitializeOffset(NewShake.RotParam.X);
        NewShake.RotSinOffset.Y = InitializeOffset(NewShake.RotParam.Y);
        NewShake.RotSinOffset.Z = InitializeOffset(NewShake.RotParam.Z);
    }
    if (!IsZero(NewShake.LocAmplitude))
    {
        NewShake.LocSinOffset.X = InitializeOffset(NewShake.LocParam.X);
        NewShake.LocSinOffset.Y = InitializeOffset(NewShake.LocParam.Y);
        NewShake.LocSinOffset.Z = InitializeOffset(NewShake.LocParam.Z);
    }
    if (NewShake.FOVAmplitude != 0)
    {
        NewShake.FOVSinOffset = InitializeOffset(NewShake.FOVParam);
    }
    return NewShake;
}

final static function float InitializeOffset(EShakeParam Param)
{
    switch (Param)
    {
        case 0:
            return FRand() * 2 * 3,141593;
        case 1:
            return 0;
    }
    return 0;
}

final function ScreenShakeStruct ComposeNewShake(float Duration, Vector newRotAmplitude, Vector newRotFrequency, Vector newLocAmplitude, Vector newLocFrequency, float newFOVAmplitude, float newFOVFrequency)
{
    local ScreenShakeStruct NewShake;
    NewShake.TimeDuration = Duration;
    NewShake.RotAmplitude = newRotAmplitude;
    NewShake.RotFrequency = newRotFrequency;
    NewShake.LocAmplitude = newLocAmplitude;
    NewShake.LocFrequency = newLocFrequency;
    NewShake.FOVAmplitude = newFOVAmplitude;
    NewShake.FOVFrequency = newFOVFrequency;
    return NewShake;
}

function StartNewShake(float Duration, Vector newRotAmplitude, Vector newRotFrequency, Vector newLocAmplitude, Vector newLocFrequency, float newFOVAmplitude, float newFOVFrequency)
{
    local ScreenShakeStruct NewShake;
    if (Duration == -1)
    {
        Shakes.Length = 0;
    }
    else
    {
        NewShake = ComposeNewShake(Duration, newRotAmplitude, newRotFrequency, newLocAmplitude, newLocFrequency, newFOVAmplitude, newFOVFrequency);
        AddScreenShake(NewShake);
    }
}

native function UpdateScreenShake(float DeltaTime, out ScreenShakeStruct Shake, out TPOV OutPOV);

function bool ModifyCamera(Camera Camera, float DeltaTime, out TPOV OutPOV)
{
    local int I;
    local ScreenShakeStruct CurrentScreenShake;
    ModifyCamera(Camera, DeltaTime, OutPOV);
    if (Shakes.Length > 0)
    {
        I = 0;
        while (I < Shakes.Length)
        {
            CurrentScreenShake = Shakes[I];
            UpdateScreenShake(DeltaTime, CurrentScreenShake, OutPOV);
            Shakes[I] = CurrentScreenShake;
            I++;
        }
        I = Shakes.Length - 1;
        while (I >= 0)
        {
            if (Shakes[I].TimeToGo <= 0)
            {
                Shakes.Remove(I, 1);
            }
            I--;
        }
    }
    UpdateScreenShake(DeltaTime, TestShake, OutPOV);
    return false;
}

defaultproperties
{
}
