﻿class SFXConsole extends Console
    within GameViewportClient
    hidecategories(Object,UIRoot)
    config(Input)
    transient
    native;

var bool bShift;

function Initialized()
{
    Initialized();
    LoadHistory();
}

function PurgeCommandFromHistory(String Command)
{
    local int Count;
    local int I;
    local int idx;
    local int J;
    local int jdx;
    local int kdx;
    if (HistoryTop >= HistoryBot)
    {
        Count = HistoryTop - HistoryBot;
    }
    else
    {
        Count = 16 - HistoryBot + HistoryTop;
    }
    I = 0;
    while (I < Count)
    {
        idx = HistoryBot + I % 16;
        if (History[idx] ~= Command)
        {
            J = I;
            while (J < Count - 1)
            {
                jdx = HistoryBot + J + 0 % 16;
                kdx = HistoryBot + J + 1 % 16;
                History[jdx] = History[kdx];
                ++J;
            }
            if (HistoryTop > 0)
            {
                HistoryTop--;
            }
            else
            {
                HistoryTop = 16;
            }
            Count--;
        }
        else
        {
            I++;
        }
    }
    if (HistoryTop > HistoryBot)
    {
        I = HistoryTop;
        while (I < 16)
        {
            History[I] = "";
            ++I;
        }
    }
    else
    {
        I = HistoryTop;
        while (I < HistoryBot)
        {
            History[I] = "";
            ++I;
        }
    }
}

function Actor GetCameraActor()
{
    local Actor HitActor;
    local Vector HitLocation;
    local Vector HitNormal;
    SFXPlayerCamera(Outer.Outer.GamePlayers[0].Actor.PlayerCamera).GetTrace(HitActor, HitLocation, HitNormal);
    return HitActor;
}

function String PreParseCommand(String Command)
{
    local array<String> Pieces;
    local String Token;
    local String outputstring;
    local int I;
    ParseStringIntoArray(Command, Pieces, " ", true);
    I = 0;
    while (I < Pieces.Length)
    {
        Token = Pieces[I];
        if (Caps(Token) == "$CAMERA")
        {
            Pieces[I] = PathName(GetCameraActor());
        }
        outputstring = outputstring @ Pieces[I];
        I++;
    }
    return outputstring;
}

function ConsoleCommand(String Command)
{
    if (HistoryTop == 0 ? !History[16 - 1] ~= Command : !History[HistoryTop - 1] ~= Command)
    {
        PurgeCommandFromHistory(Command);
        History[HistoryTop] = Command;
        HistoryTop = HistoryTop + 1 % 16;
        if (HistoryBot == -1 || HistoryBot == HistoryTop)
        {
            HistoryBot = HistoryBot + 1 % 16;
        }
    }
    HistoryCur = HistoryTop;
    Command = PreParseCommand(Command);
    if (Left(Command, 4) ~= "Say ")
    {
        Command = "Say "" $ Right(Command, Len(Command) - 4) $ """;
    }
    if (Left(Command, 8) ~= "TeamSay ")
    {
        Command = "TeamSay "" $ Right(Command, Len(Command) - 8) $ """;
    }
    SaveHistory();
    if (bEnableUI)
    {
        OutputText("\n\>\>\>" @ Command @ "\<\<\<");
    }
    else
    {
        OutputText("\n>>>" @ Command @ "<<<");
    }
    if (Command ~= "cls")
    {
        ClearOutput();
    }
    else
    {
        if (ConsoleTargetPlayer != None)
        {
            ConsoleTargetPlayer.Actor.ConsoleCommand(Command);
        }
        else
        {
            if (Outer.Outer.GamePlayers.Length > 0 && Outer.Outer.GamePlayers[0].Actor != None)
            {
                Outer.Outer.GamePlayers[0].Actor.ConsoleCommand(Command);
            }
            else
            {
                Outer.ConsoleCommand(Command);
            }
        }
    }
}

function ClearOutput()
{
    SBHead = 0;
    SBPos = 0;
    Scrollback.Remove(0, Scrollback.Length);
    if (bEnableUI)
    {
        ConsoleBufferText.SetValue("");
    }
}

function StartTyping(coerce String Text)
{
    GotoState('Typing');
    SetInputText(Text);
    SetCursorPos(Len(Text));
}

function PostRender_Console(Canvas Canvas);

function bool InputKey(int ControllerId, Name Key, EInputEvent Event, optional float AmountDepressed = 1, optional bool bGamepad = false)
{
    if (Key == ConsoleKey && Event == 0)
    {
        GotoState('Open');
        return true;
    }
    else
    {
        if (Key == TypeKey && Event == 0)
        {
            GotoState('Typing');
            return true;
        }
    }
    return false;
}

native final function LoadHistory();

native final function SaveHistory();

function bool InputChar(int ControllerId, String Unicode)
{
    return false;
}

native function String FindNextPropertyMatching(class<Object> C, String PropertyPrefix, String AfterThisProperty);

native final static function Object FindObjectUnqualified(String ObjectName, class<Object> ObjectClass);

function FindNextMatchingAutoComplete(out String CurrentString, int CurrentStringPos)
{
    local array<String> Pieces;
    local String Token;
    local String ClassName;
    local String PropertyName;
    local String Prefix;
    local String outputstring;
    local int PrefixBegin;
    local int PrefixEnd;
    local class<Object> SearchClass;
    local Object TestObj;
    ParseStringIntoArray(CurrentString, Pieces, " ", true);
    if (Pieces.Length > 0)
    {
        Token = Pieces[Pieces.Length - 1];
        if (InStr(Token, "::") != -1)
        {
            PrefixBegin = InStr(CurrentString, "::", true) + 2;
            PrefixEnd = CurrentStringPos - PrefixBegin;
            if (PrefixBegin != PrefixEnd)
            {
                Prefix = Mid(CurrentString, PrefixBegin, PrefixEnd);
            }
            PropertyName = Right(Token, Len(Token) - InStr(Token, "::", false) + 2);
            ClassName = Left(Token, InStr(Token, "::", false));
            SearchClass = Class<Object>(FindObjectUnqualified(ClassName, Class'Object'));
            if (SearchClass != None)
            {
                Token = FindNextPropertyMatching(SearchClass, Prefix, PropertyName);
                if (Token != "")
                {
                    Pieces[Pieces.Length - 1] = ClassName $ "::" $ Token;
                }
            }
        }
        else
        {
            if (Pieces.Length >= 2)
            {
                PrefixBegin = InStr(CurrentString, " ", true) + 1;
                PrefixEnd = CurrentStringPos - PrefixBegin;
                if (PrefixBegin != PrefixEnd)
                {
                    Prefix = Mid(CurrentString, PrefixBegin, PrefixEnd);
                }
                PropertyName = Token;
                ClassName = Pieces[Pieces.Length - 2];
                TestObj = FindObjectUnqualified(ClassName, Class'Object');
                SearchClass = Class<Object>(TestObj);
                if (SearchClass == None && TestObj != None)
                {
                    SearchClass = TestObj.Class;
                }
                if (SearchClass != None)
                {
                    Token = FindNextPropertyMatching(SearchClass, Prefix, PropertyName);
                    if (Token != "")
                    {
                        Pieces[Pieces.Length - 1] = Token;
                    }
                }
            }
        }
    }
    JoinArray(Pieces, outputstring, " ");
    SetInputText(outputstring);
}

function bool CommonInputKey(int ControllerId, Name Key, EInputEvent Event, float AmountDepressed, bool bGamepad)
{
    local String CopyOfTypedStr;
    local int CopyOfTypedStrPos;
    local int NextHistory;
    CopyOfTypedStr = TypedStr;
    CopyOfTypedStrPos = TypedStrPos;
    if (Event == 0)
    {
        bCaptureKeyInput = true;
    }
    if (Key == 'LeftControl')
    {
        if (Event == 1)
        {
            bCtrl = false;
        }
        else
        {
            if (Event == 0)
            {
                bCtrl = true;
            }
        }
        return true;
    }
    else
    {
        if (Key == 'LeftShift')
        {
            if (Event == 1)
            {
                bShift = false;
            }
            else
            {
                if (Event == 0)
                {
                    bShift = true;
                }
            }
            return true;
        }
        else
        {
            if (Key == 'Escape' && Event == 1)
            {
                if (TypedStr != "")
                {
                    SetInputText("");
                    SetCursorPos(0);
                    HistoryCur = HistoryTop;
                }
                return true;
            }
            else
            {
                if (Key == 'Enter' && Event == 1)
                {
                    if (TypedStr != "")
                    {
                        SetInputText("");
                        SetCursorPos(0);
                        ConsoleCommand(CopyOfTypedStr);
                    }
                    return true;
                }
                else
                {
                    if (Event != 0 && Event != 2)
                    {
                        return false;
                    }
                    else
                    {
                        if (Key == 'Up' && !bCtrl)
                        {
                            if (HistoryBot >= 0)
                            {
                                if (HistoryCur == HistoryBot)
                                {
                                    HistoryCur = HistoryTop;
                                }
                                else
                                {
                                    HistoryCur--;
                                    if (HistoryCur < 0)
                                    {
                                        HistoryCur = 16 - 1;
                                    }
                                }
                                SetInputText(History[HistoryCur]);
                                SetCursorPos(Len(History[HistoryCur]));
                            }
                            return true;
                        }
                        else
                        {
                            if (Key == 'Down' && !bCtrl)
                            {
                                if (HistoryBot >= 0)
                                {
                                    if (HistoryCur == HistoryTop)
                                    {
                                        HistoryCur = HistoryBot;
                                    }
                                    else
                                    {
                                        HistoryCur = HistoryCur + 1 % 16;
                                    }
                                    SetInputText(History[HistoryCur]);
                                    SetCursorPos(Len(History[HistoryCur]));
                                }
                                return true;
                            }
                            else
                            {
                                if (Key == 'BackSpace' && !bCtrl)
                                {
                                    if (TypedStrPos > 0)
                                    {
                                        SetInputText(Left(TypedStr, TypedStrPos - 1) $ Right(TypedStr, Len(TypedStr) - TypedStrPos));
                                        SetCursorPos(TypedStrPos - 1);
                                    }
                                    return true;
                                }
                                else
                                {
                                    if (Key == 'BackSpace' && bCtrl)
                                    {
                                        if (TypedStrPos > 0)
                                        {
                                            PurgeCommandFromHistory(TypedStr);
                                            SaveHistory();
                                            SetInputText("");
                                            SetCursorPos(0);
                                        }
                                        return true;
                                    }
                                    else
                                    {
                                        if (Key == 'Delete' && !bCtrl)
                                        {
                                            if (TypedStrPos < Len(TypedStr))
                                            {
                                                SetInputText(Left(TypedStr, TypedStrPos) $ Right(TypedStr, Len(TypedStr) - TypedStrPos - 1));
                                            }
                                            return true;
                                        }
                                        else
                                        {
                                            if (Key == 'Delete' && bCtrl)
                                            {
                                                if (TypedStrPos < Len(TypedStr))
                                                {
                                                    SetInputText(Left(TypedStr, TypedStrPos) $ Mid(TypedStr, FindBreak(TypedStr, TypedStrPos, true)));
                                                }
                                                return true;
                                            }
                                            else
                                            {
                                                if (Key == 'Left' && !bCtrl)
                                                {
                                                    SetCursorPos(Max(0, TypedStrPos - 1));
                                                    return true;
                                                }
                                                else
                                                {
                                                    if (Key == 'Right' && !bCtrl)
                                                    {
                                                        SetCursorPos(Min(Len(TypedStr), TypedStrPos + 1));
                                                        return true;
                                                    }
                                                    else
                                                    {
                                                        if (Key == 'Left' && bCtrl)
                                                        {
                                                            SetCursorPos(FindBreak(TypedStr, TypedStrPos, false));
                                                            return true;
                                                        }
                                                        else
                                                        {
                                                            if (Key == 'Right' && bCtrl)
                                                            {
                                                                SetCursorPos(FindBreak(TypedStr, TypedStrPos, true));
                                                                return true;
                                                            }
                                                            else
                                                            {
                                                                if (Key == 'Home' && !bCtrl)
                                                                {
                                                                    SetCursorPos(0);
                                                                    return true;
                                                                }
                                                                else
                                                                {
                                                                    if (Key == 'End' && !bCtrl)
                                                                    {
                                                                        SetCursorPos(Len(TypedStr));
                                                                        return true;
                                                                    }
                                                                    else
                                                                    {
                                                                        if (Key == 'Tab' && bCtrl)
                                                                        {
                                                                            if (FindNextMatchingHistory(NextHistory, TypedStr, TypedStrPos))
                                                                            {
                                                                                HistoryCur = NextHistory;
                                                                                SetInputText(History[HistoryCur]);
                                                                                SetCursorPos(CopyOfTypedStrPos);
                                                                            }
                                                                            return true;
                                                                        }
                                                                        else
                                                                        {
                                                                            if (Key == 'Tab' && bShift)
                                                                            {
                                                                                FindNextMatchingAutoComplete(TypedStr, TypedStrPos);
                                                                                SetCursorPos(CopyOfTypedStrPos);
                                                                                return true;
                                                                            }
                                                                            else
                                                                            {
                                                                                if (Key == 'V' && bCtrl)
                                                                                {
                                                                                    SetInputText(TypedStr $ RetrieveClipboard());
                                                                                    SetCursorPos(Len(TypedStr));
                                                                                }
                                                                            }
                                                                        }
                                                                    }
                                                                }
                                                            }
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    return false;
}

function bool FindNextMatchingHistory(out int outNextHistory, String CurrentString, int CurrentStringPos)
{
    local String LeftCurrentString;
    local String LeftHistoryString;
    local int Top;
    local int bot;
    local int I;
    if (CurrentString != "" && Len(CurrentString) >= CurrentStringPos && CurrentStringPos > 0)
    {
        if (HistoryCur >= 0 && HistoryCur < 16 && History[HistoryCur] == CurrentString)
        {
            if (HistoryCur > 0)
            {
                Top = HistoryCur - 1;
            }
            else
            {
                Top = 16 - 1;
            }
            if (HistoryCur < 16 - 1)
            {
                bot = HistoryCur + 1;
            }
            else
            {
                bot = 0;
            }
        }
        else
        {
            if (HistoryTop >= 0 && HistoryTop < 16 && HistoryBot >= 0 && HistoryBot < 16)
            {
                Top = HistoryTop;
                bot = HistoryBot;
            }
            else
            {
                Top = 0;
                bot = 0;
            }
        }
        LeftCurrentString = Left(CurrentString, CurrentStringPos);
        I = Top;
        while (true)
        {
            LeftHistoryString = Left(History[I], CurrentStringPos);
            if (LeftCurrentString ~= LeftHistoryString)
            {
                outNextHistory = I;
                return true;
            }
            if (I == bot)
            {
                break;
            }
            if (I > 0)
            {
                I--;
            }
            else
            {
                I = 16 - 1;
            }
        }
    }
    return false;
}

function int FindBreak(String CurrentString, int CurrentStringPos, bool bForward)
{
    local int breakCharsLen;
    local String si;
    local String sj;
    local int I;
    local int J;
    local int Start;
    local int Limit;
    local int increment;
    if (CurrentString != "")
    {
        breakCharsLen = Len(" .()='\/"");
        if (bForward)
        {
            Limit = Len(CurrentString);
            Start = Min(Limit, CurrentStringPos + 1);
            increment = 1;
        }
        else
        {
            Limit = 0;
            Start = Max(Limit, CurrentStringPos - 1);
            increment = -1;
        }
        I = Start;
        while (true)
        {
            si = Mid(CurrentString, I, 1);
            J = 0;
            while (J != breakCharsLen)
            {
                sj = Mid(" .()='\/"", J, 1);
                if (sj == si)
                {
                    return I;
                }
                J++;
            }
            if (I == Limit)
            {
                break;
            }
            I += increment;
        }
        return Limit;
    }
    return 0;
}

const breakChars = " .()='\\/\"";
native final function String RetrieveClipboard();

state Typing
{
    function bool InputChar(int ControllerId, String Unicode)
    {
        local int Character;
        if (IsUIMiniConsoleOpen())
        {
            return false;
        }
        if (!bCaptureKeyInput)
        {
            return true;
        }
        while (Len(Unicode) > 0)
        {
            Character = Asc(Left(Unicode, 1));
            Unicode = Mid(Unicode, 1);
            if (Character >= 32 && Character < 256 && Character != Asc("~") && Character != Asc("`"))
            {
                SetInputText(Left(TypedStr, TypedStrPos) $ Chr(Character) $ Right(TypedStr, Len(TypedStr) - TypedStrPos));
                SetCursorPos(TypedStrPos + 1);
            }
        }
        return true;
    }

    function bool InputKey(int ControllerId, Name Key, EInputEvent Event, optional float AmountDepressed = 1, optional bool bGamepad = false)
    {
        local String Temp;
        Temp = TypedStr;
        CommonInputKey(ControllerId, Key, Event, AmountDepressed, bGamepad);
        if (Key == 'Escape' && Event == 1)
        {
            if (Temp == "")
            {
                GotoState('None');
            }
            return true;
        }
        else
        {
            if (Key == ConsoleKey && Event == 0)
            {
                GotoState('Open');
                return true;
            }
            else
            {
                if (Key == TypeKey && Event == 0 && !bCtrl && !bShift)
                {
                    GotoState('None');
                    return true;
                }
                else
                {
                    if (Key == 'Enter' && Event == 1)
                    {
                        GotoState('None');
                        return true;
                    }
                    else
                    {
                        if (Event != 0 && Event != 2)
                        {
                            return false;
                        }
                    }
                }
            }
        }
        return true;
    }

    event PostRender_Console(Canvas Canvas)
    {
        local float XL;
        local float YL;
        local String OutStr;
        local float ClipX;
        local float ClipY;
        local float LeftPos;
        if (!IsUIMiniConsoleOpen())
        {
            Global.PostRender_Console(Canvas);
            Canvas.Font = Class'Engine'.GetSmallFont();
            OutStr = "(>" @ TypedStr;
            Canvas.StrLen(OutStr, XL, YL);
            ClipX = Canvas.ClipX;
            ClipY = Canvas.ClipY;
            LeftPos = 0;
            if (Class'WorldInfo'.IsConsoleBuild())
            {
                ClipX -= 32;
                ClipY -= 32;
                LeftPos = 32;
            }
            Canvas.SetPos(LeftPos, ClipY - 6 - YL);
            Canvas.DrawTile(DefaultTexture_Black, ClipX, YL + 6, 0, 0, 32, 32);
            Canvas.SetPos(LeftPos, ClipY - 6 - YL);
            Canvas.SetDrawColor(0, 255, 0);
            Canvas.DrawTile(DefaultTexture_White, ClipX, 2, 0, 0, 32, 32);
            Canvas.SetPos(LeftPos, ClipY - 3 - YL);
            Canvas.bCenter = false;
            Canvas.DrawText(OutStr, false);
            OutStr = "(>" @ Left(TypedStr, TypedStrPos);
            Canvas.StrLen(OutStr, XL, YL);
            Canvas.SetPos(LeftPos + XL, ClipY - 1 - YL);
            Canvas.DrawText("_");
        }
    }

    event BeginState(Name PreviousStateName)
    {
        if (bEnableUI && MiniConsoleScene != None)
        {
            Outer.UIController.OpenScene(MiniConsoleScene);
        }
        bCaptureKeyInput = false;
        HistoryCur = HistoryTop;
    }

    event EndState(Name NextStateName)
    {
        if (MiniConsoleScene != None)
        {
            Outer.UIController.CloseScene(MiniConsoleScene);
        }
    }

}

state Open
{
    function bool InputChar(int ControllerId, String Unicode)
    {
        local int Character;
        if (!bCaptureKeyInput)
        {
            return true;
        }
        while (Len(Unicode) > 0)
        {
            Character = Asc(Left(Unicode, 1));
            Unicode = Mid(Unicode, 1);
            if (Character >= 32 && Character < 256 && Character != Asc("~") && Character != Asc("`"))
            {
                SetInputText(Left(TypedStr, TypedStrPos) $ Chr(Character) $ Right(TypedStr, Len(TypedStr) - TypedStrPos));
                SetCursorPos(TypedStrPos + 1);
            }
        }
        return true;
    }

    function bool InputKey(int ControllerId, Name Key, EInputEvent Event, optional float AmountDepressed = 1, optional bool bGamepad = false)
    {
        local String Temp;
        Temp = TypedStr;
        CommonInputKey(ControllerId, Key, Event, AmountDepressed, bGamepad);
        if (Key == 'Escape' && Event == 1)
        {
            if (Temp == "")
            {
                GotoState('None');
            }
            return true;
        }
        else
        {
            if (Key == ConsoleKey && Event == 0)
            {
                GotoState('None');
                return true;
            }
            else
            {
                if (Key == TypeKey && Event == 0 && !bCtrl)
                {
                    GotoState('Typing');
                    return true;
                }
                else
                {
                    if (Event != 0 && Event != 2)
                    {
                        return false;
                    }
                    else
                    {
                        if (Key == 'Up' && bCtrl)
                        {
                            if (SBPos < Scrollback.Length - 1)
                            {
                                SBPos++;
                                if (SBPos >= Scrollback.Length)
                                {
                                    SBPos = Scrollback.Length - 1;
                                }
                            }
                            return true;
                        }
                        else
                        {
                            if (Key == 'Down' && bCtrl)
                            {
                                if (SBPos > 0)
                                {
                                    SBPos--;
                                    if (SBPos < 0)
                                    {
                                        SBPos = 0;
                                    }
                                }
                                return true;
                            }
                            else
                            {
                                if (Key == 'Home' && bCtrl)
                                {
                                    SBPos = 0;
                                }
                                else
                                {
                                    if (Key == 'End' && bCtrl)
                                    {
                                        SBPos = Scrollback.Length - 1;
                                    }
                                    else
                                    {
                                        if (Key == 'PageUp' || Key == 'MouseScrollUp')
                                        {
                                            if (SBPos < Scrollback.Length - 1)
                                            {
                                                if (bCtrl)
                                                {
                                                    SBPos += 5;
                                                }
                                                else
                                                {
                                                    SBPos++;
                                                }
                                                if (SBPos >= Scrollback.Length)
                                                {
                                                    SBPos = Scrollback.Length - 1;
                                                }
                                            }
                                            return true;
                                        }
                                        else
                                        {
                                            if (Key == 'PageDown' || Key == 'MouseScrollDown')
                                            {
                                                if (SBPos > 0)
                                                {
                                                    if (bCtrl)
                                                    {
                                                        SBPos -= 5;
                                                    }
                                                    else
                                                    {
                                                        SBPos--;
                                                    }
                                                    if (SBPos < 0)
                                                    {
                                                        SBPos = 0;
                                                    }
                                                }
                                                return true;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        return true;
    }

    event PostRender_Console(Canvas Canvas)
    {
        local float Height;
        local float XL;
        local float YL;
        local float Y;
        local String OutStr;
        local int idx;
        if (!IsUIConsoleOpen())
        {
            Canvas.Font = Class'Engine'.GetSmallFont();
            Height = Canvas.ClipY * 0,75;
            Canvas.SetDrawColor(255, 255, 255, 255);
            Canvas.SetPos(0, 0);
            Canvas.DrawTile(DefaultTexture_Black, Canvas.ClipX, Height, 0, 0, 32, 32);
            OutStr = "(>" @ TypedStr;
            Canvas.StrLen(OutStr, XL, YL);
            Canvas.SetPos(0, Height - 12 - YL);
            Canvas.SetDrawColor(0, 255, 0);
            Canvas.DrawTile(DefaultTexture_White, Canvas.ClipX, 2, 0, 0, 32, 32);
            Canvas.SetPos(0, Height);
            Canvas.DrawTile(DefaultTexture_White, Canvas.ClipX, 2, 0, 0, 32, 32);
            Canvas.SetPos(0, Height - 5 - YL);
            Canvas.bCenter = false;
            Canvas.DrawText(OutStr, false);
            OutStr = "(>" @ Left(TypedStr, TypedStrPos);
            Canvas.StrLen(OutStr, XL, YL);
            Canvas.SetPos(XL, Height - 3 - YL);
            Canvas.DrawText("_");
            idx = SBHead - SBPos;
            Y = Height - 16 - YL * 2;
            if (Scrollback.Length == 0)
            {
                return;
            }
            Canvas.SetDrawColor(255, 255, 255, 255);
            while (Y > YL && idx >= 0)
            {
                Canvas.SetPos(0, Y);
                Canvas.DrawText(Scrollback[idx], false);
                idx--;
                Y -= YL;
            }
        }
    }

    event BeginState(Name PreviousStateName)
    {
        bCaptureKeyInput = false;
        HistoryCur = HistoryTop;
        SBPos = 0;
        bCtrl = false;
        if (bEnableUI && LargeConsoleScene != None)
        {
            Outer.UIController.OpenScene(LargeConsoleScene);
        }
    }

    event EndState(Name NextStateName)
    {
        if (LargeConsoleScene != None)
        {
            Outer.UIController.CloseScene(LargeConsoleScene);
        }
    }

}

defaultproperties
{
    HistoryTop=4
    HistoryBot=5
    HistoryCur=4
    History="profile non"
    History="profile none"
    History="open ericcombat"
    History="exit"
    History="slomo 3"
    History="killcurrenttarget"
    History="dumpachievementstats"
    History="enabledamage 0"
    History="enabledamage 1"
    History="slomo 0.5"
    History="slomo 0.1"
    History="slomo 50"
    History="slomo 1"
    History="profile anim"
    History="profile combat self"
}
