﻿class SFXCustomAction_DamageReaction extends BioCustomAction
    config(Game)
    abstract;

var() transient Controller Instigator;

var() ParticleSystem PS_Impact;

var() float ImpactScale;

var() float RotationTime;

var() ERootMotionMode ERootMotionMode;

var() float fReactionDuration;

var() bool bRotateOnHit;

var() bool bRagdollOnFinish;

var() bool bAllowAnimInterrupt;

var() bool bAnimLooping;

var() float fAnimStartTime;

var() float fAnimBlendOutTime;

var() float fAnimBlendInTime;

var() float fAnimPlayRate;

var() BodyStance BS_ReactionExplore;

var() BodyStance BS_Reaction;

function Init(Vector HitLocation, Vector HitNormal, TraceHitInfo HitInfo, bool bPlayImpact, optional Controller Inst)
{
    Instigator = Inst;
    if (bPlayImpact && PS_Impact != None)
    {
        ActivateImpactEmitter(HitLocation, HitNormal, HitInfo);
    }
}

function StartCustomAction()
{
    StartCustomAction();
    if (oPawn.ActionState == 0 && BS_ReactionExplore.AnimName != 'None')
    {
        BS_Reaction = BS_ReactionExplore;
    }
    else
    {
        BS_Reaction = BS_Reaction;
    }
    if (oPawn.PlayBodyStance(BS_Reaction, fAnimPlayRate, fAnimBlendInTime, fAnimBlendOutTime, bAnimLooping, false, false, fAnimStartTime) != 0)
    {
        oPawn.SetBodyStanceAnimEndNotification(BS_Reaction, true);
        if (ERootMotionMode != 2)
        {
            oPawn.SetBodyStanceRootBoneAxisOption(BS_Reaction, 2, 2);
            oPawn.Mesh.RootMotionMode = ERootMotionMode;
            oPawn.Velocity = vect(0,0,0);
        }
        if (fReactionDuration > 0)
        {
            oPawn.SetTimer(fReactionDuration, false, 'OnCustomActionTimeUp', self);
        }
    }
    if (bRotateOnHit && oPawn.LastHitBy != None)
    {
        SetFacePreciseRotation(oPawn.LastHitBy.Pawn.Location - oPawn.Location, RotationTime);
    }
}

function OnCustomActionTimeUp()
{
    if (bAllowAnimInterrupt != false)
    {
        EndThisCustomAction();
    }
    else
    {
        oPawn.SetBodyStanceAnimLooping(BS_Reaction, false);
    }
}

function StopCustomAction()
{
    StopCustomAction();
    if (ERootMotionMode != 2)
    {
        oPawn.Mesh.RootMotionMode = oPawn.Mesh.Default.RootMotionMode;
    }
    oPawn.StopBodyStance(BS_Reaction, fAnimBlendOutTime);
    oPawn.SetBodyStanceAnimEndNotification(BS_Reaction, false);
    if (ERootMotionMode != 2)
    {
        oPawn.SetBodyStanceRootBoneAxisOption(BS_Reaction, 1, 1, 1);
    }
}

function BodyStanceAnimEndNotification(AnimNodeSequence SeqNode, float PlayedTime, float ExcessTime)
{
    if (ERootMotionMode != 2)
    {
        oPawn.Mesh.RootMotionMode = oPawn.Mesh.Default.RootMotionMode;
        oPawn.SetBodyStanceRootBoneAxisOption(BS_Reaction, 1, 1, 1);
    }
    if (bRagdollOnFinish)
    {
        oPawn.InitRagdoll();
        oPawn.SetTimer(0,1, true, 'CheckForRagdollRecovery', self);
    }
    else
    {
        EndThisCustomAction();
    }
}

function CheckForRagdollRecovery()
{
    if (!oPawn.IsInState('InRagdoll', true))
    {
        oPawn.ClearTimer('CheckForRagdollRecovery', self);
        EndThisCustomAction();
    }
}

function EndThisCustomAction()
{
    if (SFXPlayerController(oPawn.Controller) != None)
    {
        SFXPlayerController(oPawn.Controller).EndCustomAction();
    }
    else
    {
        if (SFXAI_Core(oPawn.Controller) != None)
        {
            SFXAI_Core(oPawn.Controller).EndCustomAction();
        }
    }
}

simulated function ActivateImpactEmitter(Vector HitLocation, Vector HitNormal, TraceHitInfo HitInfo)
{
    local SFXObjectPool Pool;
    local ParticleSystemComponent ImpactPSC;
    if (oPawn.Mesh != None)
    {
        Pool = SFXGame(oPawn.WorldInfo.Game).ObjectPool;
        if (Pool != None)
        {
            ImpactPSC = Pool.GetGenericParticleSystemComponent(PS_Impact);
            if (ImpactPSC != None)
            {
                Pool.AttachParticleSystemComponent(ImpactPSC, oPawn, oPawn.Mesh, HitInfo.BoneName, HitLocation, HitNormal, false);
                Pool.ApplyBloodColor(ImpactPSC, oPawn);
                ImpactPSC.SetScale(ImpactScale);
                Pool.ApplyLODLevel(ImpactPSC, HitLocation);
                ImpactPSC.SetActive(true);
            }
        }
    }
}

defaultproperties
{
    fAnimPlayRate=1
    fAnimBlendInTime=0,2
    fAnimBlendOutTime=0,2
    bAllowAnimInterrupt=true
    fReactionDuration=1
    ERootMotionMode=RMM_Ignore
    RotationTime=0,2
    ImpactScale=1
    bBreakFromCover=true
    bDisableMovement=true
    bNotifyKnockedOutOfCover=true
}
