﻿class SFXDifficulty extends Object
    config(Game)
    abstract;

var config float SquadPowerCooldownMultiplier;

var config float SquadPowerDurationMultiplier;

var config float EnemyPowerCooldownMultiplier;

var config int MaxTargetTicketsHench;

var config int MaxTargetTickets;

var config int MaxAttackTicketsHench;

var config int MaxAttackTickets;

var config float PlayerShieldRechargeModifier;

var config float MaxScore;

var config float MinScore;

var config int DifficultyLevel;

var config float BreachDuration;

var config float BleedOutInterval;

var config float AmmoDecayMultiplier;

var config float EnemyAimMultiplier;

var config float EnemyShieldBonus;

var config Vector2D FriendlyHealthMultiplierHigh;

var config Vector2D FriendlyHealthMultiplierLow;

var config Vector2D EnemyHealthMultiplierHigh;

var config Vector2D EnemyHealthMultiplierLow;

var config Vector2D HenchHealthMultiplierHigh;

var config Vector2D HenchHealthMultiplierLow;

var config Vector2D PlayerHealthMultiplierHigh;

var config Vector2D PlayerHealthMultiplierLow;

static function float GetCooldownMultiplier(Actor Owner)
{
    if (SFXPawn_Player(Owner) != None || SFXPawn_Henchman(Owner) != None)
    {
        return Default.SquadPowerCooldownMultiplier;
    }
    return Default.EnemyPowerCooldownMultiplier;
}

static function float GetDurationMultiplier(Actor Owner)
{
    if (SFXPawn_Player(Owner) != None || SFXPawn_Henchman(Owner) != None)
    {
        return Default.SquadPowerDurationMultiplier;
    }
    return 1;
}

static function float GetPlayerDifficultyScore()
{
    local SFXPRI PlayerPRI;
    PlayerPRI = Class'SFXPRI'.FindLocalPRI();
    if (PlayerPRI.bCalculatedChallenge == false)
    {
        SFXGame(Class'Engine'.GetCurrentWorldInfo().Game).GetChallengeModifier(0, PlayerPRI.ChallengeModifier);
        PlayerPRI.bCalculatedChallenge = true;
    }
    if (PlayerPRI.LockedScore > 0)
    {
        PlayerPRI.DifficultyScore = PlayerPRI.LockedScore;
    }
    PlayerPRI.DifficultyScore = FClamp(PlayerPRI.DifficultyScore, 0, 1000);
    PlayerPRI.DifficultyScore = FClamp(PlayerPRI.DifficultyScore, Default.MinScore, Default.MaxScore);
    return PlayerPRI.DifficultyScore / 500;
}

static function UpdateDifficulty(SFXPRI PRI)
{
    local float Score;
    local SFXPRI PlayerPRI;
    local Pawn PlayerPawn;
    local Pawn Pawn;
    PlayerPRI = Class'SFXPRI'.FindLocalPRI();
    PlayerPawn = Controller(PlayerPRI.Owner).Pawn;
    Pawn = Controller(PRI.Owner).Pawn;
    if (Pawn == None)
    {
        PRI.Destroy();
        return;
    }
    Score = GetPlayerDifficultyScore();
    PRI.ChallengeModifier = PlayerPRI.ChallengeModifier;
    if (PlayerController(PRI.Owner) != None)
    {
        if (Score <= 1)
        {
            PRI.CurrentHealthMultiplier = GetRangeValueByPct(Default.PlayerHealthMultiplierLow, Score);
        }
        else
        {
            PRI.CurrentHealthMultiplier = GetRangeValueByPct(Default.PlayerHealthMultiplierHigh, Score - 1);
        }
        PRI.CurrentShieldModifier = Default.PlayerShieldRechargeModifier;
        PRI.CurrentHealthMultiplier *= PRI.ChallengeModifier;
    }
    else
    {
        if (SFXAI_Henchman(PRI.Owner) != None)
        {
            if (Score <= 1)
            {
                PRI.CurrentHealthMultiplier = GetRangeValueByPct(Default.HenchHealthMultiplierLow, Score);
            }
            else
            {
                PRI.CurrentHealthMultiplier = GetRangeValueByPct(Default.HenchHealthMultiplierHigh, Score - 1);
            }
            PRI.CurrentShieldModifier = 1;
            PRI.CurrentHealthMultiplier *= PRI.ChallengeModifier;
        }
        else
        {
            if (Pawn.IsFriendly(PlayerPawn))
            {
                if (Score <= 1)
                {
                    PRI.CurrentHealthMultiplier = GetRangeValueByPct(Default.FriendlyHealthMultiplierLow, Score);
                }
                else
                {
                    PRI.CurrentHealthMultiplier = GetRangeValueByPct(Default.FriendlyHealthMultiplierHigh, Score - 1);
                }
                PRI.CurrentShieldModifier = 1;
                PRI.CurrentHealthMultiplier *= PRI.ChallengeModifier;
            }
            else
            {
                if (Score <= 1)
                {
                    PRI.CurrentHealthMultiplier = GetRangeValueByPct(Default.EnemyHealthMultiplierLow, Score);
                }
                else
                {
                    PRI.CurrentHealthMultiplier = GetRangeValueByPct(Default.EnemyHealthMultiplierHigh, Score - 1);
                }
                PRI.CurrentShieldModifier = 1;
                PRI.CurrentHealthMultiplier /= PRI.ChallengeModifier;
            }
        }
    }
    PRI.CurrentMaxAttackTickets = Default.MaxAttackTickets;
    PRI.CurrentMaxAttackTicketsHench = Default.MaxAttackTicketsHench;
    PRI.CurrentMaxTargetTickets = Default.MaxTargetTickets;
    PRI.CurrentMaxTargetTicketsHench = Default.MaxTargetTicketsHench;
}

defaultproperties
{
    EnemyAimMultiplier=1
    AmmoDecayMultiplier=1
    BleedOutInterval=0,5
    BreachDuration=1
    DifficultyLevel=-1
    EnemyPowerCooldownMultiplier=1
    SquadPowerDurationMultiplier=1
    SquadPowerCooldownMultiplier=1
}
