﻿class SFXDoor extends SkeletalMeshActor
    hidecategories(Navigation)
    native
    placeable;

enum ESFXDoorType
{
    EDT_Manual,
    EDT_Proximity,
    EDT_AutoEntrance,
    EDT_AutoExit,
};

enum ESFXDoorState
{
    EDS_Closed,
    EDS_Open,
    EDS_Hackable,
    EDS_PlotLocked,
    EDS_Disabled,
};

var(Save) const Guid MyGuid;

var /* 00000008 */ transient SkeletalMeshComponent EditorMesh;

var transient ESFXDoorState m_PendingDoorState;

var transient bool m_ProcessPendingState;

var transient bool m_bIsTransitioning;

var transient array<BioVisualEffect> m_aCurrentIconVFX;

var(SFXDoor_Sound) WwiseEvent m_Wwise_GUI_Kismet_Disabled;

var(SFXDoor_Sound) WwiseEvent m_Wwise_GUI_Kismet_Enabled;

var(SFXDoor_Sound) WwiseEvent m_Wwise_GUI_Kismet_UnLocked;

var(SFXDoor_Sound) WwiseEvent m_Wwise_GUI_Kismet_Locked;

var(SFXDoor_Sound) WwiseEvent m_Wwise_GUI_Failure_HackLocked;

var(SFXDoor_Sound) WwiseEvent m_Wwise_GUI_Failure_PlotLocked;

var(SFXDoor_Sound) WwiseEvent m_Wwise_GUI_Success_Opening;

var(SFXDoor_Sound) WwiseEvent m_Wwise_Transition_Close;

var(SFXDoor_Sound) WwiseEvent m_Wwise_Transition_Open;

// m_srTargetTipTextPlotLocked: StringRefProperty[44]
// m_srTargetTipTextHackLocked: StringRefProperty[44]
// m_srTargetTipTextClosed: StringRefProperty[44]
var(SFXDoor_GUI) BioVFXTemplate m_IconTemplateOpenTransition;

var(SFXDoor_GUI) BioVFXTemplate m_IconTemplatePlotLocked;

var(SFXDoor_GUI) BioVFXTemplate m_IconTemplateHackable;

var(SFXDoor_GUI) BioVFXTemplate m_IconTemplateClosed;

var(SFXDoor_GUI) array<Name> m_aIconSocket;

var(SFXDoor_Animation) Name m_TransitionClose;

var(SFXDoor_Animation) Name m_TransitionOpen;

var String m_sRefName;

var int m_nPrevPlotIndex;

var int m_nPrevRegionIndex;

var String m_sVarName;

var int m_nIndex;

var() array<SFXDoorMarker> m_aDoorMarker;

var(Save) bool bSaveMe;

var() bool m_bOpenWhenInteractedWhileUnlocked;

var(SFXDoor_Plot) EBioAutoSet Int;

var(SFXDoor_Plot) EBioPlotAutoSet Plot;

var(SFXDoor_Plot) EBioRegionAutoSet Region;

var() ESFXDoorType m_DoorType;

var ESFXDoorState m_PreviousDoorState;

var() ESFXDoorState m_CurrentDoorState;

function OnUse(Actor User)
{
    TriggerInteraction(User);
    switch (m_CurrentDoorState)
    {
        case 0:
            PlayWiseEvent(m_Wwise_GUI_Success_Opening);
            break;
        case 2:
            PlayWiseEvent(m_Wwise_GUI_Failure_HackLocked);
            break;
        case 3:
            PlayWiseEvent(m_Wwise_GUI_Failure_PlotLocked);
            break;
    }
    if (m_bOpenWhenInteractedWhileUnlocked && m_CurrentDoorState == 0)
    {
        SetDoorState(1, User);
    }
}

final private function PlayAnim(Name InAnimSeqName, optional bool bSkipToEnd = false)
{
    local AnimNodeSequence SeqNode;
    SkeletalMeshComponent.bIsFrozen = false;
    SeqNode = AnimNodeSequence(SkeletalMeshComponent.Animations);
    if (SeqNode != None)
    {
        if (SeqNode.AnimSeqName != InAnimSeqName)
        {
            SeqNode.SetAnim(InAnimSeqName);
        }
        if (bSkipToEnd)
        {
            SeqNode.PlayAnim(false, SeqNode.Rate, SeqNode.AnimSeq.SequenceLength);
        }
        else
        {
            SeqNode.PlayAnim(false, SeqNode.Rate, 0);
        }
    }
    m_bIsTransitioning = true;
}

event SetDoorState(ESFXDoorState ToState, optional Actor User, optional bool bInstantTransition = false)
{
    if (m_bIsTransitioning)
    {
        m_ProcessPendingState = true;
        m_PendingDoorState = ToState;
        return;
    }
    if (ToState == m_CurrentDoorState)
    {
        return;
    }
    switch (ToState);
    case 0;
    switch (m_CurrentDoorState);
    case 2;
    case 3;
    PlayWiseEvent(m_Wwise_GUI_Kismet_UnLocked);
    jump 154;
    case 1;
    CloseDoor(bInstantTransition);
    jump 154;
    case 4;
    PlayWiseEvent(m_Wwise_GUI_Kismet_Enabled);
    jump 154;
    default;
    jump 434;
    case 1;
    switch (m_CurrentDoorState);
    case 4;
    case 0;
    case 2;
    case 3;
    OpenDoor(bInstantTransition);
    jump 211;
    default;
    jump 434;
    case 2;
    PlayWiseEvent(m_Wwise_GUI_Kismet_Locked);
    switch (m_CurrentDoorState);
    case 1;
    CloseDoor(bInstantTransition);
    jump 291;
    case 4;
    PlayWiseEvent(m_Wwise_GUI_Kismet_Enabled);
    jump 291;
    default;
    jump 434;
    case 3;
    PlayWiseEvent(m_Wwise_GUI_Kismet_Locked);
    switch (m_CurrentDoorState);
    case 1;
    CloseDoor(bInstantTransition);
    jump 371;
    case 4;
    PlayWiseEvent(m_Wwise_GUI_Kismet_Enabled);
    jump 371;
    default;
    jump 434;
    case 4;
    PlayWiseEvent(m_Wwise_GUI_Kismet_Disabled);
    switch (m_CurrentDoorState);
    case 1;
    CloseDoor(bInstantTransition);
    jump 428;
    default;
    jump 434;
    default;
    m_PreviousDoorState = m_CurrentDoorState;
    m_CurrentDoorState = ToState;
    SaveDoorStates();
    if (!!m_bIsTransitioning) jump 487;
    OnTransitionEnd();
    TriggerStateChange(User);
    return;
}

function bool OpenDoor(optional bool bInstant)
{
    PlayAnim(m_TransitionOpen, bInstant);
    if (bInstant == false)
    {
        PlayWiseEvent(m_Wwise_Transition_Open);
    }
    GetModule(Class'SFXSelectionModule').m_bTargetable = false;
    SetDoorIcon(m_IconTemplateOpenTransition);
    return true;
}

function bool CloseDoor(optional bool bInstant)
{
    PlayAnim(m_TransitionClose, bInstant);
    if (bInstant == false)
    {
        PlayWiseEvent(m_Wwise_Transition_Close);
    }
    SetDoorMarkerState(false);
    return true;
}

function PostBeginPlay()
{
    local AnimNodeSequence SeqNode;
    PostBeginPlay();
    SFXSimpleUseModule(GetModule(Class'SFXSelectionModule')).__OnUsed__Delegate = OnUse;
    if (m_CurrentDoorState == 1)
    {
        OpenDoor(true);
    }
    if (SkeletalMeshComponent != None)
    {
        SeqNode = AnimNodeSequence(SkeletalMeshComponent.Animations);
        if (SeqNode != None && SeqNode.AnimSeq != None)
        {
            SeqNode.SetPosition(SeqNode.AnimSeq.SequenceLength, false);
            SeqNode.StopAnim();
        }
        SkeletalMeshComponent.bIsFrozen = true;
    }
    OnTransitionEnd();
}

function TriggerStateChange(Actor EventInstigator)
{
    local int I;
    I = 0;
    while (I < GeneratedEvents.Length)
    {
        if (ClassIsChildOf(GeneratedEvents[I].Class, Class'SFXSeqEvt_SFXDoorInteraction'))
        {
            SFXSeqEvt_SFXDoorInteraction(GeneratedEvents[I]).TriggerStateChange(m_CurrentDoorState, m_PreviousDoorState, EventInstigator);
        }
        I++;
    }
}

function TriggerInteraction(Actor EventInstigator)
{
    local int I;
    I = 0;
    while (I < GeneratedEvents.Length)
    {
        if (ClassIsChildOf(GeneratedEvents[I].Class, Class'SFXSeqEvt_SFXDoorInteraction'))
        {
            SFXSeqEvt_SFXDoorInteraction(GeneratedEvents[I]).TriggerInteraction(m_CurrentDoorState, EventInstigator);
        }
        I++;
    }
}

event SetDoorIcon(BioVFXTemplate oVFXTemplate)
{
    local BioVisualEffect Effect;
    local Name nmSocket;
    if (oVFXTemplate == None || m_aCurrentIconVFX.Length > 0)
    {
        foreach m_aCurrentIconVFX(Effect)
        {
            if (Effect != None)
            {
                Effect.SetState(2, true);
                if (Effect.pTemplate.bAllowParticlesToFinish && Effect.pTemplate.fLifeTime <= 0 && !Effect.bPauseOnDeath)
                {
                    Effect.DestroyInstance();
                }
                Effect = None;
            }
        }
        m_aCurrentIconVFX.Length = 0;
    }
    if (oVFXTemplate != None)
    {
        foreach m_aIconSocket(nmSocket)
        {
            Effect = Class'BioVisualEffect'.CreateVFXOnMesh(oVFXTemplate, self, nmSocket, 0, None, false);
            if (Effect != None)
            {
                Effect.SetState(0, true);
                m_aCurrentIconVFX.AddItem(Effect);
            }
        }
    }
}

event OnTransitionEnd()
{
    switch (m_CurrentDoorState)
    {
        case 0:
            if (m_DoorType == 0 || m_DoorType == 3)
            {
                GetModule(Class'SFXSelectionModule').m_bTargetable = true;
            }
            else
            {
                GetModule(Class'SFXSelectionModule').m_bTargetable = false;
            }
            SetDoorIcon(m_IconTemplateClosed);
            GetModule(Class'SFXSelectionModule').m_srTargetTipText = m_srTargetTipTextClosed;
            break;
        case 1:
            SetDoorMarkerState(true);
            break;
        case 2:
            GetModule(Class'SFXSelectionModule').m_bTargetable = true;
            SetDoorIcon(m_IconTemplateHackable);
            GetModule(Class'SFXSelectionModule').m_srTargetTipText = m_srTargetTipTextHackLocked;
            break;
        case 3:
            GetModule(Class'SFXSelectionModule').m_bTargetable = true;
            SetDoorIcon(m_IconTemplatePlotLocked);
            GetModule(Class'SFXSelectionModule').m_srTargetTipText = m_srTargetTipTextPlotLocked;
            break;
        case 4:
            SetDoorIcon(None);
            GetModule(Class'SFXSelectionModule').m_bTargetable = false;
            break;
    }
}

function SetDoorMarkerState(bool bOpen)
{
    local int I;
    I = 0;
    while (I < m_aDoorMarker.Length)
    {
        if (bOpen)
        {
            m_aDoorMarker[I].DoorOpened();
        }
        else
        {
            m_aDoorMarker[I].DoorClosed();
        }
        I++;
    }
}

native function PlayWiseEvent(WwiseEvent oWwiseEvent);

function SaveDoorStates()
{
    local BioGlobalVariableTable oGV;
    local int nCombined;
    local int nCurrentState;
    local int nPrevState;
    if (m_nIndex >= 0)
    {
        nCurrentState = m_CurrentDoorState;
        nPrevState = m_PreviousDoorState;
        nCombined = nPrevState << 9 | nCurrentState << 1 | 1;
        oGV = BioWorldInfo(WorldInfo).GetGlobalVariables();
        oGV.SetInt(m_nIndex, nCombined);
    }
}

event RestoreDoorStates()
{
    local BioGlobalVariableTable oGV;
    local int nPlotState;
    local int nCurrentState;
    local int nPrevState;
    if (m_nIndex >= 0)
    {
        oGV = BioWorldInfo(WorldInfo).GetGlobalVariables();
        nPlotState = oGV.GetInt(m_nIndex);
        if (1 & nPlotState != 0)
        {
            nCurrentState = 255 & nPlotState >> 1;
            nPrevState = 255 & nPlotState >> 9;
            m_CurrentDoorState = nCurrentState;
            m_PreviousDoorState = nPrevState;
        }
    }
}

defaultproperties
{
    m_DoorType=EDT_Proximity
    m_bOpenWhenInteractedWhileUnlocked=true
    bSaveMe=true
    m_nIndex=-1
    m_sVarName="Int"
    m_sRefName="Unset"
    m_TransitionOpen=trans_closed_opened
    m_TransitionClose=trans_opened_closed
    m_aIconSocket(0)=Socket_Icon01
    m_aIconSocket(1)=Socket_Icon02
    m_IconTemplateClosed=BioVFXTemplate'Holo_Door_Green_VFXTemplate'
    m_IconTemplateHackable=BioVFXTemplate'Holo_Door_Red_VFXTemplate'
    m_IconTemplatePlotLocked=BioVFXTemplate'Holo_Door_Red2_VFXTemplate'
    m_IconTemplateOpenTransition=BioVFXTemplate'Holo_Door_Green_Disappear_VFX'
    m_srTargetTipTextClosed=System.Byte[]
    m_srTargetTipTextHackLocked=System.Byte[]
    m_srTargetTipTextPlotLocked=System.Byte[]
    m_Wwise_GUI_Failure_PlotLocked=WwiseEvent'Play_gui_door_failure_locked_generic'
    m_Wwise_GUI_Failure_HackLocked=WwiseEvent'Play_gui_door_failure_hackable_generic'
    m_Wwise_GUI_Kismet_Locked=WwiseEvent'Play_gui_door_locked_generic'
    m_Wwise_GUI_Kismet_UnLocked=WwiseEvent'Play_gui_door_unlocked_generic'
    m_Wwise_GUI_Kismet_Enabled=WwiseEvent'Play_gui_door_enabled_generic'
    m_Wwise_GUI_Kismet_Disabled=WwiseEvent'Play_gui_door_disabled_generic'
    SkeletalMeshComponent=None
    LightEnvironment=None
    FacialAudioComp=None
    Components=None
    // child object FaceAudioComponent of type AudioComponent
    // child object MyLightEnvironment of type DynamicLightEnvironmentComponent
    // child object SelMod01 of type SFXSimpleUseModule
    // child object SkeletalMeshComponent0 of type SkeletalMeshComponent
}
