﻿class SFXEngine extends GameEngine
    config(Engine)
    transient
    native;

enum ESaveGameAction
{
    SaveGame_DoNothing,
    SaveGame_Load,
    SaveGame_LoadAutoSave,
    SaveGame_LoadChapterSave,
    SaveGame_Resume,
    SaveGame_Save,
    SaveGame_Overwrite,
    SaveGame_AutoSave,
    SaveGame_ChapterSave,
    SaveGame_QuickSave,
    SaveGame_LoadQuickSave,
    SaveGame_Delete,
    SaveGame_DeleteAutoSave,
    SaveGame_LoadExternalSave,
};

// srCantContinueErrorMessage: StringRefProperty[44]
var transient float LastCantContinueTime;

// CancelDeleteCorruptText: StringRefProperty[44]
// ConfirmDeleteCorruptText: StringRefProperty[44]
// CorruptCareerWarningText: StringRefProperty[44]
var transient array<int> AsyncLoadingMapping;

var const config array<PackageRemapInfo> DynamicLoadPackageRemapping;

var const config array<DynamicLoadInfo> DynamicLoadMapping;

var transient int LastDeleteGameIndex;

var transient int LastSaveGameIndex;

var transient Vector LastSaveGameLocation;

var transient ESaveGameAction LastSaveGameAction;

var() transient byte PendingModeToRemove;

var() BlazeDataStore BlazeCache;

var transient UniqueNetId m_oProfilePlayerID;

var transient UniqueNetId m_oInitialPlayerID;

var String PlayerLoadoutWeapons[6];

var transient bool m_bNGSavesExist;

var transient bool m_bSavesExist;

var transient bool m_bSavesExistInCurrentCareer;

var transient bool m_bProfileInitialized;

var transient bool m_bInLoad;

var transient bool m_bRenderingSuspended;

var transient bool bFlushInputRequested;

var() array<String> CorruptedCareers;

var() bool bPlayerLoadPosition;

var() bool bPlayerNeedsLoad;

var() bool bSavingGame;

var() bool bDoneSaving;

var() bool bCanWriteSaveToStorage;

var() SFXSaveGame ME2ImportSaveGame;

var() SFXSaveGame CurrentSaveGame;

var() String CurrentCareerName;

var() array<DoorSaveRecord> SavedDoorList;

var() array<HenchmanSaveRecord> HenchmanRecords;

var() array<KismetBoolSaveRecord> SavedKismetList;

var() array<Guid> DeadPawnList;

var() int CurrentLoadingTip;

var() int CurrentDeviceID;

var() int CurrentUserID;

var() float LastSecondsPlayed;

var() float LastSaveTime;

var() String CareerNameOverride;

var() int PendingDeleteGameIndex;

var() int PendingSaveGameIndex;

var() ESaveGameAction PendingSaveGameAction;

var() Vector PendingSaveGameLocation;

struct native BlazeDataStore
{
    var() int DaysSinceRegistration;

    var() array<byte> LoginInfo;

    var() array<byte> PersonaID;

    var() bool bNucleusRefused;

    var() bool bNucleusSuccessful;

    var() bool bCerberusRefused;

    var() bool bAutoLogin;

    var() bool bDirty;

}

enum EAsyncLoadStatus
{
    ASYNC_LOAD_ERROR,
    ASYNC_LOAD_STARTED,
    ASYNC_LOAD_INPROGRESS,
    ASYNC_LOAD_COMLETE,
};

struct native DynamicLoadInfo
{
    var String ObjectName;

    var Name SeekFreePackageName;

    var transient Object CachedObjectHandle;

    var transient Object LoadedLinkerRoot;

}

struct native PackageRemapInfo
{
    var Name PackageName;

    var Name SeekFreePackageName;

}

native final static function SkipMovie();

native function ReLaunchExecutable();

native final static function LaunchUnreaper();

native function LaunchUnreaperWithDiscCheck();

native final static function bool IsSeekFreeObjectSupported(String ObjectName);

native final static function Object LoadSeekFreeObject(String ObjectName, class<Object> ObjectClass);

native final static function Object LoadSeekFreeObjectAsync(String Object, class<Object> ObjectClass, out EAsyncLoadStatus Status);

native final static function ReleaseSeekFreeObject(String ObjectName);

native final static function float CurrentSystemTimeSeconds();

native final protected function SaveGame(Vector SaveGameLocation, optional int SlotIndex);

native final protected function ResumeGame();

native final protected function LoadGame(optional int SlotIndex);

native final protected function LoadGameExternal();

native final protected function DeleteGame(int DeleteIndex);

native final function bool ReadSaveFromFile(SFXSaveGame SaveGame, String SavePath);

native final function CacheProfileData(SFXProfileSettings Profile);

native final function String ValidCharsFilter(String Sin, bool bFilterAccentedChars);

native final function CreateCareer(String FirstName, EOriginType Origin, ENotorietyType Notoriety);

native final function ClearSaveCache();

function bool SavesExistInCurrentCareer()
{
    return m_bSavesExistInCurrentCareer;
}

function bool SavesExist()
{
    return m_bSavesExist;
}

function bool NGSavesExist()
{
    return m_bNGSavesExist;
}

native final function ScanSaves();

native final function ImportME2Character(optional bool bChapterSave, optional bool bAutoSave, optional bool bQuickSave, optional String CareerName, optional int Index);

native final function Callback_ConfirmDeleteCorruptCareers(bool bAPressed, int Context);

native final function SuspendRendering();

native final function ResumeRendering();

final event CheckForCorruptCareers()
{
    local MassEffectGuiManager GuiMan;
    local BioSFHandler_MessageBox MessageBox;
    local BioMessageBoxOptionalParams Params;
    if (CorruptedCareers.Length > 0)
    {
        GuiMan = MassEffectGuiManager(BioWorldInfo(GetCurrentWorldInfo()).GetLocalPlayerController().GetScaleFormManager());
        if (GuiMan == None)
        {
            return;
        }
        MessageBox = GuiMan.CreateMessageBox();
        MessageBox.SetInputDelegate(Callback_ConfirmDeleteCorruptCareers);
        Params.srAText = ConfirmDeleteCorruptText;
        Params.srBText = CancelDeleteCorruptText;
        Params.bNoFade = true;
        MessageBox.DisplayMessageBox(CorruptCareerWarningText, Params);
    }
}

native final function UpdateCurrentDevice(int DeviceID);

native final function int GetCurrentDevice();

native final function LoadPlayer();

native final function float GetPlayTimeSeconds();

native final function float GetCurrentTime();

native final function ForceGUIUpdate();

event bool IsCurrentDeviceValid(optional int SaveSize)
{
    local OnlineSubsystem OnlineSub;
    OnlineSub = GetOnlineSubsystem();
    if (OnlineSub != None && NotEqual_InterfaceInterface(OnlineSub.PlayerInterfaceEx, OnlinePlayerInterfaceEx(None)))
    {
        return OnlineSub.PlayerInterfaceEx.IsDeviceValid(CurrentDeviceID, SaveSize);
    }
    return false;
}

native function bool FindCurrentSaveDevice();

native function BioShowDebugMessageBox(String sMessage);

native function FlushIOHandles();

defaultproperties
{
    bCanWriteSaveToStorage=true
    bDoneSaving=true
    DynamicLoadMapping=System.Byte[]
    CorruptCareerWarningText=System.Byte[]
    ConfirmDeleteCorruptText=System.Byte[]
    CancelDeleteCorruptText=System.Byte[]
}
