﻿class SFXGameEffect_PowerArmor extends SFXGameEffect_GiveShield
    hidecategories(Object);

var array<EAICustomAction> AnimatedReactions;

var float ExplosionShieldBoost;

var float ExplosionVFXParameter;

var float ExplosionForce;

var float ExplosionDamage;

var float ExplosionRadius;

var bool bShieldDestroyed;

var ParticleSystem ShieldExplosion;

var BioPawn OwnerPawn;

var BioVisualEffect HenchVFX;

var BioVisualEffect PlayerVFX;

var BioVFXTemplate ShieldDeathCrust;

var BioVFXTemplate ShieldCrust_Female;

var BioVFXTemplate ShieldCrust_Male;

function OnApplied()
{
    local SFXShield_Base Shields;
    OnApplied();
    OwnerPawn = BioPawn(Owner);
    if (OwnerPawn == None)
    {
        return;
    }
    Shields = OwnerPawn.GetShields();
    if (Shields != None)
    {
        Shields.CurrentShields = OwnerPawn.GetMaxShields();
    }
    TurnVFXOn();
}

function OnUpdate(float DeltaSeconds)
{
    local BioWorldInfo Info;
    local BioPlayerController PC;
    OnUpdate(DeltaSeconds);
    if (OwnerPawn.GetCurrentShields() <= 0)
    {
        OnShieldDestroyed(true);
        CurrentTime = Duration + 1;
        DurationType = 1;
        return;
    }
    Info = BioWorldInfo(Owner.WorldInfo);
    if (Info != None)
    {
        PC = Info.GetLocalPlayerController();
        if (PC != None && PC.GameModeManager2.IsActive(6) || PC.GameModeManager2.IsActive(5))
        {
            TurnVFXOff();
        }
        else
        {
            TurnVFXOn();
        }
    }
}

function TurnVFXOn()
{
    if (PlayerVFX == None)
    {
        if (OwnerPawn.bIsFemale)
        {
            PlayerVFX = Class'BioVisualEffect'.CreateVFXOnMesh(ShieldCrust_Female, OwnerPawn, 'None', Duration, false);
        }
        else
        {
            PlayerVFX = Class'BioVisualEffect'.CreateVFXOnMesh(ShieldCrust_Male, OwnerPawn, 'None', Duration, false);
        }
        if (PlayerVFX != None)
        {
            PlayerVFX.SetState(0, true);
        }
    }
}

function TurnVFXOff()
{
    if (PlayerVFX != None)
    {
        PlayerVFX.SetState(2, true, true);
        PlayerVFX = None;
    }
}

function OnShieldDestroyed(bool PlayDeathCrust)
{
    if (OwnerPawn == None)
    {
        return;
    }
    if (bShieldDestroyed)
    {
        return;
    }
    TurnVFXOff();
    bShieldDestroyed = true;
    if (PlayDeathCrust)
    {
        PlayerVFX = Class'BioVisualEffect'.CreateVFXOnMesh(ShieldDeathCrust, Owner, 'None', 5);
        if (PlayerVFX != None)
        {
            PlayerVFX.SetState(0, true);
        }
        AreaExplosion();
    }
}

function AreaExplosion()
{
    local BioPawn oImpactedPawn;
    local Vector Momentum;
    local SFXGame oGame;
    local SFXObjectPool oPool;
    local SFXEmitter oEmitter;
    if (OwnerPawn == None)
    {
        return;
    }
    oGame = SFXGame(OwnerPawn.WorldInfo.Game);
    if (oGame == None)
    {
        return;
    }
    oPool = oGame.ObjectPool;
    if (oPool == None)
    {
        return;
    }
    oEmitter = SFXEmitter(oPool.GetImpactEmitter(ShieldExplosion, Owner.Location, rot(0,0,0)));
    if (oEmitter != None && oEmitter.ParticleSystemComponent != None)
    {
        oEmitter.ParticleSystemComponent.SetFloatParameter('P_Radius', ExplosionVFXParameter);
        oEmitter.ParticleSystemComponent.SetActive(true);
    }
    foreach Owner.CollidingActors(Class'BioPawn', oImpactedPawn, ExplosionRadius, Owner.Location)
    {
        if (OwnerPawn.IsHostile(oImpactedPawn))
        {
            oImpactedPawn.TakeDamage(ExplosionDamage, OwnerPawn.Controller, vect(0,0,0), vect(0,0,0), Class'SFXDamageType_PowerArmor');
            if (Class'BioPowerScriptDesign'.GetPhysicsLevel(oImpactedPawn) >= 4)
            {
                Class'BioPowerScriptDesign'.PlayRandomReaction(oImpactedPawn, Instigator, AnimatedReactions);
            }
            else
            {
                Momentum = Normal(oImpactedPawn.Location - Owner.Location) * ExplosionForce;
                oImpactedPawn.AddRagdollImpulse(Momentum, Instigator, vect(0,0,0), true);
            }
        }
    }
}

function OnRemoved()
{
    local SFXModule_GameEffectManager Manager;
    local SFXGameEffect Effect;
    OnRemoved();
    OnShieldDestroyed(false);
    if (Owner != None)
    {
        Manager = Owner.GetModule(Class'SFXModule_GameEffectManager');
        if (Manager != None)
        {
            foreach Manager.GameEffects(Effect)
            {
                if (Effect.Category == Category && Effect != self)
                {
                    Effect.DurationType = 1;
                    Effect.Duration = 0;
                }
            }
        }
    }
    if (bShieldDestroyed && ExplosionShieldBoost > 0 && OwnerPawn != None)
    {
        Class'BioPowerScriptDesign'.ApplyTemporaryGameEffect(Owner, Class'SFXGameEffect_HealShieldOverTime', 1, ExplosionShieldBoost * OwnerPawn.GetMaxShields(), Category, Pawn(Owner).Controller);
    }
}

defaultproperties
{
    ShieldCrust_Male=BioVFXTemplate'PowerArmor_VFX_Crust_MShep'
    ShieldCrust_Female=BioVFXTemplate'PowerArmor_VFX_Crust_FShep'
    ShieldDeathCrust=BioVFXTemplate'PowerArmor_Dest_VFX'
    ShieldExplosion=ParticleSystem'Flashbang_Explosion'
    AnimatedReactions(0)=AI_CustomAction_Power_Stagger
    AnimatedReactions(1)=AI_CustomAction_Power_StaggerVariant
    AnimatedReactions(2)=AI_CustomAction_Power_Knockback
    AnimatedReactions(3)=AI_CustomAction_Power_KnockbackVariant
    AnimatedReactions(4)=AI_CustomAction_Power_Reave
    ShieldClass=class'SFXShield_Armour'
}
