﻿class SFXGameModeManager extends Object
    within BioPlayerController
    config(Game)
    native;

enum EGameModes
{
    GameMode_Default,
    GameMode_PowerWheel,
    GameMode_WeaponWheel,
    GameMode_Command,
    GameMode_Vehicle,
    GameMode_Conversation,
    GameMode_Cinematic,
    GameMode_GUI,
    GameMode_Movie,
    GameMode_Galaxy,
    GameMode_Orbital,
    GameMode_CheatMenu,
};

var String sAudioGameModeRTPCName;

var config array<LocalizedKeyName> KeyNames;

var transient EGameModes CurrentMode;

var transient array<SFXGameModeBase> GameModes;

var array<class<SFXGameModeBase> > SupportedModes;

struct native LocalizedKeyName
{
    var Name Key;

// Name: StringRefProperty[44]
}

event EnableMode(EGameModes Mode, optional Name ModeSpecifier)
{
    local SFXGameModeBase GameMode;
    GameMode = GameModes[Mode];
    if (GameMode != None)
    {
        GameMode.ActivateSpecifier(ModeSpecifier);
        if (GameMode.bIsActive == false)
        {
            GameMode.Activated();
        }
        if (Mode != CurrentMode)
        {
            if (CurrentMode == 0)
            {
                ResetKeyPresses();
            }
            UpdateCurrentMode();
            return;
        }
    }
}

event DisableMode(EGameModes Mode, optional Name ModeSpecifier)
{
    local SFXGameModeBase GameMode;
    GameMode = GameModes[Mode];
    if (GameMode != None)
    {
        GameMode.DeactivateSpecifier(ModeSpecifier);
        if (GameMode.bIsActive)
        {
            ResetKeyPresses();
            GameMode.Deactivated();
        }
        if (Mode == CurrentMode)
        {
            UpdateCurrentMode();
            return;
        }
    }
}

final function SFXCameraMode GetCameraMode(SFXCameraMode OldCameraMode, out int PreserveTarget, out float TransitionTime, out SFXCameraMode_Interpolate Transition)
{
    return GameModes[CurrentMode].GetCameraMode(OldCameraMode, PreserveTarget, TransitionTime, Transition);
}

final private function Helper_SetBinding(SFXGameModeBase Mode, Name Key, String Command)
{
    local int idx;
    local KeyBind NewBind;
    if (Mode != None)
    {
        idx = Mode.Bindings.Find('Name', Key);
        if (idx != -1)
        {
            Mode.Bindings[idx].Command = Command;
        }
        else
        {
            NewBind.Name = Key;
            NewBind.Command = Command;
            Mode.Bindings.AddItem(NewBind);
        }
    }
}

final function Helper_SetStaticConsoleBinding(Name Key, String Command)
{
    local int idx;
    local StaticKeyBind NewBind;
    local BioPlayerInput Input;
    Input = BioPlayerInput(Outer.PlayerInput);
    if (Input != None)
    {
        idx = Input.StaticConsoleBinds.Find('Name', Key);
        if (idx != -1)
        {
            Input.StaticConsoleBinds[idx].Command = Command;
        }
        else
        {
            NewBind.Name = Key;
            NewBind.Command = Command;
            Input.StaticConsoleBinds.AddItem(NewBind);
        }
    }
}

final function Helper_ResetStaticConsoleBindings()
{
    local BioPlayerInput Input;
    Input = BioPlayerInput(Outer.PlayerInput);
    if (Input != None)
    {
        Input.StaticConsoleBinds = Class'BioPlayerInput'.Default.StaticConsoleBinds;
    }
}

final function Console_UpdateStickBindings(EStickConfigOptions ConfigOption)
{
    local SFXGameModeBase Mode;
    local int ModeType;
    foreach GameModes(Mode)
    {
        // unresolved reference 84087296 ;
    }
}

final function Console_UpdateTriggerBindings(ETriggerConfigOptions ConfigOption)
{
    local SFXGameModeBase Mode;
    local int ModeType;
    foreach GameModes(Mode)
    {
        // unresolved reference 84174080 ;
    }
}

final function SFXGameModeBase HACK_GetOrbitalMode()
{
    return GameModes[10];
}

final event bool HACK_CanExitGalaxyMode()
{
    return SFXGameModeGalaxy(GameModes[CurrentMode]).CanExit();
}

final event HACK_BeginExitGalaxyMap(bool resize)
{
    SFXGameModeGalaxy(GameModes[CurrentMode]).BeginExitGalaxyMap(resize);
}

native final function SFXCameraMode HACK_GetCameraMode(EGameModes GalaxyOrConversationMode);

native final function Initialize();

native function UpdateCurrentMode();

native final function bool IsActive(EGameModes Mode);

native final function bool ShouldShowHUD();

native final function bool ShouldShowSelection();

native final function bool ShouldShowDamageIndicators();

native final function bool ShouldShowRadar();

native final function bool AllowRotationUpdates();

native final function bool AllowsCameraMods();

native final function bool AllowsSaving();

native final function bool ShouldShowSubtitles();

native final function bool HasMouseAuthority();

native final function bool GetMouseVisible();

native final function bool ShouldMergeNotifications();

native final function bool ShouldQueueAndSuppressNotifications();

native final function bool ShouldShowReticles();

native final function bool ShouldPlayVocalizations();

native final function bool ShouldEnforce16x9Subtitiles();

native final function ResetBindings();

native final function ResetKeyPresses();

defaultproperties
{
    SupportedModes(0)=class'SFXGameModeDefault'
    SupportedModes(1)=class'SFXGameModePowerWheel'
    SupportedModes(2)=class'SFXGameModeWeaponWheel'
    SupportedModes(3)=class'SFXGameModeCommand'
    SupportedModes(4)=class'SFXGameModeVehicle'
    SupportedModes(5)=class'SFXGameModeConversation'
    SupportedModes(6)=class'SFXGameModeCinematic'
    SupportedModes(7)=class'SFXGameModeGUI'
    SupportedModes(8)=class'SFXGameModeMovie'
    SupportedModes(9)=class'SFXGameModeGalaxy'
    SupportedModes(10)=class'SFXGameModeOrbital'
    SupportedModes(11)=class'SFXGameModeCheatMenu'
    KeyNames=System.Byte[]
    sAudioGameModeRTPCName="Audio_Game_Mode"
}
