﻿class SFXInventoryManager extends InventoryManager
    hidecategories(Navigation)
    config(Weapon);

var float DriftRampUpDelay;

var float DriftRampUpRate;

var float DriftNoiseMagYaw;

var float DriftNoiseMagPitch;

var float DriftInterpMinSize;

var float DriftInterpResetSize;

var float DriftInterpSpeed;

var float DriftYaw;

var float DriftPitch;

var Rotator DriftTarget;

var Rotator Drift;

var Rotator TotalRecoil;

var Rotator WeaponRecoilOffset;

var transient float TimeSinceLastFire;

var transient float AccMod_WeaponFired;

var transient bool bWeaponFired;

var transient Rotator AccuracyLastViewRot;

var float Accuracy;

var() config Vector2D InterpRange;

var() config float InterpMul_ViewTurn;

var() config float InterpMod_Move;

var() config float Base_Interp_Speed;

var() config float AccMod_MaxCameraLoss;

var() config float AccMod_MouseMultiplier;

var() config float AccMod_Crouched;

var() config float AccMod_ViewTurn;

var() config float AccMod_Move;

var() config float Base_Acc_Zoom;

var() config float Base_Acc_Standing;

var class<SFXWeapon> CurrentWeaponSelection;

var config float FuelEfficiency;

var config float MaxFuel;

var config int MaxProbes;

var config int MaxMedigel;

var config int MaxPlatinum;

var config int MaxPalladium;

var config int MaxIridium;

var config int MaxEezo;

var config int MaxCredits;

var() float CurrentFuel;

var() int Probes;

var() int Platinum;

var() int Palladium;

var() int Iridium;

var() int Eezo;

var() int Medigel;

var() int Credits;

enum CREDCurveSet
{
    CREDCurve_NoReward,
    CREDCurve_Minor,
    CREDCurve_Small,
    CREDCurve_Medium,
    CREDCurve_Large,
    CREDCurve_Major,
};

simulated function SwitchToBestWeapon(optional bool bForceADifferentWeapon);

simulated function SetWeaponRecoil(float PitchRecoil)
{
    local float YawRecoil;
    local float BaseRecoilCap;
    local BioPawn Pawn;
    local SFXWeapon Weapon;
    local float fOscillation;
    Weapon = SFXWeapon(Instigator.Weapon);
    Pawn = BioPawn(Instigator);
    if (Weapon == None || Pawn == None)
    {
        return;
    }
    YawRecoil = 0;
    if (Weapon.IsZoomed())
    {
        YawRecoil = Weapon.RecoilYawBias;
    }
    YawRecoil += Sin(WorldInfo.TimeSeconds * Weapon.RecoilYawFrequency);
    YawRecoil *= PitchRecoil * Weapon.RecoilYawScale * 0,5;
    BaseRecoilCap = Weapon.RecoilCap * 182,0444;
    if (Weapon.IsZoomed())
    {
        BaseRecoilCap = Weapon.ZoomRecoilCap * 182,0444;
    }
    if (Abs(TotalRecoil.Yaw) < BaseRecoilCap)
    {
        if (TotalRecoil.Yaw + YawRecoil > BaseRecoilCap)
        {
            YawRecoil = BaseRecoilCap - TotalRecoil.Yaw;
        }
        WeaponRecoilOffset.Yaw += YawRecoil;
        TotalRecoil.Yaw += YawRecoil;
    }
    if (Abs(TotalRecoil.Pitch) < BaseRecoilCap)
    {
        if (Weapon.RecoilPitchOscillation != 0)
        {
            fOscillation = 1 - Weapon.RecoilPitchOscillation;
            fOscillation += Sin(WorldInfo.TimeSeconds * Weapon.RecoilPitchFrequency) * Weapon.RecoilPitchOscillation;
            PitchRecoil *= fOscillation;
        }
        if (TotalRecoil.Pitch + PitchRecoil > BaseRecoilCap)
        {
            PitchRecoil = BaseRecoilCap - TotalRecoil.Pitch;
        }
        WeaponRecoilOffset.Pitch += PitchRecoil;
        TotalRecoil.Pitch += PitchRecoil;
    }
}

simulated function UpdateKickback(float DeltaTime, out Rotator out_DeltaRot)
{
    local Rotator DeltaRecoil;
    local SFXWeapon Weapon;
    Weapon = SFXWeapon(Instigator.Weapon);
    if (WeaponRecoilOffset != rot(0,0,0))
    {
        DeltaRecoil.Pitch = WeaponRecoilOffset.Pitch - FInterpTo(WeaponRecoilOffset.Pitch, 0, DeltaTime, Weapon.RecoilInterpSpeed);
        DeltaRecoil.Yaw = WeaponRecoilOffset.Yaw - FInterpTo(WeaponRecoilOffset.Yaw, 0, DeltaTime, Weapon.RecoilInterpSpeed);
        if (DeltaRecoil == rot(0,0,0))
        {
            WeaponRecoilOffset.Yaw = 0;
            WeaponRecoilOffset.Pitch = 0;
            WeaponRecoilOffset.Roll = 0;
        }
        else
        {
            WeaponRecoilOffset -= DeltaRecoil;
            out_DeltaRot += DeltaRecoil;
        }
    }
}

simulated function bool ApplySniperDrift(float DeltaTime)
{
    local SFXWeapon Weapon;
    local bool IsPitchPositive;
    local float RangeScale;
    local float Range;
    local Actor Target;
    local bool bHasTargetInRange;
    if (DriftNoiseMagYaw == 0 && DriftNoiseMagPitch == 0)
    {
        return false;
    }
    Weapon = SFXWeapon(Instigator.Weapon);
    if (Weapon == None || Weapon.bUseSniperCam == false || Weapon.IsZoomed() == false)
    {
        DriftTarget.Pitch = 0;
        DriftTarget.Yaw = 0;
        DriftPitch = DriftTarget.Pitch;
        DriftYaw = DriftTarget.Yaw;
        DriftInterpSpeed = 0;
        return false;
    }
    if (Instigator != None && BioPlayerController(Instigator.Controller) != None)
    {
        Target = BioPlayerController(Instigator.Controller).m_oPlayerSelection.m_oCurrentSelectionTarget;
        if (Target != None)
        {
            Range = VSize(Target.Location - Instigator.Location);
            if (Range <= Weapon.Range_Long)
            {
                bHasTargetInRange = true;
            }
        }
    }
    RangeScale = bHasTargetInRange ? 1 : 1,5;
    IsPitchPositive = DriftTarget.Pitch > 0;
    if (Abs(DriftPitch - DriftTarget.Pitch) + Abs(DriftYaw - DriftTarget.Yaw) < DriftInterpResetSize * RangeScale)
    {
        while (Abs(DriftPitch - DriftTarget.Pitch) + Abs(DriftYaw - DriftTarget.Yaw) < DriftInterpMinSize * RangeScale)
        {
            DriftTarget.Yaw = FRand() - 0,5 * DriftNoiseMagYaw * RangeScale;
            DriftTarget.Pitch = Abs(FRand() - 0,5 * DriftNoiseMagPitch * RangeScale);
            DriftTarget.Pitch = IsPitchPositive ? -1 * DriftTarget.Pitch : DriftTarget.Pitch;
            DriftInterpSpeed = 0;
            DriftRampUpDelay = Default.DriftRampUpDelay;
        }
    }
    if (DeltaTime > 0 && DriftInterpSpeed < Default.DriftInterpSpeed)
    {
        if (DriftRampUpDelay > 0)
        {
            DriftRampUpDelay -= DeltaTime;
        }
        else
        {
            DriftInterpSpeed += Default.DriftInterpSpeed * DeltaTime * DriftRampUpRate * RangeScale;
            DriftInterpSpeed = FClamp(DriftInterpSpeed, 0, Default.DriftInterpSpeed);
        }
    }
    DriftPitch = FInterpTo(DriftPitch, DriftTarget.Pitch, DeltaTime, DriftInterpSpeed);
    DriftYaw = FInterpTo(DriftYaw, DriftTarget.Yaw, DeltaTime, DriftInterpSpeed);
    Drift.Pitch = Round(DriftPitch);
    Drift.Yaw = Round(DriftYaw);
    return true;
}

simulated function UpdateKickbackFade(float DeltaTime, out Rotator out_DeltaRot)
{
    local SFXWeapon Weapon;
    local Rotator DeltaRecoil;
    local float fFadeSpeed;
    local float fMinYaw;
    local Rotator CurrentKickback;
    Weapon = SFXWeapon(Instigator.Weapon);
    if (Weapon == None)
    {
        return;
    }
    CurrentKickback = TotalRecoil - WeaponRecoilOffset;
    if (CurrentKickback == rot(0,0,0))
    {
        return;
    }
    fFadeSpeed = Weapon.IsZoomed() ? Weapon.RecoilZoomFadeSpeed : Weapon.RecoilFadeSpeed;
    fFadeSpeed = fFadeSpeed * Weapon.GetWeaponRecoil();
    fFadeSpeed = fFadeSpeed * 182,0444;
    fFadeSpeed = fFadeSpeed * Weapon.GetRateOfFire();
    fFadeSpeed = fFadeSpeed / 60;
    DeltaRecoil.Pitch = fFadeSpeed * DeltaTime;
    DeltaRecoil.Pitch *= CurrentKickback.Pitch > 0 ? 1 : -1;
    DeltaRecoil.Yaw = 0;
    if (Weapon.GetRateOfFire() < 400 || PendingFire[0] == 0 || PendingFire[1] != 0)
    {
        DeltaRecoil.Yaw = CurrentKickback.Yaw - FInterpTo(CurrentKickback.Yaw, 0, DeltaTime, 5);
    }
    if (CurrentKickback.Pitch >= 0)
    {
        DeltaRecoil.Pitch = FClamp(DeltaRecoil.Pitch, Weapon.RecoilMinFadePitch * DeltaTime, CurrentKickback.Pitch);
        DeltaRecoil.Pitch = FClamp(DeltaRecoil.Pitch, 0, CurrentKickback.Pitch);
    }
    else
    {
        DeltaRecoil.Pitch = FClamp(DeltaRecoil.Pitch, CurrentKickback.Pitch, -1 * Weapon.RecoilMinFadePitch * DeltaTime);
        DeltaRecoil.Pitch = FClamp(DeltaRecoil.Pitch, CurrentKickback.Pitch, 0);
    }
    fMinYaw = Abs(CurrentKickback.Yaw) > Abs(CurrentKickback.Pitch) ? Weapon.RecoilMinFadeYaw : 0;
    if (CurrentKickback.Yaw >= 0)
    {
        DeltaRecoil.Yaw = FClamp(DeltaRecoil.Yaw, fMinYaw * DeltaTime, CurrentKickback.Yaw);
        DeltaRecoil.Yaw = FClamp(DeltaRecoil.Yaw, 0, CurrentKickback.Yaw);
    }
    else
    {
        DeltaRecoil.Yaw = FClamp(DeltaRecoil.Yaw, CurrentKickback.Yaw, -1 * fMinYaw * DeltaTime);
        DeltaRecoil.Yaw = FClamp(DeltaRecoil.Yaw, CurrentKickback.Yaw, 0);
    }
    if (DeltaRecoil == rot(0,0,0))
    {
        TotalRecoil.Yaw = 0;
        TotalRecoil.Pitch = 0;
        TotalRecoil.Roll = 0;
    }
    else
    {
        TotalRecoil -= DeltaRecoil;
        out_DeltaRot += DeltaRecoil;
    }
}

simulated function Tick(float DeltaTime)
{
    local BioPawn Pawn;
    local SFXWeapon Weapon;
    Tick(DeltaTime);
    if (Instigator != None && Instigator.IsHumanControlled())
    {
        UpdatePlayerAccuracy(DeltaTime);
        Pawn = BioPawn(Instigator);
        if (Pawn != None && Pawn.bWeaponDebug_Accuracy)
        {
            DebugDrawWeaponAccuracy();
        }
    }
    if (Instigator != None && Instigator.Weapon != None)
    {
        Weapon = SFXWeapon(Instigator.Weapon);
        Weapon.DissipateHeat(DeltaTime);
    }
}

simulated function DebugDrawWeaponAccuracy()
{
    local float AngleRad;
    local SFXWeapon Weapon;
    local Vector StartTrace;
    local Vector EndTrace;
    local Vector Dir;
    Weapon = SFXWeapon(Instigator.Weapon);
    if (Weapon != None)
    {
        AngleRad = Sin(Weapon.GetPlayerAimError() * 0,01745329) * 0,75;
        StartTrace = Instigator.GetWeaponStartTraceLocation();
        EndTrace = StartTrace + Instigator.GetAdjustedAimFor(Weapon, StartTrace) * Weapon.GetTraceRange();
        Dir = Normal(EndTrace - StartTrace);
        StartTrace += Dir * 20;
        DrawDebugCone(StartTrace, Dir, Weapon.GetTraceRange(), AngleRad, AngleRad, 16, MakeColor(255, 128, 64, 255), false);
    }
}

simulated function UpdatePlayerAccuracy(float DeltaTime)
{
    local float InaccuracyPct;
    local float InterpSpeed;
    local float RotRateRatio;
    local float SpeedRatio;
    local BioPawn Pawn;
    local Rotator DeltaRot;
    local Rotator ViewRot;
    local Vector ViewLoc;
    local SFXWeapon Weapon;
    local PlayerInput Input;
    local float fAccuracyBonus;
    local float AccFireTimeMultiplier;
    if (DeltaTime == 0)
    {
        return;
    }
    InterpSpeed = Base_Interp_Speed;
    Pawn = BioPawn(Instigator);
    if (Pawn == None || PlayerController(Pawn.Controller) == None)
    {
        return;
    }
    Weapon = SFXWeapon(Pawn.Weapon);
    if (Weapon != None && Weapon.IsZoomed())
    {
        InaccuracyPct = Base_Acc_Zoom;
    }
    else
    {
        InaccuracyPct = Base_Acc_Standing;
    }
    fAccuracyBonus = 1;
    if (Pawn.bIsCrouched)
    {
        fAccuracyBonus = AccMod_Crouched;
    }
    if (Weapon != None)
    {
        if (bWeaponFired)
        {
            bWeaponFired = false;
            AccFireTimeMultiplier = WorldInfo.TimeSeconds - TimeSinceLastFire;
            if (AccFireTimeMultiplier < 0,1)
            {
                AccFireTimeMultiplier *= Weapon.GetRateOfFire() / 60;
            }
            else
            {
                AccFireTimeMultiplier = 1;
            }
            TimeSinceLastFire = WorldInfo.TimeSeconds;
            if (Weapon.IsZoomed())
            {
                AccMod_WeaponFired += AccFireTimeMultiplier * Weapon.ZoomAccFirePenalty * 60 * fAccuracyBonus / Weapon.GetRateOfFire();
            }
            else
            {
                AccMod_WeaponFired += AccFireTimeMultiplier * Weapon.AccFirePenalty * 60 * fAccuracyBonus / Weapon.GetRateOfFire();
            }
        }
        if (AccMod_WeaponFired > 0)
        {
            if (Weapon.IsZoomed())
            {
                AccMod_WeaponFired = AccMod_WeaponFired - DeltaTime * Weapon.ZoomAccFireInterpSpeed;
            }
            else
            {
                AccMod_WeaponFired = AccMod_WeaponFired - DeltaTime * Weapon.AccFireInterpSpeed;
            }
        }
        if (AccMod_WeaponFired < 0)
        {
            AccMod_WeaponFired = 0;
        }
    }
    InaccuracyPct += AccMod_WeaponFired;
    SpeedRatio = VSize(Pawn.Velocity) / Pawn.CombatGroundSpeed;
    if (SpeedRatio > 0)
    {
        Input = PlayerController(Pawn.Controller).PlayerInput;
        if (Input != None)
        {
            if (Abs(Input.RawJoyRight) > 0,01 || Abs(Input.RawJoyUp) > 0,01)
            {
                InaccuracyPct += SpeedRatio * AccMod_Move;
                InterpSpeed += SpeedRatio * InterpMod_Move;
            }
        }
    }
    Pawn.Controller.GetPlayerViewPoint(ViewLoc, ViewRot);
    DeltaRot = ViewRot - AccuracyLastViewRot;
    AccuracyLastViewRot = ViewRot;
    if (AccuracyLastViewRot == rot(0,0,0))
    {
        AccuracyLastViewRot = ViewRot;
    }
    if (DeltaRot != rot(0,0,0))
    {
        RotRateRatio = RSize(DeltaRot) / DeltaTime * 10950;
        if (!WorldInfo.IsConsoleBuild())
        {
            RotRateRatio *= AccMod_MouseMultiplier;
        }
        InaccuracyPct += FMin(AccMod_MaxCameraLoss, RotRateRatio * AccMod_ViewTurn);
        InterpSpeed += RotRateRatio * RotRateRatio * InterpMul_ViewTurn;
    }
    InterpSpeed = FClamp(InterpSpeed, InterpRange.X, InterpRange.Y);
    Accuracy = FClamp(FInterpTo(Accuracy, InaccuracyPct, DeltaTime, InterpSpeed), 0, 1);
}

simulated function float GetPlayerNormalizedAccuracy()
{
    if (Instigator.IsHumanControlled() == false)
    {
        return 0;
    }
    return Accuracy;
}

simulated function float GetPlayerCrosshairValue()
{
    local SFXWeapon Weapon;
    local float CrosshairScale;
    local BioPawn Pawn;
    Weapon = SFXWeapon(Instigator.Weapon);
    if (Weapon == None || Instigator.IsHumanControlled() == false)
    {
        return 0;
    }
    if (Weapon.IsZoomed())
    {
        CrosshairScale = Lerp(Weapon.ZoomCrosshairRange.X, Weapon.ZoomCrosshairRange.Y, Accuracy);
    }
    else
    {
        CrosshairScale = Lerp(Weapon.CrosshairRange.X, Weapon.CrosshairRange.Y, Accuracy);
    }
    if (Instigator.bIsCrouched)
    {
        CrosshairScale = CrosshairScale * AccMod_Crouched;
    }
    Pawn = BioPawn(Instigator);
    if (Pawn != None && Pawn.IsHumanControlled())
    {
        if (!Pawn.IsInCover())
        {
            CrosshairScale *= SFXGame(WorldInfo.Game).gameconfig.CoverAimBonus;
        }
    }
    return CrosshairScale;
}

simulated function bool CancelWeaponChange()
{
    return false;
}

simulated function ChangedWeapon()
{
    local SFXWeapon Weapon;
    local BioPawn Pawn;
    Pawn = BioPawn(Instigator);
    if (Pawn != None)
    {
        Pawn.WeaponFromLastGameState = Instigator.Weapon;
        if (Pawn.Weapon != None && PendingWeapon != None)
        {
            Pawn.WeaponOnDeck = Instigator.Weapon;
        }
        if (BioPlayerController(Pawn.Controller) != None)
        {
            BioPlayerController(Pawn.Controller).HintSystem.HintEvent('ChangeWeapon', PendingWeapon.Class.Name);
        }
    }
    ChangedWeapon();
    Weapon = SFXWeapon(Instigator.Weapon);
    if (Weapon != None)
    {
        CurrentWeaponSelection = Weapon.Class;
    }
}

simulated function SetWeaponBySelected()
{
    local SFXWeapon Weap;
    local SFXWeapon WeapToSet;
    foreach InventoryActors(Class'SFXWeapon', Weap)
    {
        if (ClassIsChildOf(Weap.Class, CurrentWeaponSelection))
        {
            if (WeapToSet == None || Weap.GetAIRating() > WeapToSet.GetAIRating())
            {
                WeapToSet = Weap;
            }
        }
    }
    if (WeapToSet == None)
    {
        WeapToSet = SFXWeapon(GetBestWeapon());
    }
    SetCurrentWeapon(WeapToSet);
}

simulated function SetWeaponFromSlot(EAttachSlot Slot)
{
    local SFXWeapon Weap;
    Weap = GetWeaponInSlot(Slot);
    if (Weap != None)
    {
        SetCurrentWeapon(Weap);
    }
}

simulated function SetWeaponIfAvailable(SFXWeapon WeapToSwitch)
{
    local SFXWeapon Weap;
    foreach InventoryActors(Class'SFXWeapon', Weap)
    {
        if (Weap == WeapToSwitch)
        {
            SetCurrentWeapon(Weap);
            break;
        }
    }
}

simulated function SFXWeapon GetWeaponInSlot(EAttachSlot Slot, optional bool bIgnoreWielded)
{
    local SFXWeapon Weap;
    foreach InventoryActors(Class'SFXWeapon', Weap)
    {
        if (Weap.CharacterSlot == Slot)
        {
            if (!bIgnoreWielded || Weap != Instigator.Weapon)
            {
                return Weap;
            }
        }
    }
    return None;
}

simulated function Inventory CreateInventory(class<Inventory> NewInventoryItemClass, optional bool bDoNotActivate)
{
    local Inventory Inv;
    if (NewInventoryItemClass != None)
    {
        Inv = Spawn(NewInventoryItemClass, Instigator);
        if (Inv != None)
        {
            if (SFXWeapon(Inv) != None)
            {
                SFXWeapon(Inv).CharacterSlot = 5;
            }
            if (!AddInventory(Inv, bDoNotActivate))
            {
                Inv.Destroy();
                Inv = None;
            }
            jump 138;
        }
    }
    return Inv;
}

simulated function ProcessDamage(out float Damage, out TraceHitInfo HitInfo, out Vector HitLocation, out Vector Momentum, class<SFXDamageType> DamageType, Controller InstigatedBy, Actor DamageCauser)
{
    local SFXShield_Base Shield;
    local SFXArmor_Base Armor;
    foreach InventoryActors(Class'SFXShield_Base', Shield)
    {
        Shield.ApplyDamage(Damage, HitInfo, HitLocation, Momentum, DamageType, InstigatedBy, DamageCauser);
    }
    if (Damage > 0)
    {
        foreach InventoryActors(Class'SFXArmor_Base', Armor)
        {
            Armor.ApplyDamage(Damage, HitInfo, HitLocation, Momentum, DamageType, InstigatedBy, DamageCauser);
        }
    }
}

event EmptyInventory()
{
    local Inventory oListItem;
    foreach InventoryActors(Class'Inventory', oListItem)
    {
        RemoveFromInventory(oListItem);
        oListItem.Destroy();
    }
}

function AdjustResource(EInventoryResourceTypes eInvResType, int Amount, optional bool bShowNotification, optional bool bIgnoreBudget = false)
{
    local int DisplayAmount;
    local int NewAmount;
    local BioPlayerController PC;
    local String sTelemetryParam;
    if (Amount > 0)
    {
        BioWorldInfo(Instigator.WorldInfo).m_oTreasure.TriggerResourceHint(eInvResType, Amount);
    }
    switch (eInvResType)
    {
        case 0:
            if (!bIgnoreBudget)
            {
                Amount = SFXGame(Instigator.WorldInfo.Game).gameconfig.RecordTreasure(0, Amount);
            }
            NewAmount = FMin(Credits + Amount, MaxCredits);
            DisplayAmount = NewAmount - Credits;
            Credits = NewAmount;
            PC = BioPlayerController(Instigator.Controller);
            if (PC != None && bShowNotification)
            {
                PC.HintSystem.AddNotification_CreditRecovery(DisplayAmount);
            }
            break;
        case 1:
            NewAmount = FMin(Medigel + Amount, GetMaxMedigel());
            DisplayAmount = NewAmount - Medigel;
            Medigel = NewAmount;
            PC = BioPlayerController(Instigator.Controller);
            if (PC != None && bShowNotification)
            {
                PC.HintSystem.AddNotification_MedigelRecovery(DisplayAmount);
            }
            break;
        case 4:
            if (!bIgnoreBudget)
            {
                Amount = SFXGame(Instigator.WorldInfo.Game).gameconfig.RecordTreasure(4, Amount);
            }
            NewAmount = FMin(Eezo + Amount, MaxEezo);
            DisplayAmount = NewAmount - Eezo;
            Eezo = NewAmount;
            PC = BioPlayerController(Instigator.Controller);
            if (PC != None && bShowNotification)
            {
                PC.HintSystem.AddNotification_ElementZeroRecovery(DisplayAmount);
            }
            break;
        case 5:
            if (!bIgnoreBudget)
            {
                Amount = SFXGame(Instigator.WorldInfo.Game).gameconfig.RecordTreasure(5, Amount);
            }
            NewAmount = FMin(Iridium + Amount, MaxIridium);
            DisplayAmount = NewAmount - Iridium;
            Iridium = NewAmount;
            PC = BioPlayerController(Instigator.Controller);
            if (PC != None && bShowNotification)
            {
                PC.HintSystem.AddNotification_IridiumRecovery(DisplayAmount);
            }
            break;
        case 6:
            if (!bIgnoreBudget)
            {
                Amount = SFXGame(Instigator.WorldInfo.Game).gameconfig.RecordTreasure(6, Amount);
            }
            NewAmount = FMin(Palladium + Amount, MaxPalladium);
            DisplayAmount = NewAmount - Palladium;
            Palladium = NewAmount;
            PC = BioPlayerController(Instigator.Controller);
            if (PC != None && bShowNotification)
            {
                PC.HintSystem.AddNotification_PalladiumRecovery(DisplayAmount);
            }
            break;
        case 7:
            if (!bIgnoreBudget)
            {
                Amount = SFXGame(Instigator.WorldInfo.Game).gameconfig.RecordTreasure(7, Amount);
            }
            NewAmount = FMin(Platinum + Amount, MaxPlatinum);
            DisplayAmount = NewAmount - Platinum;
            Platinum = NewAmount;
            PC = BioPlayerController(Instigator.Controller);
            if (PC != None && bShowNotification)
            {
                PC.HintSystem.AddNotification_PlatinumRecovery(DisplayAmount);
            }
            break;
        case 8:
            NewAmount = FMin(Probes + Amount, GetMaxProbes());
            DisplayAmount = NewAmount - Probes;
            Probes = NewAmount;
            break;
    }
    sTelemetryParam = eInvResType @ "," @ NewAmount;
    Class'BioTelemetry'.SendEventString('RESOURCE_ACQUIRED', sTelemetryParam);
}

function int GetResource(EInventoryResourceTypes eInvResType)
{
    switch (eInvResType)
    {
        case 0:
            return Credits;
        case 1:
            return Medigel;
        case 4:
            return Eezo;
        case 5:
            return Iridium;
        case 6:
            return Palladium;
        case 7:
            return Platinum;
        case 8:
            return Probes;
        case 2:
        case 3:
        default:
            return 0;
    }
}

function float GetMaxFuel()
{
    local float TecBonus;
    if (BioWorldInfo(Instigator.WorldInfo) != None)
    {
        TecBonus = MaxFuel * SFXGame(Instigator.WorldInfo.Game).BonusList.RchShipFuelCache;
    }
    return MaxFuel + TecBonus;
}

function SetMaxFuel(float F)
{
    MaxFuel = F;
}

function int GetMaxMedigel()
{
    if (BioWorldInfo(Instigator.WorldInfo) != None)
    {
        return MaxMedigel + SFXGame(Instigator.WorldInfo.Game).BonusList.RchMedigelCache + BioWorldInfo(Instigator.WorldInfo).GetGlobalVariables().GetIntByName('nmPlotAddedMaxMedigel');
    }
    return MaxMedigel;
}

function int GetMaxProbes()
{
    return MaxProbes * SFXGame(Instigator.WorldInfo.Game).BonusList.RchShipProbesCache;
}

function bool GrantScaledCredits(int nSize, optional bool i_bTickResource = true)
{
    local Name nmColumnName;
    local BioPlayerSquad oPlayerSquad;
    local int PlayerLevel;
    local int nGrantQuantity;
    local BioPlayerController oController;
    local BioWorldInfo oWorldInfo;
    local SFXEngine oEngine;
    oEngine = None;
    oWorldInfo = BioWorldInfo(Class'Engine'.GetCurrentWorldInfo());
    if (oWorldInfo != None)
    {
        oController = oWorldInfo.GetLocalPlayerController();
    }
    if (oController != None)
    {
        oEngine = SFXEngine(oController.Player.Outer);
    }
    oPlayerSquad = BioWorldInfo(Instigator.WorldInfo).m_playerSquad;
    if (oPlayerSquad == None)
    {
        return false;
    }
    if (oEngine != None)
    {
        if (SFXGame(Instigator.WorldInfo.Game).GetPlayerLevel(oEngine.CurrentUserID, PlayerLevel) == false)
        {
            return false;
        }
    }
    switch (nSize)
    {
        case 1:
            nmColumnName = 'MinorReward';
            break;
        case 2:
            nmColumnName = 'SmallReward';
            break;
        case 3:
            nmColumnName = 'MediumReward';
            break;
        case 4:
            nmColumnName = 'LargeReward';
            break;
        case 5:
            nmColumnName = 'MajorReward';
            break;
        default:
            return false;
    }
    if (oPlayerSquad.LookupCredits(PlayerLevel, nmColumnName, nGrantQuantity))
    {
        AdjustResource(0, nGrantQuantity, i_bTickResource);
    }
    return true;
}

simulated function StartFire(byte FireModeNum)
{
    if (SFXWeapon(Instigator.Weapon) != None)
    {
        SFXWeapon(Instigator.Weapon).StartBurstFire(FireModeNum);
    }
}

defaultproperties
{
    Probes=10
    MaxCredits=9999999
    MaxEezo=9999999
    MaxIridium=9999999
    MaxPalladium=9999999
    MaxPlatinum=9999999
    MaxMedigel=3
    MaxProbes=30
    MaxFuel=1000
    FuelEfficiency=1,5
    AccMod_Move=0,4
    AccMod_ViewTurn=0,3
    AccMod_Crouched=1
    AccMod_MouseMultiplier=0,75
    AccMod_MaxCameraLoss=0,85
    Base_Interp_Speed=8
    InterpMod_Move=4
    InterpMul_ViewTurn=1
    InterpRange=(X=0,1,Y=1000)
    DriftInterpSpeed=3
    DriftInterpResetSize=10
    DriftInterpMinSize=50
    DriftRampUpRate=0,4
    DriftRampUpDelay=0,5
    PendingFire=None
    0=System.Byte[]
}
