﻿class SFXModule_AimAssist extends SFXModule
    within BioPlayerController
    hidecategories(Object)
    config(Game);

var transient float CurrentAimAssistSoftMargin;

var transient Vector CurrentAimAssistBoneLocation;

var transient Actor CurrentAimAssistTarget;

var float FrictionSmoothRate;

var transient float FrictionSmoothValue;

var transient Quat AdhesionSmoothRot;

var float AdhesionSmoothRate;

var transient Actor LastZoomSnapTarget;

var transient Vector ZoomSnapTarget;

var transient Pawn LastFrictionTarget;

var transient Pawn LastAdhesionTarget;

var transient Quat DebugAdhesion_StrafeAssist;

var transient Quat DebugAdhesion_PerfectAim;

var transient Quat DebugAdhesion_Combined;

var transient Rotator DebugAdhesion_Final;

var transient Rotator DebugAdhesion_Smoothed;

var Vector LastCamLoc;

var float LastTargetHeight;

var float LastTargetRadius;

var float LastFrictionMultiplier;

var float LastDistFromAimY;

var float LastDistFromAimZ;

var float LastDistMultiplier;

var float LastDistToTarget;

var config float EnemyMovementAdhesionFactor;

var config Vector GlobalAdhesionFactor;

var config float GlobalAimCorrectionFactor;

var config float GlobalFrictionFactor;

final function CacheCurrentAimAssistTarget()
{
    local SFXWeapon SW;
    local Vector CamLoc;
    local Rotator CamRot;
    if (SFXPlayerCamera(Outer.PlayerCamera) != None && SFXPlayerCamera(Outer.PlayerCamera).CurrentCameraMode != None)
    {
        CamLoc = SFXPlayerCamera(Outer.PlayerCamera).CurrentCameraMode.m_pov.Location;
        CamRot = SFXPlayerCamera(Outer.PlayerCamera).CurrentCameraMode.m_pov.Rotation;
    }
    SW = SFXWeapon(Outer.Pawn.Weapon);
    CurrentAimAssistTarget = Outer.GetAimAssistTarget(SW.MaxAdhesionDistance, CamLoc, CamRot, CurrentAimAssistBoneLocation, CurrentAimAssistSoftMargin);
    if (SFXPlayerCamera(Outer.PlayerCamera).m_aTraceInfo.m_oCollVectorActor != CurrentAimAssistTarget && VSizeSq(SFXPlayerCamera(Outer.PlayerCamera).m_aTraceInfo.m_vCollVectorLocation - CamLoc) < VSizeSq(CurrentAimAssistBoneLocation - CamLoc))
    {
        CurrentAimAssistTarget = None;
    }
}

final function UpdateViewRotationForAdhesion(float DeltaTime, Rotator ViewRotation)
{
    local SFXWeapon Weapon;
    Weapon = Outer.Pawn != None ? SFXWeapon(Outer.Pawn.Weapon) : None;
    if (Weapon != None)
    {
        if (Outer.AimAssist.ViewAdhesion(DeltaTime, Weapon, ViewRotation))
        {
            Outer.Pawn.Controller.SetRotation(ViewRotation);
        }
        if (LastAdhesionTarget != None && LastAdhesionTarget.bDeleteMe || LastAdhesionTarget.IsDead())
        {
            LastAdhesionTarget = None;
        }
    }
}

final function Rotator GetMagneticAimCorrection(Vector StartFireLoc, Rotator BaseAimRot, float MaxAngle, float Threshold, float Range)
{
    local BioPawn HitPawn;
    local Rotator PerfectAimRot;
    local Rotator AdjustedAimRot;
    local float CosCorrection;
    local float CorrectionRads;
    local float InterpAmount;
    local float FinalAimCorrectionFactor;
    FinalAimCorrectionFactor = GlobalAimCorrectionFactor * MaxAngle;
    HitPawn = BioPawn(SFXPlayerCamera(Outer.PlayerCamera).m_aTraceInfo.m_oCollVectorActor);
    if (HitPawn != None)
    {
        return BaseAimRot;
    }
    HitPawn = BioPawn(CurrentAimAssistTarget);
    if (HitPawn == None)
    {
        return BaseAimRot;
    }
    PerfectAimRot = CurrentAimAssistBoneLocation - StartFireLoc;
    CosCorrection = BaseAimRot Dot PerfectAimRot;
    if (CosCorrection > Cos(0,01745329 * Threshold))
    {
        CorrectionRads = Acos(CosCorrection);
        InterpAmount = 0,01745329 * FinalAimCorrectionFactor / CorrectionRads * 1 - CorrectionRads / 0,01745329 * Threshold;
        AdjustedAimRot = RLerp(BaseAimRot, PerfectAimRot, FMin(1, InterpAmount), true);
        return AdjustedAimRot;
    }
    return BaseAimRot;
}

final function Rotator GetAdjustedAimFor(Weapon W, Vector StartFireLoc)
{
    local SFXWeapon SW;
    local Rotator BaseAimRot;
    BaseAimRot = Outer.Pawn != None ? Outer.Pawn.GetBaseAimRotation() : Outer.Rotation;
    SW = SFXWeapon(W);
    if (SW == None)
    {
        return BaseAimRot;
    }
    return GetMagneticAimCorrection(StartFireLoc, BaseAimRot, SW.MaxMagneticCorrectionAngle, SW.MagneticCorrectionThresholdAngle, W.GetTraceRange());
}

function ViewFriction(float DeltaTime, SFXWeapon Weapon)
{
    local BioPawn FrictionTarget;
    local Vector CamLoc;
    local Vector X;
    local Vector Y;
    local Vector Z;
    local Rotator CamRot;
    local float FrictionMultiplier;
    if (Outer.Pawn == None || Weapon.bFrictionEnabled == false)
    {
        return;
    }
    Outer.GetPlayerViewPoint(CamLoc, CamRot);
    GetAxes(CamRot, X, Y, Z);
    FrictionTarget = BioPawn(CurrentAimAssistTarget);
    if (FrictionTarget != None)
    {
        FrictionMultiplier = GetRangeValueByPct(Weapon.FrictionMultiplierRange, CurrentAimAssistSoftMargin) * GlobalFrictionFactor;
    }
    else
    {
        FrictionMultiplier = 0;
    }
    if (FrictionMultiplier > FrictionSmoothValue)
    {
        FrictionSmoothValue = FrictionMultiplier;
    }
    else
    {
        FrictionSmoothValue = FrictionSmoothValue * 1 - FrictionSmoothRate + FrictionMultiplier * FrictionSmoothRate;
        FrictionMultiplier = FrictionSmoothValue;
    }
    if (FrictionMultiplier > 0)
    {
        Outer.PlayerInput.aTurn *= 1 - FrictionMultiplier;
        Outer.PlayerInput.aLookUp *= 1 - FrictionMultiplier;
        Outer.PlayerInput.RawJoyLookRight = Outer.PlayerInput.aTurn;
        Outer.PlayerInput.RawJoyLookUp = Outer.PlayerInput.aLookUp;
        LastFrictionTarget = FrictionTarget;
    }
    LastFrictionMultiplier = FrictionMultiplier;
    LastCamLoc = CamLoc;
}

function DebugDraw_Friction(BioHUD H)
{
    local Vector XAxis;
    local Vector YAxis;
    local Vector ZAxis;
    local Vector TargLoc;
    local int X;
    local int Y;
    local AimAssistBox AimBox;
    X = 15;
    Y = 150;
    H.Canvas.SetPos(X, Y);
    H.Canvas.DrawText("Target:" @ LastFrictionTarget @ "[" $ LastDistToTarget $ "] [" $ LastDistMultiplier $ "]");
    Y += 15;
    H.Canvas.SetPos(X, Y);
    H.Canvas.DrawText("Aim distance (Y/Z): [" $ LastDistFromAimY $ "] [" $ LastDistFromAimZ $ "]");
    Y += 15;
    H.Canvas.SetPos(X, Y);
    H.Canvas.DrawText("Target collision (R/H): [" $ LastTargetRadius $ "] [" $ LastTargetHeight $ "]");
    Y += 15;
    H.Canvas.SetPos(X, Y);
    H.Canvas.DrawText("Friction multiplier: [" $ LastFrictionMultiplier $ "]");
    Y += 15;
    if (CurrentAimAssistTarget != None)
    {
        GetAxes(Outer.PlayerCamera.CameraCache.POV.Rotation, XAxis, YAxis, ZAxis);
        foreach BioPawn(CurrentAimAssistTarget).AimAssistRegions(AimBox)
        {
            TargLoc = BioPawn(CurrentAimAssistTarget).Mesh.GetBoneLocation(BioPawn(CurrentAimAssistTarget).AimNodes[AimBox.NodeType]);
            Outer.DrawDebugLine(Outer.Pawn.Location, TargLoc, 0, 255, 0);
            Outer.DrawDebugLine(TargLoc + YAxis * AimBox.Width * 0,5 + ZAxis * AimBox.Height * 0,5, TargLoc + YAxis * AimBox.Width * 0,5 - ZAxis * AimBox.Height * 0,5, 0, 0, 255);
            Outer.DrawDebugLine(TargLoc - YAxis * AimBox.Width * 0,5 + ZAxis * AimBox.Height * 0,5, TargLoc - YAxis * AimBox.Width * 0,5 - ZAxis * AimBox.Height * 0,5, 0, 0, 255);
            Outer.DrawDebugLine(TargLoc + YAxis * AimBox.Width * 0,5 + ZAxis * AimBox.Height * 0,5, TargLoc - YAxis * AimBox.Width * 0,5 + ZAxis * AimBox.Height * 0,5, 0, 0, 255);
            Outer.DrawDebugLine(TargLoc + YAxis * AimBox.Width * 0,5 - ZAxis * AimBox.Height * 0,5, TargLoc - YAxis * AimBox.Width * 0,5 - ZAxis * AimBox.Height * 0,5, 0, 0, 255);
            Outer.DrawDebugLine(TargLoc + YAxis * AimBox.Width * 0,5 + AimBox.SoftMargin + ZAxis * AimBox.Height * 0,5 + AimBox.SoftMargin, TargLoc + YAxis * AimBox.Width * 0,5 + AimBox.SoftMargin - ZAxis * AimBox.Height * 0,5 + AimBox.SoftMargin, 0, 0, 127);
            Outer.DrawDebugLine(TargLoc - YAxis * AimBox.Width * 0,5 + AimBox.SoftMargin + ZAxis * AimBox.Height * 0,5 + AimBox.SoftMargin, TargLoc - YAxis * AimBox.Width * 0,5 + AimBox.SoftMargin - ZAxis * AimBox.Height * 0,5 + AimBox.SoftMargin, 0, 0, 127);
            Outer.DrawDebugLine(TargLoc + YAxis * AimBox.Width * 0,5 + AimBox.SoftMargin + ZAxis * AimBox.Height * 0,5 + AimBox.SoftMargin, TargLoc - YAxis * AimBox.Width * 0,5 + AimBox.SoftMargin + ZAxis * AimBox.Height * 0,5 + AimBox.SoftMargin, 0, 0, 127);
            Outer.DrawDebugLine(TargLoc + YAxis * AimBox.Width * 0,5 + AimBox.SoftMargin - ZAxis * AimBox.Height * 0,5 + AimBox.SoftMargin, TargLoc - YAxis * AimBox.Width * 0,5 + AimBox.SoftMargin - ZAxis * AimBox.Height * 0,5 + AimBox.SoftMargin, 0, 0, 127);
        }
    }
}

function bool ViewAdhesion(float DeltaTime, SFXWeapon Weapon, out Rotator out_ViewRotation)
{
    local Pawn AdhesionTarget;
    local LocalEnemy ChkEnemy;
    local bool bTargetVisible;
    local float DistToTarget;
    local float AdhesionRot;
    local float LookMag;
    local float MoveMag;
    local float Decay;
    local Vector CamLoc;
    local Vector TargetLoc;
    local Vector CamToTarget;
    local Vector YLoc;
    local Vector FutureCamLoc;
    local Vector FutureTargetLoc;
    local Vector Correction;
    local Rotator CamRot;
    local Rotator DeltaRot;
    local Rotator NewRotation;
    local Quat PerfectAimRotation;
    local Quat StrafeAssistRotation;
    local Quat Combined;
    local float CorrectionFactor;
    MoveMag = Sqrt(Outer.RemappedJoyRight * Outer.RemappedJoyRight + Outer.RemappedJoyUp * Outer.RemappedJoyUp);
    LookMag = Sqrt(Outer.PlayerInput.RawJoyLookRight * Outer.PlayerInput.RawJoyLookRight + Outer.PlayerInput.RawJoyLookUp * Outer.PlayerInput.RawJoyLookUp);
    if (Outer.Pawn == None || Weapon == None || Weapon.bAdhesionEnabled == false || LookMag == 0 && MoveMag == 0)
    {
        AdhesionSmoothRot.X = 0;
        AdhesionSmoothRot.Y = 0;
        AdhesionSmoothRot.Z = 0;
        AdhesionSmoothRot.W = 0;
        return false;
    }
    AdhesionTarget = BioPawn(CurrentAimAssistTarget);
    TargetLoc = CurrentAimAssistBoneLocation;
    foreach Outer.EnemyList(ChkEnemy)
    {
        if (ChkEnemy.Enemy == AdhesionTarget && ChkEnemy.bVisible)
        {
            bTargetVisible = true;
            break;
        }
    }
    CamLoc = SFXPlayerCamera(Outer.PlayerCamera).CurrentCameraMode.m_pov.Location;
    CamRot = Outer.Rotation;
    if (AdhesionTarget != None && bTargetVisible && VSize(AdhesionTarget.Location - Outer.Pawn.Location) >= Weapon.MinAdhesionDistance)
    {
        CamToTarget = TargetLoc - CamLoc;
        DistToTarget = VSize(CamToTarget);
        YLoc = CamLoc + CamRot * DistToTarget;
        FutureTargetLoc = YLoc + AdhesionTarget.Velocity * DeltaTime * EnemyMovementAdhesionFactor;
        FutureCamLoc = CamLoc + Outer.Pawn.Velocity * DeltaTime;
        Correction = TargetLoc + AdhesionTarget.Velocity * DeltaTime;
        Correction.Z = FutureTargetLoc.Z;
        PerfectAimRotation = QuatFindBetween(CamRot, Normal(Correction - FutureCamLoc));
        StrafeAssistRotation = QuatFindBetween(CamRot, Normal(FutureTargetLoc - FutureCamLoc));
        CorrectionFactor = FClamp(Weapon.AimCorrectionAmount * LookMag + MoveMag * 2 * 0,333, 0, 1);
        Combined = QuatSlerp(StrafeAssistRotation, PerfectAimRotation, CorrectionFactor, true);
    }
    DebugAdhesion_StrafeAssist = StrafeAssistRotation;
    DebugAdhesion_PerfectAim = PerfectAimRotation;
    DebugAdhesion_Combined = Combined;
    Decay = 1 - 1 - AdhesionSmoothRate ** DeltaTime;
    AdhesionSmoothRot = QuatSlerp(AdhesionSmoothRot, Combined, Decay, true);
    NewRotation = QuatToRotator(AdhesionSmoothRot);
    DebugAdhesion_Smoothed = NewRotation;
    NewRotation = NewRotation + CamRot;
    AdhesionRot = GetRangeValueByPct(Weapon.AdhesionRot, CurrentAimAssistSoftMargin);
    AdhesionRot = FClamp(AdhesionRot, 0, 1);
    out_ViewRotation.Yaw = RLerp(CamRot, NewRotation, AdhesionRot * GlobalAdhesionFactor.X, true).Yaw;
    out_ViewRotation.Pitch = RLerp(CamRot, NewRotation, AdhesionRot * GlobalAdhesionFactor.Y, true).Pitch;
    DeltaRot = Normalize(out_ViewRotation - CamRot);
    DebugAdhesion_Final = DeltaRot;
    if (AdhesionTarget != None)
    {
        return true;
    }
    else
    {
        return false;
    }
}

function String PrintQuat(Quat Q)
{
    return "" $ Q.X $ "," $ Q.Y $ "," $ Q.Z $ "," $ Q.W;
}

function DebugDraw_Adhesion(BioHUD H)
{
    local Vector XAxis;
    local Vector YAxis;
    local Vector ZAxis;
    local Vector Dir;
    local Vector TargLoc;
    local int X;
    local int Y;
    local Vector CamLoc;
    local Rotator CamRot;
    X = 15;
    Y = 150;
    H.Canvas.DrawColor = MakeColor(255, 255, 0, 255);
    H.Canvas.SetPos(X, Y);
    H.Canvas.DrawText("Perfect  :" $ PrintQuat(DebugAdhesion_PerfectAim));
    Y += 15;
    H.Canvas.SetPos(X, Y);
    H.Canvas.DrawText("Strafe   :" $ PrintQuat(DebugAdhesion_StrafeAssist));
    Y += 15;
    H.Canvas.SetPos(X, Y);
    H.Canvas.DrawText("Combined :" $ PrintQuat(DebugAdhesion_Combined));
    Y += 15;
    H.Canvas.SetPos(X, Y);
    H.Canvas.DrawText("Smoothed :" $ DebugAdhesion_Smoothed);
    Y += 15;
    H.Canvas.SetPos(X, Y);
    H.Canvas.DrawText("Final    :" $ DebugAdhesion_Final);
    Y += 15;
    if (LastAdhesionTarget != None)
    {
        TargLoc = LastAdhesionTarget.Location;
        Outer.DrawDebugLine(Outer.Pawn.Location, TargLoc, 0, 255, 0);
        Outer.GetPlayerViewPoint(CamLoc, CamRot);
        Dir = TargLoc - CamLoc;
        Dir.Z = 0;
        GetAxes(Dir, XAxis, YAxis, ZAxis);
    }
}

function ZoomSnap()
{
    local SFXWeapon Weapon;
    local float Dist;
    local float MaxHeightOffset;
    local float HeightOffset;
    local Vector Proj;
    local Vector Offset;
    local SFXPlayerCamera Cam;
    Weapon = SFXWeapon(Outer.Pawn.Weapon);
    if (Outer.Pawn == None || Weapon == None || Weapon.bZoomSnapEnabled == false || SFXPlayerCamera(Outer.PlayerCamera).m_aTraceInfo.m_oCollVectorActor == CurrentAimAssistTarget)
    {
        return;
    }
    LastZoomSnapTarget = Outer.GetZoomSnapTarget(Weapon.MinZoomSnapDistance, Weapon.MaxZoomSnapDistance, ZoomSnapTarget);
    if (LastZoomSnapTarget == None)
    {
        ZoomSnapTarget = vect(0,0,0);
        return;
    }
    Cam = SFXPlayerCamera(Outer.PlayerCamera);
    Dist = VSize(Cam.CameraCache.POV.Location - LastZoomSnapTarget.Location + ZoomSnapTarget);
    Proj = Cam.CameraCache.POV.Location + Cam.CameraCache.POV.Rotation * Dist;
    Offset = Proj - LastZoomSnapTarget.Location + ZoomSnapTarget;
    if (BioPawn(LastZoomSnapTarget) != None)
    {
        MaxHeightOffset = BioPawn(LastZoomSnapTarget).CylinderComponent.CollisionHeight;
        HeightOffset = Proj.Z - LastZoomSnapTarget.Location.Z;
        if (HeightOffset > 0 && HeightOffset < MaxHeightOffset)
        {
            ZoomSnapTarget.Z = HeightOffset;
            Offset.Z = 0;
        }
    }
    ZoomSnapTarget += Offset * FRand() * 0,25 + 0,05;
}

function DebugDraw_ZoomSnap(BioHUD H)
{
    local SFXWeapon Weapon;
    local LocalEnemy ChkEnemy;
    local BioPawn ChkPawn;
    local ZoomSnapInfo Info;
    local float XL;
    local float YL;
    local float Angle;
    local Vector CamLoc;
    local Vector HitLoc;
    local Vector HitNorm;
    local Rotator CamRot;
    local TraceHitInfo Hit;
    local float CircleSize;
    local Vector ScreenCoords;
    local Vector BoneLoc;
    Weapon = SFXWeapon(Outer.Pawn.Weapon);
    if (Weapon == None)
    {
        return;
    }
    Outer.GetPlayerViewPoint(CamLoc, CamRot);
    foreach Outer.EnemyList(ChkEnemy)
    {
        ChkPawn = BioPawn(ChkEnemy.Enemy);
        if (ChkPawn != None && ChkEnemy.bVisible && VSize(ChkPawn.Location - Outer.Pawn.Location) >= Weapon.MinZoomSnapDistance && VSize(ChkPawn.Location - Outer.Pawn.Location) <= Weapon.MaxZoomSnapDistance)
        {
            foreach Weapon.ZoomSnapList(Info)
            {
                if (ChkPawn.IsInCover() && Info.AimNode == 0)
                {
                    BoneLoc = ChkPawn.GetPawnViewLocation();
                }
                else
                {
                    BoneLoc = ChkPawn.Mesh.GetBoneLocation(ChkPawn.AimNodes[Info.AimNode]);
                }
                if (IsZero(BoneLoc) == false && SFXPlayerCamera(Outer.PlayerCamera).LineCheck(Hit, HitLoc, HitNorm, BoneLoc) == None)
                {
                    ScreenCoords = H.Canvas.Project(BoneLoc);
                    H.Canvas.TextSize(ChkPawn.AimNodes[Info.AimNode], XL, YL);
                    H.Canvas.SetPos(ScreenCoords.X - XL / 2, ScreenCoords.Y);
                    H.Canvas.SetDrawColor(255, 255, 255, 255);
                    Angle = Acos(Normal(BoneLoc - CamLoc) Dot CamRot) * 57,29578;
                    H.Canvas.DrawTextClipped(ChkPawn.AimNodes[Info.AimNode] @ Angle);
                    H.Canvas.SetDrawColor(255, 0, 0, 255);
                    CircleSize = Info.OuterSnapAngle / Outer.PlayerCamera.CameraCache.POV.FOV * H.Canvas.SizeX;
                    H.DrawCircle(ScreenCoords.X, ScreenCoords.Y, CircleSize - 2);
                    H.Canvas.SetDrawColor(0, 255, 0, 255);
                    CircleSize = Info.InnerSnapAngle / Outer.PlayerCamera.CameraCache.POV.FOV * H.Canvas.SizeX;
                    H.DrawCircle(ScreenCoords.X, ScreenCoords.Y, CircleSize - 2);
                }
            }
        }
    }
}

defaultproperties
{
    GlobalFrictionFactor=1
    GlobalAimCorrectionFactor=1
    GlobalAdhesionFactor=(X=0,75,Y=0,5,Z=0)
    EnemyMovementAdhesionFactor=0,75
    AdhesionSmoothRate=1
    FrictionSmoothRate=0,1
    bTickWhilePaused=true
}
