﻿class SFXModule_Damage extends SFXModule_DamageBase
    hidecategories(Object)
    config(Game)
    editinlinenew;

struct DamagePart
{
    var float Damage;

    var bool bIsDetached;

}

var() ScaledFloat DamageMultiplier;

var const float NOTICEABLE_RTPC_CHANGE_AMOUNT;

var float m_fLastHealthPct;

var transient WwiseRTPCForActorHandler m_RTPCHandler;

var() float DamageScale;

var bool bIsDead;

var bool bPartBasedDamageEnabled;

var config bool bGoreActorEnabled;

var() bool bShowImmunityOnGUI;

var() ScaledFloat MaxHealth;

var() float CurrentHealth;

var() DamagePart DamageParts[8];

function ScaleHealth()
{
    MaxHealth.StaticBonus = 1;
    if (SFXPawn_Player(Owner) != None)
    {
        MaxHealth.StaticBonus = 1 * SFXGame(Pawn(Owner).WorldInfo.Game).BonusList.RchHealthUpgradeCache;
    }
    Class'SFXGame'.ReCalculate(MaxHealth);
    MaxHealth.Value = FMax(MaxHealth.Value, 1);
    Class'SFXGame'.ReCalculate(DamageMultiplier);
}

event PostBeginPlay()
{
    PostBeginPlay();
    ScaleHealth();
}

function SFXTakeDamage(float Damage, out TraceHitInfo HitInfo, out Vector HitLocation, out Vector Momentum, class<DamageType> DamageType, Controller InstigatedBy, optional Actor DamageCauser)
{
    local SFXGame Game;
    local RB_BodySetup HitBody;
    local float HealthRatio;
    local class<SFXDamageType> DamageClass;
    local float Shields;
    local float Health;
    local float OriginalDamage;
    local float CurrentShields;
    local float ActualShieldDamage;
    local float ActualHealthDamage;
    local SFXShield_Base ShieldObj;
    local bool bWasDead;
    local SFXPRI PRI;
    local SFXPRI InstigatorPRI;
    local bool bGenHeadShotVoc;
    if (Pawn(Owner) != None)
    {
        if (Pawn(Owner).InvManager != None)
        {
            foreach Pawn(Owner).InvManager.InventoryActors(Class'SFXShield_Base', ShieldObj)
            {
                CurrentShields += ShieldObj.CurrentShields;
            }
        }
        Shields = CurrentShields;
        Shields = Shields > 0 ? Shields : 0;
        Health = CurrentHealth;
        Health = Health > 0 ? Health : 0;
    }
    OriginalDamage = Damage;
    DamageClass = Class<SFXDamageType>(DamageType);
    if (DamageClass == None)
    {
        DamageClass = Class'SFXDamageType_Default';
    }
    if (InstigatedBy != None)
    {
        InstigatorPRI = SFXPRI(InstigatedBy.PlayerReplicationInfo);
        InstigatorPRI.LastDamageCalculation.BaseDamage = Damage;
        InstigatorPRI.LastDamageCalculation.AchievementBonusDamage = 1;
        InstigatorPRI.LastDamageCalculation.CoverLeanMultiplier = 1;
        InstigatorPRI.LastDamageCalculation.CoverMultiplier = 1;
        InstigatorPRI.LastDamageCalculation.DamageScale = 1;
        InstigatorPRI.LastDamageCalculation.DifficultyMultiplier = 1;
        InstigatorPRI.LastDamageCalculation.GEMultiplier = 1;
        InstigatorPRI.LastDamageCalculation.LastBulletMultiplier = 1;
        InstigatorPRI.LastDamageCalculation.PartBasedDamageMultiplier = 1;
        InstigatorPRI.LastDamageCalculation.PassiveGEMultiplier = 1;
        InstigatorPRI.LastDamageCalculation.RagdollMultiplier = 1;
        InstigatorPRI.LastDamageCalculation.RangeMultiplier = 1;
        InstigatorPRI.LastDamageCalculation.StealthMultiplier = 1;
        InstigatorPRI.LastDamageCalculation.StormMultiplier = 1;
    }
    if (Pawn(Owner) != None)
    {
        PRI = SFXPRI(Pawn(Owner).Controller.PlayerReplicationInfo);
        if (PRI != None)
        {
            Damage /= PRI.CurrentHealthMultiplier;
            if (InstigatorPRI != None)
            {
                InstigatorPRI.LastDamageCalculation.DifficultyMultiplier = 1 / PRI.CurrentHealthMultiplier;
            }
        }
    }
    Damage *= DamageScale;
    if (InstigatorPRI != None)
    {
        InstigatorPRI.LastDamageCalculation.DamageScale = DamageScale;
    }
    OriginalDamage *= DamageScale;
    if (Pawn(Owner) != None && Pawn(Owner).IsDead())
    {
        bWasDead = true;
    }
    Game = SFXGame(Owner.WorldInfo.Game);
    if (Game != None)
    {
        Momentum *= Game.gameconfig.PhysicsMomentumMultiplier;
        if (VSize(Momentum) > Game.gameconfig.PlayerMomentumCap && Pawn(Owner) != None && BioPlayerController(Pawn(Owner).Controller) != None)
        {
            Momentum = Normal(Momentum) * Game.gameconfig.PlayerMomentumCap;
        }
        Game.ModifyDamage(Damage, HitInfo, HitLocation, Momentum, DamageClass, Owner, InstigatedBy, DamageCauser);
    }
    if (Damage > 0 && DamageClass.Default.bHealthDamage)
    {
        ApplyDamageToHealth(Damage, InstigatedBy, DamageClass, HitLocation);
        if (bPartBasedDamageEnabled)
        {
            HitBody = GetBodyFromHit(HitInfo);
            if (HitBody != None)
            {
                if (HitBody.fDamageScale >= 1,5)
                {
                    bGenHeadShotVoc = true;
                }
            }
            if (HitBody != None && HitBody.bDetachable)
            {
                DamageParts[HitBody.ePartGroup].Damage += Damage;
                HealthRatio = CurrentHealth / MaxHealth.Value;
                if (HealthRatio <= 1 - HitBody.fDetachHPThresholdPercent && DamageParts[HitBody.ePartGroup].bIsDetached == false)
                {
                    HealthRatio = DamageParts[HitBody.ePartGroup].Damage / MaxHealth.Value;
                    if (HealthRatio >= HitBody.fDetachHPPercent)
                    {
                        DamageParts[HitBody.ePartGroup].bIsDetached = true;
                        DetachLimb(HitBody);
                    }
                }
            }
        }
        if (DamageClass.Default.bSpawnWeaponImpacts)
        {
            PlayHitEffects(HitInfo, HitLocation, DamageCauser);
        }
    }
    if (Damage > 0 && DamageClass.Default.bHealthDamage || VSize(Momentum) > 0 && DamageClass.Default.bCausesPhysicsImpact)
    {
        ProcessHit(Damage, HitInfo, HitLocation, Momentum, DamageClass, InstigatedBy, DamageCauser, bGenHeadShotVoc);
    }
    if (!bWasDead && Pawn(Owner) != None)
    {
        if (Pawn(Owner).InvManager != None)
        {
            CurrentShields = 0;
            foreach Pawn(Owner).InvManager.InventoryActors(Class'SFXShield_Base', ShieldObj)
            {
                CurrentShields += ShieldObj.CurrentShields;
            }
        }
        ActualShieldDamage = Shields - CurrentShields;
        ActualHealthDamage = Health - CurrentHealth;
        SFXGame(Owner.WorldInfo.Game).ScoreDamage(InstigatedBy, Pawn(Owner).Controller, OriginalDamage, ActualShieldDamage + ActualHealthDamage, DamageClass);
        if (SFXGame(Owner.WorldInfo.Game).gameconfig.bLastDamageUpdate && InstigatedBy.Pawn.Tag == 'Player' && DamageScale > 0)
        {
            SFXGame(Owner.WorldInfo.Game).UpdateDamage(ActualShieldDamage / 1,5 + ActualHealthDamage / InstigatorPRI.LastDamageCalculation.DifficultyMultiplier * InstigatorPRI.LastDamageCalculation.RangeMultiplier);
        }
    }
}

simulated function ApplyDamageToHealth(float Damage, Controller InstigatedBy, class<SFXDamageType> DamageType, out Vector HitLocation)
{
    Damage *= DamageMultiplier.Value;
    CurrentHealth -= Damage;
    if (CurrentHealth < 1)
    {
        if (Pawn(Owner) != None && Pawn(Owner).InPlayerParty() && SFXGame(Owner.WorldInfo.Game).bMin1Health || Owner.WorldInfo.Game.PreventDeath(Pawn(Owner), InstigatedBy, DamageType, HitLocation))
        {
            CurrentHealth = 1;
        }
    }
}

function CreateGoreActor(Pawn Pawn, Name BoneName)
{
    local SFXGoreActor PhysActor;
    local Vector Offset;
    Offset = Pawn.Mesh.Translation;
    PhysActor = Pawn.Spawn(Class'SFXGoreActor', Pawn, Pawn.Location + Offset, Pawn.Rotation);
    PhysActor.CreateGoreActor(Pawn, BoneName);
}

simulated function DetachLimb(RB_BodySetup HitBody)
{
    local SFXPawn Pawn;
    local ParticleSystemComponent PSC;
    local ParticleSystem Template;
    local MaterialInstanceConstant MIC;
    Pawn = SFXPawn(Owner);
    if (Pawn != None)
    {
        if (!Pawn.bSpawnPHATInstance)
        {
            Pawn.Mesh.SetHasPhysicsAssetInstance(true);
        }
        if (bGoreActorEnabled)
        {
            CreateGoreActor(Pawn, HitBody.BoneName);
        }
        else
        {
            Pawn.Mesh.HideBone(Pawn.Mesh.MatchRefBone(HitBody.BoneName), true);
        }
        if (HitBody.MaterialParamName != 'None')
        {
            foreach Pawn.BodyMaterialInstances(MIC)
            {
                MIC.SetScalarParameterValue(HitBody.MaterialParamName, 0);
            }
        }
        if (HitBody.ePartGroup == 2)
        {
            Template = Pawn.PS_HeadGib;
        }
        else
        {
            Template = Pawn.PS_LimbGib;
        }
        if (Template != None)
        {
            PSC = SFXGame(Owner.WorldInfo.Game).ObjectPool.GetGenericParticleSystemComponent(Template);
            if (PSC != None)
            {
                SFXGame(Owner.WorldInfo.Game).ObjectPool.AttachParticleSystemComponent(PSC, Pawn, Pawn.Mesh, Pawn.Mesh.GetParentBone(HitBody.BoneName), Pawn.Mesh.GetBoneLocation(HitBody.BoneName), rot(0,0,0), false);
                SFXGame(Owner.WorldInfo.Game).ObjectPool.ApplyLODLevel(PSC, Pawn.Location);
                SFXGame(Owner.WorldInfo.Game).ObjectPool.ApplyBloodColor(PSC, Pawn);
                PSC.SetActive(true);
            }
        }
        if (BioAiController(Pawn.Controller) != None)
        {
            BioAiController(Pawn.Controller).NotifyLimbDetached(HitBody.ePartGroup);
        }
        Pawn.NotifyLimbDetached(HitBody.ePartGroup);
    }
}

simulated function ProcessHit(float Damage, out TraceHitInfo HitInfo, Vector HitLocation, Vector Momentum, class<SFXDamageType> DamageType, Controller InstigatedBy, Actor DamageCauser, bool HeadShot)
{
    local Pawn PawnOwner;
    local Controller Killer;
    PawnOwner = Pawn(Owner);
    if (PawnOwner != None)
    {
        PawnOwner.PlayHit(Damage, InstigatedBy, HitLocation, DamageType, Momentum, HitInfo);
        PawnOwner.MakeNoise(1);
        PawnOwner.NotifyTakeHit(InstigatedBy, HitLocation, Damage, DamageType, Momentum);
        if (CurrentHealth > 0)
        {
            if (InstigatedBy != None && InstigatedBy != PawnOwner.Controller)
            {
                PawnOwner.LastHitBy = InstigatedBy;
            }
        }
        else
        {
            if (!bIsDead)
            {
                bIsDead = true;
                Killer = PawnOwner.SetKillInstigator(InstigatedBy, DamageType);
                PawnOwner.TearOffMomentum = Momentum;
                PawnOwner.Died(Killer, DamageType, HitLocation);
                if (BioPawn(PawnOwner) != None)
                {
                    BioPawn(PawnOwner).DeathHitBoneIdx = HitInfo.Item;
                }
                if (HeadShot && Killer.Pawn.IsHumanControlled())
                {
                    BioPlayerController(Killer).UpdateAchievementProgression(30);
                }
            }
        }
    }
    else
    {
        if (CurrentHealth < 0)
        {
            Owner.Destroy();
        }
    }
}

function SFXTakeRadiusDamage(float Damage, float DamageRadius, bool bFullDamage, Vector HurtOrigin, float Momentum, class<DamageType> DamageType, Controller InstigatedBy, Actor DamageCauser)
{
    local float ColRadius;
    local float ColHeight;
    local float Dist;
    local float Scale;
    local Vector Dir;
    local bool bConstantFalloff;
    Owner.GetBoundingCylinder(ColRadius, ColHeight);
    Dir = Owner.Location - HurtOrigin;
    Dist = VSize(Dir);
    Dir = Normal(Dir);
    if (Class<SFXDamageType>(DamageType) != None && Class<SFXDamageType>(DamageType).Default.FalloffType == 1)
    {
        bConstantFalloff = true;
    }
    if (bFullDamage || bConstantFalloff)
    {
        Scale = 1;
    }
    else
    {
        Dist = FClamp(Dist - ColRadius, 0, DamageRadius);
        Scale = 1 - Dist / DamageRadius;
    }
    if (DamageScale > 0)
    {
        Owner.TakeDamage(Damage * Scale, InstigatedBy, Owner.Location - Dir * ColRadius, Dir * Momentum * Scale, DamageType, DamageCauser);
    }
}

final function RB_BodySetup GetBodyFromHit(out TraceHitInfo HitInfo)
{
    local Pawn OwnerPawn;
    local RB_BodySetup Body;
    local int BoneIdx;
    local int ParentIdx;
    OwnerPawn = Pawn(Owner);
    BoneIdx = HitInfo.Item;
    if (OwnerPawn != None && BoneIdx != -1)
    {
        Body = OwnerPawn.Mesh.PhysicsAsset.BodySetup[BoneIdx];
        while (BoneIdx != 0 && Body.ePartGroup == 1)
        {
            ParentIdx = OwnerPawn.Mesh.GetParentBoneIndex(BoneIdx);
            if (BoneIdx == ParentIdx)
            {
                return None;
            }
            BoneIdx = ParentIdx;
            Body = OwnerPawn.Mesh.PhysicsAsset.BodySetup[BoneIdx];
        }
        if (Body != None && Body.bPartBasedDamageEnabled)
        {
            return Body;
        }
    }
    return None;
}

final function PlayHitEffects(out TraceHitInfo HitInfo, out Vector HitLocation, Actor DamageCauser)
{
    local SFXWeapon Weapon;
    local ImpactInfo Impact;
    Weapon = SFXWeapon(DamageCauser);
    if (Weapon != None)
    {
        Impact.HitInfo = HitInfo;
        Impact.HitActor = Owner;
        Impact.HitLocation = HitLocation;
        Impact.HitNormal = Normal(Weapon.Instigator.Location - HitLocation);
        Impact.RayDir = Impact.HitNormal * -1;
        Weapon.SpawnImpactEffects(Impact);
        Weapon.SpawnImpactSounds(Impact);
    }
}

function array<EBioPartGroup> GetConstraints()
{
    local array<EBioPartGroup> output;
    local RB_BodySetup Body;
    if (Pawn(Owner).Mesh.PhysicsAsset != None)
    {
        foreach Pawn(Owner).Mesh.PhysicsAsset.BodySetup(Body)
        {
            if (Body.bPartBasedDamageEnabled && Body.bDetachable && Body.ePartGroup != 1)
            {
                if (output.Find(Body.ePartGroup) == -1)
                {
                    output.AddItem(Body.ePartGroup);
                }
            }
        }
    }
    return output;
}

function DestroyConstraint(EBioPartGroup Group)
{
    local RB_BodySetup Body;
    foreach Pawn(Owner).Mesh.PhysicsAsset.BodySetup(Body)
    {
        if (Body.ePartGroup == Group)
        {
            if (Body.bPartBasedDamageEnabled && Body.bDetachable)
            {
                DetachLimb(Body);
            }
        }
    }
}

function Tick(float TimeDelta)
{
    local float fHealthPct;
    local float fPctChange;
    Tick(TimeDelta);
    if (MaxHealth.Value <= 0)
    {
        return;
    }
    fHealthPct = CurrentHealth / MaxHealth.Value;
    if (m_RTPCHandler.m_actor == None)
    {
        m_RTPCHandler.m_actor = Owner;
        m_RTPCHandler.m_sRTPCName = "Pawn_Health";
        Class'WwiseAudioComponent'.SetRTPCWithHandler(m_RTPCHandler, 1);
    }
    fPctChange = Abs(fHealthPct - m_fLastHealthPct);
    if (fPctChange > NOTICEABLE_RTPC_CHANGE_AMOUNT)
    {
        Class'WwiseAudioComponent'.SetRTPCWithHandler(m_RTPCHandler, fHealthPct);
        m_fLastHealthPct = fHealthPct;
    }
}

defaultproperties
{
    MaxHealth=(X=3,864781E-41,Y=0,MaxLevel=10594,Level=0,Value=5,605194E-45,Bonuses=UnHood.Engine.UnPropertyArray,StaticBonus=0)
    bPartBasedDamageEnabled=true
    DamageScale=1
    m_fLastHealthPct=1
    NOTICEABLE_RTPC_CHANGE_AMOUNT=0,15
    DamageMultiplier=(X=3,864781E-41,Y=0,MaxLevel=10594,Level=0,Value=5,605194E-45,Bonuses=UnHood.Engine.UnPropertyArray,StaticBonus=1)
}
