﻿class SFXObjectPool extends Actor
    hidecategories(Navigation)
    config(Game)
    native;

const MAX_TRACERS = 15;
struct native SFXObjectPoolTracers
{
    var StaticMesh Mesh;

    var ParticleSystem Template;

    var Projectile Tracers[15];

    var int NextIdx;

}

var array<PathGoalEvaluator> PathGoalEvaluatorCache;

var array<SFXObjectPoolPathConstraintCaches> PathConstraintCache;

var int WwiseNextIdx;

var /* 00000008 */ WwiseAudioComponent WwiseACPool[20];

var array<SFXObjectPoolEmitters> EmitterPool;

var array<SFXObjectPoolPSCs> GenericPSCPool;

var array<SFXObjectPoolImpactPSCs> ImpactPSCPool;

var array<SFXObjectPoolDroppedAmmos> DroppedPool;

var array<SFXObjectPoolProjectiles> ProjectilePool;

var array<SFXObjectPoolTracers> TracerPool;

struct native SFXObjectPoolProjectiles
{
    var class<Projectile> ProjectileClass;

    var array<Projectile> Projectiles;

    var int NextIdx;

}

struct native SFXObjectPoolDroppedAmmos
{
    var class<DroppedPickup> DroppedAmmoClass;

    var array<DroppedPickup> DroppedAmmos;

    var int NextIdx;

}

const MAX_PSCS = 15;
struct native SFXObjectPoolImpactPSCs
{
    var ParticleSystem Template;

    var /* 00000008 */ ParticleSystemComponent PSysComponents[15];

    var int NextIdx;

}

struct native SFXObjectPoolPSCs
{
    var ParticleSystem Template;

    var /* 00000008 */ array<ParticleSystemComponent> PSysComponents;

    var int NextIdx;

}

struct native SFXObjectPoolEmitters
{
    var ParticleSystem Template;

    var array<Emitter> Emitters;

    var int NextIdx;

}

const MAX_WwiseAC = 20;
const NUM_CONSTRAINT_CLASSES = 11;
const MAX_INSTANCES_PER_CLASS = 3;
struct native SFXObjectPoolPathConstraintCaches
{
    var int ListIdx;

    var PathConstraint List[3];

}

const MAX_GOALEVALUATORS = 3;
native final function AttachParticleSystemComponent(ParticleSystemComponent PSC, Actor HitActor, PrimitiveComponent HitComponent, Name HitBone, Vector HitLocation, Vector HitNormal, bool bStaticLocation);

native final function ApplyLODLevel(ParticleSystemComponent PSC, Vector EffectLocation);

native final function ApplyBloodColor(ParticleSystemComponent PSC, Actor HitActor);

final private event Projectile CreatePooledTracer(StaticMesh Mesh, ParticleSystem Template, Vector SpawnLocation)
{
    local SFXTracer_Bullet Tracer;
    local StaticMeshComponent SMC;
    Tracer = Spawn(Class'SFXTracer_Bullet', SpawnLocation);
    if (Tracer != None)
    {
        Tracer.bPooled = true;
        Tracer.SetDrawScale3D(vect(2,1,1));
        SMC = StaticMeshComponent(Tracer.Mesh);
        if (SMC != None)
        {
            SMC.SetStaticMesh(Mesh);
        }
        Tracer.ProjEffectsTrailTemplate = Template;
        Tracer.Recycle();
        assert(Tracer.bStasis == true);
    }
    return Tracer;
}

final private event ReusePooledTracer(Projectile Proj)
{
    local SFXTracer_Bullet Tracer;
    Tracer = SFXTracer_Bullet(Proj);
    if (Tracer != None)
    {
        Tracer.Recycle();
        Tracer.Reuse();
        assert(Tracer.bStasis == false);
    }
}

native final function PrecacheTracer(StaticMesh Mesh, ParticleSystem Template);

native final function Projectile GetTracer(StaticMesh Mesh, ParticleSystem Template);

final private event Projectile CreatePooledProjectile(class<Projectile> ProjClass, Vector SpawnLocation)
{
    local SFXProjectile SFXProj;
    if (ClassIsChildOf(ProjClass, Class'SFXProjectile'))
    {
        SFXProj = SFXProjectile(Spawn(ProjClass, SpawnLocation));
        if (SFXProj != None)
        {
            SFXProj.bPooled = true;
            SFXProj.Recycle();
            assert(SFXProj.bStasis == true);
        }
        jump 122;
    }
    return SFXProj;
}

final private event ReusePooledProjectile(Projectile Proj)
{
    local SFXProjectile SFXProj;
    SFXProj = SFXProjectile(Proj);
    if (SFXProj != None)
    {
        SFXProj.Reuse();
        assert(SFXProj.bStasis == false);
    }
}

native final function Projectile GetProjectile(class<Projectile> ProjectileClass, Actor ProjOwner, Pawn ProjInstigator, Vector ProjLocation, Rotator ProjRotation);

final private event DroppedPickup CreatePooledDroppedAmmo(class<DroppedPickup> DroppedAmmoClass, Vector SpawnLocation)
{
    local SFXDroppedAmmo SFXDrop;
    if (ClassIsChildOf(DroppedAmmoClass, Class'DroppedPickup'))
    {
        SFXDrop = SFXDroppedAmmo(Spawn(DroppedAmmoClass, SpawnLocation));
        if (SFXDrop != None)
        {
            SFXDrop.bPooled = true;
            SFXDrop.Recycle();
            assert(SFXDrop.bStasis == true);
        }
        jump 122;
    }
    return SFXDrop;
}

final private event ReusePooledDroppedAmmo(DroppedPickup Dropped)
{
    local SFXDroppedAmmo SFXDrop;
    SFXDrop = SFXDroppedAmmo(Dropped);
    if (SFXDrop != None)
    {
        SFXDrop.Reuse();
        assert(SFXDrop.bStasis == false);
    }
}

native final function DroppedPickup GetDroppedAmmo(class<DroppedPickup> DroppedAmmoClass, Vector SpawnLocation);

native final function PrecacheImpactParticleSystemComponent(ParticleSystem Template);

native final function ParticleSystemComponent GetImpactParticleSystemComponent(ParticleSystem Template);

native final function PrecacheGenericParticleSystemComponent(ParticleSystem Template);

native final function ParticleSystemComponent GetGenericParticleSystemComponent(ParticleSystem Template);

final private event Emitter CreatePooledEmitter(ParticleSystem Template)
{
    local SFXEmitter SFXEmit;
    SFXEmit = Spawn(Class'SFXEmitter');
    if (SFXEmit != None)
    {
        SFXEmit.bPooled = true;
        SFXEmit.Initialize(Template);
        SFXEmit.Recycle();
        assert(SFXEmit.bStasis == true);
    }
    return SFXEmit;
}

final private event ReusePooledEmitter(Emitter Emit)
{
    local SFXEmitter SFXEmit;
    SFXEmit = SFXEmitter(Emit);
    if (SFXEmit != None)
    {
        SFXEmit.Reuse();
        assert(SFXEmit.bStasis == false);
    }
}

native final function PrecacheImpactEmitter(ParticleSystem Template);

native final function Emitter GetImpactEmitter(ParticleSystem Template, Vector SpawnLocation, Rotator SpawnRotation);

native final function WwiseAudioComponent GetWwiseAudioComponent();

final private event CreatePathConstraintCache()
{
    PathConstraintCache.Length = 11;
    PathGoalEvaluatorCache.Length = 3;
}

final private event CleanupPathConstraints()
{
    PathConstraintCache.Length = 0;
    PathGoalEvaluatorCache.Length = 0;
}

final event PathConstraint GetPathConstraintFromCache(class<PathConstraint> ConstraintClass, Pawn Requestor)
{
    local int CacheIdx;
    local SFXObjectPoolPathConstraintCaches PDC;
    local PathConstraint ChosenConstraint;
    if (PathConstraintCache.Length == 0)
    {
        CreatePathConstraintCache();
    }
    CacheIdx = ConstraintClass.Default.CacheIdx;
    if (CacheIdx < 0 || CacheIdx > 11)
    {
        return new(ConstraintClass);
    }
    PDC = PathConstraintCache[CacheIdx];
    if (++PathConstraintCache[CacheIdx].ListIdx >= 3)
    {
        PathConstraintCache[CacheIdx].ListIdx = 0;
    }
    ChosenConstraint = PDC.List[PDC.ListIdx];
    if (ChosenConstraint == None || ChosenConstraint.IsPendingKill())
    {
        ChosenConstraint = new(ConstraintClass);
        PDC.List[PDC.ListIdx] = ChosenConstraint;
        jump 274;
    }
    ChosenConstraint.Recycle();
    return ChosenConstraint;
}

final event PathGoalEvaluator GetPathGoalEvaluatorFromCache(class<PathGoalEvaluator> GoalEvalClass, Pawn Requestor)
{
    local int CacheIdx;
    local PathGoalEvaluator ChosenEval;
    if (PathGoalEvaluatorCache.Length == 0)
    {
        CreatePathConstraintCache();
    }
    CacheIdx = GoalEvalClass.Default.CacheIdx;
    if (CacheIdx < 0 || CacheIdx > 3)
    {
        return new(GoalEvalClass);
    }
    ChosenEval = PathGoalEvaluatorCache[CacheIdx];
    if (ChosenEval == None || ChosenEval.IsPendingKill())
    {
        ChosenEval = new(GoalEvalClass);
        PathGoalEvaluatorCache[CacheIdx] = ChosenEval;
        jump 158;
    }
    ChosenEval.Recycle();
    return ChosenEval;
}

native final function DumpPools();

defaultproperties
{
    bMovable=false
    CollisionType=System.Byte[]
}
