﻿class SFXPRI extends SFXPRI_NativeBase
    hidecategories(Navigation,Movement,Collision)
    config(Game);

var transient array<ScoreEvent> ScoreLog;

var transient float WeaponDamageBonus;

var transient float WeaponPassiveDamageBonus;

var transient float RecoilBonus;

var transient float CurrentMaxTargetTicketsHench;

var transient float CurrentMaxTargetTickets;

var transient float CurrentMaxAttackTicketsHench;

var transient float CurrentMaxAttackTickets;

var transient float CurrentShieldModifier;

var transient float CurrentHealthMultiplier;

var transient bool bCalculatedChallenge;

var transient float ChallengeModifier;

var transient float LockedScore;

var transient float DifficultyScore;

var float LastCombatTimestamp;

var float TimeLastCombat;

var float TimeInCombat;

var float DamageTakenNet;

var float DamageTakenRaw;

var float NetDamageCurrentTarget;

var float RawDamageCurrentTarget;

var int ShotsHitCurrentTarget;

var int ShotsFiredCurrentTarget;

var Actor CurrentTarget;

var int HeadShots;

var float NetDamage;

var float RawDamage;

var int ShotsHit;

var int ShotsFired;

var transient array<DamageRecordStruct> DamageRecord;

var config float PointsFor_FlamethrowerTankKill;

var config array<KillScore> PointsFor_GameEffectKills;

var config array<KillScore> PointsFor_SpecialKills;

var transient DamageCalculationAlgorithm LastDamageCalculation;

var config float PointsFor_Kill_Boss;

var config float PointsFor_Kill_SubBoss;

var config float PointsFor_Kill_Elite;

var config float PointsFor_Kill_Minion;

var config float PointsFor_HeadShot;

var config float PointsFor_DamageReceived;

var config float PointsFor_DamageDealt;

var float Multiplier;

struct DamageCalculationAlgorithm
{
    var float BaseDamage;

    var float RangeMultiplier;

    var float PassiveGEMultiplier;

    var float GEMultiplier;

    var float StealthMultiplier;

    var float LastBulletMultiplier;

    var float DifficultyMultiplier;

    var float PartBasedDamageMultiplier;

    var float DamageScale;

    var float CoverLeanMultiplier;

    var float CoverMultiplier;

    var float StormMultiplier;

    var float RagdollMultiplier;

    var float AchievementBonusDamage;

}

struct KillScore
{
    var config Name DamageClassName;

    var config String Message;

    var config int Points;

    var config EAchievementID GrinderAchievementId;

}

struct DamageRecordStruct
{
    var Controller DamageCauser;

    var float DamageDone;

}

struct ScoreEvent
{
    var String Text;

    var float ExpiryTime;

}

function UpdateScoreDifficulty()
{
    if (Owner != None && Owner.WorldInfo != None && SFXGame(Owner.WorldInfo.Game) != None)
    {
        SFXGame(Owner.WorldInfo.Game).Difficulty.UpdateDifficulty(self);
    }
}

function Tick(float DeltaTime)
{
    local int idx;
    idx = ScoreLog.Length - 1;
    while (idx >= 0)
    {
        if (ScoreLog[idx].ExpiryTime < WorldInfo.TimeSeconds)
        {
            ScoreLog.Remove(idx, 1);
        }
        idx--;
    }
}

simulated function PostBeginPlay()
{
    PostBeginPlay();
    SetTimer(1, true, 'UpdateScoreDifficulty');
}

final simulated static function SFXPRI FindLocalPRI()
{
    local GameReplicationInfo GRI;
    local PlayerReplicationInfo PRI;
    GRI = Class'Engine'.GetCurrentWorldInfo().GRI;
    if (GRI != None)
    {
        foreach GRI.PRIArray(PRI)
        {
            if (SFXPRI(PRI) != None && PRI.Owner != None && PlayerController(PRI.Owner) != None && LocalPlayer(PlayerController(PRI.Owner).Player) != None)
            {
                return SFXPRI(PRI);
            }
        }
    }
    return None;
}

simulated function Clear()
{
    ShotsFired = 0;
    ShotsHit = 0;
    RawDamage = 0;
    NetDamage = 0;
    HeadShots = 0;
    Kills = 0;
    CurrentTarget = None;
    ShotsFiredCurrentTarget = 0;
    ShotsHitCurrentTarget = 0;
    RawDamageCurrentTarget = 0;
    NetDamageCurrentTarget = 0;
    DamageTakenRaw = 0;
    DamageTakenNet = 0;
    TimeInCombat = 0;
    TimeLastCombat = 0;
    LastCombatTimestamp = 0;
}

simulated function SwitchTarget(Actor Target)
{
    CurrentTarget = Target;
    ShotsFiredCurrentTarget = 0;
    ShotsHitCurrentTarget = 0;
    RawDamageCurrentTarget = 0;
    NetDamageCurrentTarget = 0;
}

function RecordDamageDone(Controller DamageCauser, float Amount)
{
    local int idx;
    local DamageRecordStruct DRS;
    idx = 0;
    while (idx < DamageRecord.Length)
    {
        if (DamageRecord[idx].DamageCauser == DamageCauser)
        {
            DamageRecord[idx].DamageDone += Amount;
            return;
        }
        idx++;
    }
    DRS.DamageCauser = DamageCauser;
    DRS.DamageDone = Amount;
    DamageRecord.AddItem(DRS);
}

function DrawScore()
{
    local HUD HUD;
    local int idx;
    local int OldX;
    local int OldY;
    local Font OldFont;
    HUD = PlayerController(Owner).myHUD;
    OldX = HUD.Canvas.CurX;
    OldY = HUD.Canvas.CurY;
    OldFont = HUD.Canvas.Font;
    HUD.Canvas.Font = Class'Engine'.GetMediumFont();
    HUD.Canvas.CurX = 840;
    HUD.Canvas.CurY = 440;
    idx = ScoreLog.Length - 1;
    while (idx >= 0)
    {
        HUD.Canvas.SetDrawColor(255, 255, 255, ScoreLog[idx].ExpiryTime - WorldInfo.TimeSeconds * 51);
        HUD.Canvas.DrawText(ScoreLog[idx].Text);
        HUD.Canvas.CurY += 5;
        idx--;
    }
    HUD.Canvas.SetDrawColor(255, 255, 255);
    HUD.Canvas.CurX = 940;
    HUD.Canvas.CurY = 40;
    HUD.Canvas.DrawText("Score:" @ Score);
    HUD.Canvas.Font = OldFont;
    HUD.Canvas.CurX = OldX;
    HUD.Canvas.CurY = OldY;
}

function LockScore(float fScore)
{
    DifficultyScore = fScore;
    LockedScore = fScore;
}

function AddScore(int Points, optional String Message, optional bool bAddDifficultyScore)
{
    local ScoreEvent SE;
    if (Message != "")
    {
        SE.Text = "+" $ Points @ Message;
        SE.ExpiryTime = WorldInfo.TimeSeconds + 5;
        ScoreLog.AddItem(SE);
    }
    Score += Points * SFXGame(Owner.WorldInfo.Game).Difficulty.Default.DifficultyLevel + 1;
    if (bAddDifficultyScore)
    {
        DifficultyScore += Points;
    }
}

function AddGrinderAchievement(EAchievementID AchievementId, BioPawn oTarget)
{
    if (AchievementId > 0)
    {
        BioPlayerController(Owner).UpdateAchievementProgression(AchievementId, oTarget);
    }
}

function ScoreFinesseKill(BioPawn Other, class<SFXDamageType> DamageType)
{
    local KillScore KS;
    local SFXGameEffect GE;
    BioPlayerController(Owner).UpdateHeavyWeaponKill(DamageType);
    foreach PointsFor_SpecialKills(KS)
    {
        if (DamageType.Name == KS.DamageClassName)
        {
            AddScore(KS.Points, KS.Message);
            AddGrinderAchievement(KS.GrinderAchievementId, Other);
            return;
        }
    }
    foreach PointsFor_GameEffectKills(KS)
    {
        GE = Other.GetGameEffect(KS.DamageClassName);
        if (GE != None && GE.Instigator == Owner)
        {
            AddScore(KS.Points, KS.Message);
            AddGrinderAchievement(KS.GrinderAchievementId, Other);
            return;
        }
    }
}

function ResetDamageDealt()
{
    DamageRecord.Length = 0;
}

function DivyPoints(Controller Other)
{
    local SFXPRI KilledPRI;
    local SFXPRI KillerPRI;
    local DamageRecordStruct DRS;
    local float PointsForDamage;
    local float TotalDamageRecorded;
    local float DamageNormalizer;
    TotalDamageRecorded = 0;
    KilledPRI = SFXPRI(Other.PlayerReplicationInfo);
    if (KilledPRI != None)
    {
        foreach KilledPRI.DamageRecord(DRS)
        {
            if (SFXPRI(DRS.DamageCauser.PlayerReplicationInfo) != None)
            {
                TotalDamageRecorded += DRS.DamageDone;
            }
        }
        if (TotalDamageRecorded > 0)
        {
            DamageNormalizer = BioPawn(Other.Pawn).GetMaxHealth() / TotalDamageRecorded;
        }
        foreach KilledPRI.DamageRecord(DRS)
        {
            KillerPRI = SFXPRI(DRS.DamageCauser.PlayerReplicationInfo);
            PointsForDamage = DRS.DamageDone * Class'SFXPRI'.Default.PointsFor_DamageDealt * DamageNormalizer;
            switch (BioPawn(Other.Pawn).PawnType.ChallengeType)
            {
                case 1:
                    KillerPRI.AddScore(PointsForDamage + Class'SFXPRI'.Default.PointsFor_Kill_Minion, "Kill", true);
                    break;
                case 2:
                    KillerPRI.AddScore(PointsForDamage + Class'SFXPRI'.Default.PointsFor_Kill_Elite, "Killed Elite", true);
                    break;
                case 3:
                    KillerPRI.AddScore(PointsForDamage + Class'SFXPRI'.Default.PointsFor_Kill_SubBoss, "Killed Sub-Boss", true);
                    break;
                case 4:
                    KillerPRI.AddScore(PointsForDamage + Class'SFXPRI'.Default.PointsFor_Kill_Boss, "Killed Boss", true);
                    break;
            }
        }
    }
}

defaultproperties
{
    PointsFor_DamageDealt=1,5
    PointsFor_DamageReceived=1,3
    PointsFor_Kill_Minion=40
    PointsFor_Kill_Elite=40
    PointsFor_Kill_SubBoss=40
    PointsFor_Kill_Boss=40
    PointsFor_SpecialKills=System.Byte[]
    PointsFor_GameEffectKills=System.Byte[]
    PointsFor_FlamethrowerTankKill=40
    LockedScore=-1
    CurrentHealthMultiplier=1
    CurrentShieldModifier=1
    CurrentMaxAttackTickets=4
    CurrentMaxAttackTicketsHench=2
    CurrentMaxTargetTickets=6
    CurrentMaxTargetTicketsHench=3
    RecoilBonus=1
    WeaponPassiveDamageBonus=1
    WeaponDamageBonus=1
}
