﻿class SFXPawn extends BioPawn
    hidecategories(Navigation)
    config(Game)
    abstract
    placeable;

var() ForceFeedbackWaveform FootstepForceFeedback;

var() ScreenShakeStruct FootstepShake;

var VisualEffect FireChargingVFX;

// PrettyName: StringRefProperty[44]
var SFXVocalizationBank VocalizationBankOverride;

var Name nmShellCasingSocketName;

var Name nmMuzzleSocketName;

var bool bSupportsVisibleWeapons;

var bool bIsTalking;

var() ParticleSystem PS_LimbGib;

var() ParticleSystem PS_HeadGib;

struct VisualEffect
{
    var BioVFXTemplate Template;

    var BioVisualEffect Effect;

}

function PlayStepEffect(int FootDown, TraceHitInfo HitInfo, float Loudness)
{
    local PlayerController P;
    local BioPlayerController PC;
    PlayStepEffect(FootDown, HitInfo, Loudness);
    Class'BioPlayerController'.PlayLocationBasedCameraShake(Location, FootstepShake);
    foreach LocalPlayerControllers(Class'PlayerController', P)
    {
        if (CheckMaxEffectDistance(P, Location, FootstepCullDistance))
        {
            PC = BioPlayerController(P);
            PC.ClientPlayForceFeedbackWaveform(FootstepForceFeedback);
        }
    }
}

function SetTalking()
{
    bIsTalking = true;
    SetTimer(2, true, 'TalkingTimout');
}

function TalkingTimout()
{
    bIsTalking = false;
}

function CreateVisualEffect(BioVFXTemplate Template, out BioVisualEffect Effect)
{
    Effect = Class'BioVisualEffect'.CreateCrustEffect(Template, self, Template.fLifeTime);
}

function PlayFireCharging()
{
    if (FireChargingVFX.Template == None)
    {
        return;
    }
    if (FireChargingVFX.Effect == None || FireChargingVFX.Effect.bDeleteMe)
    {
        CreateVisualEffect(FireChargingVFX.Template, FireChargingVFX.Effect);
    }
    FireChargingVFX.Effect.SetState(0, true);
}

function bool CanUseWeapon(class<SFXWeapon> WeaponClass)
{
    local SFXInventoryManager oInventory;
    local SFXLoadoutData ChkLoadout;
    local class<SFXWeapon> LoadoutWeapon;
    local BioGlobalVariableTable VarTable;
    local SFXPlayerSquadLoadoutData SquadLoadout;
    oInventory = SFXInventoryManager(InvManager);
    VarTable = BioWorldInfo(WorldInfo).GetGlobalVariables();
    ChkLoadout = GetPawnLoadout();
    if (oInventory == None || VarTable == None || ChkLoadout == None)
    {
        return false;
    }
    SquadLoadout = SFXPlayerSquadLoadoutData(ChkLoadout);
    if (SquadLoadout != None)
    {
        if (SFXPawn_Player(self) != None)
        {
            return Class'SFXPlayerSquadLoadoutData'.CanPlayerUseWeaponClass(WeaponClass);
        }
        else
        {
            return Class'SFXPlayerSquadLoadoutData'.CanHenchmanUseWeaponClass(Tag, WeaponClass);
        }
    }
    foreach ChkLoadout.Weapons(LoadoutWeapon)
    {
        if (LoadoutWeapon == WeaponClass)
        {
            return true;
        }
    }
    return false;
}

function String GetShieldTypeAsDisplayString()
{
    local SFXShield_Base Shields;
    Shields = GetShields();
    if (Shields != None)
    {
        return Shields.ShieldDisplayName;
    }
    return Class'SFXShield_Base'.Default.ShieldDisplayName;
}

function ???<StringRefProperty> GetPrettyName()
{
    return PrettyName;
}

function CreatePlayerSquadWeapons(SFXPlayerSquadLoadoutData ChkLoadout, optional String LoadoutWeapons)
{
    local ELoadoutWeapons GroupIdx;
    local BioGlobalVariableTable VarTable;
    local int I;
    VarTable = BioWorldInfo(WorldInfo).GetGlobalVariables();
    if (ChkLoadout == None || VarTable == None)
    {
        return;
    }
    I = 0;
    while (I < 6)
    {
        GroupIdx = I;
        if (SFXPawn_Player(self) != None && Class'SFXPlayerSquadLoadoutData'.CanPlayerUseWeaponGroup(GroupIdx) || Class'SFXPlayerSquadLoadoutData'.CanHenchmanUseWeaponGroup(Tag, GroupIdx))
        {
            if (LoadoutWeapons[GroupIdx] == "")
            {
                CreateWeapon(Class'SFXPlayerSquadLoadoutData'.FindWeaponClass(Class'SFXPlayerSquadLoadoutData'.GetWeaponGroup(GroupIdx)[0].ClassName));
            }
            else
            {
                CreateWeapon(Class'SFXPlayerSquadLoadoutData'.FindWeaponClass(LoadoutWeapons[GroupIdx]));
            }
        }
        I++;
    }
}

defaultproperties
{
    bSupportsVisibleWeapons=true
    FootstepForceFeedback=ForceFeedbackWaveform'FootstepShakeFF0'
    HeadMesh=SkeletalMeshComponent'HeadMesh0'
    m_pAnimSetMgr=BioGestureAnimSetMgr'oAnimSetMgr'
    LightEnvironment=BioDynamicLightEnvironmentComponent'BioLightEnvComponent0'
    PowerManager=SFXPowerManager'PowerMgr'
    Mesh=None
    CylinderComponent=None
    Components=None
    // child object HeadMesh0 of type SkeletalMeshComponent
    // child object CollisionCylinder of type CylinderComponent
    // child object BioLightEnvComponent0 of type BioDynamicLightEnvironmentComponent
    // child object BioPawnSkeletalMeshComponent of type SkeletalMeshComponent
    // child object oAnimSetMgr of type BioGestureAnimSetMgr
    // child object PowerMgr of type SFXPowerManager
    // child object GEMod0 of type SFXModule_GameEffectManager
    // child object RadarModule of type SFXModule_Radar
    // child object GPModule of type SFXGamePropertiesModule
    // child object Behavior of type BioPawnBehavior
    // child object FootstepShakeFF0 of type ForceFeedbackWaveform
    // child object DmgMod0 of type SFXModule_Damage
}
