﻿class SFXPawn_Henchman extends SFXPawn
    hidecategories(Navigation)
    config(Game)
    abstract
    placeable;

var config array<HenchmanInfoStruct> HenchmenInfo;

var() Name m_nmMappedPower;

var() Texture2D m_GUI_Icon_Dead;

var() Texture2D m_GUI_Icon;

var() bool bHelmetHidesHead;

var() /* 00000008 */ SkeletalMeshComponent BreatherMesh;

var() /* 00000008 */ SkeletalMeshComponent VisorMesh;

var() /* 00000008 */ SkeletalMeshComponent HelmetMesh;

var() class<Object> DefaultWeapon;

var transient float PowerUseDelay;

struct HenchmanInfoStruct
{
    var Name ClassName;

    var Name Tag;

// PrettyName: StringRefProperty[44]
// AlternatePrettyName: StringRefProperty[44]
    var Name AlternateHenchNamePlotFlag;

    var int HenchAcquiredPlotID;

    var int HenchInSquadPlotID;

    var String HenchmanImage;

}

final function ShowHelmet(bool bShowHelmet)
{
    if (bShowHelmet)
    {
        if (HelmetMesh != None && HelmetMesh.SkeletalMesh != None && HelmetMesh.bAttached == false)
        {
            AttachComponent(HelmetMesh);
            if (bHelmetHidesHead)
            {
                DetachComponent(HeadMesh);
            }
        }
    }
    else
    {
        DetachComponent(HelmetMesh);
        if (bHelmetHidesHead)
        {
            AttachComponent(HeadMesh);
        }
    }
    ForceUpdateComponents(true, false);
}

function ForceHelmetVisibility(bool bHelmetVisible)
{
    ShowHelmet(bHelmetVisible);
}

function int GetScaledLevel()
{
    return 1;
}

simulated event FellOutOfWorld(class<DamageType> dmgType)
{
    local SFXAI_Henchman oHench;
    FellOutOfWorld(dmgType);
    oHench = SFXAI_Henchman(Controller);
    if (oHench != None)
    {
        oHench.OutOfWorld();
    }
}

singular simulated event OutsideWorldBounds()
{
    local SFXAI_Henchman oHench;
    oHench = SFXAI_Henchman(Controller);
    if (oHench != None)
    {
        oHench.OutOfWorld();
    }
}

function bool InCombat()
{
    local SFXGame GameInfo;
    GameInfo = SFXGame(WorldInfo.Game);
    if (GameInfo != None)
    {
        return GameInfo.bInCombat || GameInfo.bForceCombat;
    }
    return false;
}

function bool GetAttackOrderPower(out SFXPower Power)
{
    Power = PowerManager.GetPower(m_nmMappedPower);
    return Power != None;
}

state Downed
{
    function BeginState(Name PreviousStateName)
    {
        BeginState(PreviousStateName);
        if (SFXGame(WorldInfo.Game).gameconfig.bReviveSquadOnUse)
        {
        }
    }

    function EndState(Name NextStateName)
    {
        EndState(NextStateName);
        if (SFXGame(WorldInfo.Game).gameconfig.bReviveSquadOnUse)
        {
        }
    }

}

function PostBeginPlay()
{
    local SFXModule_Radar RadarMod;
    if (GetModule(Class'SFXModule_Radar') == None)
    {
        RadarMod = new(self,Class'SFXModule_Radar');
        RadarMod.RadarType = 1;
        Modules.AddItem(RadarMod);
    }
    PostBeginPlay();
}

function Revive(Actor User)
{
    self.Resurrect(1, false);
}

event bool Resurrect(float PercentOfHealthRegained, bool bIsInstantaneous)
{
    if (IsDead() == false)
    {
        return false;
    }
    if (Physics != 10 && Mesh == CollisionComponent)
    {
        SetPhysics(10);
    }
    return Resurrect(PercentOfHealthRegained, bIsInstantaneous);
}

function SpawnDefaultWeapons()
{
    local SFXInventoryManager InventoryManager;
    local SFXLoadoutData ChkLoadout;
    InventoryManager = SFXInventoryManager(InvManager);
    if (InventoryManager != None && InventoryManager.FindInventoryType(Class'SFXWeapon', true) == None)
    {
        ChkLoadout = GetPawnLoadout();
        CreateWeapons(ChkLoadout);
        ScaleWeapons(ChkLoadout, GetScaledLevel());
        InventoryManager.CurrentWeaponSelection = ChkLoadout.Weapons[0];
    }
}

function InitializeHenchman(int DesiredLevel)
{
    local BioWorldInfo BWI;
    local BioPlayerController PC;
    local SFXEngine Engine;
    BWI = BioWorldInfo(WorldInfo);
    PC = BWI.GetLocalPlayerController();
    if (PC != None)
    {
        Engine = SFXEngine(PC.Player.Outer);
    }
    Engine.CurrentSaveGame.LoadHenchman(self);
    SpawnDefaultWeapons();
    if (CharacterLevel != DesiredLevel)
    {
        Class'BioLevelUpSystem'.LevelUpPawn(self, DesiredLevel);
    }
}

function ValidateRagdoll();

function Texture2D GetGUIIcon(optional bool bDeadIcon = false)
{
    return bDeadIcon ? m_GUI_Icon_Dead : m_GUI_Icon;
}

event SetRTPCHelmetIsEnabled(WwiseAudioComponent WwiseComponent)
{
    if (WwiseComponent != None)
    {
        if (HelmetMesh != None && HelmetMesh.SkeletalMesh != None && HelmetMesh.bAttached == true)
        {
            WwiseComponent.SetWwiseRTPC("Helmet", 1);
        }
        else
        {
            WwiseComponent.SetWwiseRTPC("Helmet", 0);
        }
    }
}

function PlayDeathVocalization(BioPawn Killer)
{
    SFXGame(WorldInfo.Game).TriggerVocalizationEvent(23, self, Killer);
    SFXGame(WorldInfo.Game).TriggerVocalizationEvent(27, self, Killer, 2);
}

state BioPawn
{
    function UpdateCombatMode()
    {
        local BioPlayerSquad PlayerSquad;
        if (m_oBehavior == None)
        {
            return;
        }
        PlayerSquad = BioPlayerSquad(Squad);
        if (PlayerSquad != None && PlayerSquad.m_bCombatEnabled)
        {
            if (ActionState == 0)
            {
                m_oBehavior.SetActionState(1);
            }
        }
    }

}

event int GetMaxTargetTickets()
{
    local int Tickets;
    Tickets = m_nMaxTargetTickets;
    if (PlayerReplicationInfo != None)
    {
        Tickets = SFXPRI(PlayerReplicationInfo).CurrentMaxTargetTicketsHench;
    }
    return Tickets;
}

event int GetMaxAttackTickets()
{
    local int Tickets;
    Tickets = m_nMaxAttackTickets;
    if (PlayerReplicationInfo != None)
    {
        Tickets = SFXPRI(PlayerReplicationInfo).CurrentMaxAttackTicketsHench;
    }
    return Tickets;
}

static function GetDefaultLoadout(Name HTag, out String DLoadout)
{
    local array<LoadoutWeaponInfo> WeaponGroup;
    local LoadoutWeaponInfo WeaponInfo;
    local int idx;
    local int WIdx;
    local int WRating;
    local String WClassName;
    idx = 0;
    while (idx < 6)
    {
        if (Class'SFXPlayerSquadLoadoutData'.CanHenchmanUseWeaponGroup(HTag, idx))
        {
            WeaponGroup = Class'SFXPlayerSquadLoadoutData'.GetWeaponGroup(idx);
            WRating = -1;
            WIdx = 0;
            while (WIdx < WeaponGroup.Length)
            {
                WeaponInfo = WeaponGroup[WIdx];
                if (WeaponInfo.Rating >= WRating)
                {
                    if (Class'SFXPlayerSquadLoadoutData'.CanHenchmanUseWeaponClass2(HTag, WeaponInfo.ClassName))
                    {
                        WRating = WeaponInfo.Rating;
                        WClassName = WeaponInfo.ClassName;
                    }
                }
                WIdx++;
            }
            if (WClassName != "")
            {
                DLoadout[idx] = WClassName;
            }
        }
        idx++;
    }
}

function CreateWeapons(SFXLoadoutData ChkLoadout)
{
    local class<SFXWeapon> WeaponClass;
    local SFXPlayerSquadLoadoutData PlayerLoadout;
    local int I;
    local BioPlayerController PC;
    local SFXEngine Engine;
    PC = BioWorldInfo(WorldInfo).GetLocalPlayerController();
    Engine = SFXEngine(PC.Player.Outer);
    PlayerLoadout = SFXPlayerSquadLoadoutData(ChkLoadout);
    if (PlayerLoadout != None)
    {
        if (Engine != None)
        {
            Engine.CurrentSaveGame.EnsureHenchmanRecordExists(self);
            I = 0;
            while (I < Engine.HenchmanRecords.Length)
            {
                if (Engine.HenchmanRecords[I].Tag == Tag)
                {
                    CreatePlayerSquadWeapons(PlayerLoadout, Engine.HenchmanRecords[I].LoadoutWeapons);
                }
                I++;
            }
        }
        else
        {
            CreatePlayerSquadWeapons(PlayerLoadout);
        }
    }
    foreach ChkLoadout.Weapons(WeaponClass)
    {
        CreateWeapon(WeaponClass);
    }
}

defaultproperties
{
    PowerUseDelay=5
    HelmetMesh=SkeletalMeshComponent'HelmetMesh0'
    HenchmenInfo=System.Byte[]
    bShouldSpawnWeapons=false
    bOverrideBodyMats=true
    m_bAlwaysTick=true
    bScalePowers=false
    HeadMesh=SkeletalMeshComponent'HeadMesh0'
    m_pAnimSetMgr=BioGestureAnimSetMgr'oAnimSetMgr'
    LightEnvironment=BioDynamicLightEnvironmentComponent'BioLightEnvComponent0'
    PowerManager=SFXPowerManager'PowerMgr'
    Mesh=None
    CylinderComponent=None
    Components=None
    // child object BioLightEnvComponent0 of type BioDynamicLightEnvironmentComponent
    // child object oAnimSetMgr of type BioGestureAnimSetMgr
    // child object Behavior of type BioPawnBehavior
    // child object CollisionCylinder of type CylinderComponent
    // child object GPModule of type SFXGamePropertiesModule
    // child object DmgMod1 of type SFXModule_DamageParty
    // child object GEMod0 of type SFXModule_GameEffectManager
    // child object RadarMod1 of type SFXModule_Radar
    // child object RadarModule of type SFXModule_Radar
    // child object PowerMgr of type SFXPowerManager
    // child object BioPawnSkeletalMeshComponent of type SkeletalMeshComponent
    // child object HeadMesh0 of type SkeletalMeshComponent
    // child object HelmetMesh0 of type SkeletalMeshComponent
}
