﻿class SFXPlayerCamera extends SFXCameraNativeBase
    hidecategories(Navigation)
    transient;

var transient float m_fLastAspectRatio;

var transient float LastTransitionTime;

var float TransitionTimeout;

var transient Rotator KickBackModifier;

var transient bool m_bLastConstrainAspect;

var transient bool bDebugging;

var transient bool bCurrentTransitionIsModal;

var(Camera) transient SFXCameraModifier_ScreenShake ScreenShakeModifier;

var InterpCurveFloat LongEaseInCurve;

var InterpCurveFloat TaperOverTimeCurve;

var InterpCurveFloat SlowRampUpCurve;

var InterpCurveFloat StandardCurve;

var transient float DefaultTransitionTime;

var transient SFXCameraMode FreeCam;

var() SFXCameraTransition_FaceTarget FaceTargetTransition;

function PostBeginPlay()
{
    PostBeginPlay();
    ScreenShakeModifier = SFXCameraModifier_ScreenShake(CreateCameraModifier(Class'SFXCameraModifier_ScreenShake'));
    FreeCam = new(self,Class'BioCameraBehaviorFree');
    FaceTargetTransition = new(self,Class'SFXCameraTransition_FaceTarget');
    FaceTargetTransition.Curve = StandardCurve;
    InstantTransition = new(self,Class'SFXCameraTransition_GalaxyMap');
    SlideTransition = new(self,Class'SFXCameraMode_Interpolate');
    SlideTransition.Curve = StandardCurve;
}

function AddScreenShake(ScreenShakeStruct Shake)
{
    if (!bFreeCamActive)
    {
        ScreenShakeModifier.AddScreenShake(Shake);
    }
}

function CameraModifier CreateCameraModifier(class<CameraModifier> ModifierClass)
{
    local CameraModifier NewMod;
    NewMod = new(Outer,ModifierClass);
    NewMod.Init();
    NewMod.AddCameraModifier(self);
    return NewMod;
}

function UpdateViewTarget(out TViewTarget OutVT, float DeltaTime)
{
    local CameraActor CamActor;
    local BioWorldInfo BWorldInfo;
    if (bDisabled && !bFreeCamActive)
    {
        OutVT.POV = CameraCache.POV;
        if (m_fLastAspectRatio >= 0)
        {
            OutVT.AspectRatio = m_fLastAspectRatio;
            bConstrainAspectRatio = m_bLastConstrainAspect;
        }
        return;
    }
    ResetHiddenActors();
    CamActor = CameraActor(OutVT.Target);
    if (CamActor != None && !bFreeCamActive)
    {
        CamActor.GetCameraView(DeltaTime, OutVT.POV);
        bConstrainAspectRatio = bConstrainAspectRatio || CamActor.bConstrainAspectRatio;
        OutVT.AspectRatio = CamActor.AspectRatio;
    }
    else
    {
        if (TimeSince(LastTransitionTime) > TransitionTimeout)
        {
            PickCameraMode(DeltaTime);
        }
        UpdateCameraManager(DeltaTime, OutVT.AspectRatio, OutVT.POV);
        BWorldInfo = BioWorldInfo(WorldInfo);
        if (BWorldInfo != None)
        {
            BWorldInfo.UpdateEnvironmentEffects(DeltaTime);
        }
    }
    ApplyCameraModifiers(DeltaTime, OutVT.POV);
    m_bLastConstrainAspect = bConstrainAspectRatio;
    m_fLastAspectRatio = OutVT.AspectRatio;
}

function RotateToFace(Vector AimPoint, float Time)
{
    FaceTargetTransition.TargetLocation = AimPoint;
    SetBehavior(CurrentCameraMode, FaceTargetTransition, Time, true);
}

function PlayCameraTransition(SFXCameraMode_Interpolate Transition, float Time)
{
    local BioPlayerController PC;
    local SFXGameModeManager GMM;
    local SFXGameModeBase GM;
    local int idx;
    PC = BioPlayerController(Owner);
    GMM = PC.GameModeManager2;
    idx = 0;
    while (idx < GMM.GameModes.Length)
    {
        GM = GMM.GameModes[idx];
        if (SFXGameModeDefault(GM) != None)
        {
            SetBehavior(SFXGameModeDefault(GM).CombatCam, Transition, Time, false);
        }
        idx++;
    }
    bCurrentTransitionIsModal = true;
}

function PickCameraMode(float DeltaTime)
{
    local int PreserveTarget;
    local BioPlayerController PC;
    local BioPlayerInput Input;
    local bool bNoChange;
    local SFXCameraMode NewCameraMode;
    local SFXCameraMode_Interpolate Transition;
    local float TransitionTime;
    PC = BioPlayerController(PCOwner);
    Input = PC != None ? BioPlayerInput(PC.PlayerInput) : None;
    if (PC == None || Input == None)
    {
        return;
    }
    PreserveTarget = 1;
    TransitionTime = DefaultTransitionTime;
    Transition = SlideTransition;
    if (bFreeCamActive)
    {
        NewCameraMode = FreeCam;
    }
    else
    {
        NewCameraMode = PC.GameModeManager2.GetCameraMode(CurrentCameraMode, PreserveTarget, TransitionTime, Transition);
    }
    bNoChange = NewCameraMode == CurrentCameraMode || SFXCameraMode_Interpolate(CurrentCameraMode) != None && SFXCameraMode_Interpolate(CurrentCameraMode).To == NewCameraMode;
    if (!bNoChange)
    {
        if (CurrentCameraMode != None && CurrentCameraMode.CameraName == 'ConversationCam' || CurrentCameraMode.CameraName == 'GalaxyCam')
        {
            TransitionTime = 0;
            PreserveTarget = 0;
        }
        SetBehavior(NewCameraMode, Transition, TransitionTime, PreserveTarget);
    }
}

function UpdateCameraManager(float fDeltaTime, out float fAspectRatio, out TPOV POV)
{
    local SFXCameraMode_Interpolate Transition;
    if (PCOwner == None || PCOwner.WorldInfo == None || CurrentCameraMode == None)
    {
        return;
    }
    Transition = SFXCameraMode_Interpolate(CurrentCameraMode);
    if (Transition != None && Transition.bComplete)
    {
        bCurrentTransitionIsModal = false;
        SwitchTo(Transition.To);
        Transition = None;
    }
    AspectRatio = fAspectRatio;
    POV = ViewTarget.POV;
    if (m_bIgnoreSlowMo && WorldInfo.TimeDilation != 0)
    {
        if (bFreeCamActive)
        {
            fDeltaTime /= WorldInfo.TimeDilation;
        }
    }
    if (CurrentCameraMode != None)
    {
        CurrentCameraMode.Tick(fDeltaTime);
        POV = CurrentCameraMode.m_pov;
        POV.Rotation += CurrentCameraMode.GetCurrentShake();
        POV.Rotation += KickBackModifier;
        POV.Rotation = Normalize(POV.Rotation);
        ViewTarget.POV = POV;
        fAspectRatio = CurrentCameraMode.m_fAspectRatio;
        AspectRatio = fAspectRatio;
        bConstrainAspectRatio = CurrentCameraMode.m_bConstrainAspectRatio;
        if (CurrentCameraMode.IsCollisionEnabled())
        {
            CameraCache.POV = POV;
            TraceCamera();
            CurrentCameraMode.CollisionDistance = 0;
            CurrentCameraMode.DoCameraCollision(POV.Location, POV.Rotation, CurrentCameraMode.Owner);
        }
    }
    POV.Rotation.Yaw = POV.Rotation.Yaw % 65536;
    m_aTraceInfo.m_bCollisionDirty = true;
    TraceCamera();
}

function SetBehavior(SFXCameraMode Mode, SFXCameraMode_Interpolate NewTransition, float Time, bool PreserveTarget)
{
    local SFXCameraMode PreviousBehavior;
    local SFXCameraMode_Interpolate PrevTransition;
    if (bCurrentTransitionIsModal)
    {
        return;
    }
    NewTransition = new(self,NewTransition.Class);
    NewTransition;
    NewTransition.From = None;
    NewTransition.To = None;
    if (Mode == None)
    {
        return;
    }
    PrevTransition = SFXCameraMode_Interpolate(CurrentCameraMode);
    PreviousBehavior = CurrentCameraMode;
    Mode.Initialize(BioPlayerController(PCOwner));
    if (NewTransition != None && PreviousBehavior != None)
    {
        NewTransition.Initialize(BioPlayerController(PCOwner));
        NewTransition.InitializeTransition(PreviousBehavior, Mode, Time, PreserveTarget);
        if (NewTransition.CheckLoop(None, 4))
        {
            return;
        }
        SwitchTo(NewTransition);
        LastTransitionTime = WorldInfo.TimeSeconds;
    }
    else
    {
        SwitchTo(Mode);
    }
    if (CurrentCameraMode != None)
    {
        ViewTarget.POV = CurrentCameraMode.m_pov;
    }
}

function SwitchTo(SFXCameraMode NewMode)
{
    if (CurrentCameraMode != None)
    {
        CurrentCameraMode.MakeInactive();
    }
    CurrentCameraMode = NewMode;
    if (CurrentCameraMode != None)
    {
        CurrentCameraMode.MakeActive();
    }
    CurrentCameraMode.Tick(0);
}

final function Rotator GetBaseRotation()
{
    return PCOwner.Rotation;
}

final function SetBaseRotation(Rotator NewRotation)
{
    if (CurrentCameraMode != None)
    {
        CurrentCameraMode.m_pov.Rotation = NewRotation;
        PCOwner.SetRotation(NewRotation);
    }
}

defaultproperties
{
    DefaultTransitionTime=0,2
    StandardCurve=(Points=?,InterpMethod=?)
    SlowRampUpCurve=(Points=?,InterpMethod=?)
    TaperOverTimeCurve=(Points=?,InterpMethod=?)
    LongEaseInCurve=(Points=?,InterpMethod=?)
    TransitionTimeout=0,25
    m_fLastAspectRatio=-1
    DefaultAspectRatio=1,777778
}
