﻿class SFXPlayerSquadLoadoutData extends SFXLoadoutData
    hidecategories(Object)
    config(Game)
    native;

enum ELoadoutWeapons
{
    LoadoutWeapons_AssaultRifles,
    LoadoutWeapons_Shotguns,
    LoadoutWeapons_SniperRifles,
    LoadoutWeapons_AutoPistols,
    LoadoutWeapons_HeavyPistols,
    LoadoutWeapons_HeavyWeapons,
};

struct native LoadoutWeaponInfo
{
    var() String ClassName;

    var() Name UnlockPlotId;

    var() int Rating;

    var() bool bNotRegularWeaponGUI;

}

var config array<SpecialWeaponInfo> HenchmenSpecialWeapons;

var config array<SpecialWeaponInfo> PlayerSpecialWeapons;

var config array<BonusWeaponInfo> PlayerBonusWeapons;

var config array<LoadoutInfo> HenchLoadoutInfo;

var config array<LoadoutInfo> PlayerLoadoutInfo;

var int MaxWeapons;

var(Weapons) array<UnlockableWeaponClass> UnlockableStandardWeapons;

var(Weapons) array<ELoadoutWeapons> StandardWeapons;

var array<PlotWeapon> PlotWeapons;

var(Weapons) array<PlotWeaponEditor> PlotWeaponsEditor;

var config array<LoadoutWeaponInfo> HeavyWeapons;

var config array<LoadoutWeaponInfo> HeavyPistols;

var config array<LoadoutWeaponInfo> AutoPistols;

var config array<LoadoutWeaponInfo> SniperRifles;

var config array<LoadoutWeaponInfo> Shotguns;

var config array<LoadoutWeaponInfo> AssaultRifles;

struct native PlotWeaponEditor
{
    var() class<SFXWeapon> WeaponClass;

    var() Name UnlockPlotId;

    var() Name EquippedPlotId;

}

struct native PlotWeapon
{
    var String WeaponClassName;

    var Name UnlockPlotId;

    var Name EquippedPlotId;

}

struct native UnlockableWeaponClass
{
    var() ELoadoutWeapons WeaponType;

    var() Name UnlockPlotId;

}

struct native LoadoutInfo
{
    var Name ClassName;

    var array<ELoadoutWeapons> WeaponClasses;

}

struct native BonusWeaponInfo
{
    var ELoadoutWeapons WeaponClass;

    var Name UnlockPlotName;

}

struct native SpecialWeaponInfo
{
    var Name WeaponClassName;

    var Name HenchmanClassName;

    var Name UnlockPlotName;

}

static function array<LoadoutWeaponInfo> GetWeaponGroup(int idx)
{
    if (idx == 0)
    {
        return Default.AssaultRifles;
    }
    else
    {
        if (idx == 1)
        {
            return Default.Shotguns;
        }
        else
        {
            if (idx == 2)
            {
                return Default.SniperRifles;
            }
            else
            {
                if (idx == 3)
                {
                    return Default.AutoPistols;
                }
                else
                {
                    if (idx == 4)
                    {
                        return Default.HeavyPistols;
                    }
                    else
                    {
                        return Default.HeavyWeapons;
                    }
                }
            }
        }
    }
}

static function GetWeaponCategory(class<SFXWeapon> WClass, out int GroupIdx, out int EntryIdx)
{
    GetWeaponCategoryByName(PathName(WClass), GroupIdx, EntryIdx);
}

static function GetWeaponCategoryByName(String WClassName, out int GroupIdx, out int EntryIdx)
{
    local LoadoutWeaponInfo LWI;
    EntryIdx = 0;
    while (EntryIdx < Default.AssaultRifles.Length)
    {
        LWI = Default.AssaultRifles[EntryIdx];
        if (LWI.ClassName == WClassName)
        {
            GroupIdx = 0;
            return;
        }
        EntryIdx++;
    }
    EntryIdx = 0;
    while (EntryIdx < Default.Shotguns.Length)
    {
        LWI = Default.Shotguns[EntryIdx];
        if (LWI.ClassName == WClassName)
        {
            GroupIdx = 1;
            return;
        }
        EntryIdx++;
    }
    EntryIdx = 0;
    while (EntryIdx < Default.SniperRifles.Length)
    {
        LWI = Default.SniperRifles[EntryIdx];
        if (LWI.ClassName == WClassName)
        {
            GroupIdx = 2;
            return;
        }
        EntryIdx++;
    }
    EntryIdx = 0;
    while (EntryIdx < Default.AutoPistols.Length)
    {
        LWI = Default.AutoPistols[EntryIdx];
        if (LWI.ClassName == WClassName)
        {
            GroupIdx = 3;
            return;
        }
        EntryIdx++;
    }
    EntryIdx = 0;
    while (EntryIdx < Default.HeavyPistols.Length)
    {
        LWI = Default.HeavyPistols[EntryIdx];
        if (LWI.ClassName == WClassName)
        {
            GroupIdx = 4;
            return;
        }
        EntryIdx++;
    }
    EntryIdx = 0;
    while (EntryIdx < Default.HeavyWeapons.Length)
    {
        LWI = Default.HeavyWeapons[EntryIdx];
        if (LWI.ClassName == WClassName)
        {
            GroupIdx = 5;
            return;
        }
        EntryIdx++;
    }
    GroupIdx = -1;
    EntryIdx = -1;
}

static function class<SFXWeapon> FindWeaponClass(String WeaponClassName)
{
    local class<SFXWeapon> FoundClass;
    FoundClass = Class<SFXWeapon>(FindObject(WeaponClassName, Class'Class'));
    if (FoundClass == None && Class'SFXEngine'.IsSeekFreeObjectSupported(WeaponClassName))
    {
        FoundClass = Class<SFXWeapon>(Class'SFXEngine'.LoadSeekFreeObject(WeaponClassName, Class'Class'));
    }
    if (FoundClass == None)
    {
        FoundClass = Class<SFXWeapon>(DynamicLoadObject(WeaponClassName, Class'Class'));
    }
    return FoundClass;
}

static function int GetPluralPrettyName(int idx)
{
    switch (idx)
    {
        case 0:
            return 326505;
        case 1:
            return 326503;
        case 2:
            return 326504;
        case 3:
            return 326501;
        case 4:
            return 326502;
        default:
            return 0;
    }
}

static function bool CanPlayerUseWeaponGroup(ELoadoutWeapons WeaponGroup)
{
    local BioPlayerController PlayerController;
    local BioWorldInfo WorldInfo;
    local LoadoutInfo Info;
    local BioGlobalVariableTable VarTable;
    local BonusWeaponInfo BonusInfo;
    WorldInfo = BioWorldInfo(Class'Engine'.GetCurrentWorldInfo());
    if (WorldInfo != None)
    {
        PlayerController = WorldInfo.GetLocalPlayerController();
    }
    if (PlayerController == None)
    {
        return false;
    }
    foreach Default.PlayerLoadoutInfo(Info)
    {
        if (Info.ClassName == PlayerController.Pawn.Class.Name)
        {
            if (Info.WeaponClasses.Find(WeaponGroup) != -1)
            {
                return true;
            }
            else
            {
                VarTable = WorldInfo.GetGlobalVariables();
                foreach Default.PlayerBonusWeapons(BonusInfo)
                {
                    if (BonusInfo.WeaponClass == WeaponGroup)
                    {
                        return VarTable.GetIntByName(BonusInfo.UnlockPlotName) != 0;
                    }
                }
            }
        }
    }
    return false;
}

static function Name GetHenchmanClassname(Name HenchTag)
{
    local int idx;
    local Name HenchClassName;
    idx = 0;
    while (idx < Class'SFXPawn_Henchman'.Default.HenchmenInfo.Length)
    {
        if (Class'SFXPawn_Henchman'.Default.HenchmenInfo[idx].Tag == HenchTag)
        {
            HenchClassName = Class'SFXPawn_Henchman'.Default.HenchmenInfo[idx].ClassName;
        }
        idx++;
    }
    return HenchClassName;
}

static function bool CanHenchmanUseWeaponGroup(Name HenchTag, ELoadoutWeapons WeaponGroup)
{
    local LoadoutInfo Info;
    local Name HenchClassName;
    HenchClassName = GetHenchmanClassname(HenchTag);
    foreach Default.HenchLoadoutInfo(Info)
    {
        if (Info.ClassName == HenchClassName)
        {
            return Info.WeaponClasses.Find(WeaponGroup) != -1;
        }
    }
    return false;
}

static function bool CanPlayerUseWeaponClass(class<SFXWeapon> WeaponClass)
{
    local int WeaponGroup;
    local int WeaponID;
    local SpecialWeaponInfo SpecialInfo;
    local BioWorldInfo WorldInfo;
    local BioGlobalVariableTable VarTable;
    WorldInfo = BioWorldInfo(Class'Engine'.GetCurrentWorldInfo());
    if (WorldInfo == None)
    {
        return false;
    }
    GetWeaponCategory(WeaponClass, WeaponGroup, WeaponID);
    if (WeaponClass.Default.bNotRegularWeaponGUI == false && CanPlayerUseWeaponGroup(WeaponGroup))
    {
        return true;
    }
    else
    {
        VarTable = WorldInfo.GetGlobalVariables();
        foreach Default.PlayerSpecialWeapons(SpecialInfo)
        {
            if (PathName(WeaponClass) == SpecialInfo.WeaponClassName && VarTable.GetIntByName(SpecialInfo.UnlockPlotName) != 0)
            {
                return true;
            }
        }
    }
    return false;
}

static function bool CanHenchmanUseWeaponClass(Name HenchTag, class<SFXWeapon> Weapon)
{
    return CanHenchmanUseWeaponClass2(HenchTag, PathName(Weapon));
}

static function bool CanHenchmanUseWeaponClass2(Name HenchTag, Name WeaponClassName)
{
    local int WeaponGroup;
    local int WeaponID;
    local SpecialWeaponInfo SpecialInfo;
    local BioWorldInfo WorldInfo;
    local BioGlobalVariableTable VarTable;
    local LoadoutWeaponInfo LWI;
    local Name HenchClassName;
    HenchClassName = GetHenchmanClassname(HenchTag);
    WorldInfo = BioWorldInfo(Class'Engine'.GetCurrentWorldInfo());
    if (WorldInfo == None)
    {
        return false;
    }
    GetWeaponCategoryByName(WeaponClassName, WeaponGroup, WeaponID);
    LWI = GetWeaponGroup(WeaponGroup)[WeaponID];
    VarTable = WorldInfo.GetGlobalVariables();
    if (CanHenchmanUseWeaponGroup(HenchTag, WeaponGroup) && LWI.bNotRegularWeaponGUI == false && LWI.UnlockPlotId == 'None' || VarTable.GetIntByName(LWI.UnlockPlotId) != 0)
    {
        return true;
    }
    else
    {
        foreach Default.HenchmenSpecialWeapons(SpecialInfo)
        {
            if (SpecialInfo.HenchmanClassName == HenchClassName)
            {
                if (WeaponClassName == SpecialInfo.WeaponClassName && VarTable.GetIntByName(SpecialInfo.UnlockPlotName) != 0)
                {
                    return true;
                }
            }
        }
    }
    return false;
}

defaultproperties
{
    AssaultRifles=System.Byte[]
    Shotguns=System.Byte[]
    SniperRifles=System.Byte[]
    AutoPistols=System.Byte[]
    HeavyPistols=System.Byte[]
    HeavyWeapons=System.Byte[]
    MaxWeapons=5
    PlayerLoadoutInfo=System.Byte[]
    HenchLoadoutInfo=System.Byte[]
    PlayerBonusWeapons=System.Byte[]
    PlayerSpecialWeapons=System.Byte[]
    HenchmenSpecialWeapons=System.Byte[]
}
