﻿class SFXPower extends Object
    abstract
    native;

var transient BioPower OldPower;

var transient BioPawn MyPawn;

var int WheelDisplayIndex;

var array<UnlockRequirement> UnlockRequirements;

var array<int> RankCosts;

var array<RankInfo> Ranks;

// TalentDescription: StringRefProperty[44]
var class<Object> EvolvedPowerClass2;

var class<Object> EvolvedPowerClass1;

var float TimeUntilNextUse;

var float DelayBetweenUses;

var float DelayBeforeFirstUse;

var class<SFXProjectile_Power> ProjectileClass;

var class<Object> PowerScriptClass;

var ESFXVocalizationEventID VocalizationEvent;

var EBioCapMode Discipline;

var EBioCapabilityTypes CapabilityType;

var EBioPowerType PowerType;

var float MaxRank;

var float Rank;

var Vector2D ConeHalfAngle;

var Vector2D ProjectileRadius;

var Vector2D ProjectileSpeed;

var Vector2D VFXIntensity;

var array<float> Force;

var float DynamicDamageBonus;

var array<float> Damage;

var float DynamicDurationBonus;

var array<float> EffectDuration;

var array<float> ImpactRadius;

var Vector2D MaximumRange;

var Vector2D MinimumRange;

var bool DisplayInCharacterRecord;

var bool IsHenchmenUnique;

var bool AISelectable;

var bool DisplayInHUD;

var bool CanMoveWhileCasting;

var bool StopOnFirstHit;

var bool AimingIgnoresObstructions;

var bool BlockedByObjects;

var bool UsesSharedCooldown;

var float DynamicCooldownBonus;

var Vector2D GlobalCooldownTime;

var Vector2D CooldownTime;

var float ReleasePlayRate;

var Vector2D CastingTime;

var BioPowerVFXAppearance VFX;

var AnimSet Animations;

var int Icon;

// ImpactText: StringRefProperty[44]
// Description: StringRefProperty[44]
// DisplayName: StringRefProperty[44]
var Name BaseName;

var Name PowerName;

struct native RankInfo
{
    var int Icon;

// Name: StringRefProperty[44]
// Description: StringRefProperty[44]
// unlockBlurb: StringRefProperty[44]
}

struct native UnlockRequirement
{
    var class<Object> PowerClass;

    var float Rank;

// CustomUnlockText: StringRefProperty[44]
}

native function float GetScaledValue(Vector2D ValueToScale, optional int nRankToUse);

native function float GetArrayValue(out array<float> ArrayValues, optional int nRankToUse);

native function GetStringFromStringRef(out String TheString, optional bool bParse, optional int nParseIndex);

native function GetUnlockDetails(out String sUnlockDetails);

native function bool IsTargetInRange(Actor Target);

native function BioPowerScript GetPowerScript();

native function float GetCooldownRemaining();

native function float GetTotalCooldownTime();

native function bool IsEnabled();

native function bool IsSuppressed();

native function bool GetDescription(int nRankIndex, out String sDescription);

native function bool GetUnlockBlurb(int nRankIndex, out String sUnlockBlurb);

native function bool GetParsedString(int nRankIndex, out String sOutput);

native final private function bool ParseString(int nRankIndex, out String sParsedString);

native final private function ProcessToken(int nRankIndex, out String sToken);

function float GetDamage()
{
    local float fDamage;
    local float fStaticBonus;
    fDamage = GetArrayValue(Damage);
    if (MyPawn != None && MyPawn.InPlayerParty())
    {
        fStaticBonus = 1;
        GetDamageResearchBonus(fStaticBonus);
        fDamage *= fStaticBonus + DynamicDamageBonus - 1;
    }
    return fDamage;
}

function GetDamageResearchBonus(out float fStaticDamageBonus)
{
    if (MyPawn != None && MyPawn.InPlayerParty())
    {
        if (Discipline == 1)
        {
            fStaticDamageBonus *= SFXGame(MyPawn.WorldInfo.Game).BonusList.RchBioDmgUpgradeCache;
        }
        else
        {
            if (Discipline == 2)
            {
                fStaticDamageBonus *= SFXGame(MyPawn.WorldInfo.Game).BonusList.RchTechDmgUpgradeCache;
            }
        }
        if (MyPawn.Tag == 'hench_convict')
        {
            fStaticDamageBonus = fStaticDamageBonus - 1 + SFXGame(MyPawn.WorldInfo.Game).BonusList.RchJackUpgradeCache;
        }
        else
        {
            if (MyPawn.Tag == 'hench_professor')
            {
                fStaticDamageBonus = fStaticDamageBonus - 1 + SFXGame(MyPawn.WorldInfo.Game).BonusList.RchMordinUpgradeCache;
            }
        }
    }
}

function RecalculateDamageBonus()
{
    local SFXModule_GameEffectManager Manager;
    if (MyPawn != None)
    {
        Manager = MyPawn.GetModule(Class'SFXModule_GameEffectManager');
        if (Manager != None)
        {
            DynamicDamageBonus = Manager.GetEffectValue(Class'SFXGameEffect_PowerDamageBonus', 1);
        }
    }
}

function float GetForce()
{
    return GetArrayValue(Force);
}

function float GetDuration()
{
    local float fDuration;
    local float fStaticBonus;
    fDuration = GetArrayValue(EffectDuration);
    if (MyPawn != None && MyPawn.InPlayerParty())
    {
        fStaticBonus = 1;
        GetDurationResearchBonus(fStaticBonus);
        fDuration *= fStaticBonus + DynamicDurationBonus - 1;
    }
    return fDuration * GetDifficultyDurationMultiplier();
}

function GetDurationResearchBonus(out float fStaticDamageBonus)
{
    if (MyPawn != None && MyPawn.InPlayerParty())
    {
        if (Discipline == 1)
        {
            fStaticDamageBonus *= SFXGame(MyPawn.WorldInfo.Game).BonusList.RchBioticDurationCache;
        }
        else
        {
            if (Discipline == 2)
            {
                fStaticDamageBonus *= SFXGame(MyPawn.WorldInfo.Game).BonusList.RchTechDurationCache;
            }
        }
    }
}

function RecalculateDurationBonus()
{
    local SFXModule_GameEffectManager Manager;
    if (MyPawn != None)
    {
        Manager = MyPawn.GetModule(Class'SFXModule_GameEffectManager');
        if (Manager != None)
        {
            DynamicDurationBonus = Manager.GetEffectValue(Class'SFXGameEffect_PowerDurationBonus', 1);
        }
    }
}

function float GetDifficultyDurationMultiplier()
{
    return SFXGame(MyPawn.WorldInfo.Game).Difficulty.GetDurationMultiplier(MyPawn);
}

native function Tick(float fDeltaTime);

function RecalculateCooldownBonus()
{
    local SFXModule_GameEffectManager Manager;
    if (MyPawn != None)
    {
        Manager = MyPawn.GetModule(Class'SFXModule_GameEffectManager');
        if (Manager != None)
        {
            DynamicCooldownBonus = Manager.GetEffectValue(Class'SFXGameEffect_PowerCooldownModifier', 1);
        }
    }
}

defaultproperties
{
    ReleasePlayRate=1
    DynamicCooldownBonus=1
    UsesSharedCooldown=true
    BlockedByObjects=true
    CanMoveWhileCasting=true
    DisplayInHUD=true
    AISelectable=true
    DisplayInCharacterRecord=true
    DynamicDurationBonus=1
    DynamicDamageBonus=1
    MaxRank=4
    ProjectileClass=class'SFXProjectile_Power'
    RankCosts(0)=1
    RankCosts(1)=2
    RankCosts(2)=3
    RankCosts(3)=4
}
