﻿class SFXPowerLevelUpHelper extends Object;

var transient int m_nPointsSpent;

var transient array<NewPowerInfo> m_UpdatedPowers;

var transient array<SFXPower> m_powers;

var transient BioPawn m_Pawn;

var transient BioWorldInfo m_WorldInfo;

struct NewPowerInfo
{
    var class<Object> PowerClass;

    var float CurrentRank;

    var int WheelDisplayIndex;

}

function bool Initialize(BioWorldInfo WorldInfo)
{
    if (WorldInfo == None)
    {
        return false;
    }
    m_WorldInfo = WorldInfo;
    return true;
}

function bool SetPawn(BioPawn Pawn)
{
    local int nIndex;
    if (m_WorldInfo == None)
    {
        return false;
    }
    if (Pawn == None || Pawn.PowerManager == None)
    {
        return false;
    }
    m_Pawn = Pawn;
    m_powers.Length = 0;
    m_Pawn.PowerManager.GetCharacterRecordPowers(m_powers);
    m_UpdatedPowers.Length = 0;
    nIndex = 0;
    if (nIndex < m_powers.Length)
    {
        // unknown bytecode 54 ;
    }
}

function AutoLevelUp()
{
    local SFXLoadoutData Loadout;
    local int nLevelUpIndex;
    local int nIndex;
    if (m_Pawn == None || m_Pawn.PawnType == None)
    {
        return;
    }
    Loadout = m_Pawn.GetPawnLoadout();
    if (Loadout == None)
    {
        return;
    }
    nLevelUpIndex = 0;
    while (nLevelUpIndex < Loadout.PowerLevelUpInfo.Length)
    {
        nIndex = 0;
        while (nIndex < m_UpdatedPowers.Length)
        {
            if (m_UpdatedPowers[nIndex].PowerClass == Loadout.PowerLevelUpInfo[nLevelUpIndex].PowerClass)
            {
                break;
            }
            nIndex++;
        }
        if (nIndex == m_UpdatedPowers.Length)
        {
        }
        else
        {
            while (m_UpdatedPowers[nIndex].CurrentRank < Loadout.PowerLevelUpInfo[nLevelUpIndex].Rank)
            {
                if (IncreaseRank(nIndex) == false)
                {
                    break;
                }
            }
            if (CanEvolvePower(nIndex))
            {
                EvolvePower(m_powers[nIndex], Loadout.PowerLevelUpInfo[nLevelUpIndex].EvolvedPowerClass);
            }
            if (GetRemainingPoints() == 0)
            {
                break;
            }
        }
        nLevelUpIndex++;
    }
    Class'BioTelemetry'.SendEventName('AutoLevelUp', m_Pawn.Tag);
}

function bool IncreaseRank(int nPowerIndex)
{
    local int nCostToIncrease;
    local int nRankIndex;
    local String sUnlockBlurb;
    if (nPowerIndex < 0 || nPowerIndex >= m_UpdatedPowers.Length)
    {
        return false;
    }
    if (CanIncreaseRank(nPowerIndex, nCostToIncrease) == false)
    {
        return false;
    }
    m_UpdatedPowers[nPowerIndex].CurrentRank += 1;
    if (m_UpdatedPowers[nPowerIndex].CurrentRank == 1)
    {
        m_UpdatedPowers[nPowerIndex].WheelDisplayIndex = GetNextWheelDisplayIndex();
    }
    m_nPointsSpent += nCostToIncrease;
    nRankIndex = m_UpdatedPowers[nPowerIndex].CurrentRank - 1;
    if (nRankIndex < m_powers[nPowerIndex].Ranks.Length && m_powers[nPowerIndex].Ranks[nRankIndex].Icon > 0)
    {
        if (CanEvolvePower(nPowerIndex) == false)
        {
            if (m_WorldInfo != None && m_WorldInfo.EventNotifier != None)
            {
                m_powers[nPowerIndex].GetUnlockBlurb(nRankIndex, sUnlockBlurb);
                m_WorldInfo.EventNotifier.AddTalentNotifyString(m_powers[nPowerIndex].Ranks[nRankIndex].Icon, sUnlockBlurb, m_Pawn);
            }
        }
    }
    return true;
}

function bool CanIncreaseRank(int nPowerIndex, out int nCostToIncrease)
{
    local SFXPower Power;
    local int nNextRank;
    local int nRemainingPoints;
    if (nPowerIndex < 0 || nPowerIndex >= m_UpdatedPowers.Length)
    {
        return false;
    }
    Power = m_powers[nPowerIndex];
    if (Power == None)
    {
        return false;
    }
    if (GetAvailableRank(nPowerIndex) <= m_UpdatedPowers[nPowerIndex].CurrentRank)
    {
        return false;
    }
    nNextRank = m_UpdatedPowers[nPowerIndex].CurrentRank + 1;
    if (nNextRank - 1 >= Power.RankCosts.Length)
    {
        return false;
    }
    nCostToIncrease = Power.RankCosts[nNextRank - 1];
    nRemainingPoints = GetRemainingPoints();
    if (nRemainingPoints < nCostToIncrease)
    {
        return false;
    }
    return true;
}

function int GetRemainingPoints()
{
    if (m_Pawn == None)
    {
        return 0;
    }
    return m_Pawn.TalentPoints - m_nPointsSpent;
}

function int GetAvailableRank(int nPowerIndex)
{
    local SFXPower Power;
    if (nPowerIndex < 0 || nPowerIndex >= m_powers.Length)
    {
        return 0;
    }
    Power = m_powers[nPowerIndex];
    if (Power == None)
    {
        return 0;
    }
    if (IsUnlocked(nPowerIndex) == false)
    {
        return 0;
    }
    return Power.MaxRank;
}

function bool IsUnlocked(int nPowerIndex)
{
    local SFXPower Power;
    local int nRequiredIndex;
    local int nIndex;
    local class<Object> RequiredPowerClass;
    local bool bFound;
    if (nPowerIndex < 0 || nPowerIndex >= m_powers.Length)
    {
        return false;
    }
    Power = m_powers[nPowerIndex];
    if (Power == None)
    {
        return false;
    }
    if (m_Pawn == None || m_Pawn.PowerManager == None)
    {
        return false;
    }
    if (Power.UnlockRequirements.Length == 0)
    {
        return true;
    }
    nRequiredIndex = 0;
    while (nRequiredIndex < Power.UnlockRequirements.Length)
    {
        RequiredPowerClass = Power.UnlockRequirements[nRequiredIndex].PowerClass;
        bFound = false;
        nIndex = 0;
        while (nIndex < m_UpdatedPowers.Length && bFound == false)
        {
            if (m_UpdatedPowers[nIndex].PowerClass == RequiredPowerClass)
            {
                if (m_UpdatedPowers[nIndex].CurrentRank >= Power.UnlockRequirements[nRequiredIndex].Rank)
                {
                    return true;
                }
                else
                {
                    bFound = true;
                }
            }
            nIndex++;
        }
        if (bFound == false)
        {
            nIndex = 0;
            while (nIndex < m_Pawn.PowerManager.Powers.Length && bFound == false)
            {
                if (m_Pawn.PowerManager.Powers[nIndex].Class == RequiredPowerClass)
                {
                    if (m_Pawn.PowerManager.Powers[nIndex].Rank >= Power.UnlockRequirements[nRequiredIndex].Rank)
                    {
                        return true;
                    }
                    else
                    {
                        bFound = true;
                    }
                }
                nIndex++;
            }
        }
        nRequiredIndex++;
    }
    return false;
}

function bool CanEvolvePower(int nPowerIndex)
{
    if (nPowerIndex < 0 || nPowerIndex >= m_powers.Length)
    {
        return false;
    }
    if (m_Pawn == None || m_Pawn.PowerManager == None)
    {
        return false;
    }
    if (m_UpdatedPowers[nPowerIndex].CurrentRank < m_Pawn.PowerManager.EvolveRank)
    {
        return false;
    }
    if (m_powers[nPowerIndex].EvolvedPowerClass1 == None && m_powers[nPowerIndex].EvolvedPowerClass2 == None)
    {
        return false;
    }
    return true;
}

function ApplyChanges()
{
    local int nIndex;
    local BioPowerScript PowerScript;
    if (m_Pawn == None)
    {
        return;
    }
    m_Pawn.RemoveTalentPoints(m_nPointsSpent);
    m_nPointsSpent = 0;
    nIndex = 0;
    while (nIndex < m_powers.Length)
    {
        if (m_powers[nIndex].Rank != m_UpdatedPowers[nIndex].CurrentRank)
        {
            m_powers[nIndex].Rank = m_UpdatedPowers[nIndex].CurrentRank;
            m_powers[nIndex].WheelDisplayIndex = m_UpdatedPowers[nIndex].WheelDisplayIndex;
            PowerScript = m_powers[nIndex].GetPowerScript();
            if (PowerScript != None)
            {
                PowerScript.OnPowerRankIncreased();
            }
        }
        nIndex++;
    }
}

function CancelChanges()
{
    local int nIndex;
    m_nPointsSpent = 0;
    nIndex = 0;
    while (nIndex < m_powers.Length)
    {
        m_UpdatedPowers[nIndex].CurrentRank = m_powers[nIndex].Rank;
        nIndex++;
    }
}

function EvolvePower(SFXPower OriginalPower, class<Object> EvolvedPowerClass)
{
    local SFXPower EvolvedPower;
    local SFXPower PowerInList;
    local int nIndex;
    local BioPlayerController PlayerController;
    local BioPlayerInput PlayerInput;
    local String sUnlockBlurb;
    local int nNewPowerIndex;
    local int nOldPowerIndex;
    if (OriginalPower == None)
    {
        return;
    }
    if (m_Pawn == None || m_Pawn.PowerManager == None)
    {
        return;
    }
    if (EvolvedPowerClass != OriginalPower.EvolvedPowerClass1 && EvolvedPowerClass != OriginalPower.EvolvedPowerClass2)
    {
        return;
    }
    ApplyChanges();
    EvolvedPower = m_Pawn.PowerManager.AddPower(EvolvedPowerClass);
    if (EvolvedPower == None)
    {
        return;
    }
    EvolvedPower.Rank = 4;
    OriginalPower.Rank = 0;
    EvolvedPower.WheelDisplayIndex = OriginalPower.WheelDisplayIndex;
    nIndex = 0;
    while (nIndex < m_powers.Length)
    {
        PowerInList = m_powers[nIndex];
        if (PowerInList != None && PowerInList.Class == OriginalPower.Class)
        {
            m_powers[nIndex] = EvolvedPower;
            m_UpdatedPowers[nIndex].PowerClass = EvolvedPower.Class;
            m_UpdatedPowers[nIndex].CurrentRank = EvolvedPower.Rank;
            break;
        }
        nIndex++;
    }
    if (EvolvedPower.Ranks.Length >= 4)
    {
        if (m_WorldInfo != None && m_WorldInfo.EventNotifier != None)
        {
            EvolvedPower.GetUnlockBlurb(3, sUnlockBlurb);
            m_WorldInfo.EventNotifier.AddTalentNotifyString(EvolvedPower.Ranks[3].Icon, sUnlockBlurb, m_Pawn);
        }
    }
    PlayerController = BioPlayerController(m_Pawn.Controller);
    if (PlayerController != None)
    {
        PlayerController.UnlockAchievement(49);
        PlayerInput = BioPlayerInput(PlayerController.PlayerInput);
        if (PlayerInput != None)
        {
            if (PlayerInput.m_nmMappedPower == OriginalPower.Class.Name)
            {
                PlayerInput.m_nmMappedPower = EvolvedPower.Class.Name;
            }
            if (PlayerInput.m_nmMappedPower2 == OriginalPower.Class.Name)
            {
                PlayerInput.m_nmMappedPower2 = EvolvedPower.Class.Name;
            }
            if (PlayerInput.m_nmMappedPower3 == OriginalPower.Class.Name)
            {
                PlayerInput.m_nmMappedPower3 = EvolvedPower.Class.Name;
            }
        }
    }
    if (m_WorldInfo != None && m_WorldInfo.LocalPlayerController != None)
    {
        nNewPowerIndex = m_Pawn.PowerManager.Powers.Find(EvolvedPower);
        nOldPowerIndex = m_Pawn.PowerManager.Powers.Find(OriginalPower);
        if (nNewPowerIndex >= 0 && nOldPowerIndex >= 0)
        {
            nIndex = 0;
            while (nIndex < 8)
            {
                if (m_WorldInfo.LocalPlayerController.m_aHotKeyDefines[nIndex].nmPawn == m_Pawn.Tag && m_WorldInfo.LocalPlayerController.m_aHotKeyDefines[nIndex].nPowerID == nOldPowerIndex)
                {
                    m_WorldInfo.LocalPlayerController.m_aHotKeyDefines[nIndex].nPowerID = nNewPowerIndex;
                    break;
                }
                ++nIndex;
            }
        }
    }
}

function CancelEvolution(SFXPower OriginalPower)
{
    local int nIndex;
    if (OriginalPower == None)
    {
        return;
    }
    nIndex = 0;
    while (nIndex < m_UpdatedPowers.Length)
    {
        if (m_UpdatedPowers[nIndex].PowerClass == OriginalPower.Class)
        {
            m_nPointsSpent -= OriginalPower.RankCosts[m_UpdatedPowers[nIndex].CurrentRank - 1];
            m_UpdatedPowers[nIndex].CurrentRank -= 1;
        }
        nIndex++;
    }
}

function float GetCurrentRank(int nPowerIndex)
{
    return m_UpdatedPowers[nPowerIndex].CurrentRank;
}

function float GetCurrentRankByPower(SFXPower Power)
{
    local int nPowerIndex;
    nPowerIndex = 0;
    while (nPowerIndex < m_UpdatedPowers.Length)
    {
        if (m_powers[nPowerIndex] == Power)
        {
            return m_UpdatedPowers[nPowerIndex].CurrentRank;
        }
        ++nPowerIndex;
    }
    return -1;
}

function int GetNextWheelDisplayIndex()
{
    local array<SFXPower> aWheelPowers;
    m_Pawn.PowerManager.GetPowerWheelPowers(aWheelPowers);
    return aWheelPowers.Length;
}

function bool CanMakePurchase()
{
    local int nIndex;
    local int nCostToIncrease;
    local bool bCanMakePurchase;
    bCanMakePurchase = false;
    nIndex = 0;
    while (nIndex < m_powers.Length)
    {
        bCanMakePurchase = CanIncreaseRank(nIndex, nCostToIncrease);
        if (bCanMakePurchase)
        {
            break;
        }
        nIndex++;
    }
    return bCanMakePurchase;
}

defaultproperties
{
}
