﻿class SFXPowerManager extends Object
    config(Game)
    native;

struct native PowerSaveInfo
{
    var Name PowerName;

    var float CurrentRank;

    var String PowerClassName;

    var int WheelDisplayIndex;

}

var transient float m_fSharedCooldownTotal;

var transient float m_fSharedCooldownRemaining;

var transient config int EvolveRank;

var transient config int UnlockedRankIcon;

var transient config int LockedRankIcon;

var transient config int PassiveRankIcon;

// RequiredUnlockText: StringRefProperty[44]
// RankUnlockText: StringRefProperty[44]
var transient array<SFXPower> Powers;

var transient BioPawn MyPawn;

function SFXPower AddPower(class<Object> PowerClass)
{
    local SFXPower Power;
    local SFXPower PowerInList;
    local int nIndex;
    if (PowerClass == None)
    {
        return None;
    }
    nIndex = 0;
    while (nIndex < Powers.Length)
    {
        PowerInList = Powers[nIndex];
        if (PowerInList != None && PowerInList.Class == PowerClass)
        {
            return None;
        }
        nIndex++;
    }
    Power = SFXPower(new(MyPawn,PowerClass));
    if (Power == None)
    {
        return None;
    }
    Power.MyPawn = MyPawn;
    Powers.AddItem(Power);
    if (MyPawn != None && MyPawn.m_oBehavior != None)
    {
        Power.OldPower = new(MyPawn,Class'BioPower');
        if (Power.OldPower != None)
        {
            Power.OldPower.Owner = MyPawn;
            Power.OldPower.NewPower = Power;
            Power.OldPower.InitializePower(Power.PowerName);
            MyPawn.m_oBehavior.m_powers.AddItem(Power.OldPower);
        }
    }
    nIndex = 0;
    while (nIndex < Powers.Length)
    {
        PowerInList = Powers[nIndex];
        if (PowerInList != None && PowerInList.OldPower != None && PowerInList.OldPower.Script != None)
        {
            PowerInList.OldPower.Script.OnPowerAdded(Power);
        }
        nIndex++;
    }
    return Power;
}

function SFXPower AddPowerByClassName(String PowerClassName)
{
    local class<SFXPower> PowerClass;
    PowerClass = Class<SFXPower>(FindObject(PowerClassName, Class'Class'));
    if (PowerClass == None)
    {
        if (Class'SFXEngine'.IsSeekFreeObjectSupported(PowerClassName))
        {
            PowerClass = Class<SFXPower>(Class'SFXEngine'.LoadSeekFreeObject(PowerClassName, Class'Class'));
        }
    }
    if (PowerClass == None)
    {
        return None;
    }
    return AddPower(PowerClass);
}

function bool RemovePower(class<Object> PowerClass)
{
    local SFXPower Power;
    if (PowerClass == None)
    {
        return false;
    }
    Power = GetPowerByClass(PowerClass);
    if (Power == None)
    {
        return false;
    }
    if (MyPawn != None && MyPawn.m_oBehavior != None)
    {
        if (Power.OldPower != None)
        {
            if (Power.OldPower.IsActive())
            {
                return false;
            }
            MyPawn.m_oBehavior.m_powers.RemoveItem(Power.OldPower);
        }
    }
    Powers.RemoveItem(Power);
    return true;
}

native function SFXPower GetPower(Name nmPowerName);

native function SFXPower GetPowerByClass(class<Object> PowerClass);

function GetCharacterRecordPowers(out array<SFXPower> PowerList)
{
    local int nIndex;
    local SFXPower EvolvedPower;
    local array<SFXPower> EvolvedPowers;
    nIndex = 0;
    while (nIndex < Powers.Length)
    {
        if (Powers[nIndex].DisplayInCharacterRecord)
        {
            if (EvolvedPowers.Find(Powers[nIndex]) == -1)
            {
                if (HasPowerBeenEvolved(Powers[nIndex], EvolvedPower))
                {
                    PowerList[PowerList.Length] = EvolvedPower;
                    EvolvedPowers[EvolvedPowers.Length] = EvolvedPower;
                }
                else
                {
                    PowerList[PowerList.Length] = Powers[nIndex];
                }
            }
        }
        nIndex++;
    }
}

function bool HasPowerBeenEvolved(SFXPower Power, out SFXPower EvolvedPower)
{
    local int nIndex;
    if (Power == None)
    {
        return false;
    }
    if (Power.EvolvedPowerClass1 == None && Power.EvolvedPowerClass2 == None)
    {
        return false;
    }
    nIndex = 0;
    while (nIndex < Powers.Length)
    {
        if (Powers[nIndex].Class == Power.EvolvedPowerClass1 || Powers[nIndex].Class == Power.EvolvedPowerClass2)
        {
            if (Powers[nIndex].Rank > 0)
            {
                EvolvedPower = Powers[nIndex];
                return true;
            }
        }
        nIndex++;
    }
    return false;
}

native function float GetSharedCooldownRemaining();

native function SetSharedCooldownRemaining(float fCooldown);

native function float GetSharedCooldownTotal();

native function SetSharedCooldownTotal(float fCooldown);

native function Tick(float fDeltaTime);

function SavePowers(out array<PowerSaveInfo> PowerList)
{
    local int nIndex;
    nIndex = 0;
    if (nIndex < Powers.Length)
    {
        // unknown bytecode 54 ;
    }
}

function LoadPowers(out array<PowerSaveInfo> PowerList)
{
    local int nIndex;
    local SFXPower Power;
    local class<SFXPower> PowerClass;
    local class<Object> EvolvedPowerClass;
    local BioPowerScript PowerScript;
    nIndex = 0;
    while (nIndex < PowerList.Length)
    {
        Power = None;
        PowerClass = None;
        if (Len(PowerList[nIndex].PowerClassName) > 0)
        {
            PowerClass = Class<SFXPower>(FindObject(PowerList[nIndex].PowerClassName, Class'Class'));
            if (PowerClass != None)
            {
                Power = GetPowerByClass(PowerClass);
            }
        }
        else
        {
            Power = GetPower(PowerList[nIndex].PowerName);
        }
        if (Power == None)
        {
            if (IsEvolvedPower(PowerList[nIndex].PowerClassName, EvolvedPowerClass))
            {
                Power = AddPower(EvolvedPowerClass);
            }
            else
            {
                if (SFXPawn_Player(MyPawn) != None)
                {
                    if (PowerClass == None)
                    {
                        if (Class'SFXEngine'.IsSeekFreeObjectSupported(PowerList[nIndex].PowerClassName))
                        {
                            PowerClass = Class<SFXPower>(Class'SFXEngine'.LoadSeekFreeObject(PowerList[nIndex].PowerClassName, Class'Class'));
                        }
                    }
                    if (PowerClass != None && PowerClass.Default.IsHenchmenUnique)
                    {
                        Power = AddPower(PowerClass);
                    }
                }
            }
        }
        if (Power != None)
        {
            Power.Rank = PowerList[nIndex].CurrentRank;
            Power.WheelDisplayIndex = PowerList[nIndex].WheelDisplayIndex;
        }
        nIndex++;
    }
    if (MyPawn != None)
    {
        MyPawn.OnPowersLoaded();
    }
    foreach Powers(Power)
    {
        PowerScript = Power.GetPowerScript();
        if (PowerScript != None)
        {
            PowerScript.OnPowersLoaded();
        }
    }
}

function bool IsEvolvedPower(Name nmClassName, out class<Object> PowerClass)
{
    local int nIndex;
    local class<SFXPower> EvolvedPowerClass;
    local array<String> PathArray;
    local Name nmShortClassName;
    ParseStringIntoArray(nmClassName, PathArray, ".", true);
    nmShortClassName = PathArray[PathArray.Length - 1];
    nIndex = 0;
    while (nIndex < Powers.Length)
    {
        EvolvedPowerClass = Class<SFXPower>(Powers[nIndex].EvolvedPowerClass1);
        if (EvolvedPowerClass != None && EvolvedPowerClass.Name == nmShortClassName)
        {
            PowerClass = Powers[nIndex].EvolvedPowerClass1;
            return true;
        }
        EvolvedPowerClass = Class<SFXPower>(Powers[nIndex].EvolvedPowerClass2);
        if (EvolvedPowerClass != None && EvolvedPowerClass.Name == nmShortClassName)
        {
            PowerClass = Powers[nIndex].EvolvedPowerClass2;
            return true;
        }
        nIndex++;
    }
    return false;
}

event OnSharedCooldownFinished()
{
    local SFXAI_Core AIController;
    AIController = MyPawn != None ? SFXAI_Core(MyPawn.Controller) : None;
    if (AIController != None)
    {
        AIController.NotifySharedCooldownFinished();
    }
}

function StartFirstTimeDelay()
{
    local SFXPower Power;
    local int nIndex;
    nIndex = 0;
    while (nIndex < Powers.Length)
    {
        Power = Powers[nIndex];
        if (Power != None && Power.DelayBeforeFirstUse > 0)
        {
            Power.TimeUntilNextUse = Power.DelayBeforeFirstUse;
        }
        nIndex++;
    }
}

event GetPowerWheelPowers(out array<SFXPower> PowerList)
{
    local int nIndex;
    local SFXPower EvolvedPower;
    local array<SFXPower> EvolvedPowers;
    ProcessWheelDisplayOrder();
    nIndex = 0;
    while (nIndex < Powers.Length)
    {
        if (Powers[nIndex].IsEnabled() != false && Powers[nIndex].DisplayInHUD != false)
        {
            if (EvolvedPowers.Find(Powers[nIndex]) == -1)
            {
                if (HasPowerBeenEvolved(Powers[nIndex], EvolvedPower))
                {
                    PowerList[PowerList.Length] = EvolvedPower;
                    EvolvedPowers[EvolvedPowers.Length] = EvolvedPower;
                }
                else
                {
                    PowerList[PowerList.Length] = Powers[nIndex];
                }
            }
        }
        nIndex++;
    }
}

function ProcessWheelDisplayOrder()
{
    local int nPower;
    local int nSortedIndex;
    local int nHighIndex;
    local bool bAddToEnd;
    local array<int> aSortedIndices;
    nHighIndex = -1;
    nPower = 0;
    while (nPower < Powers.Length)
    {
        if (Powers[nPower].DisplayInHUD != false && Powers[nPower].IsEnabled() != false)
        {
            bAddToEnd = true;
            if (Powers[nPower].WheelDisplayIndex < nHighIndex)
            {
                nSortedIndex = 0;
                while (nSortedIndex < aSortedIndices.Length)
                {
                    if (Powers[aSortedIndices[nSortedIndex]].WheelDisplayIndex >= Powers[nPower].WheelDisplayIndex)
                    {
                        aSortedIndices.InsertItem(nSortedIndex, nPower);
                        bAddToEnd = false;
                        break;
                    }
                    ++nSortedIndex;
                }
            }
            if (bAddToEnd != false)
            {
                aSortedIndices.AddItem(nPower);
                nHighIndex = Powers[nPower].WheelDisplayIndex;
            }
        }
        ++nPower;
    }
    nHighIndex = 0;
    while (nHighIndex < aSortedIndices.Length)
    {
        Powers[aSortedIndices[nHighIndex]].WheelDisplayIndex = nHighIndex;
        ++nHighIndex;
    }
}

function RefundAllTalentPoints()
{
    local int nIndex;
    local int nRankIndex;
    local SFXPower Power;
    local int TalentRefund;
    local class<Object> EvolvedPowerClass;
    nIndex = Powers.Length - 1;
    while (nIndex >= 0)
    {
        Power = Powers[nIndex];
        if (Power != None && Power.DisplayInCharacterRecord)
        {
            nRankIndex = 0;
            while (nRankIndex < Power.Rank)
            {
                TalentRefund += Power.RankCosts[nRankIndex];
                nRankIndex++;
            }
            Power.Rank = 0;
            if (IsEvolvedPower(Power.Class.Name, EvolvedPowerClass))
            {
                RemovePower(Power.Class);
            }
        }
        nIndex--;
    }
    if (MyPawn != None)
    {
        MyPawn.AddTalentPoints(TalentRefund);
    }
}

static function int GetRefundAmount(SFXPower Power)
{
    return GetRefundAmount2(Power.Class, Power.Rank);
}

static function int GetRefundAmount2(class<SFXPower> Power, int Rank)
{
    local int Refund;
    local int Index;
    Refund = 0;
    if (Power != None)
    {
        Index = 0;
        while (Index < Rank)
        {
            Refund += Power.Default.RankCosts[Index];
            Index++;
        }
    }
    return Refund;
}

defaultproperties
{
    RankUnlockText=System.Byte[]
    RequiredUnlockText=System.Byte[]
    LockedRankIcon=28
    UnlockedRankIcon=29
    EvolveRank=4
}
