﻿class SFXProjectile_Explosive extends SFXProjectile
    hidecategories(Navigation);

var const float ExplosionEffectRadius;

var const float ExplosionParticleLifetime;

var const float CustomGravityScaling;

var const float TerminalVelocity;

var const float TossZ;

var const float AccelRate;

var const float MaxEffectDistance;

var const Name DecalDissolveParamName;

var const float DurationOfDecal;

var const float DecalHeight;

var const float DecalWidth;

var const MaterialInterface ExplosionDecal;

var const ParticleSystem ProjExplosionTemplate;

var const float fFuseLength;

var const WwiseEvent WwiseDuckEvent;

var const float DuckDistanceThreshold;

var bool bSuppressExplosionFX;

var const bool bAdvanceExplosionEffect;

var const bool bDuckAudio;

var const WwiseEvent ExplosionSound;

simulated event Landed(Vector HitNormal, Actor FloorActor)
{
    HitWall(HitNormal, FloorActor, None);
}

simulated function bool CanSplash()
{
    return false;
}

simulated event SetInitialState()
{
    bScriptInitialized = true;
    if (Role < 3 && AccelRate != 0)
    {
        GotoState('WaitingForVelocity');
    }
    else
    {
        GotoState(InitialState != 'None' ? InitialState : 'Auto');
    }
}

simulated function Recycle()
{
    Recycle();
    bSuppressExplosionFX = Default.bSuppressExplosionFX;
}

function Init(Vector Direction)
{
    Init(Direction);
    Velocity = Speed * Direction;
    Velocity.Z += TossZ;
    Acceleration = AccelRate * Normal(Velocity);
    SetTimer(fFuseLength + FRand() * 0,5, false);
}

simulated function bool CheckMaxEffectDistance(PlayerController P, Vector SpawnLocation, optional float CullDistance)
{
    local float Dist;
    if (P.ViewTarget == None)
    {
        return true;
    }
    if (P.Rotation Dot SpawnLocation - P.ViewTarget.Location < 0)
    {
        return VSize(P.ViewTarget.Location - SpawnLocation) < ExplosionEffectRadius;
    }
    Dist = VSize(SpawnLocation - P.ViewTarget.Location);
    if (CullDistance > 0 && CullDistance < Dist * P.LODDistanceFactor)
    {
        return false;
    }
    return true;
}

simulated function bool EffectIsRelevant(Vector SpawnLocation, bool bForceDedicated, optional float CullDistance)
{
    local PlayerController PC;
    if (WorldInfo.NetMode != 1 && SpawnLocation == Location)
    {
        foreach LocalPlayerControllers(Class'PlayerController', PC)
        {
            if (PC.ViewTarget != None && VSize(PC.ViewTarget.Location - Location) < 256)
            {
                return true;
            }
        }
    }
    return EffectIsRelevant(SpawnLocation, bForceDedicated, CullDistance);
}

simulated function SetExplosionEffectParameters(ParticleSystemComponent ProjExplosion);

simulated function SpawnExplosionEffects(Vector HitLocation, Vector HitNormal)
{
    local Rotator EffectRotation;
    local SFXEmitter ProjExplosionEmitter;
    local MaterialInstanceTimeVarying MITV_Decal;
    local Vector ListenerPosition;
    if (WorldInfo.NetMode != 1)
    {
        if (ProjExplosionTemplate != None && EffectIsRelevant(Location, false, MaxEffectDistance))
        {
            EffectRotation = HitNormal + rot(-16384,0,0);
            ProjExplosionEmitter = SFXEmitter(SFXGame(WorldInfo.Game).ObjectPool.GetImpactEmitter(ProjExplosionTemplate, Location, EffectRotation));
            if (ProjExplosionEmitter != None)
            {
                ProjExplosionEmitter.SetLifeTime(ExplosionParticleLifetime);
                ProjExplosionEmitter.ParticleSystemComponent.SetActive(true);
            }
            if (ExplosionDecal != None && Pawn(ImpactedActor) == None)
            {
                if (MaterialInstanceTimeVarying(ExplosionDecal) != None)
                {
                    MITV_Decal = new(self,Class'MaterialInstanceTimeVarying');
                    MITV_Decal.SetParent(ExplosionDecal);
                    WorldInfo.MyDecalManager.SpawnDecal(MITV_Decal, HitLocation, -HitNormal, DecalWidth, DecalHeight, 10, false);
                    MITV_Decal.SetScalarStartTime(DecalDissolveParamName, DurationOfDecal);
                }
                else
                {
                    WorldInfo.MyDecalManager.SpawnDecal(ExplosionDecal, HitLocation, -HitNormal, DecalWidth, DecalHeight, 10, true);
                }
            }
        }
        if (ExplosionSound != None && !bSuppressAudio)
        {
            SFXGame(WorldInfo.Game).PlayTransientSound(ExplosionSound, HitLocation);
        }
        if (bDuckAudio)
        {
            ListenerPosition = Class'WwiseAudioComponent'.GetMicPosition();
            if (VSize(ListenerPosition - HitLocation) < DuckDistanceThreshold)
            {
                PlaySound(WwiseDuckEvent);
            }
        }
        bSuppressExplosionFX = true;
    }
}

state WaitingForVelocity
{
    simulated function Tick(float DeltaTime)
    {
        if (!IsZero(Velocity))
        {
            Acceleration = AccelRate * Normal(Velocity);
            GotoState(InitialState != 'None' ? InitialState : 'Auto');
        }
    }

}

simulated function Timer()
{
    Explode(Location, vect(0,0,1));
    ShutDown();
}

simulated event HitWall(Vector HitNormal, Actor Wall, PrimitiveComponent WallComp)
{
    Explode(Location, HitNormal);
    ShutDown();
}

simulated function Explode(Vector HitLocation, Vector HitNormal)
{
    if (GetDamage() > 0 || MomentumTransfer > 0 && GetDamageRadius() > 0)
    {
        if (Role == 3)
        {
            MakeNoise(1);
        }
        ProjectileHurtRadius(GetDamage(), GetDamageRadius(), MomentumTransfer, HitLocation, HitNormal);
    }
    if (WorldInfo.NetMode != 1 && !bSuppressExplosionFX)
    {
        SpawnExplosionEffects(Location, HitNormal);
    }
    Explode(HitLocation, HitNormal);
}

simulated function ProcessTouch(Actor Other, Vector HitLocation, Vector HitNormal)
{
    if (SFXPawn_Player(Instigator) != None && SFXPawn_Henchman(Other) != None)
    {
        return;
    }
    if (Pawn(Other) == None && BioArtPlaceable(Other) == None && StaticMeshActor(Other) == None && BioPhysicsActor(Other) == None && DynamicSMActor(Other) == None)
    {
        return;
    }
    if (StaticMeshActor(Other) != None || BioPhysicsActor(Other) != None || DynamicSMActor(Other) != None)
    {
        if (VSizeSq(Location - Owner.Location) < 40000)
        {
            return;
        }
    }
    if (GetDamageRadius() > 0)
    {
        Explode(HitLocation, HitNormal);
    }
    else
    {
        Other.TakeDamage(Damage, InstigatorController, HitLocation, MomentumTransfer * Normal(Velocity), MyDamageType, self);
    }
    if (IsShuttingDown() == false)
    {
        ShutDown();
    }
}

function DoImpact(Actor InImpactedActor, Controller InInstigatorController, float BaseDamage, float InDamageRadius, float Momentum, Vector HurtOrigin, bool bFullDamage)
{
    local SFXWeapon Weapon;
    local class<DamageType> DamageTypeToUse;
    Weapon = SFXWeapon(Outer);
    if (Weapon == None)
    {
        Weapon = SFXWeapon(Owner);
    }
    DamageTypeToUse = Weapon.GetDamageType(0);
    InImpactedActor.TakeRadiusDamage(InInstigatorController, BaseDamage, InDamageRadius, DamageTypeToUse, Momentum, HurtOrigin, true, self);
}

simulated function bool ProjectileHurtRadius(float InDamageAmount, float InDamageRadius, float Momentum, Vector HurtOrigin, Vector HitNormal)
{
    local bool bCausedDamage;
    local bool bInitializedAltOrigin;
    local bool bFailedAltOrigin;
    local Actor Victim;
    local Vector AltOrigin;
    local Vector OutLocation;
    local Vector OutNormal;
    if (bHurtEntry)
    {
        return false;
    }
    bHurtEntry = true;
    bCausedDamage = false;
    if (ImpactedActor != None && ImpactedActor != self)
    {
        DoImpact(ImpactedActor, InstigatorController, InDamageAmount, InDamageRadius, Momentum, HurtOrigin, true);
        if (ImpactedActor != None)
        {
            bCausedDamage = ImpactedActor.bProjTarget;
        }
    }
    foreach CollidingActors(Class'Actor', Victim, InDamageRadius, HurtOrigin, true)
    {
        if (Victim.CollisionComponent != None && !Victim.bWorldGeometry && Victim != self && Victim != ImpactedActor && Victim.bProjTarget || NavigationPoint(Victim) == None)
        {
            if (Victim.Trace(OutLocation, OutNormal, HurtOrigin, Victim.CollisionComponent.Bounds.Origin, false, 1) != None)
            {
                if (!bInitializedAltOrigin)
                {
                    bInitializedAltOrigin = true;
                    AltOrigin = HurtOrigin + Class'BioPawn'.Default.MaxStepHeight * HitNormal;
                    if (Victim.Trace(OutLocation, OutNormal, HurtOrigin, AltOrigin, false, 1) != None)
                    {
                        if (Velocity == vect(0,0,0))
                        {
                            bFailedAltOrigin = true;
                        }
                        else
                        {
                            AltOrigin = HurtOrigin - Class'BioPawn'.Default.MaxStepHeight * Normal(Velocity);
                            bFailedAltOrigin = Victim.Trace(OutLocation, OutNormal, HurtOrigin, AltOrigin, false) != None;
                        }
                    }
                }
                if (bFailedAltOrigin || Victim.Trace(OutLocation, OutNormal, HurtOrigin, Victim.CollisionComponent.Bounds.Origin) != None)
                {
                    continue;
                }
            }
            DoImpact(Victim, InstigatorController, InDamageAmount, InDamageRadius, Momentum, HurtOrigin, false);
            bCausedDamage = bCausedDamage || Victim.bProjTarget;
        }
    }
    bHurtEntry = false;
    return bCausedDamage;
}

defaultproperties
{
    DuckDistanceThreshold=500
    WwiseDuckEvent=WwiseEvent'Set_Ducking_Medium'
    fFuseLength=5
    ProjExplosionTemplate=ParticleSystem'Generic_Grenade'
    ExplosionDecal=DecalMaterial'DECAL_Blast_Generic'
    DurationOfDecal=4
    DecalDissolveParamName=DissolveAmount
    MaxEffectDistance=10000
    TerminalVelocity=3500
    CustomGravityScaling=1
    ExplosionParticleLifetime=3
    ExplosionEffectRadius=1000
    MeshScaleTime=0,1
    ProjEffectsHeadTemplate=ParticleSystem'GrenadeLauncher_Projectile'
    Speed=8000
    MaxSpeed=8000
    bSwitchToZeroCollision=true
    bBlockedByInstigator=false
    MomentumTransfer=600
    CylinderComponent=None
    Components=None
    // child object CollisionCylinder of type CylinderComponent
}
