﻿class SFXProjectile_Power extends SFXProjectile_Explosive
    hidecategories(Navigation);

var transient array<Actor> ImpactedActors;

var transient Rotator CameraRotation;

var transient Vector CameraLocation;

var transient Vector TargetLocation;

var transient Actor TargetActor;

var transient Vector LastLocation;

var transient int ImpactedCount;

var transient int ReservationID;

var transient bool ShowPowerAiming;

var transient bool bUsePowerReservation;

var transient Vector WorldImpactNormal;

var transient Vector WorldImpactPoint;

var transient Actor Caster;

var transient BioPower Power;

var transient BioVisualEffect ProjectileVFX;

function float GetDamageRadius()
{
    return Power.GetImpactRadius();
}

simulated function Timer();

simulated event HitWall(Vector HitNormal, Actor Wall, PrimitiveComponent WallComp)
{
    if (VSizeSq(Location - Owner.Location) < 40000 && VSizeSq(Location - TargetLocation) > 40000)
    {
        SetLocation(Location + Velocity * 0,1);
        return;
    }
    HitWall(HitNormal, Wall, WallComp);
}

simulated function ProcessTouch(Actor Other, Vector HitLocation, Vector HitNormal)
{
    if (Caster != None && Other == Caster)
    {
        return;
    }
    if (SFXPawn_Player(Instigator) != None && SFXPawn_Henchman(Other) != None)
    {
        return;
    }
    if (Pawn(Other) == None && BioArtPlaceable(Other) == None && StaticMeshActor(Other) == None && BioPhysicsActor(Other) == None && DynamicSMActor(Other) == None)
    {
        return;
    }
    if (StaticMeshActor(Other) != None || BioPhysicsActor(Other) != None || DynamicSMActor(Other) != None)
    {
        if (VSizeSq(Location - Owner.Location) < 40000)
        {
            return;
        }
    }
    Explode(HitLocation, HitNormal);
    if (IsShuttingDown() == false)
    {
        ShutDown();
    }
}

function DoPowerImpact(Actor oActor, Controller InstigatedBy, int nImpactedCount)
{
    ImpactedActors.AddItem(oActor);
    if (Power.Script != None)
    {
        Power.Script.m_vProjectileVelocity = Velocity;
        Power.Script.m_vProjectileLocation = Location;
        if (Power.Script.OnImpact(InstigatedBy.Pawn, 1, oActor, nImpactedCount))
        {
            if (oActor.bWorldGeometry == false)
            {
                Power.DoImpactVFX(6, oActor, WorldImpactPoint, WorldImpactNormal, 0, 'None');
            }
            MakeNoise(Power.GetImpactNoise(), 'NoiseType_PowerImpact');
        }
    }
    GenerateOnCastAt(oActor);
}

simulated function bool ProjectileHurtRadius(float InDamageAmount, float InDamageRadius, float Momentum, Vector HurtOrigin, Vector HitNormal)
{
    local bool bInitializedAltOrigin;
    local bool bFailedAltOrigin;
    local Actor Victim;
    local Vector AltOrigin;
    local Vector OutLocation;
    local Vector OutNormal;
    local int nImpactedCount;
    local bool bCausedDamage;
    if (bHurtEntry)
    {
        return false;
    }
    bHurtEntry = true;
    if (ImpactedActor != None && ImpactedActor != self && ImpactedActors.Find(ImpactedActor) == -1)
    {
        DoPowerImpact(ImpactedActor, InstigatorController, nImpactedCount++);
        bCausedDamage = true;
    }
    foreach CollidingActors(Class'Actor', Victim, InDamageRadius, HurtOrigin)
    {
        if (Victim.CollisionComponent != None && Victim.bWorldGeometry == false && Victim != self && Victim != ImpactedActor && Victim.bProjTarget || NavigationPoint(Victim) == None && ImpactedActors.Find(Victim) == -1)
        {
            if (Victim.Trace(OutLocation, OutNormal, HurtOrigin, Victim.CollisionComponent.Bounds.Origin, false) != None)
            {
                if (!bInitializedAltOrigin)
                {
                    bInitializedAltOrigin = true;
                    AltOrigin = HurtOrigin + Class'BioPawn'.Default.MaxStepHeight * HitNormal;
                    if (Victim.Trace(OutLocation, OutNormal, HurtOrigin, AltOrigin, false) != None)
                    {
                        if (Velocity == vect(0,0,0))
                        {
                            bFailedAltOrigin = true;
                        }
                        else
                        {
                            AltOrigin = HurtOrigin - Class'BioPawn'.Default.MaxStepHeight * Normal(Velocity);
                            bFailedAltOrigin = Victim.Trace(OutLocation, OutNormal, HurtOrigin, AltOrigin, false) != None;
                        }
                    }
                }
                if (bFailedAltOrigin || Victim.Trace(OutLocation, OutNormal, HurtOrigin, Victim.CollisionComponent.Bounds.Origin) != None)
                {
                    continue;
                }
            }
            DoPowerImpact(Victim, InstigatorController, nImpactedCount++);
            bCausedDamage = true;
        }
    }
    bHurtEntry = false;
    return bCausedDamage;
}

simulated function Explode(Vector HitLocation, Vector HitNormal)
{
    local float fRadius;
    if (Owner.WorldInfo != None && BioWorldInfo(Owner.WorldInfo).m_oPowerManager != None)
    {
        if (IsZero(WorldImpactPoint))
        {
            WorldImpactPoint = HitLocation;
            WorldImpactNormal = HitNormal;
        }
        Power.DoImpactVFX(7, None, WorldImpactPoint, WorldImpactNormal, 0);
        fRadius = Power.GetImpactRadius();
        if (fRadius > 0)
        {
            Explode(HitLocation, HitNormal);
        }
        if (bUsePowerReservation)
        {
            BioWorldInfo(WorldInfo).m_oPowerManager.ClearReservation(ReservationID);
        }
    }
    if (ProjectileVFX != None)
    {
        ProjectileVFX.SetState(2);
    }
    if (Power != None && Power.Script != None)
    {
        Power.Script.OnProjectileExploded(Instigator, HitLocation, HitNormal, Velocity);
    }
}

event GenerateOnCastAt(Actor Target)
{
    if (BioPawn(Target) != None)
    {
        BioPawn(Target).OnCastAt(BioPawn(Owner), Power.NewPower);
    }
    else
    {
        if (BioArtPlaceable(Target) != None)
        {
            BioArtPlaceable(Target).OnCastAt(BioPawn(Owner), Power.NewPower);
        }
        else
        {
            if (SFXPointOfInterest(Target) != None)
            {
                SFXPointOfInterest(Target).OnCastAt(BioPawn(Owner), Power.NewPower);
            }
        }
    }
}

function bool InitializePowerProjectile(Actor oCaster, float fTravelSpeed, float fRadius, BioPower oPower)
{
    local float fIntensity;
    local Vector vRadius;
    local Vector vTravelSpeed;
    local BioWorldInfo TheWorldInfo;
    local BioPlayerController PlayerController;
    local BioCheatManager CheatManager;
    Caster = oCaster;
    Power = oPower;
    DamageRadius = fRadius;
    Speed = fTravelSpeed;
    MaxSpeed = fTravelSpeed;
    vRadius.X = fRadius;
    vRadius.Y = fRadius;
    vRadius.Z = fRadius;
    vTravelSpeed.X = fTravelSpeed;
    vTravelSpeed.Y = fTravelSpeed;
    vTravelSpeed.Z = fTravelSpeed;
    ProjectileVFX = Class'BioVisualEffect'.CreateVFXOnPosition(Power.GetVFXTemplate(5), Location, Rotation, LifeSpan);
    if (ProjectileVFX != None)
    {
        fIntensity = Power.GetVFXIntensity();
        ProjectileVFX.SetHardAttach(true);
        ProjectileVFX.SetBase(self);
        ProjectileVFX.SetVectorParameter('ProjectileRadius', vRadius);
        ProjectileVFX.SetVectorParameter('ProjectileSpeed', vTravelSpeed);
        ProjectileVFX.SetScalarParameter('Intensity', fIntensity);
        ProjectileVFX.SetSound(fIntensity - 1);
        ProjectileVFX.SetState(0, true);
    }
    TheWorldInfo = BioWorldInfo(Caster.WorldInfo);
    if (TheWorldInfo != None)
    {
        PlayerController = TheWorldInfo.GetLocalPlayerController();
        if (PlayerController != None)
        {
            CheatManager = BioCheatManager(PlayerController.CheatManager);
            if (CheatManager != None)
            {
                ShowPowerAiming = CheatManager.m_bShowPowerAiming;
            }
        }
    }
    Init(Rotation);
    return true;
}

simulated function Tick(float DeltaTime)
{
    local Vector Extent;
    Tick(DeltaTime);
    if (ShowPowerAiming && Location != LastLocation)
    {
        Extent.X = 2;
        Extent.Y = 2;
        Extent.Z = 2;
        DrawDebugBox(Location, Extent, 0, 0, 255, true);
        LastLocation = Location;
    }
}

simulated function Recycle()
{
    Recycle();
    ProjectileVFX = Default.ProjectileVFX;
    Power = Default.Power;
    Caster = Default.Caster;
    WorldImpactPoint = Default.WorldImpactPoint;
    WorldImpactNormal = Default.WorldImpactNormal;
    bUsePowerReservation = Default.bUsePowerReservation;
    ReservationID = Default.ReservationID;
    ImpactedCount = Default.ImpactedCount;
    ShowPowerAiming = Default.ShowPowerAiming;
    LastLocation = Default.LastLocation;
    TargetActor = Default.TargetActor;
    TargetLocation = Default.TargetLocation;
    CameraLocation = Default.CameraLocation;
    CameraRotation = Default.CameraRotation;
    ImpactedActors.Length = 0;
}

simulated function ShutDown()
{
    ImpactedActors.Length = 0;
    ShutDown();
}

defaultproperties
{
    ProjExplosionTemplate=None
    ProjEffectsHeadTemplate=None
    Speed=1300
    MaxSpeed=1300
    CylinderComponent=None
    Components=None
    // child object CollisionCylinder of type CylinderComponent
}
