﻿class SFXSFHandler_Load extends SFXSFHandler_Save
    config(UI)
    transient
    native;

const LoadOption_Initialize = 1;
const LoadOption_LoadGame = 3;
const LoadOption_DeleteGame = 4;
const LoadOption_QuitToMainMenu = 5;
const LoadOption_ChangeStorageDevice = 11;
const LoadOption_InitializeCareers = 6;
const LoadOption_SelectCareer = 7;
const LoadOption_DeleteCareer = 8;
struct native SaveGUICareerRecord
{
    var() String CareerName;

    var() String FirstName;

    var() EOriginType Origin;

    var() ENotorietyType Notoriety;

    var() String ClassName;

    var() SaveTimeStamp CreationDate;

    var() bool bActiveCareer;

    var() int DeviceID;

}

var() transient bool bFromMainMenu;

var() transient array<String> CorruptedSaveGames;

var() ELoadGuiMode LoadMode;

// CancelDeleteCorruptText: StringRefProperty[44]
// ConfirmDeleteCorruptText: StringRefProperty[44]
// CorruptSaveWarningText: StringRefProperty[44]
// CancelDeleteCareerText: StringRefProperty[44]
// ConfirmDeleteCareerText: StringRefProperty[44]
// DeleteCareerText: StringRefProperty[44]
// CharNameText: StringRefProperty[44]
// CancelLoadGameText: StringRefProperty[44]
// ConfirmLoadGameText: StringRefProperty[44]
// LoadGameLoseProgressText: StringRefProperty[44]
var() array<SaveGameRecord> LegacyRecords;

var() array<SaveGUICareerRecord> CareerList;

enum ELoadGuiMode
{
    LoadGuiMode_Default,
    LoadGuiMode_Legacy,
    LoadGuiMode_NGPlus,
};

native function InitializeLoadList(optional int CareerIdx);

native function InitializeCareers();

native function DeleteCareer(int CareerIdx);

function Callback_ConfirmLoadGame(bool bAPressed, int Context)
{
    local BioPlayerController PC;
    local SFXEngine Engine;
    if (bAPressed)
    {
        PC = BioPlayerController(oPanel.oParentManager.GetPlayerController());
        if (PC != None)
        {
            if (LoadMode == 2)
            {
                Engine = SFXEngine(PC.Player.Outer);
                if (Engine != None)
                {
                    if (SaveList[Context].Type == 3)
                    {
                        Engine.ImportME2Character(true, CareerNameOverride);
                    }
                    else
                    {
                        if (SaveList[Context].Type == 2)
                        {
                            Engine.ImportME2Character(true, CareerNameOverride);
                        }
                        else
                        {
                            if (SaveList[Context].Type == 1)
                            {
                                Engine.ImportME2Character(true, CareerNameOverride);
                            }
                            else
                            {
                                Engine.ImportME2Character(CareerNameOverride, SaveList[Context].SaveIndex);
                            }
                        }
                    }
                }
            }
            else
            {
                if (LoadMode == 1)
                {
                    if (SaveList[Context].Type == 4)
                    {
                        PC.ConsoleCommand("defer importcharacter " $ LegacyRecords[Context].m_filename $ " " $ LegacyRecords[Context].m_DeviceId $ " forcenewgame");
                    }
                }
                else
                {
                    if (SaveList[Context].Type == 3)
                    {
                        PC.LoadGame(true, false, false, CareerNameOverride, 0);
                    }
                    else
                    {
                        if (SaveList[Context].Type == 2)
                        {
                            PC.LoadGame(false, true, false, CareerNameOverride, 0);
                        }
                        else
                        {
                            if (SaveList[Context].Type == 1)
                            {
                                PC.LoadGame(false, false, true, CareerNameOverride, 0);
                            }
                            else
                            {
                                PC.LoadGame(false, false, false, CareerNameOverride, SaveList[Context].SaveIndex);
                            }
                        }
                    }
                }
            }
        }
    }
    bWaitingOnMsgBox = false;
}

function Callback_ConfirmDeleteCareer(bool bAPressed, int Context)
{
    local BioPlayerController PC;
    if (bAPressed)
    {
        PC = BioPlayerController(oPanel.oParentManager.GetPlayerController());
        if (PC != None)
        {
            DeleteCareer(Context);
            PC.SetTimer(0,1, false, 'InitializeCareers', self);
        }
    }
    bWaitingOnMsgBox = false;
}

function ResetGui()
{
    InitializeLoadList();
}

native function Callback_ConfirmDeleteCorruptSaves(bool bAPressed, int Context);

event CheckForCorruptSaves()
{
    local MassEffectGuiManager GuiMan;
    local BioMessageBoxOptionalParams Params;
    if (CorruptedSaveGames.Length > 0)
    {
        GuiMan = MassEffectGuiManager(oPanel.oParentManager);
        if (GuiMan == None)
        {
            return;
        }
        Params.srAText = ConfirmDeleteCorruptText;
        Params.srBText = CancelDeleteCorruptText;
        Params.bNoFade = true;
        GuiMan.QueueNamedMessageBox('LoadMessageBox', 3, CorruptSaveWarningText, Params, Callback_ConfirmDeleteCorruptSaves);
        bWaitingOnMsgBox = true;
    }
}

function HandleEvent(byte Command, const out array<String> lstArguments)
{
    local MassEffectGuiManager GuiMan;
    local BioPlayerController PC;
    local SFXEngine Engine;
    local array<ASParams> args;
    local BioMessageBoxOptionalParams Params;
    local int SaveIdx;
    local bool bStartInCareerSelection;
    if (bWaitingOnMsgBox)
    {
        return;
    }
    GuiMan = MassEffectGuiManager(oPanel.oParentManager);
    if (GuiMan == None)
    {
        return;
    }
    PC = BioPlayerController(GuiMan.Player);
    if (PC == None)
    {
        return;
    }
    Engine = SFXEngine(PC.Player.Outer);
    switch (Command)
    {
        case 1:
            args.Length = 7;
            args[0].nVar = 0;
            args[0].Type = 0;
            args[1].bVar = LoadMode == 1;
            args[1].Type = 3;
            args[2].bVar = LoadMode == 2;
            args[2].Type = 3;
            args[3].nVar = GuiMode;
            args[3].Type = 0;
            args[4].bVar = bFromMainMenu;
            args[4].Type = 3;
            bFromMainMenu = false;
            args[5].nVar = ScreenLayout;
            args[5].Type = 0;
            if (Engine != None && Engine.CurrentCareerName == "" && LoadMode == 0)
            {
                bStartInCareerSelection = true;
            }
            args[6].bVar = bStartInCareerSelection;
            args[6].Type = 3;
            oPanel.InvokeMethodArgs("loadSave.SetLoadSave", args);
            if (LoadMode != 2 && !bStartInCareerSelection)
            {
                InitializeLoadList();
            }
            break;
        case 3:
            SaveIdx = lstArguments[0];
            if (SaveIdx >= 0 && SaveIdx < SaveList.Length)
            {
                if (GuiMode == 1 || GuiMode == 2)
                {
                    Callback_ConfirmLoadGame(true, SaveIdx);
                }
                else
                {
                    Params.srAText = ConfirmLoadGameText;
                    Params.srBText = CancelLoadGameText;
                    Params.bNoFade = true;
                    GuiMan.QueueNamedMessageBox('LoadMessageBox', 3, LoadGameLoseProgressText, Params, Callback_ConfirmLoadGame, SaveIdx);
                    bWaitingOnMsgBox = true;
                }
            }
            break;
        case 4:
            SaveIdx = lstArguments[0];
            if (SaveIdx >= 0 && SaveIdx < SaveList.Length)
            {
                Params.srAText = ConfirmDeleteGameText;
                Params.srBText = CancelDeleteGameText;
                Params.bNoFade = true;
                GuiMan.QueueNamedMessageBox('LoadMessageBox', 3, DeleteGameText, Params, Callback_ConfirmDelete, SaveIdx);
                bWaitingOnMsgBox = true;
            }
            break;
        case 6:
            InitializeCareers();
            break;
        case 7:
            SaveIdx = lstArguments[0];
            if (SaveIdx >= 0 && SaveIdx < CareerList.Length)
            {
                InitializeLoadList(SaveIdx);
            }
            break;
        case 8:
            SaveIdx = lstArguments[0];
            if (SaveIdx >= 0 && SaveIdx < CareerList.Length)
            {
                Params.srAText = ConfirmDeleteCareerText;
                Params.srBText = CancelDeleteCareerText;
                Params.bNoFade = true;
                GuiMan.QueueNamedMessageBox('LoadMessageBox', 3, DeleteCareerText, Params, Callback_ConfirmDeleteCareer, SaveIdx);
                bWaitingOnMsgBox = true;
            }
            break;
        case 5:
            if (GuiMode == 2)
            {
                GuiMan.HackReloadMainMenu();
            }
            else
            {
                GuiMan.ShowMainMenu(4);
                GuiMan.RemovePanel(oPanel, true);
            }
            break;
    }
}

event OnPanelAdded()
{
    local OnlineSubsystem OnlineSub;
    OnPanelAdded();
    OnlineSub = Class'GameEngine'.GetOnlineSubsystem();
    if (OnlineSub != None)
    {
        SystemInt = OnlineSub.SystemInterface;
        if (NotEqual_InterfaceInterface(SystemInt, OnlineSystemInterface(None)))
        {
            SystemInt.AddStorageDeviceChangeDelegate(OnStorageDeviceChanged);
        }
    }
}

event OnPanelRemoved()
{
    local OnlineSubsystem OnlineSub;
    OnlineSub = Class'GameEngine'.GetOnlineSubsystem();
    if (OnlineSub != None)
    {
        SystemInt = OnlineSub.SystemInterface;
        if (NotEqual_InterfaceInterface(SystemInt, OnlineSystemInterface(None)))
        {
            SystemInt.ClearStorageDeviceChangeDelegate(OnStorageDeviceChanged);
        }
    }
    OnPanelRemoved();
}

function OnStorageDeviceChanged()
{
    local MassEffectGuiManager GuiMan;
    GuiMan = MassEffectGuiManager(oPanel.oParentManager);
    if (GuiMan != None)
    {
        GuiMan.RemoveNamedMessageBox('LoadMessageBox');
        bWaitingOnMsgBox = false;
    }
    oPanel.InvokeMethod("loadSave.InitializeCharacterSelectionScreen");
}

function GameSessionEnded()
{
    local OnlineSubsystem OnlineSub;
    OnlineSub = Class'GameEngine'.GetOnlineSubsystem();
    if (OnlineSub != None)
    {
        SystemInt = OnlineSub.SystemInterface;
        if (NotEqual_InterfaceInterface(SystemInt, OnlineSystemInterface(None)))
        {
            SystemInt.ClearStorageDeviceChangeDelegate(OnStorageDeviceChanged);
        }
    }
    GameSessionEnded();
}

defaultproperties
{
    LoadGameLoseProgressText=System.Byte[]
    ConfirmLoadGameText=System.Byte[]
    CancelLoadGameText=System.Byte[]
    CharNameText=System.Byte[]
    DeleteCareerText=System.Byte[]
    ConfirmDeleteCareerText=System.Byte[]
    CancelDeleteCareerText=System.Byte[]
    CorruptSaveWarningText=System.Byte[]
    ConfirmDeleteCorruptText=System.Byte[]
    CancelDeleteCorruptText=System.Byte[]
}
