﻿class SFXSFHandler_Personalization extends BioSFHandler
    config(UI)
    transient
    native;

enum ESFXPersonalizationOption
{
    SFXPersOpt_Type,
    SFXPersOpt_Helmet,
    SFXPersOpt_Torso,
    SFXPersOpt_Shoulder,
    SFXPersOpt_Arm,
    SFXPersOpt_Leg,
    SFXPersOpt_Spec,
    SFXPersOpt_Tint1,
    SFXPersOpt_Tint2,
    SFXPersOpt_Pattern,
    SFXPersOpt_PatternColor,
    SFXPersOpt_Casual,
};

var int HelmetID;

var int PatternColorID;

var int PatternID;

var int Tint2ID;

var int Tint1ID;

var int LegID;

var int ArmID;

var int ShoulderID;

var int TorsoID;

var int FullBodyID;

var int CasualID;

var config int CasualOptionTitle;

var config int PatternColorOptionTitle;

var config int PatternOptionTitle;

var config int Tint2OptionTitle;

var config int Tint1OptionTitle;

var config int SpecOptionTitle;

var config int LegOptionTitle;

var config int ArmOptionTitle;

var config int ShoulderOptionTitle;

var config int TorsoOptionTitle;

var config int HelmetOptionTitle;

var config int TypeOptionTitle;

var bool m_bStopScroll;

var config float RotationDegreesPerSecond;

var int m_nRotating;

var transient SFXPawn_Player m_oPlayerPawn;

var transient SFXPawn_Player m_oUIWorldPawn;

var transient BioWorldInfo m_WorldInfo;

delegate ExternalCallback_OnComplete();

function SetExternalCallback_OnComplete(delegate<ExternalCallback_OnComplete> pDelegate)
{
    __ExternalCallback_OnComplete__Delegate = pDelegate;
}

function ClearDelegates()
{
    __ExternalCallback_OnComplete__Delegate = None;
}

event OnPanelAdded()
{
    OnPanelAdded();
    oPanel.SetExternalInterface(self);
    PlayGuiSound('PersonalizeOpen');
}

function OnPanelRemoved()
{
    PlayGuiSound('PersonalizeExit');
    OnPanelRemoved();
    if (m_WorldInfo != None)
    {
        if (m_oPlayerPawn != None)
        {
            if (m_WorldInfo.m_UIWorld != None)
            {
                m_WorldInfo.m_UIWorld.DestroyPawn(m_oPlayerPawn);
            }
        }
        if (m_WorldInfo.m_UIWorld != None)
        {
            m_WorldInfo.m_UIWorld.ResetActors();
            m_WorldInfo.m_UIWorld.TriggerEvent('LightsOff', m_WorldInfo);
        }
    }
    ClearDelegates();
}

function InitializeUIWorld()
{
    local BioPlayerController PC;
    if (oWorldInfo != None)
    {
        m_WorldInfo = BioWorldInfo(oWorldInfo);
        PC = m_WorldInfo.GetLocalPlayerController();
        if (PC != None)
        {
            m_oPlayerPawn = SFXPawn_Player(PC.Pawn);
        }
    }
    CasualID = m_oPlayerPawn.CasualID;
    FullBodyID = m_oPlayerPawn.FullBodyID;
    TorsoID = m_oPlayerPawn.TorsoID;
    ShoulderID = m_oPlayerPawn.ShoulderID;
    ArmID = m_oPlayerPawn.ArmID;
    LegID = m_oPlayerPawn.LegID;
    Tint1ID = m_oPlayerPawn.Tint1ID;
    Tint2ID = m_oPlayerPawn.Tint2ID;
    PatternID = m_oPlayerPawn.PatternID;
    PatternColorID = m_oPlayerPawn.PatternColorID;
    HelmetID = m_oPlayerPawn.HelmetID;
    m_WorldInfo.m_UIWorld.TriggerEvent('SetupPersonalization', m_WorldInfo);
    m_WorldInfo.m_UIWorld.SpawnPawn(m_oPlayerPawn, 'PersonalizationSpawnPoint', 'PersonalizationPawn');
}

function HandleInputEvent(BioGuiEvents Event, optional float fValue = 1)
{
    local array<ASParams> lstParams;
    local ASParams aParam;
    switch (Event)
    {
        case 5:
            if (Abs(fValue) <= 0,0001)
            {
                if (m_bStopScroll == true)
                {
                    oPanel.InvokeMethod("StopScroll");
                    m_bStopScroll = false;
                }
                return;
            }
            aParam.Type = 1;
            aParam.fVar = -fValue;
            lstParams.AddItem(aParam);
            oPanel.InvokeMethodArgs("ScrollInfoText", lstParams);
            m_bStopScroll = true;
            break;
        case 4:
            if (fValue < -0,0001)
            {
                m_nRotating = 1;
            }
            else
            {
                if (fValue > 0,0001)
                {
                    m_nRotating = -1;
                }
                else
                {
                    m_nRotating = 0;
                }
            }
            break;
        default:
            HandleInputEvent(Event, fValue);
            return;
    }
}

event Update(float fDeltaT)
{
    local Rotator rotCurrentPreviewRotation;
    Update(fDeltaT);
    if (GetUIWorldPlayerPawn() == None)
    {
        return;
    }
    if (m_nRotating != 0)
    {
        rotCurrentPreviewRotation = GetUIWorldPlayerPawn().Rotation;
        rotCurrentPreviewRotation.Yaw += m_nRotating * fDeltaT * 182,0444 * RotationDegreesPerSecond;
        rotCurrentPreviewRotation = Normalize(rotCurrentPreviewRotation);
        m_WorldInfo.m_UIWorld.RotatePawn(m_oPlayerPawn, rotCurrentPreviewRotation);
    }
}

function SFXPawn_Player GetUIWorldPlayerPawn()
{
    if (m_WorldInfo == None)
    {
        return None;
    }
    if (m_oUIWorldPawn == None)
    {
        m_oUIWorldPawn = SFXPawn_Player(m_WorldInfo.m_UIWorld.GetSpawnedPawn(m_oPlayerPawn));
        if (m_oUIWorldPawn != None)
        {
            m_oUIWorldPawn.bInPersonalization = true;
        }
    }
    if (m_oUIWorldPawn == None)
    {
        return m_oPlayerPawn;
    }
    return m_oUIWorldPawn;
}

function String GetCleanString(int nStringRef)
{
    if (nStringRef <= 0)
    {
        return "";
    }
    return "$" $ nStringRef;
}

final function bool IsAvailable(CustomizableElement Element)
{
    if (Element.PlotFlag <= 0)
    {
        return true;
    }
    return m_WorldInfo.GetGlobalVariables().GetBool(Element.PlotFlag);
}

function AddDetailEntry(int nEntryIndex, int nTitleText, out array<CustomizableElement> Elements, int nIndex)
{
    local array<ASParams> lstParams;
    local int nOpt;
    local String sTemp;
    local bool bFirstElement;
    bFirstElement = true;
    nIndex = GetIdxByAppearanceID(nIndex, Elements);
    lstParams.Length = 4;
    lstParams[0].Type = 0;
    lstParams[1].Type = 2;
    lstParams[2].Type = 2;
    lstParams[3].Type = 0;
    lstParams[0].nVar = nEntryIndex;
    lstParams[1].sVar = GetCleanString(nTitleText);
    sTemp = "";
    nOpt = 0;
    while (nOpt < Elements.Length)
    {
        if (IsAvailable(Elements[nOpt]))
        {
            if (!bFirstElement)
            {
                sTemp = sTemp @ ",";
            }
            sTemp = sTemp @ GetCleanString(Elements[nOpt].Name);
            bFirstElement = false;
        }
        ++nOpt;
    }
    lstParams[2].sVar = sTemp;
    lstParams[3].nVar = nIndex;
    oPanel.InvokeMethodArgs("AddDetailEntry", lstParams);
}

function ApplyChanges()
{
    local SFXPawn_Player oUIPawn;
    local SFXEngine oEngine;
    oUIPawn = GetUIWorldPlayerPawn();
    if (oUIPawn == m_oPlayerPawn)
    {
        return;
    }
    PlayGuiSound('PersonalizeSelect');
    m_oPlayerPawn.CopyPawnAppearance(oUIPawn);
    BioPlayerController(GetPlayerController()).UnlockAchievement(48);
    if (HelmetID != m_oPlayerPawn.HelmetID)
    {
        Class'BioTelemetry'.SendEventInt('CHANGED_PERSONALIZATION_DATA_HELMET', m_oPlayerPawn.HelmetID);
    }
    if (CasualID != m_oPlayerPawn.CasualID)
    {
        Class'BioTelemetry'.SendEventInt('CHANGED_PERSONALIZATION_DATA_CASUAL', CasualID);
    }
    if (FullBodyID != m_oPlayerPawn.FullBodyID)
    {
        Class'BioTelemetry'.SendEventInt('CHANGED_PERSONALIZATION_DATA_FULLBODY', m_oPlayerPawn.FullBodyID);
    }
    if (TorsoID != m_oPlayerPawn.TorsoID)
    {
        Class'BioTelemetry'.SendEventInt('CHANGED_PERSONALIZATION_DATA_TORSO', m_oPlayerPawn.TorsoID);
    }
    if (ShoulderID != m_oPlayerPawn.ShoulderID)
    {
        Class'BioTelemetry'.SendEventInt('CHANGED_PERSONALIZATION_DATA_SHOULDER', m_oPlayerPawn.ShoulderID);
    }
    if (ArmID != m_oPlayerPawn.ArmID)
    {
        Class'BioTelemetry'.SendEventInt('CHANGED_PERSONALIZATION_DATA_ARM', m_oPlayerPawn.ArmID);
    }
    if (LegID != m_oPlayerPawn.LegID)
    {
        Class'BioTelemetry'.SendEventInt('CHANGED_PERSONALIZATION_DATA_LEG', m_oPlayerPawn.LegID);
    }
    if (Tint1ID != m_oPlayerPawn.Tint1ID)
    {
        Class'BioTelemetry'.SendEventInt('CHANGED_PERSONALIZATION_DATA_TINT1', m_oPlayerPawn.Tint1ID);
    }
    if (Tint2ID != m_oPlayerPawn.Tint2ID)
    {
        Class'BioTelemetry'.SendEventInt('CHANGED_PERSONALIZATION_DATA_TINT2', m_oPlayerPawn.Tint2ID);
    }
    if (PatternID != m_oPlayerPawn.PatternID)
    {
        Class'BioTelemetry'.SendEventInt('CHANGED_PERSONALIZATION_DATA_PATTERN', m_oPlayerPawn.PatternID);
    }
    if (PatternColorID != m_oPlayerPawn.PatternColorID)
    {
        Class'BioTelemetry'.SendEventInt('CHANGED_PERSONALIZATION_DATA_PATTERNCOLOR', m_oPlayerPawn.PatternColorID);
    }
    CasualID = m_oPlayerPawn.CasualID;
    FullBodyID = m_oPlayerPawn.FullBodyID;
    TorsoID = m_oPlayerPawn.TorsoID;
    ShoulderID = m_oPlayerPawn.ShoulderID;
    ArmID = m_oPlayerPawn.ArmID;
    LegID = m_oPlayerPawn.LegID;
    Tint1ID = m_oPlayerPawn.Tint1ID;
    Tint2ID = m_oPlayerPawn.Tint2ID;
    PatternID = m_oPlayerPawn.PatternID;
    PatternColorID = m_oPlayerPawn.PatternColorID;
    HelmetID = m_oPlayerPawn.HelmetID;
    oEngine = SFXEngine(GetPlayerController().Player.Outer);
    if (oEngine != None)
    {
        oEngine.PendingSaveGameAction = 7;
        oEngine.PendingSaveGameLocation = m_oPlayerPawn.Location;
    }
}

function CancelChanges()
{
    local SFXPawn_Player oUIPawn;
    oUIPawn = GetUIWorldPlayerPawn();
    if (oUIPawn == m_oPlayerPawn)
    {
        return;
    }
    PlayGuiSound('PersonalizeCancel');
    oUIPawn.CasualID = CasualID;
    oUIPawn.FullBodyID = FullBodyID;
    oUIPawn.TorsoID = TorsoID;
    oUIPawn.ShoulderID = ShoulderID;
    oUIPawn.ArmID = ArmID;
    oUIPawn.LegID = LegID;
    oUIPawn.Tint1ID = Tint1ID;
    oUIPawn.Tint2ID = Tint2ID;
    oUIPawn.PatternID = PatternID;
    oUIPawn.PatternColorID = PatternColorID;
    oUIPawn.HelmetID = HelmetID;
    oUIPawn.UpdateAppearance();
}

function onExIntDoInitialize()
{
    InitializeUIWorld();
}

function onExIntExit()
{
    if (__ExternalCallback_OnComplete__Delegate == None)
    {
        MassEffectGuiManager(oPanel.oParentManager).HidePersonalizationGUI();
    }
    else
    {
        ExternalCallback_OnComplete();
    }
}

function onExIntSelectCategory(int nCategory)
{
    local SFXPawn_Player pPawn;
    local array<CustomizableElement> BodyTypes;
    local int idx;
    local CustomizableElement N7Armor;
    local int AppearanceID;
    pPawn = GetUIWorldPlayerPawn();
    if (pPawn == None)
    {
        return;
    }
    if (nCategory == 0)
    {
        BodyTypes.Length = pPawn.FullBodyAppearances.Length;
        idx = 0;
        while (idx < BodyTypes.Length)
        {
            BodyTypes[idx] = pPawn.FullBodyAppearances[idx];
            idx++;
        }
        N7Armor.bCustomizable = true;
        N7Armor.Name = pPawn.N7ArmourName;
        N7Armor.Description = pPawn.N7ArmourDescription;
        N7Armor.Id = -1;
        BodyTypes.InsertItem(0, N7Armor);
        if (pPawn.CombatAppearance == 1)
        {
            AppearanceID = pPawn.FullBodyID;
        }
        else
        {
            AppearanceID = -1;
        }
        AddDetailEntry(0, TypeOptionTitle, BodyTypes, AppearanceID);
        onExIntDetailItemChange(0, GetIdxByAppearanceID(AppearanceID, BodyTypes));
    }
    else
    {
        if (nCategory == 1)
        {
            AddDetailEntry(11, CasualOptionTitle, pPawn.CasualAppearances, GetIdxByAppearanceID(pPawn.CasualID, pPawn.CasualAppearances));
            onExIntDetailItemChange(11, GetIdxByAppearanceID(pPawn.CasualID, pPawn.CasualAppearances));
        }
        else
        {
            return;
        }
    }
}

final function int GetAppearanceIDByIdx(int Index, out array<CustomizableElement> AppearanceData)
{
    local int I;
    if (Index >= 0 && Index < AppearanceData.Length)
    {
        I = 0;
        while (I <= Index)
        {
            if (!IsAvailable(AppearanceData[I]))
            {
                Index++;
            }
            ++I;
        }
        return AppearanceData[Index].Id;
    }
    return 0;
}

final function int GetIdxByAppearanceID(int Id, out array<CustomizableElement> AppearanceData)
{
    local int idx;
    local int I;
    local int J;
    J = AppearanceData.Find('Id', Id);
    idx = 0;
    if (J != -1)
    {
        I = 0;
        while (I < J)
        {
            if (IsAvailable(AppearanceData[I]))
            {
                idx++;
            }
            ++I;
        }
        return idx;
    }
    return 0;
}

function onExIntDetailItemChange(int nOptIndex, int nOptionIndex)
{
    local array<ASParams> lstParams;
    local ASParams stParam;
    local array<CustomizableElement> aChangedElement;
    local SFXPawn_Player pPawn;
    local SFXShield_Base oShields;
    local int nSelectionID;
    local int I;
    nSelectionID = -1;
    pPawn = GetUIWorldPlayerPawn();
    if (pPawn == None || pPawn == m_oPlayerPawn)
    {
        return;
    }
    switch (nOptIndex)
    {
        case 0:
            pPawn.bUseCasualAppearance = false;
            if (nOptionIndex == 0)
            {
                pPawn.CombatAppearance = 0;
            }
            else
            {
                nOptionIndex--;
                pPawn.CombatAppearance = 1;
                pPawn.FullBodyID = GetAppearanceIDByIdx(nOptionIndex, pPawn.FullBodyAppearances);
                aChangedElement = pPawn.FullBodyAppearances;
                nSelectionID = pPawn.FullBodyID;
            }
            pPawn.UpdateAppearance();
            break;
        case 11:
            pPawn.bUseCasualAppearance = true;
            aChangedElement = pPawn.CasualAppearances;
            pPawn.CasualID = GetAppearanceIDByIdx(nOptionIndex, pPawn.CasualAppearances);
            nSelectionID = pPawn.CasualID;
            pPawn.UpdateAppearance();
            break;
        case 1:
            aChangedElement = pPawn.HelmetAppearances;
            pPawn.HelmetID = GetAppearanceIDByIdx(nOptionIndex, pPawn.HelmetAppearances);
            nSelectionID = pPawn.HelmetID;
            pPawn.UpdateAppearance();
            break;
        case 2:
            aChangedElement = pPawn.TorsoAppearances;
            pPawn.TorsoID = GetAppearanceIDByIdx(nOptionIndex, pPawn.TorsoAppearances);
            nSelectionID = pPawn.TorsoID;
            pPawn.UpdateAppearance();
            break;
        case 3:
            aChangedElement = pPawn.ShoulderAppearances;
            pPawn.ShoulderID = GetAppearanceIDByIdx(nOptionIndex, pPawn.ShoulderAppearances);
            nSelectionID = pPawn.ShoulderID;
            pPawn.UpdateAppearance();
            break;
        case 4:
            aChangedElement = pPawn.ArmAppearances;
            pPawn.ArmID = GetAppearanceIDByIdx(nOptionIndex, pPawn.ArmAppearances);
            nSelectionID = pPawn.ArmID;
            pPawn.UpdateAppearance();
            break;
        case 5:
            aChangedElement = pPawn.LegAppearances;
            pPawn.LegID = GetAppearanceIDByIdx(nOptionIndex, pPawn.LegAppearances);
            nSelectionID = pPawn.LegID;
            pPawn.UpdateAppearance();
            break;
        case 6:
            aChangedElement = pPawn.SpecAppearances;
            pPawn.SpecID = GetAppearanceIDByIdx(nOptionIndex, pPawn.SpecAppearances);
            nSelectionID = pPawn.SpecID;
            pPawn.UpdateAppearance();
            break;
        case 7:
            aChangedElement = pPawn.Tint1Appearances;
            pPawn.Tint1ID = GetAppearanceIDByIdx(nOptionIndex, pPawn.Tint1Appearances);
            nSelectionID = pPawn.Tint1ID;
            pPawn.UpdateAppearance();
            break;
        case 8:
            aChangedElement = pPawn.Tint2Appearances;
            pPawn.Tint2ID = GetAppearanceIDByIdx(nOptionIndex, pPawn.Tint2Appearances);
            nSelectionID = pPawn.Tint2ID;
            pPawn.UpdateAppearance();
            break;
        case 9:
            aChangedElement = pPawn.PatternAppearances;
            pPawn.PatternID = GetAppearanceIDByIdx(nOptionIndex, pPawn.PatternAppearances);
            nSelectionID = pPawn.PatternID;
            pPawn.UpdateAppearance();
            break;
        case 10:
            aChangedElement = pPawn.PatternColorAppearances;
            pPawn.PatternColorID = GetAppearanceIDByIdx(nOptionIndex, pPawn.PatternColorAppearances);
            nSelectionID = pPawn.PatternColorID;
            pPawn.UpdateAppearance();
            break;
    }
    if (nOptIndex == 0)
    {
        if (pPawn.CombatAppearance == 0)
        {
            AddDetailEntry(1, HelmetOptionTitle, pPawn.HelmetAppearances, pPawn.HelmetID);
            AddDetailEntry(2, TorsoOptionTitle, pPawn.TorsoAppearances, pPawn.TorsoID);
            AddDetailEntry(3, ShoulderOptionTitle, pPawn.ShoulderAppearances, pPawn.ShoulderID);
            AddDetailEntry(4, ArmOptionTitle, pPawn.ArmAppearances, pPawn.ArmID);
            AddDetailEntry(5, LegOptionTitle, pPawn.LegAppearances, pPawn.LegID);
            AddDetailEntry(6, SpecOptionTitle, pPawn.SpecAppearances, pPawn.SpecID);
            AddDetailEntry(7, Tint1OptionTitle, pPawn.Tint1Appearances, pPawn.Tint1ID);
            AddDetailEntry(8, Tint2OptionTitle, pPawn.Tint2Appearances, pPawn.Tint2ID);
            AddDetailEntry(9, PatternOptionTitle, pPawn.PatternAppearances, pPawn.PatternID);
            AddDetailEntry(10, PatternColorOptionTitle, pPawn.PatternColorAppearances, pPawn.PatternColorID);
            jump 1926;
        }
    }
    stParam.Type = 2;
    stParam.sVar = "";
    if (nSelectionID >= 0)
    {
        I = 0;
        while (I < aChangedElement.Length)
        {
            if (aChangedElement[I].Id == nSelectionID)
            {
                stParam.sVar = GetCleanString(aChangedElement[I].Description);
            }
            ++I;
        }
    }
    lstParams.AddItem(stParam);
    oPanel.InvokeMethodArgs("SetInfoText", lstParams);
    pPawn.RecoverFromBleedout();
    oShields = pPawn.GetShields();
    if (oShields != None)
    {
        oShields.CurrentShields = pPawn.GetMaxShields();
    }
    return;
}

defaultproperties
{
    RotationDegreesPerSecond=90
    TypeOptionTitle=253994
    HelmetOptionTitle=253996
    TorsoOptionTitle=312098
    ShoulderOptionTitle=253997
    ArmOptionTitle=253998
    LegOptionTitle=253999
    SpecOptionTitle=313497
    Tint1OptionTitle=254002
    Tint2OptionTitle=254003
    PatternOptionTitle=312099
    PatternColorOptionTitle=312100
    CasualOptionTitle=253995
}
