﻿class SFXShield_Base extends Inventory
    hidecategories(Navigation)
    config(Weapon)
    abstract;

const MINSHIELDBARS = 0;
const MAXSHIELDBARS = 6;
var ResistanceType Resistance;

var() config float HardenedDamageMultiplier;

var() config Vector2D PenetrationHeavy;

var() config Vector2D PenetrationMedium;

var() config Vector2D PenetrationLight;

var() config Vector2D PenetrationVeryLight;

var() config Vector2D PenetrationDefault;

var() config float ShieldPointsPerBar;

var() float BoostedShieldMax;

var() config float PartialBreakPct;

var() config Vector2D ShieldRechargeTime;

var() config ScaledFloat ShieldRegenDelay;

var() config ScaledFloat MaxShields;

var() float CurrentShields;

var transient float ShieldRegenTimer;

var() WwiseEventPairObject ShieldsDownSound;

var() PhysicalMaterial PhysMat;

var() Vector ShieldOffset;

var() float ShieldScale;

var() ParticleSystem PS_Explode;

var() ParticleSystem PS_CriticalHit;

var() ParticleSystem PS_Breach;

var() ParticleSystem PS_Recharged;

var() ParticleSystem PS_Impact;

// ShieldDisplayName: StringRefProperty[44]
var() config bool bAbsorbDamageOnBreak;

var() config bool bFastCoverRegen;

var() config bool bHardened;

var() config bool bRechargeable;

var() bool bShowBarrierOnGUI;

enum ResistanceType
{
    ResistanceType_Shield,
    ResistanceType_Armour,
    ResistanceType_Biotic,
};

delegate ShieldDestroyedPowerCallback(Actor ShieldActor, SFXShield_Base Shield);

simulated function ScaleShields()
{
    MaxShields.StaticBonus = 1;
    if (Pawn(Owner).InPlayerParty())
    {
        MaxShields.StaticBonus += SFXGame(Pawn(Owner).WorldInfo.Game).BonusList.RchShieldUpgradeCache - 1;
        MaxShields.StaticBonus += SFXGame(Pawn(Owner).WorldInfo.Game).BonusList.RchQuarianShieldCache - 1;
    }
    if (SFXPRI(Pawn(Owner).PlayerReplicationInfo) != None)
    {
        ShieldRegenDelay.StaticBonus = SFXPRI(Pawn(Owner).PlayerReplicationInfo).CurrentShieldModifier;
    }
    Class'SFXGame'.ReCalculate(ShieldRegenDelay);
    Class'SFXGame'.ReCalculate(MaxShields);
}

simulated function float GetShieldRegenDelay()
{
    return ShieldRegenDelay.Value;
}

simulated function float GetShieldRechargeTime()
{
    local float Mod;
    Mod = 1;
    if (SFXPRI(Pawn(Owner).PlayerReplicationInfo) != None)
    {
        Mod = SFXPRI(Pawn(Owner).PlayerReplicationInfo).CurrentShieldModifier;
    }
    return Lerp(ShieldRechargeTime.X, ShieldRechargeTime.Y, MaxShields.Level / MaxShields.MaxLevel) * Mod;
}

function Tick(float DeltaSeconds)
{
    local float ShieldMax;
    local BioPawn Pawn;
    local float RechargeTime;
    if (bFastCoverRegen)
    {
        if (ShieldRegenTimer > 0 && BioPawn(Owner) != None)
        {
            if (!BioPawn(Owner).IsInCoverLeaning() && BioPawn(Owner).IsInCover())
            {
                ShieldRegenTimer -= SFXGame(WorldInfo.Game).gameconfig.PlayerCoverRegenTimerMultiplier - 1 * DeltaSeconds;
            }
        }
    }
    if (ShieldRegenTimer > 0)
    {
        ShieldRegenTimer -= DeltaSeconds;
    }
    if (Pawn(Owner) != None && Pawn(Owner).IsDead())
    {
        CurrentShields = 0;
        return;
    }
    ShieldMax = MaxShields.Value;
    if (bRechargeable == false || CurrentShields >= ShieldMax || ShieldRegenTimer > 0)
    {
        return;
    }
    Pawn = BioPawn(Instigator);
    if (Pawn != None && Pawn.bHasShields == false)
    {
        Pawn.bHasShields = true;
    }
    RechargeTime = GetShieldRechargeTime();
    if (RechargeTime > 0)
    {
        CurrentShields += ShieldMax / GetShieldRechargeTime() * DeltaSeconds;
        CurrentShields = FClamp(CurrentShields, 0, ShieldMax);
        if (CurrentShields >= ShieldMax)
        {
            ActivatePSC(PS_Recharged, ShieldScale, ShieldOffset);
            if (Instigator.IsHumanControlled())
            {
                Instigator.StopSound(ShieldsDownSound);
            }
        }
    }
}

simulated function ApplyDamage(out float Damage, out TraceHitInfo HitInfo, out Vector HitLocation, out Vector Momentum, class<SFXDamageType> DamageType, Controller InstigatedBy, Actor DamageCauser)
{
    local SFXWeapon Weapon;
    local float ResearchDamageBonus;
    if (Damage <= 0)
    {
        return;
    }
    ShieldRegenTimer = GetShieldRegenDelay();
    if (CurrentShields <= 0)
    {
        return;
    }
    ResearchDamageBonus = 1;
    if (InstigatedBy.Pawn != None && SFXPawn_Player(InstigatedBy.Pawn) != None || SFXPawn_Henchman(InstigatedBy.Pawn) != None)
    {
        if (DamageType.Default.bResearchBonus_SniperArmorDamage && Resistance == 1)
        {
            ResearchDamageBonus *= SFXGame(InstigatedBy.Pawn.WorldInfo.Game).BonusList.RchSniperArmorDmgCache;
        }
        else
        {
            if (DamageType.Default.bResearchBonus_AssaultRifleResistanceDamage)
            {
                ResearchDamageBonus *= SFXGame(InstigatedBy.Pawn.WorldInfo.Game).BonusList.RchARResistanceDamageCache;
            }
            else
            {
                if (DamageType.Default.bResearchBonus_AutoPistolResistanceDamage && Resistance == 0 || Resistance == 2)
                {
                    ResearchDamageBonus *= SFXGame(InstigatedBy.Pawn.WorldInfo.Game).BonusList.RchAutoPistolResistanceDmgCache;
                }
                else
                {
                    if (DamageType.Default.bResearchBonus_HeavyPistolArmorDamage && Resistance == 1)
                    {
                        ResearchDamageBonus *= SFXGame(InstigatedBy.Pawn.WorldInfo.Game).BonusList.RchHPistolArmorDmgCache;
                    }
                    else
                    {
                        if (DamageType.Default.bResearchBonus_ShotgunResistanceDamage && Resistance == 0)
                        {
                            ResearchDamageBonus *= SFXGame(InstigatedBy.Pawn.WorldInfo.Game).BonusList.RchShotgunResistanceDmgCache;
                        }
                    }
                }
            }
        }
    }
    if (ResearchDamageBonus > 0)
    {
        Damage *= ResearchDamageBonus;
    }
    ApplyDamageToShields(Damage, DamageType, Momentum, HitLocation, HitInfo, InstigatedBy);
    if (ResearchDamageBonus > 0)
    {
        Damage /= ResearchDamageBonus;
    }
    if (DamageType.Default.bSpawnWeaponImpacts)
    {
        Weapon = SFXWeapon(DamageCauser);
        if (Weapon != None)
        {
            if (Weapon.bSuppressImpactFX == false)
            {
                ActivateWeaponImpact(Weapon, HitInfo, HitLocation, Weapon.PS_DefaultImpactEffect, SkeletalMeshComponent(HitInfo.HitComponent));
                ActivateWeaponImpact(Weapon, HitInfo, HitLocation, PS_Impact, SkeletalMeshComponent(HitInfo.HitComponent));
            }
            if (Weapon.bSuppressAudio == false)
            {
                PlayShieldSound(Weapon, HitInfo, HitLocation);
            }
        }
    }
    BioPawn(Owner).PlayShieldHit(Damage, InstigatedBy, HitLocation, DamageType, Momentum, HitInfo);
}

simulated protected function ApplyDamageToShields(out float Damage, class<SFXDamageType> DamageType, Vector Momentum, Vector HitLocation, TraceHitInfo HitInfo, Controller InstigatedBy)
{
    local float ShieldDamage;
    local float DamageResistance;
    local BioPawn Pawn;
    local float RandomNumber;
    local float ShieldBreakValue;
    local Vector CriticalHitLocation;
    local SFXGameEffect Effect;
    Pawn = BioPawn(Instigator);
    DamageResistance = GetDamageResistance(DamageType);
    ShieldDamage = Damage * DamageResistance;
    if (Pawn != None && SFXPawn_Player(Pawn) != None)
    {
        ShieldDamage *= SFXGame(Pawn.WorldInfo.Game).BonusList.RchShieldDamageCache;
    }
    if (CurrentShields > ShieldDamage)
    {
        ShieldBreakValue = MaxShields.Value * PartialBreakPct;
        if (Instigator.IsHumanControlled() && DamageType.Default.bNoShieldBreakVFX == false)
        {
            if (CurrentShields >= ShieldBreakValue && CurrentShields - ShieldDamage < ShieldBreakValue)
            {
                ActivatePSC(PS_CriticalHit);
            }
        }
        CurrentShields -= ShieldDamage;
        Damage = 0;
    }
    else
    {
        if (Pawn != None && SFXPawn_Player(Pawn) != None || SFXPawn_Henchman(Pawn) != None && SFXGame(Pawn.WorldInfo.Game).BonusList.RchShieldRestoreCache > 0)
        {
            if (FRand() < SFXGame(Pawn.WorldInfo.Game).BonusList.RchShieldRestoreChance)
            {
                CurrentShields = MaxShields.Value * SFXGame(Pawn.WorldInfo.Game).BonusList.RchShieldRestoreCache;
                ActivatePSC(PS_Recharged, ShieldScale, ShieldOffset);
                if (Instigator.IsHumanControlled())
                {
                    Instigator.StopSound(ShieldsDownSound);
                }
                Damage = 0;
                return;
            }
        }
        ActivatePSC(PS_Breach, ShieldScale, ShieldOffset);
        if (Instigator.IsHumanControlled())
        {
            CriticalHitLocation = vect(0,0,65);
            if (DamageType.Default.bNoShieldBreakVFX == false)
            {
                ActivatePSC(PS_CriticalHit, 1, CriticalHitLocation);
            }
            Instigator.PlaySound(ShieldsDownSound);
            SFXGame(WorldInfo.Game).TriggerVocalizationEvent(28, Pawn);
        }
        if (Pawn != None)
        {
            if (Pawn.Rotation Dot Momentum < 0)
            {
                RandomNumber = FRand();
                if (RandomNumber > 0,7)
                {
                    Pawn.PlayAnimatedReaction(37, InstigatedBy, true, HitLocation, -Normal(Momentum), HitInfo);
                }
                else
                {
                    if (RandomNumber > 0,4)
                    {
                        Pawn.PlayAnimatedReaction(35, InstigatedBy, true, HitLocation, -Normal(Momentum), HitInfo);
                    }
                    else
                    {
                        if (RandomNumber > 0,1)
                        {
                            Pawn.PlayAnimatedReaction(35, InstigatedBy, true, HitLocation, -Normal(Momentum), HitInfo);
                        }
                        else
                        {
                            Pawn.PlayAnimatedReaction(36, InstigatedBy, true, HitLocation, -Normal(Momentum), HitInfo);
                        }
                    }
                }
            }
        }
        ShieldDamage -= CurrentShields;
        Damage = ShieldDamage / GetDamageResistance(DamageType);
        if (SFXPawn_Player(Instigator) != None || SFXPawn_Henchman(Instigator) != None)
        {
            Effect = BioPawn(Owner).GetModule(Class'SFXModule_GameEffectManager').CreateEffect(Class'SFXGameEffect_DamageMultiplier', 'ShieldBreach', SFXGame(Owner.WorldInfo.Game).Difficulty.Default.BreachDuration, 1, 0,35);
            if (Effect != None)
            {
                Effect.OnApplied();
            }
            Damage = 0;
        }
        CurrentShields = 0;
        Pawn.bHasShields = false;
        ShieldDestroyedPowerCallback(Owner, self);
        SFXGame(WorldInfo.Game).TriggerVocalizationEvent(28, Pawn, None);
    }
    if (bAbsorbDamageOnBreak && !DamageType.Default.bIgnoresBleedout)
    {
        Damage = 0;
    }
}

simulated function float GetDamageResistance(class<SFXDamageType> DamageType)
{
    local float fResistance;
    local SFXModule_GameEffectManager GEModule;
    local float DamageReduction;
    if (DamageType == None)
    {
        return 1;
    }
    fResistance = 1;
    switch (Resistance)
    {
        case 1:
            fResistance = DamageType.Default.Resistance.Armour;
            break;
        case 2:
            fResistance = DamageType.Default.Resistance.Biotic;
            break;
        case 0:
            fResistance = DamageType.Default.Resistance.Shield;
            break;
    }
    fResistance = fResistance + DamageType.Default.ShieldBonus;
    if (Owner != None)
    {
        GEModule = Owner.GetModule(Class'SFXModule_GameEffectManager');
        if (GEModule != None)
        {
            DamageReduction = GEModule.GetEffectValue(Class'SFXGameEffect_ShieldDamageReduction');
            fResistance = fResistance - fResistance * DamageReduction;
        }
    }
    return FClamp(fResistance, 0, 10);
}

final simulated function ActivatePSC(ParticleSystem Template, optional float ImpactScale = 1, optional Vector Offset)
{
    local SFXObjectPool Pool;
    local ParticleSystemComponent PSC;
    if (Owner != None && Template != None)
    {
        Pool = SFXGame(Owner.WorldInfo.Game).ObjectPool;
        if (Pool != None)
        {
            PSC = Pool.GetGenericParticleSystemComponent(Template);
            if (PSC != None)
            {
                Owner.AttachComponent(PSC);
                PSC.SetScale(ImpactScale);
                PSC.SetTranslation(Offset);
                Pool.ApplyLODLevel(PSC, Owner.Location);
                PSC.SetActive(true);
            }
        }
    }
}

final simulated function ActivateWeaponImpact(SFXWeapon Weapon, out TraceHitInfo HitInfo, out Vector HitLocation, ParticleSystem Template, SkeletalMeshComponent SMC)
{
    local SFXObjectPool Pool;
    local ParticleSystemComponent PSC;
    local Vector HitNormal;
    if (Weapon != None && SMC != None && Template != None)
    {
        HitNormal = Normal(Weapon.Instigator.Location - HitLocation);
        if (Pawn(Owner) != None)
        {
            HitLocation += HitNormal * Pawn(Owner).CylinderComponent.CollisionRadius;
        }
        Pool = SFXGame(Owner.WorldInfo.Game).ObjectPool;
        if (Pool != None)
        {
            PSC = Pool.GetImpactParticleSystemComponent(Template);
            if (PSC != None)
            {
                Pool.AttachParticleSystemComponent(PSC, SMC.Owner, SMC, HitInfo.BoneName, HitLocation, HitNormal, false);
                Pool.ApplyLODLevel(PSC, HitLocation);
                PSC.SetScale(GetImpactScale(Weapon, HitLocation));
                PSC.SetVectorParameter('Shield_Damage', vect(1,1,1) * 1 - CurrentShields / MaxShields.Value);
                PSC.SetActive(true);
            }
        }
    }
}

function PlayShieldSound(SFXWeapon Weapon, TraceHitInfo HitInfo, Vector HitLocation)
{
    local WwiseEvent ShieldSound;
    ShieldSound = Weapon.GetImpactSound(PhysMat);
    if (ShieldSound != None)
    {
        Instigator.PlaySound(ShieldSound, false, false, false, HitLocation);
    }
}

function float GetImpactScale(SFXWeapon Weapon, Vector HitLocation)
{
    local float Range;
    local float Scale;
    Scale = 1;
    if (Weapon.bIsAutomatic == false)
    {
        Scale = 1,5;
    }
    if (Weapon.bUseSniperCam == false && SFXPawn_Player(Owner) == None && SFXPawn_Henchman(Owner) == None)
    {
        Range = VSize(HitLocation - Weapon.Owner.Location);
        Scale = FClamp(Scale * Range + 3000 / 4000, Scale, Scale * 1,5);
    }
    return Scale;
}

final function int GetShieldBars()
{
    if (ShieldPointsPerBar == 0)
    {
        return 0;
    }
    return Clamp(MaxShields.Value / ShieldPointsPerBar, 0, 6);
}

function GivenTo(Pawn thisPawn, optional bool bDoNotActivate)
{
    local BioPawn BP;
    BP = BioPawn(Instigator);
    if (BP.GetCurrentShields() > 0)
    {
        BP.bHasShields = true;
    }
}

function ItemRemovedFromInvManager()
{
    local BioPawn BP;
    BP = BioPawn(Instigator);
    if (BP.GetCurrentShields() > 0)
    {
        BP.bHasShields = true;
    }
}

state Active
{
    simulated function BeginState(Name PreviousStateName)
    {
        Class'SFXGame'.ReCalculate(MaxShields);
        CurrentShields = MaxShields.Value;
    }

}

defaultproperties
{
    ShieldDisplayName=System.Byte[]
    PS_Impact=ParticleSystem'Generic_Imp_ForceField'
    PS_Recharged=ParticleSystem'Shield_Booster'
    PS_Breach=ParticleSystem'Shield__Break'
    PS_CriticalHit=ParticleSystem'Blood_Red_CriticalHit_Misty'
    PS_Explode=ParticleSystem'Flash_Explosion'
    ShieldScale=1
    PhysMat=PhysicalMaterial'PHM_GEN_Shields'
    ShieldsDownSound=WwiseEventPairObject'Play_vfx_shield_down_Pair'
    MaxShields=(X=3,864781E-41,Y=0,MaxLevel=10594,Level=0,Value=5,605194E-45,Bonuses=UnHood.Engine.UnPropertyArray,StaticBonus=0)
    ShieldRegenDelay=(X=3,864781E-41,Y=0,MaxLevel=10594,Level=0,Value=5,605194E-45,Bonuses=UnHood.Engine.UnPropertyArray,StaticBonus=5)
    ShieldRechargeTime=(X=2,Y=2)
    PartialBreakPct=0,55
    ShieldPointsPerBar=40
    Components=None
    ***Deleted***=System.Byte[]
}
