﻿class SFXVocalizationManagerNativeBase extends Actor
    hidecategories(Navigation)
    config(Game)
    native;

struct native SFXVocalization
{
    var BioPawn Speaker;

    var SFXVocalizationEvent Event;

    var ESFXVocalizationRole Role;

    var int Specific;

}

struct native SFXVocalizationEventProperties
{
    var() float ChanceToPlay;

    var() float MinTimeBetweenSec;

    var float TimeLastPlayed;

    var float delay;

    var float MaxWitnessDistSq;

    var bool bQueueIfBlocked;

    var bool bCanInterrupt;

    var bool bCanPlayIfDead;

    var bool bCanPlayIfRagdolled;

    var array<ESFXVocalizationRole> Roles;

}

var transient array<SFXVocalizationEvent> QueuedEvents;

var array<SFXVocalizationEventProperties> EventProperties;

var transient array<BioPawn> m_aIgnorePawn;

var transient int DebugCounter;

var transient bool bAlwaysPlay;

var transient bool bDebugging;

var SFXVocalizationBank OverridesAllIfSet;

struct native SFXVocalizationEvent
{
    var BioPawn Instigator;

    var BioPawn Recipient;

    var int Id;

    var float DelayTimeRemainingSec;

    var int DebugIndex;

}

simulated protected event bool PlaySFXVocalizationSLineInternal(const out SFXVocalizationEvent Event, const out SFXVocalizationLine LineToPlay, BioPawn Speaker, optional float DelaySec)
{
    local BioWorldInfo BWI;
    BWI = BioWorldInfo(WorldInfo);
    if (BWI == None)
    {
        return false;
    }
    if (LineToPlay.Sound != None)
    {
        Speaker.PlayFOVO(LineToPlay.Sound);
        Class'BioRemoteLogger'.SendVocalizationEvent(Speaker.Tag, LineToPlay.Sound.Name);
    }
    return true;
}

event SFXVocalizationBank GetPawnVocalizationBank(BioPawn P)
{
    if (OverridesAllIfSet != None)
    {
        return OverridesAllIfSet;
    }
    if (SFXPawn(P) != None && SFXPawn(P).VocalizationBankOverride != None)
    {
        return SFXPawn(P).VocalizationBankOverride;
    }
    if (P != None && P.PawnType != None)
    {
        return P.PawnType.VocalizationBank;
    }
    return None;
}

function AddToIgnoreList(BioPawn oPawn)
{
    if (m_aIgnorePawn.Find(oPawn) == -1)
    {
        m_aIgnorePawn.AddItem(oPawn);
    }
}

function RemoveFromIgnoreList(BioPawn oPawn)
{
    m_aIgnorePawn.RemoveItem(oPawn);
}

native function bool HandleSFXVocalizationEvent(const out SFXVocalizationEvent Event);

native function bool SpecificLocationMatches(ESFXVocalizationLocation Type, BioPawn BP, BioPawn Speaker);

native function bool SpecificCharacterTypeMatches(ECharacterType Type, BioPawn Pawn);

native function bool SpecificAffiliationMatches(int idx, BioPawn Pawn);

native function bool SpecificGenderMatches(ESFXVocalizationGender Type, BioPawn Pawn);

native function bool SpecificWeaponMatches(ESFXVocalizationWeapon Type, BioPawn Pawn);

native function bool SpecificChallengeMatches(EChallengeType Type, BioPawn Pawn);

native function bool SpecificNameMatches(ESFXVocalizationName Type, BioPawn Pawn);

event String DebugOutput(int Type, int Value)
{
    local Object En;
    local String output;
    En = Class'SFXVocalizationTypes'.Default.EnumForType[Type];
    output = Type @ "---" @ GetEnum(En, Value);
    return output;
}

simulated event TickEvents(float DeltaTime);

defaultproperties
{
    RemoteRole=System.Byte[]
    TickGroup=System.Byte[]
    bReplicateMovement=false
    bMovable=false
    CollisionType=System.Byte[]
}
